:- prolog_flag(character_escapes,_,on).

:- use_module(library(terms),[subsumes_chk/2,term_variables/2,cyclic_term/1,
			      variant/2,term_hash/2,subsumes/2]).
:- use_module(library(lists),[member/2,append/3,select/3,same_length/2,
			      memberchk/2,reverse/2,is_list/1,sum_list/2,nth/3]).

:- use_module(library(ugraphs),[vertices_edges_to_ugraph/3,top_sort/2,
				add_vertices/3,transpose/2,vertices/2,colouring/3]).
:- use_module(library(assoc),[assoc_to_list/2,ord_list_to_assoc/2,get_assoc/5,
                              put_assoc/4,empty_assoc/1,get_assoc/3,map_assoc/3,
			      del_assoc/4,list_to_assoc/2]).
:- use_module(library(system),[file_exists/2]).

subsumeschk(X,Y) :- subsumes_chk(X,Y).
nth1(A,B,C) :- nth(A,B,C).
sumlist(L,S) :- sub_list(L,S).
transpose_ugraph(G,T) :- transpose(G,T).
avl_to_list(A,L) :- assoc_to_list(A,L).
ord_list_to_avl(L,A) :- ord_list_to_assoc(L,A).
avl_change(K,A1,V1,A2,V2) :- get_assoc(K,A1,V1,A2,V2).
avl_store(K,A1,V,A2) :- put_assoc(K,A1,V,A2).
empty_avl(E) :- empty_assoc(E).
avl_fetch(K,A,V) :- get_assoc(K,A,V).
avl_map(P,A1,A2) :- map_assoc(P,A1,A2).
avl_delete(K,A1,V,A2) :- del_assoc(K,A1,V,A2).
list_to_avl(L,A) :- list_to_assoc(L,A).

sp_put_attributes(TypePos,TFS,Goals,Goals) :-
  '$put_attributes'(TypePos,TFS).

:- multifile user:term_expansion/2.

term_expansion(end_of_file,Code) :- term_expansion_eof(Code).

term_expansion('--->'(WordStart,DescOrGoal),Code) :-
  term_expansion_lex(Code,WordStart,DescOrGoal).

dynamic_bind_mgsat(Code,Code).

:- meta_predicate static_abolish(:).
static_abolish(Spec) :- abolish(Spec).
static_abolish_tree(List) :- abolish(List).

filter_iqs([],[],[]).
filter_iqs([ResGoal|Residue],Iqs,FSResidue) :-
  ResGoal = _-Goal,
  filter_iq(Goal,ResGoal,Residue,Iqs,FSResidue).

filter_iq(prolog:dif__int(FS1,FS2),_,Residue,[ineq(FS1,FS2)|Iqs],FSResidue) :-
  !,filter_iqs(Residue,Iqs,FSResidue).
filter_iq('$put_attributes'(Var,Term),_,Residue,Iqs,FSResidue) :-
  functor(Term,tfs,Arity), (Arity == 2 ; Arity == 4),
  !,'$put_attributes'(Var,Term),  % this is wasting heap cells if we called
  % this on frozen goals (from frozen_term/2) rather than on a residuation.
  filter_iqs(Residue,Iqs,FSResidue).
filter_iq(_Goal,ResGoal,Residue,Iqs,[ResGoal|FSResidue]) :-
  filter_iqs(Residue,Iqs,FSResidue).

:- meta_predicate call_residue(:,-,-).
call_residue(Goal,_Term,Residue) :-
  call_residue(Goal,Residue).

:- meta_predicate call_with_residue(:,-,-).
call_with_residue(Goal,_Term,Residue) :-
  call_residue(Goal,Residue).

residuate_term(_Term,true). % dummy predicate for SP3.


