% SP4: subsumes_chk/2 is now subsumeschk/2
:- use_module(library(terms),[subsumeschk/2,term_variables/2,cyclic_term/1,
	  		      variant/2,term_hash/2,subsumes/2]).
% SP4: nth/3 is now nth1/3, sum_list/2 is now sumlist/2
:- use_module(library(lists),[select/3,same_length/2,
			      reverse/2,is_list/1,sumlist/2,nth1/3]).

% SP4: transpose/2 is now transpose_ugraph/2, colouring/3 is gone.
:- use_module(library(ugraphs),[vertices_edges_to_ugraph/3,top_sort/2,
				add_vertices/3,transpose_ugraph/2,vertices/2]).
:- ensure_loaded(ale_home('colouring4.pl')).

% SP4: the old lib(assoc) is now lib(avl)
:- use_module(library(avl),[avl_to_list/2,ord_list_to_avl/2,avl_change/5,avl_store/4,empty_avl/1,
                            avl_fetch/3,avl_map/3,avl_delete/4,list_to_avl/2]).

% SP4: file_exists/2 moved to library(file_systems)
:- use_module(library(file_systems),[file_exists/2]).

sp_put_attributes(TypePos,TFS,Goals,GoalsRest) :-
  Goals = ['$put_attributes'(TypePos,TFS)|GoalsRest].

% ------------------------------------------------------------------------------
% bind_sequence_to_head(Seq:<goal_seq>,Goals:<goal>s,GoalsRest:<goal>s
% ------------------------------------------------------------------------------
%
% ------------------------------------------------------------------------------
bind_sequence_to_head(true,Goals,Goals) :- !.
bind_sequence_to_head((G1,G2),Goals,GoalsRest) :-
  !,bind_sequence_to_head(G1,Goals,GoalsMid),
  bind_sequence_to_head(G2,GoalsMid,GoalsRest).
bind_sequence_to_head(G,[G|GoalsRest],GoalsRest).

% SP4: term_expansion/[2,4] are now term_expansion/6.
:- multifile user:term_expansion/6.

term_expansion(end_of_file,_Layout,Ids,Code,[],[ale_eof|Ids]) :-
  nonmember(ale_eof,Ids),
  term_expansion_eof(Code).

term_expansion('--->'(WordStart,DescOrGoal),Layout,Ids,Code,Layout2,[ale_lex|Ids]) :-
  nonmember(ale_lex,Ids),
  ( Layout == [] -> Layout2 = []  % unclear why Layout is often missing, but it is
  ; Layout = [First|Rest], Layout2 = [First,[First|Rest]]
  ),
  term_expansion_lex(Code,WordStart,DescOrGoal).

dynamic_bind_mgsat(Code,CodeMid) :-
  ( ale_flag(mgsat,offline) -> Code = [(:- dynamic (bind_mgsat)/3)|CodeMid]
  ; Code = CodeMid  % SP4: cannot abolish static predicates
  ).

:- meta_predicate static_abolish(:,+).
static_abolish(Spec) :- abolish(Spec,[force(true)]).
static_abolish_tree(List) :- abolish(List,[force(true),tree(true)]).

filter_iqs(ResidueSeq,Iqs,FSResidue) :-
  filter_iqs(ResidueSeq,Iqs,[],FSResidue,[]).

filter_iqs(true,Iqs,Iqs,FSResidue,FSResidue) :- !.
filter_iqs((Goal1,Goal2),Iqs,IqsRest,FSResidue,FSResidueRest) :-
  !,filter_iqs(Goal1,Iqs,IqsMid,FSResidue,FSResidueMid),
  filter_iqs(Goal2,IqsMid,IqsRest,FSResidueMid,FSResidueRest).
filter_iqs(Goal,Iqs,IqsRest,FSResidue,FSResidueRest) :-
  % SP4: residues are produced by frozen/2 now, so they have a different syntax
  filter_iq(Goal,Iqs,IqsRest,FSResidue,FSResidueRest). 

filter_iq(prolog:dif__int(FS1,FS2),[ineq(FS1,FS2)|Iqs],Iqs,FSResidue,FSResidue) :-  !.
filter_iq(user:'$put_attributes'(_Var,Term),Iqs,Iqs,FSResidue,FSResidue) :-
  % SP4: '$put_attributes'/2 calls are module-prefixed in residues
  functor(Term,tfs,Arity), (Arity == 2 ; Arity == 4),
  !. % SP4: attributes now are either already attached or attached before the call to filter_iqs/3
filter_iq(ResGoal,Iqs,Iqs,[ResGoal|FSResidue],FSResidue).

:- meta_predicate call_residue(:,-,-).    %SP4: no more call_residue/2
call_residue(Goal,Term,Residue) :-
  call(Goal),
  residuate_term(Term,Residue).

:- meta_predicate call_with_residue(:,:,-).
call_with_residue(Goal,Residue,Residue) :-
  call(Goal),
  call(Residue).

%SP4: we can no longer assert, consult or findall variables with attributes or suspensions.
% residuate_term/2 binds structure to the attributes and suspensions so that they will survive
% these operations on their carrier terms.  Technically, this is not residuation, because
% they are still attached to the original FS, but copying will strip them off the FS.
residuate_term(FS,ResidueSeq) :-
  term_variables(FS,FSVars),
  residuate_term_iter(FSVars,[],[],UnSortedResidue),
  sort(UnSortedResidue,Residue),
  goal_list_to_seq(Residue,ResidueSeq).

residuate_term_iter(Vars,OldVars,OldResidue,Residue) :-
  residuate_term_iter_act(Vars,OldVars,AllVars,NextResidue,OldResidue,NewResidue),
  ( NextResidue == [] -> Residue = OldResidue
  ; term_variables(NextResidue,NextVars),
    residuate_term_iter(NextVars,AllVars,NewResidue,Residue)
  ).

residuate_term_iter_act([],Vars,Vars,[],Residue,Residue).
residuate_term_iter_act([V|Vs],OldVars,AllVars,NextResidue,OldResidue,NewResidue) :-
  residuate_term_iter_act2(OldVars,V,Vs,AllVars,NextResidue,OldResidue,NewResidue).

residuate_term_iter_act([],O,OldVars,[O|OldVars],[],Residue,Residue).
residuate_term_iter_act([V|Vs],O,OldVars,AllVars,NextResidue,OldResidue,NewResidue) :-
  residuate_term_iter_act2(V,Vs,O,OldVars,AllVars,NextResidue,OldResidue,NewResidue).

residuate_term_iter_act2([],V,Vs,[V|Vs],[RSeq|NextResidue],OldResidue,NewResidue) :-
  frozen(V,RSeq),  % the attriburte_goal/2 hook that we added will cause attributes to be collected here, too
                % we could actually handle attribute residues specially, since we know to look at the Int
  bind_sequence_to_head(RSeq,MidResidue,OldResidue),
  map_frozen(Vs,NextResidue,MidResidue,NewResidue).
residuate_term_iter_act2([O|Os],V,Vs,AllVars,NextResidue,OldResidue,NewResidue) :-
  compare(Comp,O,V),
  residuate_term_iter_act3(Comp,O,Os,V,Vs,AllVars,NextResidue,OldResidue,NewResidue).

residuate_term_iter_act2(V,Vs,O,Os,AllVars,NextResidue,OldResidue,NewResidue) :-
  compare(Comp,O,V),
  residuate_term_iter_act3(Comp,O,Os,V,Vs,AllVars,NextResidue,OldResidue,NewResidue).

residuate_term_iter_act3(=,V,Os,_V,Vs,[V|AllVars],NextResidue,OldResidue,NewResidue) :-
  residuate_term_iter_act(Vs,Os,AllVars,NextResidue,OldResidue,NewResidue).
residuate_term_iter_act3(<,O,Os,V,Vs,[O|AllVars],NextResidue,OldResidue,NewResidue) :-
  residuate_term_iter_act2(Os,V,Vs,AllVars,NextResidue,OldResidue,NewResidue).
residuate_term_iter_act3(>,O,Os,V,Vs,[V|AllVars],[RSeq|NextResidue],OldResidue,NewResidue) :-
  frozen(V,RSeq),
  bind_sequence_to_head(RSeq,MidResidue,OldResidue),
  residuate_term_iter_act(Vs,O,Os,AllVars,NextResidue,MidResidue,NewResidue).

map_frozen([],[],Residue,Residue).
map_frozen([V|Vs],[R|NextResidue],OldResidue,NewResidue) :-
  frozen(V,R),
  map_frozen(Vs,NextResidue,[R|OldResidue],NewResidue).

% SP4: tab/1 is no longer provided
tab(N) :- N >= 0, tab_act(N).
tab_act(0) :- !.
tab_act(N) :-
  space, NewN is N - 1,
  tab_act(NewN).

space :- write(' ').
