%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: print_mrs.pl,v $
%%  $Revision: 1.12 $
%%      $Date: 2006/09/18 08:22:44 $
%%     Author: Stefan Mueller (Stefan.Mueller@uni-bremen.de)
%%    Purpose: Prints an MRS, used by chart_display and itsdb
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: print_mrs.pl,v $
%% Revision 1.12  2006/09/18 08:22:44  stefan
%% integrated utool
%%
%% Revision 1.11  2004/06/16 17:42:15  stefan
%% generalized printing of event and index features (Felix grammar)
%%
%% Revision 1.10  2004/03/10 14:34:29  stefan
%% now MRSes that look exactly like the LKB MRSes are produced.
%%
%% Revision 1.9  2004/02/29 21:05:33  stefan
%% liszt_path and hcons_path can now be configured
%%
%% Revision 1.8  2004/02/19 15:13:26  stefan
%% removed the code that produced a the warning because of unspecified indices
%%
%% Revision 1.7  2004/02/09 20:32:31  stefan
%% changed to new MRS format, added output for pron_rel
%%
%% Revision 1.6  2004/01/10 03:43:07  dm
%% fixed bugs in print_mrs - it didn't do dereferencing
%%
%% Revision 1.5  2003/07/11 06:36:15  stefan
%% Hm, seem to have forgotten to check some of this in.
%% Sorry
%%
%% Revision 1.4  2003/05/20 20:39:00  stefan
%% generalized code for MRS output of MERGE
%%
%% Revision 1.3  2003/05/19 15:28:51  stefan
%% simplified code since I now know that [incr TSDB()] ignores
%% newlines.
%%
%% Revision 1.2  2003/05/14 11:40:55  stefan
%% Added feature ordering for relation output
%% Added error message if H-Cons is not a closed list
%%
%%
%% Revision 1.1  2003/05/13 09:10:50  stefan
%% put MRS printing in a different file
%% included a (wrong) fix to rule/1 in ale_redefinitions.pl
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- ensure_loaded([trale_home('chart_display/utool-interface')]).

print_mrs(FS) :-
        print_mrs(FS,screen,MRS,[]),
        name(MRSA,MRS),
        write(MRSA).

% FS = Feature Structure, Format= screen; tsdb, MRS,MRSRest = Difflists

/*

according to lkb-2003-10-01/src/mrs/basemrs.lisp the syntax of MRS output should be:

MRS -> [ LTOP INDEX LISZT HCONS ]

LTOP -> top: VAR

INDEX -> index: VAR

LISZT -> liszt: < REL* >

HCONS -> hcons: < QEQ* >

REL -> [ PREDNAME handel: VAR FEATPAIR* ]

FEATPAIR -> FEATNAME: VAR | CFEATNAME: CONSTNAME

VAR -> VARNAME | VARNAME [ VARTYPE EXTRAPAIR* ]

EXTRAPAIR -> PATHNAME: CONSTNAME

QEQ -> VARNAME RELNNAME VARNAME

*/

/*

Example from LinGO

[ LTOP: h1 INDEX: e2 [ EVENT DIVISIBLE: BOOL E.TENSE: PAST* E.ASPECT: NO_ASPECT* E.MOOD: INDICATIVE* ] RELS: < [ prpstn_m_rel LBL: h1 MARG: h3 ] [ proper_q_rel LBL: h4 ARG0: x5 [ REF-IND PNG.GEN: REAL_GENDER PNG.PN: 3SG* DIVISIBLE: - ] RSTR: h6 BODY: h7 ] [ named_rel LBL: h8 ARG0: x5 CARG: "ABRAMS" ] [ _bark_v_rel LBL: h9 ARG0: e2 ARG1: x5 ] > HCONS: < h3 QEQ h9 h6 QEQ h8 > ]

*/

/*

Output to Trale on screen

<h1,e2:anything,
{
 h3:udef_q(arg0:x4:third:sg:human,rstr:h5,body:h6)
 h7:man(arg0:x4)
 h8:telephone(arg0:i9:third:sg)
 h10:past_indicative_or_progressive(arg0:e2,arg3:h11)
 h11:do(arg0:e2,arg1:x4,arg3:h8)
}
{
 h1  =q h3
 h5  =q h7
} >

Output to Utool:

psoa(h1,e2,
[
 rel('every_q',h3,
     [ attrval('ARG0',x4),
       attrval('RSTR',h5),
       attrval('BODY',h6)]),
 rel('mutmaßlich',h7,
     [ attrval('ARG0',e8),
       attrval('ARG1',h9)]),
 rel('mörder_rel',h10,
     [ attrval('ARG0',x4)]),
 rel('assertion',h11,
     [ attrval('ARG0',e2),
       attrval('ARG3',h12)]),
 rel('pres_ind_conj',h12,
     [ attrval('ARG0',e2),
       attrval('ARG3',h13)]),
 rel('lachen_rel',h13,
     [ attrval('ARG0',e2),
       attrval('ARG1',x4)]),
 rel('wieder',h14,
     [ attrval('ARG1',h15)])],
 hcons([
 qeq(h1,h3),
 qeq(h5,h7),
 qeq(h9,h10),
 qeq(h15,h13),
        qeq(h15,h3)
 ]))

*/


print_mrs(FS,Format,MRS,MRSRest) :-
        retractall(mynum(_)),
        assert(mynum(0)),

        gtop_path_(TopPath),
        path_fs2pathval(TopPath,FS,bot,TopFS,TopVType),

        liszt_path_(LPath),
        path_fs2pathval(LPath,FS,bot,RelsFS,RVType),

        empty_assoc(EIndices),
        inst(RelsFS,RVType,TopFS,TopVType,EIndices,Inds0,TRef),

        % Print 
        (Format = screen ->
            format_to_chars('<~w,',[TRef],MRS,C1)
        ; Format = utool ->
            format_to_chars('psoa(~w,',[TRef],MRS,C1)
        ;   format_to_chars('[ LTOP:~w INDEX: ',[TRef],MRS,C1)
        ),

        ind_path_(IndPath),
        path_fs2pathval(IndPath,FS,bot,IndFS,IndVType),

        write_o(IndFS,IndVType,RelsFS,RVType,Inds0,Inds1,Format,C1,C2),
        (Format = screen ->
            format_to_chars(',~n{',[],C2,C3)
        ;Format = utool ->
            format_to_chars(',~n[',[],C2,C3)
        ;   format_to_chars(' RELS: < ',[],C2,C3)
        ),

        write_rels(RelsFS,RVType,Inds1,Inds2,Format,C3,C4),
        (Format = screen ->
            format_to_chars('~n}~n{',[],C4,C5)
        ;Format = utool ->
            format_to_chars('],~n hcons([',[],C4,C5)
        ;   format_to_chars('> HCONS: < ',[],C4,C5)
        ),

        hcons_path_(HPath),
        path_fs2pathval(HPath,FS,bot,HCons,HCVType),

        write_hcons(HCons,HCVType,Inds2,_Inds3,Format,C5,C6),
        (Format = screen ->
            format_to_chars('~n} >~n~n',[],C6,MRSRest)
        ;Format = utool ->
            format_to_chars('~n ]))',[],C6,MRSRest)
        ;   format_to_chars(' > ]',[],C6,MRSRest)
        ),
	!.

print_mrs(_FS,_Format,MRS,MRSRest) :-
	 format_to_chars('~N* ERROR: MRS: output failed.~n',[],MRS,MRSRest),
	 assert(error(mrs,'MRS: output failed.')).



get_index(FS,IndicesIn,IndicesOut,Index) :-
  get_assoc(FS,IndicesIn,Index) 
  -> IndicesOut = IndicesIn
   ; put_assoc(FS,IndicesIn,Index,IndicesOut).

% Tag already instantiated
inst(_,_,FS,_,IndicesIn,IndicesOut,Ind) :-
        get_index(FS,IndicesIn,IndicesOut,Ind),
        nonvar(Ind),!.

% Tag belongs to a handle.
inst(_,_,FS,FSRestr,IndicesIn,IndicesOut,H) :-
        get_index(FS,IndicesIn,IndicesOut,H),
        get_type(FS,FSRestr,handle),
        mygennum(Num),
        sformat(H,'h~w',[Num]).

% Tag belongs to an event.
inst(_,_,FS,FSRestr,IndicesIn,IndicesOut,E) :-
        get_index(FS,IndicesIn,IndicesOut,E),
        event_type_(EventT),
        get_type(FS,FSRestr,Type),
        sub_type(EventT,Type),
        mygennum(Num),
        sformat(E,'e~w',[Num]).

% An index that does not appear as argument of a quantifier.
inst(Rels,RelsRestr,FS,FSRestr,IndicesIn,IndicesOut,I) :-
        get_index(FS,IndicesIn,IndicesMid,I),
        index_type_(IndexT),
        get_type(FS,FSRestr,Type),
        sub_type(IndexT,Type),

        check_rels_for_quant(Rels,RelsRestr,IndicesMid,IndicesOut,I),
        
        !, 
        mygennum(Num),
        sformat(I,'i~w',[Num]).

% index
inst(_,_,FS,FSRestr,IndicesIn,IndicesOut,I) :-
        get_index(FS,IndicesIn,IndicesOut,I),
        index_type_(IndexT),
        get_type(FS,FSRestr,Type),
        sub_type(IndexT,Type),!,
        mygennum(Num),
        sformat(I,'x~w',[Num]).

% event or index
inst(_,_,FS,FSRestr,IndicesIn,IndicesOut,I) :-
        get_index(FS,IndicesIn,IndicesOut,I),
        event_or_index_type_(EIndexT),
        get_type(FS,FSRestr,Type),
        sub_type(EIndexT,Type),
        mygennum(Num),
        sformat(I,'u~w',[Num]).


% Index is not allowed to be bound by a quantifier.
check_rels_for_quant(RelsFS,RelsRestr,IndicesIn,IndicesOut,Index) :- 
  get_type(RelsFS,RelsRestr,LType),
  check_rels_for_quant_act(LType,RelsFS,RelsRestr,IndicesIn,IndicesOut,Index).

check_rels_for_quant_act(LType,_RelsFS,_RelsRestr,IndicesIn,IndicesOut,_Index) :- 
  sub_type(e_list,LType),
  !, IndicesOut = IndicesIn.
check_rels_for_quant_act(LType,Rels,RelsRestr,IndicesIn,IndicesOut,Index) :-
  sub_type(ne_list,LType),
  !,featvald(hd,Rels,RelsRestr,Rel,RelRestr),
    get_type(Rel,RelRestr,RType),
    quant_type_(QuantT),
        % If we have a quantifier, its ARG0 should be different from the Index.
    ( sub_type(QuantT,RType) ->
      featvald(arg0,Rel,RelRestr,Arg0,_),
      get_index(Arg0,IndicesIn,IndicesMid,Arg0Index),
%     write(Index), write(Arg0Index),nl,nl,
      Arg0Index \== Index   % GBP: why after printing?
    ; IndicesMid = IndicesIn
    ),
    featvald(tl,Rels,RelsRestr,Rels,RelsRestr),
    check_rels_for_quant(Rels,RelsRestr,IndicesMid,IndicesOut,Index).

check_rels_for_quant_act(LType,RelsFS,_,_,_,_) :-
  raise_exception(ale(liszt_uninst(RelsFS,LType))).


% instantiated handle, event, index
write_o(FS,_,_Rels,_RVType,IndicesIn,IndicesOut,_Format,Chars,CharsRest) :-
        get_index(FS,IndicesIn,IndicesOut,Ind),
        atom(Ind),
        !,format_to_chars('~w',[Ind],Chars,CharsRest).

% an atomic value, for instance (a_ karl) as the value of the feature named
write_o(FS,_,_Rels,_RVType,Indices,Indices,Format,Chars,CharsRest) :-
        nonvar(FS), FS = (a_ Val),
        !,(Format = utool ->
            format_to_chars('''~w''',[Val],Chars,CharsRest)
          ; format_to_chars('~w',[Val],Chars,CharsRest)
          ).

write_o(Handle,HVType,_Rels,_RVType,IndicesIn,IndicesOut,_Format,Chars,CharsRest) :-
        get_type(Handle,handle),
        !,inst(_,_,Handle,HVType,IndicesIn,IndicesOut,H), 
                                       % GBP - I think H is what you meant
        format_to_chars('~w',[H],Chars,CharsRest).

% special case for utool output, we just instantiate the varibale/handle and print it.
% no features are needed.
write_o(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,utool,Chars,CharsRest) :-
        inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
        format_to_chars('~w',[Result],Chars,CharsRest),!.


                                
write_o(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
        get_type(IFS,IVType,Type),
        event_type_(EventT),
        sub_type(EventT,Type),!,
        (Format = screen -> ( % Stefan's grammar
                                %feat_fs2featval(temo,IRef-ISVs,_-Temo),
                                featvald(sort,IFS,IVType,SortFS,SortVType),
                                get_type(SortFS,SortVType,Sort),
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w:~w',[Result,Sort],Chars,CharsRest)
                            ;   % Merge
                                featvald(e,IFS,IVType,E,EVType),
                                featvald(aspect,E,EVType,AspectFS,AVType),
                                get_type(AspectFS,AVType,Aspect),
                                featvald(mood,E,EVType,MoodFS,MVType),
                                get_type(MoodFS,MVType,Mood),
                                featvald(tense,E,EVType,TenseFS,TVType),
                                get_type(TenseFS,TVType,Tense),
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w:~w:~w:~w',[Result,Aspect,Mood,Tense],Chars,CharsRest)
                            ;   % all the rest
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w',[Result],Chars,CharsRest)
                            )
        ; write_var(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Chars,CharsRest)
        ).

% grammars that have a SORT feature
%
% The following should be generalized. To Do.
write_o(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
        get_type(IFS,IVType,Type),
        ref_ind_type_(IndexT),
        sub_type(IndexT,Type),!,
        
        (Format = screen -> (
                                % Stefan's grammars with genus
                              featvald(per,IFS,IVType,PerFS,PerVType),
                              get_type(PerFS,PerVType,Per),
                              featvald(num,IFS,IVType,NumFS,NumVType),
                              get_type(NumFS,NumVType,Num),
                              featvald(gen,IFS,IVType,GenFS,GenVType),
                              get_type(GenFS,GenVType,Gen),
                              featvald(sort,IFS,IVType,SortFS,SVType),
                              get_type(SortFS,SVType,Sort),
                              inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                              format_to_chars('~w:~w:~w:~w:~w',[Result,Per,Num,Gen,Sort],Chars,CharsRest)
                            ;
                                % Stefan's grammars without genus
                              featvald(per,IFS,IVType,PerFS,PerVType),
                              get_type(PerFS,PerVType,Per),
                              featvald(num,IFS,IVType,NumFS,NumVType),
                              get_type(NumFS,NumVType,Num),
                              featvald(sort,IFS,IVType,SortFS,SVType),
                              get_type(SortFS,SVType,Sort),
                              inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                              format_to_chars('~w:~w:~w:~w',[Result,Per,Num,Sort],Chars,CharsRest)
                            
                            ;   % Merge
                              featvald(png,IFS,IVType,PNG,PNGVType),
                              featvald(pn,PNG,PNGVType,PNFS,PNVType),
                              get_type(PNFS,PNVType,PN),
                              featvald(gen,PNG,PNGVType,GenFS,GenVType),
                              get_type(GenFS,GenVType,Gen),
                              inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                              format_to_chars('~w:~w:~w',[Result,PN,Gen],Chars,CharsRest)
                            ;   % all the rest
                              inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                              format_to_chars('~w',[Result],Chars,CharsRest)
                            )            
        ; write_var(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Chars,CharsRest)
        ).

write_o(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
        get_type(IFS,IVType,Type),
        index_type_(IndexT),
        sub_type(IndexT,Type),!,
        (Format = screen -> ( % Stefan's grammar with genus
                                featvald(per,IFS,IVType,PerFS,PVType),
                                get_type(PerFS,PVType,Per),
                                featvald(num,IFS,IVType,NumFS,NVType),
                                get_type(NumFS,NVType,Num),
                                featvald(gen,IFS,IVType,GenFS,GVType),
                                get_type(GenFS,GVType,Gen),
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w:~w:~w:~w',[Result,Per,Num,Gen],Chars,CharsRest)
                            ;
                                % Stefan's grammar without genus
                                featvald(per,IFS,IVType,PerFS,PVType),
                                get_type(PerFS,PVType,Per),
                                featvald(num,IFS,IVType,NumFS,NVType),
                                get_type(NumFS,NVType,Num),
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w:~w:~w',[Result,Per,Num],Chars,CharsRest)
                            
                            ;   % Merge
                                featvald(png,IFS,IVType,PNG,PNGVType),
                                featvald(pn,PNG,PNGVType,PNFS,PNVType),
                                get_type(PNFS,PNVType,PN),
                                featvald(gen,PNG,PNGVType,GenFS,GenVType),
                                get_type(GenFS,GenVType,Gen),
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w:~w:~w',[Result,PN,Gen],Chars,CharsRest)
                            ;   % all the rest
                                inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result),
                                format_to_chars('~w',[Result],Chars,CharsRest)
                            )
        ; write_var(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Chars,CharsRest)
        ).

write_o(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,_Format,Chars,CharsRest) :-
        get_type(IFS,IVType,Type),
        event_or_index_type_(EIndexT),
        sub_type(EIndexT,Type),!,
        inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,IRef),
        format_to_chars('~w',[IRef],Chars,CharsRest).

write_o(IFS,IVType,_Rels,_RVType,Indices,Indices,_Format,Chars,CharsRest) :-
        get_type(IFS,IVType,Type),
        sub_type(vtype,Type),
        !,format_to_chars('~w',[Type],Chars,CharsRest).

write_o(_FS,_,_Rels,_,Indices,Indices,_Format,Chars,Chars).

/*
write_o(FS,Chars,Chars) :-
        get_type(FS,Type),
	format_to_chars('MRS: found ~w expected event/index.',[Type],Codes),
	atom_codes(Atom,Codes),
        format(user_error,'~N* ERROR: ',[]),
	format(user_error,'~w~n',[Atom]),
        assert(error(mrs,Atom)).
*/
%,
%        fail.

write_rels(Rels,RVType,IndicesIn,IndicesOut,Format,CharsIn,CharsOut) :-
        write_rels(Rels,RVType,Rels,RVType,IndicesIn,IndicesOut,Format,CharsIn,CharsOut).

% write_rels(Rels,LVType,Rels,RVType,IndicesIn,IndicesOut,Format,CharsIn,CharsOut) :-
% GP: this means that the first and third args are always the same - not just your initial conditions
write_rels(LRels,LVType,RRels,RVType,IndicesIn,IndicesOut,Format,CharsIn,CharsOut) :-
  get_type(LRels,LVType,LType),
  write_rels_act(LType,LRels,LVType,RRels,RVType,IndicesIn,IndicesOut,Format,CharsIn,CharsOut).

write_rels_act(T,_,_,_,_,IndicesIn,IndicesOut,_Format,Chars,CharsRest) :- 
  sub_type(e_list,T),
  !, IndicesOut = IndicesIn, Chars = CharsRest.

write_rels_act(T,LRels,LVType,RRels,RsVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
  sub_type(ne_list,T),
  !,featvald(hd,LRels,LVType,Rel,RVType),
    write_rel(Rel,RVType,RRels,RsVType,IndicesIn,IndicesMid,Format,Chars,C1),
    featvald(tl,LRels,LVType,Rest,RestVType),
    get_type(Rest,RestType),
    ((sub_type(ne_list,RestType), Format = utool) ->
        format_to_chars(',',[],C1,C2)
    ; C2 = C1
    ),
    write_rels(Rest,RestVType,RRels,RsVType,IndicesMid,IndicesOut,Format,C2,CharsRest).

write_rels_act(LType,Rels,_,_,_,_,_,_,_,_) :-
  raise_exception(ale(liszt_uninst(Rels,LType))).

/*
If the list is a diff-list it is always open
        format(user_error,"~N* ERROR: MRS: LISZT open.~n",[]),
        assert(error(mrs,'MRS: LISZT open.')).
*/


write_rel(R,RVType,Rels,RsVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
        get_type(R,RVType,RType),
        approps(RType,FRs,_),
        build_keyed_feats(FRs,R,KeyedFeats),
        label_feat_(LabelFeat),
        select(fval(LabelFeat,H,HRestr),KeyedFeats,KeyedFeats0),
        order_features(KeyedFeats0,KeyedFeats1),

        inst(Rels,RsVType,H,HRestr,IndicesIn,IndicesMid,HRef),

        pred_feat_(PredFeat),
        ( select(fval(PredFeat,(a_ APred),_),KeyedFeats1,KeyedFeats2) -> true
        ; KeyedFeats2 = KeyedFeats1
        ),
        ( var(APred) -> Pred = RType
        ; Pred = APred
        ),

        (Format = screen ->
            format_to_chars('~n ~w:~w(',[HRef,Pred],Chars,C1),
            write_args(KeyedFeats2,Rels,RsVType,IndicesMid,IndicesOut,Format,C1,C2),
            format_to_chars(')',[],C2,CharsRest)
        ; Format = utool ->
            format_to_chars('~n rel(''~w'',~w,~n     [ ',[Pred,HRef],Chars,C1),
            write_args(KeyedFeats2,Rels,RsVType,IndicesMid,IndicesOut,Format,C1,C2),
            format_to_chars('])',[],C2,CharsRest)
            
        ;   format_to_chars('[ ~w LBL: ~w ',[Pred,HRef],Chars,C1),
            write_args(KeyedFeats2,Rels,RsVType,IndicesMid,IndicesOut,Format,C1,C2),
            format_to_chars('] ',[],C2,CharsRest)
        ).


write_args([],_Rels,_RsVType,Indices,Indices,_Format,Chars,Chars).
write_args([fval(F,V,VType)|T],Rels,RsVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
        (Format = screen -> 
            format_to_chars('~w:',[F],Chars,C1)
        ; Format = utool ->
            % utool needs exactly ARG0, RSTR and BODY,
            % so we have to do an uppercase conversion here.
            name(F,FChars),
            upper(FChars,UpperFChars),
            name(UF,UpperFChars),
            format_to_chars('attrval(''~w'',',[UF],Chars,C1)
        ;   format_to_chars('~w: ',[F],Chars,C1)
        ),
        write_o(V,VType,Rels,RsVType,IndicesIn,IndicesMid,Format,C1,C2),
        (Format = utool ->
            format_to_chars(')',[],C2,C3)
        ; C3=C2
        ),
        (T = [_|_] ->
            (Format = screen ->
                format_to_chars(',',[],C3,C4)
            ; Format = utool ->
                format_to_chars(',~n       ',[],C3,C4)
            ;   format_to_chars(' ',[],C3,C4)
            )
        ; C4 = C3
        ),
        write_args(T,Rels,RsVType,IndicesMid,IndicesOut,Format,C4,CharsRest).




write_hcons(HCons,HCVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
  get_type(HCons,HCVType,HCType),
  write_hcons_act(HCType,HCons,HCVType,IndicesIn,IndicesOut,Format,Chars,CharsRest).

write_hcons_act(HCType,_HCons,_HCVType,IndicesIn,IndicesOut,_Format,Chars,CharsRest) :-
  sub_type(e_list,HCType),
  !, IndicesOut = IndicesIn, Chars = CharsRest.

write_hcons_act(HCType,HC,HCVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
  sub_type(ne_list,HCType),
  !,featvald(hd,HC,HCVType,Rel,RelVType),
    write_hc(Rel,RelVType,IndicesIn,IndicesMid,Format,Chars,C1),
    featvald(tl,HC,HCVType,HCs,HCsVType),
    get_type(HCs,HCsType),
    ((sub_type(ne_list,HCsType), Format = utool) ->
        format_to_chars(',',[],C1,C2)
    ; C2 = C1
    ),
    write_hcons(HCs,HCsVType,IndicesMid,IndicesOut,Format,C2,CharsRest).

write_hcons_act(_HCType,_HCons,_,_,_,_,Chars,Chars) :-
  format(user_error,'~N* ERROR: MRS: H-CONS open.~n',[]),
  assert(error(mrs,'MRS: H-CONS open.')).


/*
If the list is a diff-list it is always open
        format(user_error,"~N* ERROR: MRS: H-CONS open.~n",[]),
        assert(error(mrs,'MRS: H-CONS open.')).
*/

write_hc(Rel,RVType,IndicesIn,IndicesOut,Format,Chars,CharsRest) :-
        outscoped_feat_(OSFeat),
        featvald(OSFeat,Rel,RVType,OutFS,OutVType),
        inst(_,_,OutFS,OutVType,IndicesIn,IndicesMid,Out), 
            % GBP: these first two uninstantiated args look very dangerous
        sc_arg_feat_(SC_ARG),
        featvald(SC_ARG,Rel,RVType,ScFS,ScVType),
        inst(_,_,ScFS,ScVType,IndicesMid,IndicesOut,Sc),
            % GBP: these first two uninstantiated args look very dangerous
        get_type(Rel,RVType,Type),
        (Format = screen -> ( Type = qeq ->
                                format_to_chars('~n ~w ~5+=q ~w',[Sc,Out],Chars,CharsRest)
                            ; Type = geq ->
                                format_to_chars('~n ~w >= ~w',[Sc,Out],Chars,CharsRest)
                            ; Type = gt ->
                                format_to_chars('~n ~w >  ~w',[Sc,Out],Chars,CharsRest)
                            ; Type = eq ->
                                format_to_chars('~n ~w =  ~w',[Sc,Out],Chars,CharsRest)
                            )
        ; Format = utool ->
            ( (Type = qeq; Type = geq) ->
                format_to_chars('~n ~w(~w,~w)',[Type,Sc,Out],Chars,CharsRest)
            ; format(user_error,'* ERROR: Handle constraints of type ~w are not supported by Utool',[Type]),
                fail
            )
        ; format_to_chars('~w ~w ~w ',[Sc,Type,Out],Chars,CharsRest)
        ).




% VAR -> VARNAME | VARNAME [ VARTYPE EXTRAPAIR* ]

/*
write_var(Ind-_,Chars,CharsRest) :-
        atom(Ind),
        format_to_chars('~w',[Ind],Chars,CharsRest).

% an atomic value, for instance (a_ karl) as the value of the feature named
write_var(_-(a_ Val),Chars,CharsRest) :-
        format_to_chars('~w',[Val],Chars,CharsRest).

write_var(H-SVs,Chars,CharsRest) :-
        get_type(H-SVs,handle),
        inst(_,H-handle,_),
        format_to_chars('~w',[H],Chars,CharsRest).

*/

write_var(_IFS,_IVType,_Rels,_RVType,Indices,Indices,Chars,Chars) :- !. % debugging

write_var(IFS,IVType,Rels,RVType,IndicesIn,IndicesOut,Chars,CharsRest) :-
        get_type(IFS,IVType,Type),

        approps(Type,FRs,_),
        build_keyed_feats(FRs,IFS,KeyedFeats),
        order_features(KeyedFeats,KeyedFeats1),

        mrs_map_type(Type,MRSType), % specific LKB conventions for RMRS
        
        ( % variable: x1, e2 or something that like
            inst(Rels,RVType,IFS,IVType,IndicesIn,IndicesOut,Result) ->
            %inst(_,IRef-ISVs,Result) ->
            format_to_chars('~w [ ~w ',[Result,MRSType],Chars,C1),
%            trace,
            write_pairs(KeyedFeats1,Rels,RVType,IndicesIn,IndicesOut,C1,C2),
            format_to_chars(']',[],C2,CharsRest)
        ; % constant
            format_to_chars('~w',[Type],Chars,CharsRest)
        ).



write_var(_IFS,_IVType,_Rels,_RVType,Indices,Indices,Chars,Chars).



write_pairs([],_Rels,_RVType,Indices,Indices,Chars,Chars).
write_pairs([fval(F,VFS,_)|T],Rels,RVType,IndicesIn,IndicesOut,Chars,CharsRest) :-
        format_to_chars('~w: ',[F],Chars,C1),
        fs2type_feats(VFS,VType,VVs),
        write_var(VVs,VType,Rels,RVType,IndicesIn,IndicesOut,C1,C2),
        format_to_chars(' ',[],C2,C3),
        write_pairs(T,C3,CharsRest).

mrs_map_type(Type,e) :-
        event_type_(EventT),
        sub_type(EventT,Type),!.


mrs_map_type(Type,x) :-
        index_type_(IndexT),
        sub_type(IndexT,Type),!.

mrs_map_type(Type,i) :-
        event_or_index_type_(EIndexT),
        sub_type(EIndexT,Type),!.

mrs_map_type(_Type,u).





mrs_utool(Command,FS,ErrorMessage) :-
        mrs_utool(Command,FS,_Result,ErrorMessage).

mrs_utool(Command,FS,Result,ErrorMessage) :-

    print_mrs(FS,utool,MRS,[]),
    ( get_phon(FS,Phon) ->
        utool_request(Command,Phon,'mrs-prolog',MRS,'term-prolog',Result,ErrorMessage)
    ;   utool_request(Command,'',  'mrs-prolog',MRS,'term-prolog',Result,ErrorMessage)
    ).




get_phon(FS,Phon) :- 
  phon_feat_(PhonFeat),
  feat_fs2featval(PhonFeat,FS,PhonFS),
  get_phon_atom_list(PhonFS,PhonList),
  wordlist2string(PhonList,String),
  name(Phon,String).

get_phon_atom_list(PhonFS,[]) :- get_type(PhonFS,e_list).

get_phon_atom_list(PhonFS,[First|PhonT]) :-
  feat_fs2featval(hd,PhonFS,(a_ First)),
  feat_fs2featval(tl,PhonFS,PhonFST),
  get_phon_atom_list(PhonFST,PhonT).
