% SP4 no longer has ugraph colouring/3
% Here is the definition from SP3

colouring(Graph, K, Colouring) :-
        (   integer(K), K >= 0 -> true
        ;   prolog:illarg(domain(integer,>=(0)),
                          colouring(Graph,K,Colouring), 2)
        ),
        colour_map(Graph, Colouring),
        colour_map(Graph, Graph1, Colouring, Colouring),
        colouring(Graph1, K, 0, [], Stack),
        colour_stack(Stack).

colouring([], _, _, Stk0, Stk) :- !, Stk0 = Stk.
colouring(Graph, K, InUse, Stk0, Stk) :-
        select_vertex(Graph, K, Compare, -, 0+0, V-Ns),
        graph_del_vertices(Graph, [V], [V], Graph1),
        (   Compare = < ->
            colouring(Graph1, K, InUse, [V-Ns|Stk0], Stk)
        ;   M is min(K,InUse+1),
            vertex_colour(Ns, 1, M, V),
            add_colour(Graph1, Ns, V, Graph2),
            InUse1 is max(V,InUse),
            colouring(Graph2, K, InUse1, Stk0, Stk)
	).

select_vertex([], _, >=, Pair, _, Pair).
select_vertex([V-Neibs|Graph], K, Comp, Pair0, Rank0, Pair) :-
	evaluate_vertex(Neibs, 0, Rank),
	(   Rank < K -> Comp = <, Pair = V-Neibs
	;   Rank @> Rank0 ->
	    select_vertex(Graph, K, Comp, V-Neibs, Rank, Pair)
	;   select_vertex(Graph, K, Comp, Pair0, Rank0, Pair)
	).

evaluate_vertex([V|Neibs], Deg, Rank) :- var(V), !,
	Deg1 is Deg+1,
	evaluate_vertex(Neibs, Deg1, Rank).
evaluate_vertex(Neibs, Deg, Sat+Deg) :-
	prolog:length(Neibs, 0, Sat).

add_colour([], _, _, []).
add_colour([V-Neibs|Graph], [W|Vs], C, [V-Neibs1|Graph1]) :- V==W, !,
	ord_add_element(Neibs, C, Neibs1),
	add_colour(Graph, Vs, C, Graph1).
add_colour([Pair|Graph], Vs, C, [Pair|Graph1]) :-
	add_colour(Graph, Vs, C, Graph1).

vertex_colour([V|Vs], I, M, Colour) :- V@<I, !,
	vertex_colour(Vs, I, M, Colour).
vertex_colour([I|Vs], I, M, Colour) :- !,
	I<M, J is I+1, vertex_colour(Vs, J, M, Colour).
vertex_colour(_, I, _, I).
vertex_colour(Vs, I, M, Colour) :-
	I<M, J is I+1, vertex_colour(Vs, J, M, Colour).

colour_stack([]).
colour_stack([V-Neibs|Stk]) :- sort(Neibs, Set), colour_stack(Set, 1, V, Stk).

colour_stack([I|Is], I, V, Stk) :- !, J is I+1, colour_stack(Is, J, V, Stk).
colour_stack(_, V, V, Stk) :- colour_stack(Stk).

colour_map([], []).
colour_map([V-_|Graph], [V-_|Colouring]) :- colour_map(Graph, Colouring).

colour_map([], [], [], _).
colour_map([V-Ns|Graph], [C-Ns1|Graph1], [V-C|Cols], Colouring) :-
	map_colours(Ns, Colouring, Ns1),
	colour_map(Graph, Graph1, Cols, Colouring).

map_colours([], _, []).
map_colours(Ns, Colouring, Ns1) :-
	Ns = [X|_],
	Colouring = [V-_|_],
	compare(C, X, V),
	map_colours(C, Ns, Colouring, Ns1).

map_colours(=, [_|Xs], [_-Y|Colouring], [Y|Ns1]) :-
	map_colours(Xs, Colouring, Ns1).
map_colours(>, Ns, [_|Colouring], Ns1) :-
	map_colours(Ns, Colouring, Ns1).

graph_del_vertices(G1, [], Set, G) :- !,
	graph_del_vertices(G1, Set, G).
graph_del_vertices([], _, _, []).
graph_del_vertices([V1-N1|G1], [V2|Vs], Set, G) :-
	compare(C, V1, V2),
	graph_del_vertices(C, V1, N1, G1, V2, Vs, Set, G).

graph_del_vertices(<, V1, N1, G1, V2, Vs, Set, [V1-N|G]) :-
	ord_subtract(N1, Set, N),
	graph_del_vertices(G1, [V2|Vs], Set, G).
graph_del_vertices(=, _, _, G1, _, Vs, Set, G) :-
	graph_del_vertices(G1, Vs, Set, G).
graph_del_vertices(>, V1, N1, G1, _, Vs, Set, G) :-
	graph_del_vertices([V1-N1|G1], Vs, Set, G).

graph_del_vertices([], _, []).
graph_del_vertices([V1-N1|G1], Set, [V1-N|G]) :-
	ord_subtract(N1, Set, N),
	graph_del_vertices(G1, Set, G).
