; Purpose: Emacs lisp mode for starting trale under emacs. 
;
; Author: Detmar Meurers
;
; Version: $Id: trale-emacs-interface.el,v 1.11 2005/03/31 17:34:24 stefan Exp $
;
; Historical note: We now add our stuff on top of the ordinary
; prolog.el mode that comes with sicstus. This improves on our
; original gtrale.el mode (which directly used comint) in that it
; allows use of the source level debugger and other facilities
; provided by the prolog.el mode of sicstus. Different from gtrale.el,
; this new code should also be compatible with both xemacs and gnu
; emacs.

; Load the prolog mode
(load "prolog")


;; Per Mildner, 31.08.2012

;; This problem has been fixed in the prolog.el that comes with SICStus
;; 4. It is caused by an incompatible change in Emacs.

;; If you can not use the latest SICStus emacs code, a workaround is to
;; put the following code in your .emacs file, right after loading the
;; sicstus_emacs_init file:

;;; BEGIN CODE
(put 'prolog-build-case-strings 'sicstus
     '(lambda ()
        (setq prolog-upper-case-string
              (concat '(65 66 67 68 69 70 71 72 73 74
                           75 76 77 78 79 80 81 82 83
                           84 85 86 87 88 89 90 192 193
                           194 195 196 197 198 199 200
                           201 202 203 204 205 206 207
                           208 209 210 211 212 213 214
                           216 217 218 219 220 221 222))

              prolog-lower-case-string
              (concat '(97 98 99 100 101 102 103 104 105
                           106 107 108 109 110 111 112
                           113 114 115 116 117 118 119
                           120 121 122 224 225 226 227
                           228 229 230 231 232 233 234
                           235 236 237 238 239 240 241
                           242 243 244 245 246 248
                           249 250 251 252 253 254)))))
;;; END CODE



(load "trale")


; (setq prolog-system 'sicstus)



; Define the start-trale function, which is called in the shell script
; "trale" that's used for starting trale

(defun start-trale (program-name options startup-filename)
; wait for emacs to get to its feet:
  (sit-for 1)
; set the variables from the arguments and start prolog
  (setq prolog-program-name program-name)
  (setq sicstus-program-switches (split-string options))
  (run-prolog nil)
; wait until we see a prolog prompt
;  (wait-until-prompt)
; make the prolog window take up the entire frame:
  (delete-other-windows)

  (setq mode-name "trale")
  (setq major-mode 'trale-interactive-mode)
  
; load startup files
  (unless (equal startup-filename "")
    (let* ((startup-load-command (concat "compile('" 
					 startup-filename
					 "').\n")))
      (prolog-process-insert-string (get-process "prolog") startup-load-command)
      (process-send-string "prolog" startup-load-command))
    
    (setq trale-grammar-name (filename-only startup-filename))
    (setq trale-grammar-directory (dirname-only startup-filename))
    
    )
 
  ;(setq local-abbrev-table prolog-mode-abbrev-table)
  (setq local-abbrev-table text-mode-abbrev-table)
  (use-local-map trale-mode-map)
  (prolog-inferior-menu)

  (setq trale-standalone-flag nil)

 
  (run-hooks 'trale-mode-hook))


(defun start-standalone-trale (startup-filename)
; wait for emacs to get to its feet:
  (sit-for 1)
  (setq prolog-system 'sicstus)
  (setq prolog-program-name "sicstus")
 ; make the prolog window take up the entire frame:
  (delete-other-windows) 

  (switch-to-buffer (make-comint "prolog" startup-filename))
  
  (setq mode-name "trale")
  (setq major-mode 'trale-interactive-mode)
  ;(setq local-abbrev-table prolog-mode-abbrev-table)
  (setq local-abbrev-table text-mode-abbrev-table)
  (use-local-map trale-mode-map)
  (prolog-inferior-menu)
;  (set-syntax-table text-mode-syntax-table)
;  (set-syntax-table prolog-mode-syntax-table)

  ; needed for fontification
  (trale-prolog-mode-variables)
  (prolog-build-case-strings)
  (prolog-set-atom-regexps)

  (setq trale-standalone-flag t)

  (run-hooks 'trale-mode-hook))


;; taken from definition of prolog-ensure-process in prolog.el
(defun wait-until-prompt ()
  (goto-char (point-max))
  (while
      (save-excursion
	(not
	 (re-search-backward
	  (concat "\\(" prolog-prompt-regexp-i "\\)" "\\=")
	  nil t)))
    (sit-for 0.1)))

; Specifications for the source level debugger (from Gerald):

(setq prolog-use-sicstus-sd t)

(setq pltrace-port-arrow-assoc '( 
	("call" . ">>>") ("exit" . "+++") ("ndexit" . "?++")
	("redo" . "<<<") ("fail" . "---") ("exception" . "==>")
        ("null" . nil)))

(setq auto-mode-alist
      (cons '("\\.pl$" . prolog-mode) auto-mode-alist))


;; for the chart display prompt
;; ^C^N -> next prompt ^C^P -> previous prompt
;; typing enter after the last input sentence parses this sentence

(setq sicstus-prompt-regexp ">>> *\\|| [ ?][- ] *")


;;                 "\\(^! \\(.*error.*$\\)?\\)\\|\\(!!! [0-9]* expected !!!\\)"


;; Do we need Prolog debugging?
;; If this is enabled, we get inferences with emacs after loading
;; a grammar, if we want to go to interactive mode immediately.
;; St. M. 13.02.2005
; GBP - NICE ... please don't disable the debugger again.
; (setq prolog-use-sicstus-sd nil)