;;   trale.el: a user interface to TRALE written in Emacs Lisp
;; It was implemented by Stefan Mller and heavily borrows from
;; ale.el by
;;   Copyright (C) 1995 Olivier Laurens  <laurens@cs.sfu.ca>
;;     
;;   Author: Olivier Laurens, 
;;           Natural Language Lab, 
;;           School of Computing Science 
;;           Simon Fraser University, Burnaby, BC, Canada V5A 1S6
;;   Version: 1.0
;;   Date: June 1 1995
;;
;;This program is free software; you can redistribute it and/or
;;modify it under the terms of the GNU General Public License
;;as published by the Free Software Foundation; either version 2
;;of the License, or (at your option) any later version.
;;
;;This program is distributed in the hope that it will be useful,
;;but WITHOUT ANY WARRANTY; without even the implied warranty of
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;GNU General Public License for more details.
;;
;;You should have received a copy of the GNU General Public License
;;along with this program; if not, write to the Free Software
;;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;
;;
;;If you use this software and find bugs, or if you modify it, please send a
;;description of the bug or a copy of the code modfied by you to laurens@cs.sfu.ca
;;There is no support or technical assistance with this software. Upgraded versions
;;may appear in the future but without any guarantee.
;;
;;
;;
;;
;;  To switch to the relevant mode automatically when you visit your grammar, put
;;  the following on the first line of the file:
;;               % -*-trale-prolog-*-
;;
;;
;;


(require 'comint)
(require 'easymenu)

(defvar trale-standalone-flag nil "A flag that is used in trale-itsdb to determine how to set up the CPUs for itsdb")

;;-------------------------------------------------------------------
;; Menu stuff (both for the editing buffer and for the inferior
;; prolog buffer)
;;-------------------------------------------------------------------

(defvar trale-itsdb-started nil "A flag that indicates whether [incr TSDB()] was sucessfully loaded.")
(defvar trale-draw-glbs     nil "A flag that indicates whether glb types in signatures and hierarchies are drawn.")
(defvar trale-draw-intros   nil "A flag that indicates whether intro types in hierarchies are drawn.")

;; you may configure the value in your .emacs
(defvar trale-number-of-processes 1 "The number of TRALE processes that [incr TSDB()] starts.")

;; These host may be any hosts somewhere on earth. 

;; If you want to use a single CPU machine locally, use this (default):
;(defvar trale-pvm-hosts   '(localhost) "A list of host names that [incr tsdb()] tries to invoce via PVM.")

;; Use this for a double processor machine.
(defvar trale-pvm-hosts   '(localhost localhost) "A list of host names that [incr tsdb()] tries to invoce via PVM.")


;; Menu for the prolog editing buffers and the trale buffer
(defvar trale-prolog-menu
  '(list
   "Trale"
   '("Compile"
     ["Grammar" trale-compile-gram  :active t]    ; everything
     "-"
     ["Signature" trale-compile-sig :active t])
   "-"
   '("Show"
     ["Lexical Rule" trale-lexrule :active t]
     ["Macro" trale-macro :active t]
     ["Rule" trale-rule :active t]
     ["Submacros" trale-submacros :active t]
     ["Type" trale-type :active t]
     ["Word" trale-word :active t])
   '("Draw"
     ["Hierarchy"  trale-hierarchy :active t]
     ["Signature"  trale-signature :active t]
     "-"
     ["Show GLBs"        trale-toggle-glbs   :style toggle :selected trale-draw-glbs]
     ["Show Intro Types" trale-toggle-intros :style toggle :selected trale-draw-intros]
     )
   "-"
   ["Test items"    trale-test-item-region :active t]
   '("[incr tsdb()]"
     ["Start [incr tsdb()]" trale-itsdb t]
     ["Reload Grammar for [incr tsdb()]" trale-load-grammar-for-itsdb :active trale-itsdb-started]
     "-"
     ["Set hosts to be invoked." trale-set-pvm-hosts t]
)
;   "-"
;   '("Options"
;     ["Show Chart Edges"        trale-toggle-chartdisplay :style toggle :selected trale-use-chartdisplay]
;     ["Show Feature Structures" trale-toggle-fs-otput     :style toggle :selected trale-fs-output]
;     )

   )
  "The definition for the menu in the editing buffers."
  )


(defun trale-prolog-menu ()
  "Add the menu in the editing buffer of Trale files."
  (let ((menu (prolog-make-menu (eval trale-prolog-menu))))
    (cond
     ((eq prolog-emacs 'gnuemacs)
      (easy-menu-define prolog-menu-map (current-local-map) "" menu))
     ((eq prolog-emacs 'xemacs)
      (easy-menu-add menu))
     ;; The default
     (t
      (easy-menu-define prolog-menu-map (current-local-map) "" menu))
     )))

;; the used menu for trale stuff is identical to trale-prolog-menu, 
;; but could be different
(defun trale-menu ()
  "Add the menu in the Trale buffer."
  (let ((menu (prolog-make-menu (eval trale-prolog-menu))))
    (cond
     ((eq prolog-emacs 'gnuemacs)
      (easy-menu-define run-prolog-menu-map (current-local-map) "" menu))
     ((eq prolog-emacs 'xemacs)
      (easy-menu-add menu))
     ;; The default
     (t
      (easy-menu-define run-prolog-menu-map (current-local-map) "" menu))
     )))

(add-hook 'trale-prolog-mode-hook 'trale-prolog-menu)
(add-hook 'trale-mode-hook 'trale-menu)

(defun trale-mode-keybindings-common (map)
  "Define keybindings common to both Trale modes in MAP."
  (define-key map "\C-c\C-a" 'trale-compile-gram)               ; Compile Grammar (everything)
  (define-key map "\C-c\C-h" 'trale-hierarchy)                  ; draw hierarchy
  (define-key map "\C-c\C-g" 'trale-compile-grammar)            ; Compile Grammar without signature ^C^G

;  ("Compile Signature ^C^i"  ("\C-ci" "\C-c\C-i") 'trale-compile-sig)
;  ("Compile Lexicon ^C^l"  ("\C-cl" "\C-c\C-l") 'trale-compile-lex)
;  ("Compile Rules ^C^R"    ("\C-cr" "\C-c\C-r") 'trale-prolog-compile-buffer-rules)
;  ("Compile Buffer ^C^K"   ("\C-ck" "\C-c\C-k") 'trale-prolog-compile-buffer)

  (define-key map "\C-c\C-r" 'trale-rule)
  (define-key map "\C-c\C-m" 'trale-macro)
;  (define-key map "\C-c\C-t" 'trale-type)
  (define-key map "\C-c\C-t" 'trale-test-item-region)
  (define-key map "\C-c\C-o" 'trale-signature)
  (define-key map "\C-c\C-i" 'trale-itsdb)
  (define-key map "\C-c\C-l" 'trale-load-grammar-for-itsdb)

)

;;;;;;;;;;;;;;;; Menu Generator from ale.el ;;;;;;;;;;;;;;;;;;;;;;;;
;; still needed for some ale modes: trale-text-mode

(defun make-multi-modal-menu (map desc)
   (if (null desc) t
	   (define-key map (vconcat [menu-bar] (cons (make-symbol (car desc)) nil))
	      (cons (car desc) (make-sparse-keymap (car desc))))
	   (make-mouse-menu map (reverse (cdr desc))
		(vconcat [menu-bar] (cons (make-symbol (car desc)) nil)))
	   (make-dumb-menu map (cdr desc))))

(defun make-mouse-menu (map desc menuhead)
  (if (null desc) t 
      (define-key map 
         (vconcat menuhead (cons (make-symbol (car (car desc))) nil))
         (cons (car (car desc)) (car (cdr (car (cdr (cdr (car desc))))))))
      (make-mouse-menu map (cdr desc) menuhead)))

(defun make-mouse-menu-only (map desc)
   (define-key map (vconcat [menu-bar] (cons (make-symbol (car desc)) nil))
	 (cons (car desc) (make-sparse-keymap (car desc))))
   (make-mouse-menu map (reverse (cdr desc))
		(vconcat [menu-bar] (cons (make-symbol (car desc)) nil))))

(defun make-dumb-menu (map desc)
  (if (null desc) t
      (define-key map "
" 'execute-command-on-current-line)
      (make-dumb-menu1 map desc)))

(defun make-dumb-menu1 (map desc)
  (if (null desc) t
      (define-the-keys map (car (cdr (car desc))) (car (cdr (car (cdr (cdr (car desc)))))))
      (make-dumb-menu1 map (cdr desc))))

(defun define-the-keys (map keys command)
  (if (null keys) nil
      (define-key map (car keys) command)
      (define-the-keys map (cdr keys) command)))
       
(defun old-execute-command-on-current-line ()
   "Searches for a character enclosed in < > on the current line and executes the command associated with this character in the current local keymap"
   (interactive)
   (beginning-of-line)
   (search-forward "<")
   (eval (cons (local-key-binding (char-to-string (following-char))) nil)))

 
(defun execute-command-on-current-line ()
   "executes the command on the current line"
   (interactive)
   (let ((line-limit (progn (end-of-line) (point))))
      (beginning-of-line)
      (if (search-forward "<" line-limit t 1)
          (eval (cons (local-key-binding (char-to-string (following-char))) nil))
          (let ((ss (current-line-string)))
             (let ((sss (substring ss (string-match "[^ ].*" ss)
                                   (string-match "$" ss))))
                 (if (get-buffer sss) (switch-to-buffer sss) nil))))))

(defun display-dumb-menu (desc)
  (if (null desc) t
      (insert "                   " (car desc))
      (insert "\n***********************************************\n")
      (display-menu-items (cdr desc))
      (insert "***********************************************\n")))

(defun display-menu-items (desc)
  (if (null desc) t 
      (insert "           " (car (car desc)))
      (insert "\n")
      (display-menu-items (cdr desc))))


(defun show-menu-window (desc)
   (if (null desc) t
       (switch-to-buffer-other-window
          (get-buffer-create (concat (car desc) "-Menu")))
       (shrink-window (- (window-height) (+ 2 (length (cdr desc)))))
       (display-menu-items (cdr desc))
       (goto-char 1)))

(defun show-commands-summary (opts) 
   (goto-char 1) 
   (insert-options (cdr opts) 0))

(defun insert-options (opts optlen)
  (if (null opts) (insert "\n\n")
      (if (>= (+ optlen 2 (length (car (car opts)))) (window-width))
          (progn (insert "\n")
                 (setq optlen 0))
          t)
      (insert (car (car opts)) "  ") 
      (insert-options (cdr opts) (+ optlen 2 (length (car (car opts)))))))



;;;;;;;;;;;;;; Ale General Mode;;;;;;;;;;;;;;;;;;

(defvar trale-mode-map nil "Keymap for Trale mode.")

(if trale-mode-map
  ()
  (if (not (boundp 'prolog-mode-map)) (load-library "prolog") t)
  (if prolog-inferior-mode-map
	     ()
	   (setq prolog-inferior-mode-map (copy-keymap comint-mode-map))
	   (prolog-mode-keybindings-common prolog-inferior-mode-map)
	   (prolog-mode-keybindings-inferior prolog-inferior-mode-map)
           )
  (setq trale-mode-map (copy-keymap prolog-inferior-mode-map)) ;comint-mode-map))
  (trale-mode-keybindings-common trale-mode-map)
;  (make-mouse-menu-only trale-mode-map trale-mode-menu)
;  (make-dumb-menu1 trale-mode-map (cdr trale-mode-menu))
)


(defun trale-mode ()
  "Major mode for running TRALE commands. 
\\{trale-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq buffer-auto-save-file-name nil)
  (setq buffer-read-only nil)
  (erase-buffer)
  (setq mode-name "Trale")
  (setq major-mode 'trale-mode)
  (setq local-abbrev-table text-mode-abbrev-table)
  (use-local-map trale-mode-map)
  (set-syntax-table text-mode-syntax-table)
  ;(display-dumb-menu trale-options)
  (setq buffer-read-only t)
  (run-hooks 'trale-mode-hook))


;;;;;;;;;;;;;;;; Trale Text Mode ;;;;;;;;;;;;;;;;;;;;;;

(defun trale-text-mode ()
  "Major mode for handling text in conjunction with TRALE.
Special commands:\\{trale-text-mode-map}."
  (interactive)
  (kill-all-local-variables)
  (setq buffer-auto-save-file-name nil)
  (setq mode-name "Trale-Text")
  (setq major-mode 'trale-text-mode)
  (setq local-abbrev-table text-mode-abbrev-table)
  (use-local-map trale-text-mode-map)
  (set-syntax-table text-mode-syntax-table)
  (run-hooks 'trale-text-mode-hook))

(defvar trale-text-mode-map nil
  "Keymap for Trale's text mode.")

(setq trale-text-mode-options '(" Trale "
  ("Parse Current Line ^J" ("\C-j")             'trale-parse-current-line)
  ("Parse Region ^Cp"      ("\C-c\C-p" "\C-cp") 'trale-parse-region)
  ("Go Back to Trale ^C^X"   ("\C-c\C-x" "\C-cx") 'go-back-ale)))

(if trale-text-mode-map
  ()
  (setq trale-text-mode-map (copy-keymap text-mode-map))
  (make-mouse-menu-only trale-text-mode-map trale-text-mode-options)
  (make-dumb-menu1 trale-text-mode-map (cdr trale-text-mode-options)))

;;;;;;;;;;;;;;;; Trale Prolog Mode ;;;;;;;;;;;;;;;;;;;;;;

;;;###autoload
(defun trale-prolog-mode ()
  "Major mode for handling TRALE Grammar code.
Special commands:\\{trale-prolog-mode-map}."
  (interactive)
  (kill-all-local-variables)
  (if (not (boundp 'prolog-mode-map)) (load-library "prolog") t)
  (setq buffer-auto-save-file-name nil)
  (setq mode-name "trale-prolog")
  (setq major-mode 'trale-prolog-mode)
  (setq local-abbrev-table prolog-mode-abbrev-table)
  (use-local-map trale-prolog-mode-map)
  (set-syntax-table prolog-mode-syntax-table)
;  (setq trale-grammar-name (filename-only buffer-file-name))
;  (setq trale-grammar-directory (dirname-only buffer-file-name))

  ; needed for fontification
  (trale-prolog-mode-variables)
  (prolog-build-case-strings)
  (prolog-set-atom-regexps)
    
  (run-hooks 'trale-prolog-mode-hook)

  ;; [PM] Turn on font lock if appropriate.
  ;; It is better to do it here than relying on the settings for
  ;; global font locking since:
  ;; . XEmacs will only turn font-lock on from find-file-hooks and
  ;;   thus not for prolog-mode buffer not visiting files
  ;; . For buffers not visiting files FSF Emacs will turn it on from
  ;;   post-command-hook which will cause an annoying delay and
  ;;   flicker for the first character typed after prolog-mode is
  ;;   turned on. This was an issue with SICStus source debugger but
  ;;   is a general problem.
  ;; Do it after prolog-mode-hooks since hooks may turn on font-lock
  (cond ((fboundp 'turn-on-font-lock-if-enabled) ; FSF Emacs
         (turn-on-font-lock-if-enabled))
        ((fboundp 'font-lock-set-defaults) ; XEmacs
         ;; Available in FSF Emacs as well but with different
         ;; semantics
         ;; In XEmacs is is safe to call multiple times and fills a
         ;; similar role as turn-on-font-lock-if-enabled
         (font-lock-set-defaults))))




(defvar trale-prolog-mode-map nil
  "Keymap for Trale prolog mode.")

(if trale-prolog-mode-map
  ()
  (if (not (boundp 'prolog-mode-map)) (load-library "prolog") t)
  (setq trale-prolog-mode-map (copy-keymap prolog-mode-map))
  
  (trale-mode-keybindings-common trale-prolog-mode-map)

;  (make-mouse-menu-only trale-prolog-mode-map trale-prolog-mode-menu)
;  (make-dumb-menu1 trale-prolog-mode-map (cdr trale-prolog-mode-menu)))

)

;;;;;;;;;;;;;;;;;;;;; Run Trale ;;;;;;;;;;;;;;;;;;;;;;

(defvar trale-command "trale" "*The shell command that runs TRALE")
(defvar trale-process "prolog")
(defvar trale-grammar-directory "~/" "*The path where the grammar lives; End the path with /")
(defvar trale-grammar-name "theory" "*The filename of the grammar")
(defvar trale-startup-file nil "*The name of a file to consult into prolog upon startup")

(setq prolog-menu '(" Trale "
     ("Defreeze Prolog ^C^G" ("\C-cg" "\C-c\C-g") 'free-prolog)
     ("Back to Trale ^C^X"     ("\C-cx" "\C-c\C-x") 'go-back-trale)
     ("Quit Trale ^C^Q"        ("\C-cq" "\C-c\C-q") 'trale-quit)))

(defvar do-startup t)

;;;###autoload
(defun run-trale ()
  "Run TRALE as an inferior (prolog) process"
  (interactive)
  (setq default-directory trale-grammar-directory)
  (load-library "prolog")
  (message "loading Trale...")
  (switch-to-buffer (make-comint trale-process trale-command))
  (if (and trale-startup-file do-startup)
      (progn
          (send-command
		     (get-buffer-process (concat "*" trale-process "*"))
  		     (concat "consult('" trale-startup-file "').\n"))
          (setq do-startup nil)))
  (inferior-prolog-mode)
  (goto-char (point-max))
  (send-command
	    (get-buffer-process (concat "*" trale-process "*"))
	    "true.\n")
  (make-mouse-menu-only inferior-prolog-mode-map prolog-menu)
  (make-dumb-menu1 inferior-prolog-mode-map (cdr prolog-menu))
  (goto-char (point-max))
  (set-marker (process-mark (get-buffer-process (buffer-name))) (point))
  (set-marker comint-last-input-start (point))
  (set-marker comint-last-input-end (point))
  (message "loading Trale...done.")
  (switch-to-buffer (get-buffer-create "trale-interaction"))
  (trale-mode))

(defun go-back-trale ()
   "Returns to Trale menu"
   (interactive) 
   (if (get-buffer-process (concat "*" trale-process "*")) 
        t
        (run-trale)))

;;;;;;;;;;;;;;;; Hooks Interface ;;;;;;;;;;;;;;;;;;;;;;

(defun first-n-match (lst mtch)
  (if (null lst) 
   (not nil)
   (if (equal (car lst) (car mtch)) 
    (first-n-match (cdr lst) (cdr mtch)))))

(defun strip-control-m (instri)
  (let ((match (string-match "\C-m" instri)))
    (if (not match) instri
      (concat 
         (substring instri 0 match)
         (strip-control-m (substring instri (+ match 1)))))))

(defun strip-yes (instri)
  (let ((match (string-match "\nyes\n" instri)))
    (if (not match) instri
      (concat 
         (substring instri 0 match)
         (substring instri (+ match 5))))))

(defun keep-output (proc out)
  (setq proc-out (concat proc-out (strip-yes (strip-control-m out))))
  (set-variable 'case-fold-search nil)
  (if (> (length proc-out) 4) 
      (if (equal (substring proc-out -5) "| ?- ")
          (progn (setq finished-processing t)
                 (insert (substring proc-out 0 (- (length proc-out) 5)))
                 (setq proc-out nil))
          t)
      t)
  (if (equal (substring out -1) "\n") 
      (if (not finished-processing)
          (progn (insert proc-out)
                 (setq proc-out nil)) t) t))


(defun send-command-and-get-result (proc command res-buffer)
  (set-buffer (get-buffer-create res-buffer))
  (setq buffer-read-only nil)
  (erase-buffer)
  (switch-to-buffer res-buffer)
  (send-command proc command)
  (goto-char 0))

(defvar trale-frames nil)

(defun send-command-and-get-result-other-frame (proc command res-buffer)
  (set-buffer (get-buffer-create res-buffer))
  (setq buffer-read-only nil)
  (erase-buffer)
  (if window-system 
      (if (get-buffer-window res-buffer t) 
          (raise-frame (window-frame 
                           (get-buffer-window res-buffer t)))
          (switch-to-buffer-other-frame res-buffer)
          (setq trale-frames 
                (cons res-buffer trale-frames)))
      (switch-to-buffer res-buffer))
  (send-command proc command)
  (goto-char 0))

(setq default-filter nil)  ;; set to 'rlogin-filter if running ale on a remote host
                           ;; 'rlogin-filter catters for the spurious ^M

(defun send-command (proc command)
  (setq finished-processing nil)
  (setq proc-out nil)
  (set-process-filter proc nil)
  (set-process-filter proc 'keep-output)
  (process-send-string proc command)
  (while (not finished-processing) 
        (accept-process-output proc))
  (set-process-filter proc default-filter))

(defun abort-trale-and-emacs ()
  "interrupt processing of both Emacs and Trale"
  (interactive)
  (free-prolog) (keyboard-quit))

;;;;;;;;;;;;;;;; TRALE (and Prolog) Commands ;;;;;;;;;;;;;;;;;;;

(defun free-prolog ()
  " Give the hand back to the prolog prompt "
  (interactive)
  (process-send-string (get-buffer-process (concat "*" trale-process "*")) "")
  (process-send-string (get-buffer-process (concat "*" trale-process "*")) "a\n")
  (set-process-filter (get-buffer-process (concat "*" trale-process "*")) default-filter))

(defun which-grammar-name ()
   (let ((gname (read-file-name "which grammar? " trale-grammar-directory 
                        (concat trale-grammar-directory trale-grammar-name) 
                        t trale-grammar-name)))
      (setq trale-grammar-name (filename-only gname))
      (setq trale-grammar-directory (dirname-only gname)))
   (save-excursion 
        (set-buffer (concat "*" trale-process "*"))
        (trale-set-wd)))

(defun trale-set-wd ()
  (setq default-directory trale-grammar-directory)
  (process-send-string (get-buffer-process (concat "*" trale-process "*"))
           (concat "unix(cd(\'" trale-grammar-directory "\')).\n")))

(defun filename-only (str)
  (let ((x 0) (y 0))
    (while x
        (setq y x)
        (setq x (string-match "/" str (+ x 1))))
    (substring str (+ y 1))))

(defun dirname-only (str)
  (let ((x 0) (y 0))
    (while x
        (setq y x)
        (setq x (string-match "/" str (+ x 1))))
    (substring str 0 (+ y 1))))


(defun show-trale-buffer ()
   (if (get-buffer-window (concat "*" trale-process "*") t) 
       (raise-frame (window-frame (get-buffer-window (concat "*" trale-process "*") t)))
       (switch-to-buffer-other-window "*prolog*")))

(defun trale-compile-sig ()
   "compile signature in trale"
   (interactive)
   ;;(show-trale-buffer)
   
   ; just one window
   (switch-to-buffer (concat "*" trale-process "*"))
   (process-send-string 
    (get-buffer-process (concat "*" trale-process "*")) (concat "prolog_mode.\ncompile_sig('" trale-grammar-name "'),goto_old_mode.\n" )))

(defun trale-compile-gram ()
   "compile grammar and signature in trale"
   (interactive)
   ;;(show-trale-buffer)
   
   ; just one window
   (switch-to-buffer (concat "*" trale-process "*"))
   ; this asks for the grammar file. We assume that it is `theory.pl'
   ;(let ((whn (which-grammar-name)))
   (process-send-string (get-buffer-process (concat "*" trale-process "*")) 
                        (concat "prolog_mode.\ncompile_gram('" trale-grammar-name "'),goto_old_mode.\n" )))
   
   
(defun trale-compile-grammar ()
   "compile grammar in trale"
   (interactive)
   (switch-to-buffer (concat "*" trale-process "*"))
   (process-send-string (get-buffer-process (concat "*" trale-process "*")) 
                        (concat "prolog_mode.\ncompile_grammar('" trale-grammar-name "'),goto_old_mode.\n" )))

(defun trale-compile-rules ()
   "compile rules in trale"
   (interactive)
   (switch-to-buffer (concat "*" trale-process "*"))
   (process-send-string (get-buffer-process (concat "*" trale-process "*")) 
                        (concat "prolog_mode.\ncompile_rules('" trale-grammar-name "'),goto_old_mode.\n" )))

(defun trale-compile-lex ()
   "compile lexicon in trale"
   (interactive)
   (switch-to-buffer (concat "*" trale-process "*"))
   (process-send-string (get-buffer-process (concat "*" trale-process "*")) 
                        (concat "prolog_mode.\ncompile_lex('" trale-grammar-name "'),goto_old_mode.\n" )))


(defun trale-compile-dcs ()
   "compile grammar in trale"
   (interactive)
   (show-trale-buffer)
   (let ((whn (which-grammar-name)))
      (process-send-string
	(get-buffer-process (concat "*" trale-process "*"))
	  (concat "ascii2atom(\""
		  (concat trale-grammar-directory trale-grammar-name)
		  "\", [], [], File), compile_dcs(File),fail;told.\n\n"))))

(defun trale-compile-empty ()
   "compile grammar in trale"
   (interactive)
   (show-trale-buffer)
   (let ((whn (which-grammar-name)))
      (process-send-string
	(get-buffer-process (concat "*" trale-process "*"))
	  (concat "ascii2atom(\""
		  (concat trale-grammar-directory trale-grammar-name)
		  "\", [], [], File), compile_empty(File),fail;told.\n\n"))))


(defun trale-parse ()
    "read a string to parse and send it to Trale."
    (interactive)
    (let ((pstr (read-from-minibuffer "Enter string to parse: " 
                     nil nil nil nil)))
       (trale-parse-string pstr)))

(defun trale-parse-string (pstr)
   (save-excursion
     (set-buffer (get-buffer-create "trale-previous-parses"))
     (end-of-buffer)
     (insert-if-not-there (concat pstr "\n")))
   (message (concat "Parsing..." pstr))
   (send-command-and-get-result
      (get-buffer-process (concat "*" trale-process "*"))
      (concat "parse_string(\"" pstr "\"), last_number_of_parses.\n")
      "**trale-temp**")
   (goto-char 0)
   (re-search-forward "^[0-9]* Parse(s)$")
   (message (current-line-string))
   (kill-buffer (get-buffer "**trale-temp**"))
   (if trale-show-derivation-on-parse (trale-derivation)
       (if trale-pp-on-parse (trale-last-parse) t)))

(defun trale-parse-current-line ()
   "sends the current line to parse to Trale"
   (interactive)
   (trale-parse-string (current-line-string)))

(defun trale-parse-region ()
   "sends the contents of the region to parse to Trale"
   (interactive)
   (trale-parse-string 
       (remove-cr (buffer-substring 
                      (region-beginning) 
                      (region-end)))))

(defun trale-reparse ()
   "Displays strings previously parsed and reparses the one on the line where the point is"
   (interactive)
   (switch-to-buffer "trale-previous-parses")
   (trale-text-mode))

(defun insert-if-not-there (str)
  (save-excursion
     (goto-char 0)
     (if (re-search-forward (concat "^" str) (point-max) t)
	 t
	 (goto-char (point-max))
	 (insert str))))

(defun remove-cr (str)
  (let ((newline-pos (string-match "\n" str)))
     (if (null newline-pos)
	 str
	 (concat 
	    (substring str 0 newline-pos)
	    (remove-cr (substring str (+ 1 newline-pos)))))))



(defvar trale-lexrule-hist nil "History for the Trale Lexrule")

(defun trale-lexrule ()
    "display information about a Trale lexrule."
    (interactive)
    (show-trale-buffer)
    (let ((lexrule (read-string "Enter lexrule to display: " 
                                nil trale-lexrule-hist)))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\nlex_rule(" lexrule "),goto_old_mode.\n"))))


(defvar trale-rule-hist nil "History for the Trale Rule")

(defun trale-rule ()
    "display information about a Trale rule."
    (interactive)
    (show-trale-buffer)
    (let ((rule (read-string "Enter rule to display: " 
                             nil trale-rule-hist)))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\nwrapped_rule(" rule "),goto_old_mode.\n"))))


(defvar trale-macro-hist nil "History for the Trale Macro")

(defun trale-macro ()
    "display information about a Trale macro."
    (interactive)
    (show-trale-buffer)
    (let ((macro (read-string "Enter macro to display: " 
                              nil trale-macro-hist)))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\nwrapped_macro(" macro "),goto_old_mode.\n"))))

(defun trale-submacros ()
    "display submacros of a Trale macro."
    (interactive)
    (show-trale-buffer)
    (let ((macro (read-string "Enter macro for displaying submacros [complete hierarchy]: " 
                              nil trale-macro-hist)))
      (if (equal macro "")
          (process-send-string
           (get-buffer-process (concat "*" trale-process "*"))
           "prolog_mode.\nshow_all_macros,goto_old_mode.\n")
        (process-send-string
         (get-buffer-process (concat "*" trale-process "*"))
         (concat "prolog_mode.\nshow_all_submacros(" macro "),goto_old_mode.\n")))))

(defvar trale-type-hist nil "History for the Trale type")

(defun trale-type ()
    "display information about a Trale type."
    (interactive)
    (show-trale-buffer)
    (let ((type (read-string "Enter type to display: " 
                             nil trale-type-hist)))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\nwrapped_show_type(" type "),goto_old_mode.\n"))))

(defvar trale-word-hist nil "History for the Trale word")

(defun trale-word ()
    "display information about a Trale word."
  (interactive)
    (let ((word (read-string "Enter word to display [list of all words]: " 
                             nil trale-word-hist "all-words")))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\n"
                (cond ((equal word "all-words") "show_all_words")
                      (t                        (concat "wrapped_lex(" word ")") ))
               ",goto_old_mode.\n"))))


(defvar trale-hierarchy-hist nil "History for the Trale hierarchy")

(defun trale-hierarchy ()
    "display Trale's type hierarchy."
    (interactive)
    (let ((type (read-string "Enter type to display [bot]: " 
                             nil trale-hierarchy-hist "bot")))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\n"
               (cond (trale-draw-glbs   "glbs_on,")
                     (t                 "glbs_off,"))
               (cond (trale-draw-intros "intros_on,")
                     (t                 "intros_off,"))
               "draw_hierarchy(" type "),goto_old_mode.\n"))))

(defvar trale-signature-hist nil "History for the Trale signature")

(defun trale-signature ()
    "display Trale's signature."
    (interactive)
    (let ((type (read-string "Enter type to display [bot]: " 
                             nil trale-signature-hist "bot")))
      (process-send-string
       (get-buffer-process (concat "*" trale-process "*"))
       (concat "prolog_mode.\n"
               (cond (trale-draw-glbs "glbs_on,")
                     (t               "glbs_off,"))
               "draw_signature(" type "),goto_old_mode.\n"))))
    

(defun trale-toggle-glbs ()
    "toggle the trale-draw-glbs flag."
    (interactive)
    (setq trale-draw-glbs (not trale-draw-glbs))
)

(defun trale-toggle-intros ()
    "toggle the trale-draw-glbs flag."
    (interactive)
    (setq trale-draw-intros (not trale-draw-intros))
)

(defvar trale-item-hist nil "History for the Trale item numbers")

(defun trale-test-item-region ()
    "display information about a Trale type."
    (interactive)
    (show-trale-buffer)
    (let ((item1 (read-string "Enter first item to test [test all]: " 
                              nil trale-item-hist)))
      (if (equal item1 "")
          (process-send-string
           (get-buffer-process (concat "*" trale-process "*"))
           "prolog_mode.\ntestt(all),goto_old_mode.\n")
        (let ((item2 (read-string "Enter last item to test: " 
                                  nil trale-item-hist "1000")))
          (process-send-string
           (get-buffer-process (concat "*" trale-process "*"))
           (concat "prolog_mode.\ntestt([" item1 "," item2 "]),goto_old_mode.\n"))))))



(defun trale-load-grammar-for-itsdb ()
  "reloads the current grammar and initializes CPUs for [incr tsdb()]. Old processes are killed."
  (interactive)
  (switch-to-buffer "*common-lisp*")
  (save-window-excursion
    (fi:eval-in-lisp
     "(progn
       (tsdb::tsdb :cpus :kill)
       (tsdb::tsdb :cpus :temp)
       nil)")))

;; (defun list-to-string (list)
;; (let ((current list)
;;         (result ""))
;; ;;(print current)
;;      (while current
;;            (set 'hostname (car current))
;;            (concat result (string hostname))
;;            (if (cdr current)
;;                concat result " ")
;;            (set 'current  (cdr current)))
;;     result)
;; )

(defun trale-set-pvm-hosts ()
   "Set the number of processes that [incr TSDB()] will start"
   (interactive)
;;   (setq default (list-to-string trale-pvm-hosts))
;; (read-string prompt initial history default inherit)
   (setq trale-pvm-hosts (split-string (read-string "Enter hostnames that [incr TSDB()] invokes (localhost localhost for double core machines): " 
                                                             nil nil "localhost"))))


;; the variable delphin-home
;; and the function lkb have to be defined in the .emacs file
;; of the user, i.e. you have to install [incr tsdb()] properly.
;; If you install delphin with the install script and put the recommended
;; lines in your .emacs, a dot.emacs file that comes with delphin should
;; be loaded. This file contains the definition of delphin-home.
(defun trale-itsdb ()
  (interactive)
  (split-window)
  (lkb)
  (defconst lingo-home (concat (getenv "DELPHINHOME") "/lkb"))
  (save-window-excursion
    (fi:eval-in-lisp
     (format 
      "(and
        (load \"%s/src/general/defsystem.lisp\")
        (load \"%s/src/general/loadup.lisp\")
        (load-system \"tsdb\")
        nil)"
      lingo-home lingo-home)))

   (cond 
    ((getenv "TRALE_LANGUAGE")
     (setf language-coding-flag (concat "\"-L" (getenv "TRALE_LANGUAGE") "\""))) ;; removed a space after the L option, do not know why
                                                                                 ;; but the getenv seems to add space. St. M. 31.08.2012
    ((getenv "TRALE_UNICODE")
     (setf language-coding-flag "\"-u\""))
    (t
     (setf language-coding-flag "")))

;; (print language-coding-flag)

   (setf call "(and
                      (setf tsdb::*pvm-cpus*
                           (list")
   (set 'current trale-pvm-hosts)
   (while current
     (set 'hostname (car current))
     (set 'current  (cdr current))
     (cond 
      (trale-standalone-flag ;; provided by the script that starts trale,
                             ;; set in trale-emacs-interface.el in start-trale/start-standalone-trale
      
      ;; This differs from the non-standalone call by using the "-i" flag and by not calling load_tsdb.
      ;; The stuff that is usually loaded with load_tsdb is contained in the executable already.
       (setf call (concat call (format "(tsdb::make-cpu 
                                        :host \"%s\"
                                        :spawn \"%s\"
                                        :options '(\"-i\" \"-s\" %s \"-c\" \"%s\" \"-e itsdb:connect_tsdb\")
                                        :class :temp :name \"trale\" :threshold 2)"
                  hostname (getenv "TRALE_EXE") language-coding-flag (getenv "FULL_PATH_TO_GRAMMAR")))))


     (t (setf call (concat call (format "(tsdb::make-cpu 
                                         :host \"%s\"
                                         :spawn \"%s\"
                                         :options '(\"-s\" %s \"-c\" \"%s\" \"-e load_tsdb,itsdb:connect_tsdb\")
                                         :class :temp :name \"trale\" :threshold 2)"
                  hostname (getenv "TRALE_EXE") language-coding-flag (getenv "FULL_PATH_TO_GRAMMAR")))))))

   (setf call (concat call "))
            (tsdb::tsdb :cpus :temp)
             nil)"))   

  ;; wait till lkb window is really there and close it here.
  ;; If we close it earlier, we get problems, if X is slow.
   (fi:eval-in-lisp "(and (clim:execute-frame-command clim-user:*lkb-top-frame* '(clim-user::com-close-to-replace)) (setq temp t))")  

   (save-window-excursion
     (fi:eval-in-lisp
      call))

   (setq trale-itsdb-started t)
)

(defun shrink-command (str)
  (cond ((string-match "trale-chart-edge" str)
         (concat "show_edge(" (substring str 14) ")"))
        ((string-match "trale-parse-result" str)
         "pp_parses")
        (t nil)))

(defun trale-hide-type ()
  "ale hides the type where the point is"
  (interactive)
  (let ((cw (strip-bar (current-word))))
     (cond ((eq (length cw) 0) t)
           ((has-ucase-letters cw) t)
	   (t (message (concat "Hiding type " cw "..."))
	      (send-command-and-get-result 
		  (get-buffer-process (concat "*" trale-process "*"))
		  (concat "type(" cw "), hide_type('" cw "'), draw_sig(bot,0,10); true.\n")
		  (buffer-name))
	      (trale-signature-mode)
	      (message (concat "Hiding type " cw "...done."))))))

(defun trale-unhide-type ()
  "ale unhides the type where the point is"
  (interactive)
  (let ((cw (strip-bar (current-word))))
     (cond ((eq (length cw) 0) t)
           ((has-ucase-letters cw) t)
           (t (message (concat "Restoring type " cw "..."))
	      (send-command-and-get-result 
		  (get-buffer-process (concat "*" trale-process "*"))
		  (concat "type(" cw "), unhide_type('" cw "'), draw_sig(bot,0,10); true.\n")
		  (buffer-name))
	      (trale-signature-mode)
	      (message (concat "Restoring type " cw "...done."))))))

(defun strip-bar (str)
   (if (equal (string-match "|" str) 0) (substring str 1 (length str)) str))


(defun has-ucase-letters (str)
    (set-variable 'case-fold-search nil) 
    (string-match "[A-Z]" str))

(defun trale-show-type ()
  "ale shows the type near the point"
  (interactive)
  (let ((cw (strip-bar (current-word))))
       (cond ((eq (length cw) 0) t)
             ((has-ucase-letters cw) t)
	     (t (message (concat "Showing type " cw "..."))
		(send-command-and-get-result 
		    (get-buffer-process (concat "*" trale-process "*"))
		    (concat "show_type(" cw "), fail ; true.\n")
		    (buffer-name))
		(message (concat "Hit f to return to the signature"))))))

(defun prolog-prompt ()
  "ale shows the most general satisfier of the current type"
  (interactive)
  (free-prolog)
  (switch-to-buffer (concat "*" trale-process "*")))

(defun trale-quit ()
   "Terminate Trale, kills associated prolog process, \
kills all associated buffers and windows"
   (interactive)
   (if (y-or-n-p "Do you REALLY want to QUIT Trale? ")
       (progn 
		 (free-prolog)
		 (process-send-string 
			  (get-buffer-process (concat "*" trale-process "*"))
			  "lt_close, halt. \n")
		 (kill-buffer (concat "*" trale-process "*"))
		 (kill-trale-buffers)
			 (setq inferior-prolog-mode-map nil)   
				  ;; this is to reset the map to its original state 
				  ;; in case run-prolog is called
			 (message " ")
         (setq do-startup t))
	   t))


(defun trale-prolog-compile-region (start end)
  "Compiles the region."
   (interactive "r")
   (let ((old-buffer (buffer-name)))
      (if (get-buffer-process (concat "*" trale-process "*")) t
	  (run-trale)
	  (switch-to-buffer old-buffer)))
   (save-excursion
     (sicstus-write-region start end temp-pred-file))
   (trale-set-wd)
   (process-send-string 
           (get-buffer-process (concat "*" trale-process "*"))
           (build-prolog-command "compile")))

(defun trale-prolog-compile-buffer-gram ()
  "Compiles the grammar buffer in prolog and trale-compiles the grammar."
  (interactive)
  (trale-prolog-compile-buffer)
  (trale-set-wd)
  (process-send-string
        (get-buffer-process (concat "*" trale-process "*"))
        "compile_gram,fail;told.\n\n"))

(defun trale-prolog-compile-buffer-grammar ()
  "Compiles the grammar buffer in prolog and trale-compiles the grammar."
  (interactive)
  (trale-prolog-compile-buffer)
  (trale-set-wd)
  (process-send-string
        (get-buffer-process (concat "*" trale-process "*"))
        "compile_grammar,fail;told.\n\n"))

(defun trale-prolog-compile-buffer-lex ()
  "Compiles the grammar buffer in prolog and trale-compiles the lexicon."
  (interactive)
  (trale-prolog-compile-buffer)
  (trale-set-wd)
  (process-send-string
        (get-buffer-process (concat "*" trale-process "*"))
        "compile_lex,fail;told.\n\n"))

(defun trale-prolog-compile-buffer-rules ()
  "Compiles the grammar buffer in prolog and trale-compiles the grammar rules."
  (interactive)
  (trale-prolog-compile-buffer)
  (trale-set-wd)
  (process-send-string
        (get-buffer-process (concat "*" trale-process "*"))
        "compile_rules,fail;told.\n\n"))

(defun trale-prolog-compile-buffer-dcs ()
  "Compiles the grammar buffer in prolog and trale-compiles the definite clauses."
  (interactive)
  (trale-prolog-compile-buffer)
  (trale-set-wd)
  (process-send-string
        (get-buffer-process (concat "*" trale-process "*"))
        "compile_dcs,fail;told.\n\n"))

(defun trale-prolog-compile-buffer ()
  "Consults the grammar buffer in prolog."
  (interactive)
  (setq trale-grammar-name (filename-only (buffer-file-name)))
  (setq trale-grammar-directory (dirname-only (buffer-file-name)))
  (trale-prolog-compile-region (point-min) (point-max))
  (show-trale-buffer))


(defun current-line-string ()
  "returns the text of the current line as a string"
  (interactive)
  (save-excursion
      (beginning-of-line)
      (set-mark (point))
      (end-of-line)
      (buffer-substring (region-beginning) (region-end))))



(defun trale-hide-line ()
  (interactive)
  (if selective-display
      t
      (setq selective-display t))
  (toggle-read-only)
  (search-backward "\C-j" () t)
  (replace-match "\C-m")
  (next-line 1)
  (toggle-read-only))

(defun trale-show-line ()
  (interactive)
  (if selective-display
      t
      (setq selective-display t))
  (toggle-read-only)
  (search-forward "\C-m" () t)
  (replace-match "\C-j")
  (toggle-read-only))


;; [PM] This is so wrong. The faces should be customizable
;; [St. M.] This is so wrong. ...
(defun trale-font-lock-keywords ()
  "Set up font lock keywords for the TRALE system." 
  (interactive)
  (if (or window-system
          ;; [PM] XEmacs does font-lock on terminals as well
          ;;      Also, window-system is bogus in a multi-display
          ;;      environment
          (eq prolog-emacs 'xemacs))
      (progn
        (require 'font-lock)
        ;; This is only for backward compatibility. Not needed with
        ;; Emacs 19.34 or later.
        (if (boundp 'font-lock-background-mode)
            ()
          (make-local-variable 'font-lock-background-mode)
          (setq font-lock-background-mode 'light)) ; Assume light bg
        (if (boundp 'font-lock-display-type)
            ()
          (make-local-variable 'font-lock-display-type)
          (setq font-lock-display-type 'color)) ; Assume color

        ;; Create faces
        ;; Format: (FACE FOREGROUND BACKGROUND BOLD-P ITALIC-P UNDERLINE-P)
        (let* ((dark-bg (eq font-lock-background-mode 'dark))
               (faces
                (cond
                 ((memq font-lock-display-type '(mono monochrome))
                  '((prolog-warning-face nil nil t t nil)
                    (prolog-builtin-face nil nil nil nil t)
                    (prolog-redo-face nil nil nil t nil)
                    (prolog-exit-face nil nil nil nil t)
                    (prolog-exception-face nil nil t t t)))
                 ((memq font-lock-display-type '(grayscale greyscale
                                                           grayshade greyshade))
                  '((prolog-warning-face nil nil t t nil)
                    (prolog-builtin-face nil nil nil nil t)
                    (prolog-redo-face nil nil nil t nil)
                    (prolog-exit-face nil nil nil nil t)
                    (prolog-exception-face nil nil t t t)))
                 (dark-bg 		; dark colour background
                  '((prolog-warning-face "red" nil t nil nil)
                    (prolog-builtin-face "LightSkyBlue" nil nil nil nil)
                    (prolog-redo-face "darkorchid" nil nil nil nil)
                    (prolog-exit-face "green" nil nil nil nil)
                    (prolog-exception-face "black" "Khaki" t nil nil)))
                 (t			; light colour background
                  '((prolog-warning-face "red" nil t nil nil)
                    (prolog-builtin-face "Orchid" nil nil nil nil)
                    (prolog-redo-face "darkorchid" nil nil nil nil)
                    (prolog-exit-face "ForestGreen" nil nil nil nil)
                    (prolog-exception-face "black" "Khaki" t nil nil))))))

          (while faces
            (if (fboundp 'font-lock-make-face)
                ;; The preferred way
                (font-lock-make-face (car faces))
              ;; The clumsy way
              (let ((facename (nth 0 (car faces)))
                    (fg (nth 1 (car faces)))
                    (bg (nth 2 (car faces)))
                    (bold (nth 3 (car faces)))
                    (ital (nth 4 (car faces)))
                    (under (nth 5 (car faces))))
                (make-face facename)
                (if fg (set-face-foreground facename fg))
                (if bg (set-face-background facename bg))
                (if bold (make-face-bold facename))
                (if ital (make-face-italic facename))
                (if bold (make-face-bold facename))
                (set-face-underline-p facename under)
                ;; This is needed under Emacs 20 for some reason.
                (set facename facename)
                ))
            (setq faces (cdr faces))))
        ;; Font Lock Patterns
        (let (
              ;; "Native" Prolog patterns
              (head-predicates
               (list (format "^%s" prolog-atom-regexp)
                     0 font-lock-function-name-face))
              (variables
                                        ;xx
                                        ;'("\\<\\([_A-Z][a-zA-Z0-9_]\\)"
                                        ;'("\\<\\([A-Z][a-zA-Z0-9_]\\)"
                                        ;0 font-lock-variable-name-face))
               (list (format "\\<\\([_%s][%s%s0-9_]*\\)"
                             prolog-upper-case-string
                             prolog-lower-case-string
                             prolog-upper-case-string)
                     1 'font-lock-variable-name-face))

              (important-elements
               (list "[][}{!;|]\\|\\*->"
                     0 'font-lock-keyword-face))
              (important-elements-1
               '("[^-*]\\(->\\)" 1 font-lock-keyword-face))
              (trale-important-elements
               '(" \\(if\\) " 1 font-lock-keyword-face))
              (trale-important-elements-1
               '(" \\(if\\)$" 1 font-lock-keyword-face))
              (predspecs		; module:predicate/cardinality
               (list (format "\\<\\(%s:\\|\\)%s/[0-9]+"
                             prolog-atom-regexp prolog-atom-regexp)
                     0 font-lock-function-name-face 'prepend))
              (keywords                 ; directives (queries)
               (list
                (concat
                 "^[?:]- *\\("
                 (prolog-make-keywords-regexp prolog-keywords-i)
                 "\\)\\>")
                1 'prolog-builtin-face))

              (quoted_atom (list prolog-quoted-atom-regexp
                                 2 'font-lock-string-face 'append))
              (string (list prolog-string-regexp
                            1 'font-lock-string-face 'append))

              ;; SICStus specific patterns
              (sicstus-object-methods
               (list
                'prolog-font-lock-object-matcher
                1 font-lock-function-name-face))

              ;; Inferior mode specific patterns
              (prompt
               (list prolog-prompt-regexp-i 0 'font-lock-keyword-face))
              (trace-exit
               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Exit\\):"
                 1 prolog-exit-face))
              (trace-fail
               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Fail\\):"
                 1 prolog-warning-face))
              (trace-redo
               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Redo\\):"
                 1 prolog-redo-face))
              (trace-call
               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Call\\):"
                 1 font-lock-function-name-face))
              (trace-exception
               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Exception\\):"
                 1 prolog-exception-face))
              (error-message-identifier
               '(
                 ;; 3.8 "{\\([A-Z]* ?ERROR:\\)"
                 ;; any line that starts with an exclamation mark
                 ;; extend any lines containing 'error' to the end of line
                 "\\(^! \\(.*error.*$\\)?\\)"
                 1 prolog-exception-face prepend))
              (trale-error-message-identifier
               '(
                 "\\(\\*\\*ERROR:.*\\)"
                 1 prolog-warning-face prepend))
              (test-suite-error-message-identifier
               '(
                 "\\(\\!\\!\\! [0-9]* expected \\!\\!\\!\\)"
                 1 prolog-warning-face prepend))
              (test-suite-ok-message-identifier
               '(
                 "(\\(ok\\))"
                 1 prolog-exit-face prepend))
              (error-whole-messages
               '(
                 ;; 3.8 "{\\([A-Z]* ?ERROR:.*\\)}[ \t]*$"
                 "\\(^! .*\\)$"           ; any line that starts with an exclamation mark
                 1 font-lock-comment-face append))
              (error-warning-messages
               ;; Mostly errors that SICStus asks the user about how to solve,
               ;; such as "NAME CLASH:" for example.
               '("^[A-Z ]*[A-Z]+:" 0 prolog-warning-face))
              (ale-warning-messages
               '("^{\\(ALE: [A-Za-z]*:\\)" 1 prolog-warning-face))
              (trale-residue-messages
               '(
                 "\\(residues: [_0-9,]*[1-9]+[ 0-9,]*\\)"
                 1 prolog-warning-face prepend))
              (warning-messages
               '(
                 ;; 3.8 ;; We allow one newline in the warning message
                 ;; 3.8 "\\({ ?\\(Warning\\|WARNING\\) ?:[^\n]*\n?[^\n]*}\\)[ \t]*$" 
                 "\\(^\\* .*\\)$"         ; any line that starts with an asterisk
                 ;; 3.8 2
                 1
                 prolog-warning-face prepend))
              )

          ;; Make font lock list
          (delq
           nil
           (cond
            ((eq major-mode 'trale-prolog-mode)  ;; the grammar code
             (list
              head-predicates
              quoted_atom
              string
              variables                    ;; I do not understand, why this does not work. St. M.
              important-elements
              important-elements-1
              trale-important-elements
              trale-important-elements-1
              predspecs
              keywords
              sicstus-object-methods
                                        ; nil                     ; types
                                        ; nil                     ; modes
                                        ; nil                     ; directives
              ))
            ((eq major-mode 'trale-interactive-mode)  ;; the Prolog window
             (list
              prompt
              error-message-identifier
              trale-error-message-identifier
              test-suite-error-message-identifier
              test-suite-ok-message-identifier
              error-whole-messages
              error-warning-messages
              warning-messages
              ale-warning-messages
              trale-residue-messages
              predspecs
              trace-exit
              trace-fail
              trace-redo
              trace-call
              trace-exception))
            ((eq major-mode 'prolog-inferior-mode)
             (list
              error-message-identifier
              error-whole-messages
              error-warning-messages
              warning-messages
              predspecs))))
          ))))


;; emacs 22.3
(add-hook 'trale-mode-hook
           (function (lambda ()
                       (make-local-variable 'font-lock-defaults)
                       (setq font-lock-defaults '(trale-font-lock-keywords nil nil ((?_ . "w"))))
                       )))

;; older versions of emacs
;; (add-hook 'trale-mode-hook
;;           (function (lambda ()
;;                       (make-local-variable 'font-lock-defaults)
;;                       (setq font-lock-defaults '(trale-font-lock-keywords nil nil t))
;;                       )))

; (add-hook 'trale-prolog-mode-hook
;           (function (lambda ()
;                       (make-local-variable 'font-lock-defaults)
;                       (setq font-lock-defaults '(trale-font-lock-keywords nil t))


;; copy of prolog-mode-variables but with a different call
;; of trale-font-lock-keywords


 (defun trale-prolog-mode-variables ()
  "Set some common variables to Prolog code specific values."
  (setq local-abbrev-table prolog-mode-abbrev-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "[ \t]*$\\|" page-delimiter)) ;'%%..'
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'adaptive-fill-mode)
  (setq adaptive-fill-mode nil)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'prolog-indent-line)
  (make-local-variable 'comment-start)
  (setq comment-start "%")
  (make-local-variable 'comment-end)
  (setq comment-end "")
  (make-local-variable 'comment-start-skip)
  ;; This complex regexp makes sure that comments cannot start
  ;; inside quoted atoms or strings
  (setq comment-start-skip 
	(format "^\\(\\(%s\\|%s\\|[^\n\'\"%%]\\)*\\)\\(/\\*+ *\\|%%+ *\\)" 
		prolog-quoted-atom-regexp prolog-string-regexp))
  (make-local-variable 'comment-column)
  (make-local-variable 'comment-indent-function)
  (setq comment-indent-function 'prolog-comment-indent)
  (make-local-variable 'comment-indent-function)
  (setq comment-indent-function 'prolog-comment-indent)
  (make-local-variable 'parens-require-spaces)
  (setq parens-require-spaces nil)
  ;; Initialize Prolog system specific variables
  (let ((vars '(prolog-keywords prolog-types prolog-mode-specificators
		prolog-determinism-specificators prolog-directives
		prolog-program-name prolog-program-switches
		prolog-consult-string prolog-compile-string prolog-eof-string
		prolog-prompt-regexp prolog-continued-prompt-regexp
		prolog-help-function)))
    (while vars
      (set (intern (concat (symbol-name (car vars)) "-i"))
	   (prolog-find-value-by-system (eval (car vars))))
      (setq vars (cdr vars))))
  (if (null prolog-program-name-i)
;      (setq compile-command prolog-compile-string-i))
      (progn
	(make-local-variable 'compile-command)
	(setq compile-command prolog-compile-string-i)))
  (make-local-variable 'font-lock-defaults)      
  (setq font-lock-defaults
    '(trale-font-lock-keywords nil nil nil))
  )          


;;;;;; End of trale.el ;;;;;;

