/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2006 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @author Martin Lazarov */

#ifndef LRS_DEBUG_H
#define LRS_DEBUG_H

#include <string>

#include <qglobal.h>

#if QT_VERSION >= 0x40000
#  include <qtcpsocket.h>
#else
#  include <qsocket.h>
#endif

#include <qapplication.h>

namespace LRS {
  /** Simple debugging facility. */
  struct Debug {
    static void trace(const std::string& msg) {

#if QT_VERSION >= 0x40000
      static QTcpSocket* s;
      if (!s) {
        s = new QTcpSocket(qApp);
      }
      if (s && s->state() != QAbstractSocket::ConnectedState) {
        s->setTextModeEnabled(true);
        s->connectToHost("localhost", 7788);
      }
      if (s) {
        s->write(msg.c_str(), msg.size());
        s->write("\n", 1);
        s->flush();
      }
#else
      static QSocket* s;
      if (!s) {
        s = new QSocket(qApp);
      }
      if (s && s->state() != QSocket::Connected) {
        s->connectToHost("localhost", 7788);
      }
      if (s) {
        s->writeBlock(msg.c_str(), msg.size());
        s->writeBlock("\n", 1);
        s->flush();
      }
#endif
    
    }
  };
};

/*

import java.net.*;
import java.io.*;

public class EchoPort {
  public static void main(String[] args) throws Exception {
    ServerSocket ss = new ServerSocket(Integer.parseInt(args[0]));
    while (true) {
      Socket s = ss.accept();
      try {
        BufferedReader reader = new BufferedReader(
          new InputStreamReader(s.getInputStream()));
        String line;
        while ((line = reader.readLine()) != null) {
          System.out.println(line);
          System.out.flush();
        }
      }
      finally {
        s.close();
      }
    }
  }
}


*/

#endif // !LRS_DEBUG_H
