/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2006 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @author Martin Lazarov */

#include "LRS_TreeLayout.h"

namespace LRS {

TreeLayout::TreeLayout(int treePosX, int treePosY, 
  int minNodeHDistance, int minNodeVDistance)
  : _rootTreePosX(treePosX), _rootTreePosY(treePosY),
    _minNodeHDistance(minNodeHDistance), _minNodeVDistance(minNodeVDistance)
{}

void TreeLayout::layoutNode(Node* node) {
  computeTreeSize(*node);
  computeNodePosition(*node, _rootTreePosX, _rootTreePosX);
}

void TreeLayout::computeTreeSize(Node& node) {
  if (node.isTerminal()) {
    node.setTreeSize(node.width(), node.height());
  }
  else {
    int sumSubtreeWidth = 0;
    int maxSubtreeHeight = 0;
    
    for (unsigned i = 0; i < node.numChildren(); ++i) {
      Node* child = node.child(i);
      computeTreeSize(*child);

      if (maxSubtreeHeight < child->treeHeight())
        maxSubtreeHeight = child->treeHeight();
      sumSubtreeWidth += child->treeWidth();
    }
    
    int w = sumSubtreeWidth + (node.numChildren() - 1) * _minNodeHDistance;
    if (w < node.width())
      w = node.width();
    int h = node.height() + _minNodeVDistance + maxSubtreeHeight;
    node.setTreeSize(w, h);
  }
}

void TreeLayout::computeNodePosition(Node& node, int treePosX, int treePosY) {
  if (node.isTerminal()) {
    node.setPosition(treePosX, treePosY);
  }
  else {
    int sumSubtreeWidth = 0;
    
    for (unsigned i = 0; i < node.numChildren(); ++i) {
      sumSubtreeWidth += node.child(i)->treeWidth();
    }
    
    int childrenSpan = sumSubtreeWidth + (node.numChildren() - 1) * _minNodeHDistance;
    
    int nextX = treePosX + (node.treeWidth() - childrenSpan) / 2;
    int nextY = treePosY + node.height() + _minNodeVDistance;
    //Node* prevChild = 0;
    
    for (unsigned i = 0; i < node.numChildren(); ++i) {
      Node* child = node.child(i);
      //if (prevChild == 0 || !child->isTerminal())
      computeNodePosition(*child, nextX, nextY);
      //else
      //  child->setPosition(prevChild->x() + prevChild->width() + _minNodeHDistance, nextY);
      nextX += _minNodeHDistance + child->treeWidth();
      //prevChild = child;
    }
    Node* firstChild = node.child(0);
    Node* lastChild  = node.child(node.numChildren() - 1);
    
    int m1 = firstChild->x() + firstChild->width() / 2;
    int m2 = lastChild->x() + lastChild->width() / 2;
    int x = m1 + (m2 - m1) / 2 - node.width() / 2;
    node.setPosition(x, treePosY);
  }
}

}
