/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @file

   Declarations for the <code>GListBox</code> class.

   @author Holger Wunsch */


#ifndef __GLISTBOX_H
#define __GLISTBOX_H


#include <q3listbox.h>
#include <q3valuevector.h>


/** This class is an extension of Qt's <code>QListBox</code> class. In addition
    to all the functionality of <code>QListBox</code>, this class provides a
    method to query all selected items at once.

    @author Holger Wunsch */
class GListBox: public Q3ListBox {
 public:

  /** Creates a new list box. See the documentation of <code>QListBox</code> for
      more information. 
      @param parent the parent widget
      @param name the name of the list box.
      @param f widget flags */
  GListBox(QWidget* parent = NULL, const char* name = NULL, Qt::WFlags f = 0);


  /** Returns the number of currently selected items.
      @return the number of selected items in this list. */
  int numSelected(void);


  /** Returns a list of the currently selected items in an integer array.
      @return a pointer to a vector contains the indices of all
      currently selected items. */
  Q3ValueVector<int> selectedItems(void);
};


#endif
