/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of class GSVGWindow.

    @author Holger Wunsch */


#include <q3scrollview.h>
#include <qfile.h>
#include <q3accel.h>
#include <qicon.h>
#include <qimage.h>
#include <q3popupmenu.h>
#include <qfileinfo.h>
//Added by qt3to4:
#include <QPixmap>
//#include <kapplication.h>
#include <qapplication.h>
//#include <kicontheme.h>
//#include <kiconloader.h>

#include "gsvgwindow.h"
#include "global.h"
#include "config.h"

namespace {

struct KIcon {
  enum { User };
};

struct MyIconLoader {
  QString _iconPath;
  MyIconLoader() { _iconPath = qApp->applicationDirPath() + "/pics/"; }
  QPixmap loadIcon(QString iconName, int) {
    iconName += ".xpm";
    return QPixmap(_iconPath + iconName);
  }
};

}

//#include "gsvgwindow.moc"


/** Creates a new instance.
    @param svgFileName the name of the SVG file to be displayed.
    @param holder the <code>GDataHolder</code> that owns this viewer.
    @param parent the parent widget. */
GSVGWindow::GSVGWindow(const QString& svgFileName, GDataHolder* holder, QWidget* parent) :
  GDataViewer(holder, parent) {
  QFile svgFile(svgFileName);
  QFileInfo svgFileInfo(svgFileName);
  QString help;

  if (globalOptions->showAppName)
    help = svgFileInfo.fileName() + "  -  " + APP_NAME + " " + APP_VERSION;
  else
    help = svgFileInfo.fileName();
  //setPlainCaption(help);
  setCaption(help);

  svgWidget = new GSVGWidget(&svgFile, this);
  setCentralWidget(svgWidget);

  mainMenu = menuBar();

  Q3PopupMenu* menu;
  QPixmap pix;
  QImage image;
  QIcon icon;
  //KIconLoader* loader = KApplication::kApplication()->iconLoader();
  MyIconLoader myIconLoader;
  MyIconLoader *loader = &myIconLoader;

  // File menu
  menu = new Q3PopupMenu;

  /*  pix = loader->loadIcon("fileprint", KIcon::User);
      image = pix.convertToImage();
      pix.convertFromImage(image.smoothScale(16, 16));
      icon = QIconSet(pix);
      menu->insertItem (icon, "&Print", this,
      SLOT(printSlot()), ALT + Key_P );
      
      menu->addSeparator(); */

  pix = loader->loadIcon("filefloppy", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  menu->insertItem(icon, "&Save...", this,
                       SLOT(saveSlot()), Qt::ALT + Qt::Key_S);

  menu->addSeparator();

  pix = loader->loadIcon("closewindow", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  menu->insertItem (icon, "&Close This Window", this,
                    SLOT(closeSlot()), Qt::ALT + Qt::Key_W );

  mainMenu->insertItem("&File", menu);

  // Data menu
  menu = new Q3PopupMenu;

  pix = loader->loadIcon("prevstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  menu->insertItem(icon, "&Previous Data", this, 
                   SLOT(previousStructureSlot()), Qt::Key_P);
  pix = loader->loadIcon("nextstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  menu->insertItem(icon, "&Next Data", this, 
                   SLOT(nextStructureSlot()), Qt::Key_N);

  mainMenu->insertItem("&Data", menu);

  // Window menu
  menu = new Q3PopupMenu;

  pix = loader->loadIcon("raisewindow", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  menu->insertItem(icon, "Raise &Main Window", this,
                   SLOT(raiseMainWindowSlot()), Qt::ALT + Qt::Key_M);

  menu->addSeparator();
 
  pix = loader->loadIcon("closeviews", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  menu->insertItem(icon, "Close &All Data Windows", parent,
                       SLOT(closeAllSlot()), Qt::ALT + Qt::Key_A);
  
  mainMenu->insertItem("&Window", menu);

  // Toolbar
  mainToolBar = new GRALE::_Q3ToolBar(this);
  addToolBar(mainToolBar);
  
  mainToolBar->setLabel("Type-Hierarchy Viewer Toolbar");

  pix = loader->loadIcon("filefloppy", KIcon::User);
  
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), this, SLOT(saveSlot()), TRUE, "Save");
  //insertButton(mainToolBar, pix, -1, SIGNAL(clicked()), this, SLOT(saveSlot()), "Save");

  /*  pix = loader->loadIcon("fileprint", KIcon::User);
      mainToolBar->insertButton(pix, TID_PRINT, SIGNAL(clicked()), this, SLOT(printSlot()),
      TRUE, "Print");
  */

  mainToolBar->addSeparator();

  pix = loader->loadIcon("zoomin", KIcon::User);
  mainToolBar->insertButton(pix, TID_ZOOM_IN, SIGNAL(clicked()), this, SLOT(slotZoomIn()),TRUE, "Zoom In");
  //insertButton(mainToolBar, pix, TID_ZOOM_IN, SIGNAL(clicked()), this, SLOT(slotZoomIn()), "Zoom In");

  pix = loader->loadIcon("zoomout", KIcon::User);
  mainToolBar->insertButton(pix, TID_ZOOM_OUT, SIGNAL(clicked()), this, SLOT(slotZoomOut()), TRUE, "Zoom Out");
  //insertButton(mainToolBar, pix, TID_ZOOM_OUT, SIGNAL(clicked()), this, SLOT(slotZoomOut()), "Zoom Out");

  mainToolBar->addSeparator();

  pix = loader->loadIcon("prevstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), this, SLOT(previousStructureSlot()), TRUE, "Previous Data");
  //insertButton(mainToolBar, pix, -1, SIGNAL(clicked()), this, SLOT(previousStructureSlot()), "Previous Data");

  pix = loader->loadIcon("nextstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), this, SLOT(nextStructureSlot()), TRUE, "Next Data");
  //insertButton(mainToolBar, pix, -1, SIGNAL(clicked()), this, SLOT(nextStructureSlot()), "Next Data");

  mainToolBar->addSeparator();

  pix = loader->loadIcon("closewindow", KIcon::User);
  mainToolBar->insertButton(pix, TID_CLOSE, SIGNAL(clicked()), this, SLOT(closeSlot()), TRUE, "Close This Window");
  //insertButton(mainToolBar, pix, TID_CLOSE, SIGNAL(clicked()), this, SLOT(closeSlot()), "Close This Window");

  mainToolBar->addSeparator();

  pix = loader->loadIcon("raisewindow", KIcon::User);
  mainToolBar->insertButton(pix, TID_RAISE_MAINWINDOW, SIGNAL(clicked()), this, SLOT(raiseMainWindowSlot()),
  //insertButton(mainToolBar, pix, TID_RAISE_MAINWINDOW, SIGNAL(clicked()), this, SLOT(raiseMainWindowSlot()),
                        "Raise Main Window");

  // Accelerators
  Q3Accel* accel = new Q3Accel(this);
 
  accel->connectItem(accel->insertItem(Qt::Key_Plus),
                     this, SLOT(slotZoomIn()));
  accel->connectItem(accel->insertItem(Qt::Key_Minus),
                     this, SLOT(slotZoomOut()));

  QObject::connect(this, SIGNAL(previousStructureSignal(GDataViewer*)), 
                   parent, SLOT(previousStructureSlot(GDataViewer*)));
  QObject::connect(this, SIGNAL(nextStructureSignal(GDataViewer*)), 
                   parent, SLOT(nextStructureSlot(GDataViewer*)));

  scaling = 50;

  resize(400, 300);
}


/** Destroys this instance. */
GSVGWindow::~GSVGWindow(void) {
}


/** Zooms the SVG picture in. */
void GSVGWindow::slotZoomIn(void) {
  if (scaling < 100) {
    scaling += 10;
    svgWidget->scale(scaling);
    //mainToolBar->setItemEnabled(TID_ZOOM_OUT, true);
  }
  
  if (scaling == 100)
    //mainToolBar->setItemEnabled(TID_ZOOM_IN, false)
    ;
}


/** Zooms the SVG picture out. */
void GSVGWindow::slotZoomOut(void) {
  if (scaling > 20) {
    scaling -= 10;
    svgWidget->scale(scaling);
    //mainToolBar->setItemEnabled(TID_ZOOM_IN, true);
  }
  
  if (scaling == 20)
    //mainToolBar->setItemEnabled(TID_ZOOM_OUT, false)
    ;
}
  

/** Raises the main window. */
void GSVGWindow::raiseMainWindowSlot(void) {
  parentWidget()->setActiveWindow();
  parentWidget()->raise();
}


/** Closes this window and emits a closeSignal. */
void GSVGWindow::closeSlot(void) {
  close();
}


/** Prints the SVG picture. */
void GSVGWindow::printSlot(void) {
  svgWidget->print();
}


/** Handles a user's request to display the previous available data package,
    replacing this viewer. */
void GSVGWindow::previousStructureSlot(void) {
  emit previousStructureSignal(this);
}


/** Handles a user's request to display the next available data package,
    replacing this viewer. */
void GSVGWindow::nextStructureSlot(void) {
  emit nextStructureSignal(this);
}


/** Saves the data package. */
void GSVGWindow::saveSlot(void) {
  holder->save(this);
}
