/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file

   Definition of the GSVGWindow class.

   @author Holger Wunsch */


#ifndef __GSVGWINDOW_H
#define __GSVGWINDOW_H

#include <qevent.h>
//#include <kmenubar.h>
#include <qmenubar.h>
#include <q3toolbar.h>

#include "gdataviewer.h"
#include "gsvgwidget.h"

#include "tools.h"


/* Toolbar IDs */
#define TID_SEPARATOR         0   /**< Dummy toolbar ID to be used with separators. */
#define TID_ZOOM_IN           1   /**< Zoom In toolbar ID. */
#define TID_ZOOM_OUT          2   /**< Zoom Out toolbar ID. */
#define TID_CLOSE             3   /**< Close window toolbar ID. */
#define TID_RAISE_MAINWINDOW  4   /**< Raise mainwindow toolbar ID. */
#define TID_PRINT             5   /**< Print toolbar ID. */


/** A GSVGWindow contains a GSVGWidget that displays an SVG picture.
    Controls are provided to scale the picture within certain limits.

    @author Holger Wunsch */
class GSVGWindow: public GDataViewer {
  Q_OBJECT;


  /** The main toolbar. */
  //KToolBar* mainToolBar;
  GRALE::_Q3ToolBar* mainToolBar;


  /** The main menu. */
  //KMenuBar* mainMenu;
  QMenuBar* mainMenu;


  /** The SVG Widget contained in this window. */
  GSVGWidget* svgWidget;


  /** The current scaling factor (in percent of the SVG-picture's 
      original size. */
  int scaling;


 public slots:
  /** Zooms the SVG picture in. */
  void slotZoomIn(void);


  /** Zooms the SVG picture out. */
  void slotZoomOut(void);


  /** Raises the main window. */
  void raiseMainWindowSlot(void);


  /** Closes this window and emits a closeSignal. */
  void closeSlot(void);


  /** Prints the SVG picture. */
  void printSlot(void);


  /** Handles a user's request to display the previous available data package,
      replacing this viewer. */
  void previousStructureSlot(void);


  /** Handles a user's request to display the next available data package,
      replacing this viewer. */
  void nextStructureSlot(void);


  /** Saves the data package. */
  void saveSlot(void);


 signals:
  /** Emitted on a user's request to display the next available data package,
      replacing this viewer. */
  void nextStructureSignal(GDataViewer* viewer);


  /** Emitted on a user's request to display the previous available data package,
      replacing this viewer. */
  void previousStructureSignal(GDataViewer* viewer);


 public:
  /** Creates a new instance.
      @param svgFileName the name of the SVG file to be displayed.
      @param holder the <code>GDataHolder</code> that owns this viewer.
      @param parent the parent widget. */
  GSVGWindow(const QString& svgFileName, GDataHolder* holder, QWidget* parent = 0);


  /** Destroys this instance. */
  virtual ~GSVGWindow(void);
};

#endif
