/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of class MainWindow.

    @author Holger Wunsch */

#include <algorithm> // moved here because of conflicts with the max macro

#include <stdio.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
//Added by qt3to4:
#include <Q3PopupMenu>
#include <QCloseEvent>

#include "mainw.h"
#include "config.h"
#include "global.h"
#include "vcanvas.h"

#define Inherited QDialog

#include <qstatusbar.h>
#include <qapplication.h>
#include <q3mainwindow.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include <qpixmap.h>
#include <qimage.h>
#include <qnamespace.h>
#include <q3accel.h>

#include "node.h"
#include "compare.h"
#include "dataparser.h"
#include "gdataexchange.h"
#include "gspacingdialog.h"
#include "gsvgwindow.h"
#include "tools.h"

#include <fstream>
#include <iterator>
#include "LRS_MainWindow.h"
#include "LRS_ExpressionParser.h"


struct ConnectionStatusIcon : QLabel {
  QPixmap _iconConnected;
  QPixmap _iconDisconnected;
  
  ConnectionStatusIcon(const QPixmap& iconConnected, const QPixmap& iconDisconnected)
    : _iconConnected(iconConnected),
      _iconDisconnected(iconDisconnected)
  {
    connected(false);
  }
  void connected(bool isConnected) {
    if (isConnected) {
      setPixmap(_iconConnected);
    }
    else {
      setPixmap(_iconDisconnected);
    }
    if (parentWidget())
      parentWidget()->update();
  }
};


/** <code>true</code> if the user changed application options. */
extern bool optionChanged;

MainWindow::MainWindow(QWidget * parent, const char *name)
  : Q3MainWindow(parent, name)
{
  QPixmap pix;
  QImage image;
  QIcon icon;

  GRALE::_IconLoader iconLoader;
  GRALE::_IconLoader* loader = &iconLoader;
  
  using GRALE::KIcon;

  QString help(APP_NAME);
  //help += " ";
  //help += APP_VERSION;
  
  Q3Accel* accel = new Q3Accel(this);

  //setPlainCaption(help);
  setCaption(help);

  resize(400, 300);
  mainMenu = menuBar();

  // File Menu
  fileMenu = new Q3PopupMenu;
  fileMenu->setCheckable(true);
  // & Fr unterstrichenen Buchstaben ...

  //###########################################
  // FileMenu
  
  pix = loader->loadIcon("fileopen", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  fileMenu->insertItem (icon, "&Load Data...", this,
			SLOT( slotLoad() ),
			#ifndef _WIN32
			Qt::ALT + Qt::Key_L
			#else
			Qt::CTRL + Qt::SHIFT + Qt::Key_L
			#endif
			);
  pix = loader->loadIcon("filefloppy", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);

  fileMenu->insertItem (icon, "&Save Data...", this,
			SLOT( slotSave() ),
			#ifndef _WIN32
			Qt::ALT + Qt::Key_S
			#else
			0 // no accelerator
			#endif
			, MID_FILE_SAVE
		);
  fileMenu->setItemEnabled(MID_FILE_SAVE, false);
  
  pix = loader->loadIcon("fileopen", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  fileMenu->insertItem (icon, "Load L&RS Data...", this,
			SLOT( slotLoadLRS() )
			#ifndef _WIN32
			, Qt::ALT + Qt::Key_R
			#endif
			);
  
  fileMenu->insertSeparator();

  fileMenu->insertItem ("E&xit GRALE", this,
			SLOT( close() )
			#ifndef _WIN32
			, Qt::ALT + Qt::Key_X
			#endif
			);

  // Additional accelerators for the file menu
  accel->connectItem(accel->insertItem(Qt::ALT + Qt::Key_Q),
		     this,
		     SLOT(close()));
  
  //###########################################
  //  Data Menu
  dataMenu = new Q3PopupMenu;
  dataMenu->setCheckable(true);

  dataMenu->insertItem("&Show Data", 
		       this, SLOT(showDataSlot()), Qt::CTRL + Qt::Key_S, MID_DATA_SHOW);
  dataMenu->setItemEnabled(MID_DATA_SHOW, false);

  pix = loader->loadIcon("showtree", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);

  dataMenu->insertItem (icon, "Show &Tree",
			this,
			SLOT( showTreeSlot()), Qt::CTRL + Qt::Key_T, MID_DATA_SHOW_TREE );
  dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, false);
  pix = loader->loadIcon("showstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);

  dataMenu->insertItem (icon, "Show &Feature Structure",
			this,
			SLOT( showStructureSlot()), Qt::CTRL + Qt::Key_F, MID_DATA_SHOW_STRUC);
  dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, false);
  dataMenu->insertSeparator();
  pix = loader->loadIcon("deletedata", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);

  dataMenu->insertItem (icon, "&Delete Data",
			this,
			SLOT( deleteDataSlot()), Qt::CTRL + Qt::Key_D, MID_DATA_DELETE_DATA );
  dataMenu->setItemEnabled(MID_DATA_DELETE_DATA, false);
  dataMenu->insertSeparator();
  dataMenu->insertItem("Delete &All Data",
		       this,
		       SLOT(deleteAllDataSlot()), Qt::SHIFT + Qt::CTRL + Qt::Key_D, MID_DATA_DELETE_ALL_DATA);
  dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, false);

  optionMenu = new Q3PopupMenu;

  
  colorMenu = new Q3PopupMenu;
  colorMenu->setCheckable(true);
  colorMenu->insertItem("&Attributes...", this, 
			SLOT(changeNormalColorSlot()));
  colorMenu->insertItem("&Types...", this,
			SLOT(changeTypeColorSlot()));
  colorMenu->insertItem("Te&xt...", this,
			SLOT(changeTextColorSlot()));
  colorMenu->insertItem("Inner T&ree Nodes...", this,
			SLOT(changeTreeColorSlot()));
  colorMenu->insertItem("&Edges...", this,
			SLOT(changeTreeLineColorSlot()));
  colorMenu->insertItem("Lea&ves...", this,
			SLOT(changeLeafColorSlot()));
  colorMenu->insertItem("Edge L&abels...", this,
			SLOT(changeArcLabelColorSlot()));
  colorMenu->insertItem("Edge Label &Background...", this,
			SLOT(changeArcLabelBgColorSlot()));
  colorMenu->insertItem("&Hidden Nodes...", this,
			SLOT(changeHiddenColorSlot()));
  colorMenu->insertItem("Highlighted &Structures...", this,
			SLOT(changeMarkerColorSlot()));
  colorMenu->insertItem("&Found Structures...", this,
			SLOT(changeFoundColorSlot()));
  colorMenu->insertItem("&Differing Structures...", this,
			SLOT(changeDifferentNodesColorSlot()));
  colorMenu->insertItem("Err&ors...", this, 
			SLOT(changeErrorColorSlot()));
  /*  colorMenu->insertSeparator();
      colorMenu->insertItem("&Background color", this, 
      SLOT(changeBackgroundColorSlot()));*/
  
  QObject::connect(this,
	  SIGNAL(changeColorSignal(int, QWidget*)),
	  globalOptions, 
	  SLOT(changeColorSlot(int, QWidget*)));
  
  fontMenu = new Q3PopupMenu;
  fontMenu->insertItem("&Customize All...", this,
		       SLOT(changeAllFontsSlot()));
  fontMenu->insertSeparator();
  fontMenu->insertItem("&Attributes...", this, 
		       SLOT(changeNormalFontSlot()));
  fontMenu->insertItem("&Types...", this, 
		       SLOT(changeTypeFontSlot()));
  fontMenu->insertItem("Inner T&ree Nodes...", this, 
		       SLOT(changeTreeFontSlot()));
  fontMenu->insertItem("&Leaves...", this, 
		       SLOT(changeLeafFontSlot()));
  fontMenu->insertItem("&Edge L&abels...", this, 
		       SLOT(changeArcLabelFontSlot()));
  fontMenu->insertItem("Te&xt...", this, 
		       SLOT(changeTextFontSlot()));
  fontMenu->insertItem("&Errors...", this, 
		       SLOT(changeErrorFontSlot()));
  
  QObject::connect(this,
	  SIGNAL(changeFontSignal(int)),
	  globalOptions, 
	  SLOT(changeFontSlot(int)));
  
  optionMenu->insertItem("Auto-&Expand Tags", this, SLOT(toggleAutoExpandSlot()), 0, MID_OPTIONS_EXPAND_REENT);
  optionMenu->setItemChecked(MID_OPTIONS_EXPAND_REENT, globalOptions->expandReentrancies);
  optionMenu->insertItem("&Windows Fit Structure Size", this, SLOT(toggleFitSlot()), 0, MID_OPTIONS_FIT);
  optionMenu->setItemChecked(MID_OPTIONS_FIT, globalOptions->fitToDrawing);
  optionMenu->insertItem("&Highlight Structures", this, SLOT(toggleHighlightSlot()), 0, MID_OPTIONS_HIGHLIGHT);
  optionMenu->setItemChecked(MID_OPTIONS_HIGHLIGHT, globalOptions->highlightStructures);
  optionMenu->insertItem("&Automatically Open Windows", this, SLOT(toggleAutoOpenSlot()), 0, MID_OPTIONS_AUTOOPEN);
  optionMenu->setItemChecked(MID_OPTIONS_AUTOOPEN, globalOptions->autoOpenWindows);
  optionMenu->insertSeparator();
  optionMenu->insertItem("S&tructure Spacing...", this,
			  SLOT(optionsSlot()));
  optionMenu->insertSeparator();
  optionMenu->insertItem("&Colors", colorMenu);
  optionMenu->insertItem("&Fonts", fontMenu);
  optionMenu->insertSeparator();
  optionMenu->insertItem("&Save Options", this, SLOT(saveOptionsSlot()));

  windowMenu = new Q3PopupMenu();

  pix = loader->loadIcon("closeviews", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);

  windowMenu->insertItem(icon, "Close All &Data Windows", this, SLOT(closeAllSlot()), 0, MID_WINDOW_CLOSEALL);
  windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, false);

  mainMenu->insertItem("&File", fileMenu);
  mainMenu->insertItem("&Data", dataMenu);
  mainMenu->insertItem("&Options", optionMenu);
  mainMenu->insertItem("&Window", windowMenu);

  helpMenu = new Q3PopupMenu();
  pix = loader->loadIcon("GRALE", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  helpMenu->insertItem(icon, "About &GRALE...", this, SLOT(aboutGRALESlot()), 0, MID_ABOUT_GRALE);

  mainMenu->insertItem("&Info", helpMenu);
  
  // status bar
  {
    QPixmap pxm1 = loader->loadIcon("connected", KIcon::User);
    QPixmap pxm2 = loader->loadIcon("disconnected", KIcon::User);
    
    image = pxm1.convertToImage();
    pxm1.convertFromImage(image.smoothScale(16, 16));
    
    image = pxm2.convertToImage();
    pxm2.convertFromImage(image.smoothScale(16, 16));
    
    sbItemConnectionInfo = new QLabel();
    sbItemConnectionIcon = new ConnectionStatusIcon(pxm1, pxm2);
    
    statusBar()->addPermanentWidget(sbItemConnectionInfo);
    statusBar()->addPermanentWidget(sbItemConnectionIcon);
    
    statusBar()->setSizeGripEnabled(false);
    statusBar()->setSizeGripEnabled(false);
  }

  /***********************/
  /* And now the toolbar */
  /***********************/

  // Create toolbar...
  mainToolBar = new GRALE::_Q3ToolBar(this);
  addToolBar(mainToolBar);

  // Add toolbar buttons
  mainToolBar->setLabel("GRALE Main Toolbar");

  pix = loader->loadIcon("fileopen", KIcon::User);
  mainToolBar->insertButton(pix, TID_FILE_OPEN, SIGNAL(clicked()), this, SLOT(slotLoad()),
			TRUE, "Load Data");
  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  mainToolBar->insertButton(pix, TID_FILE_SAVE, SIGNAL(clicked()), this, SLOT(slotSave()),
			TRUE, "Save Data");
  mainToolBar->setItemEnabled(TID_FILE_SAVE, false);

  mainToolBar->insertSeparator(-1, TID_SEPARATOR);

  pix = loader->loadIcon("showtree", KIcon::User);
  mainToolBar->insertButton(pix, TID_DATA_SHOW_TREE, SIGNAL(clicked()), this, SLOT(showTreeSlot()),
			TRUE, "Show Tree");
  mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, false);
  

  pix = loader->loadIcon("showstruc", KIcon::User);
  mainToolBar->insertButton(pix, TID_DATA_SHOW_STRUC, SIGNAL(clicked()), this, SLOT(showStructureSlot()),
			TRUE, "Show Structure");
  mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, false);
  mainToolBar->insertSeparator(-1, TID_SEPARATOR);

  pix = loader->loadIcon("deletedata", KIcon::User);
  mainToolBar->insertButton(pix, TID_DATA_DELETE_DATA, SIGNAL(clicked()), this, SLOT(deleteDataSlot()),
			TRUE, "Delete Data");
  mainToolBar->setItemEnabled(TID_DATA_DELETE_DATA, false);

  mainToolBar->insertSeparator(-1, TID_SEPARATOR);

  pix = loader->loadIcon("closeviews", KIcon::User);
  mainToolBar->insertButton(pix, TID_WINDOW_CLOSEALL, SIGNAL(clicked()), this, SLOT(closeAllSlot()),
			TRUE, "Close All Data Windows");
  mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, false);

  listBox = new GListBox(this);
  listBox->setAutoScrollBar(TRUE);
  listBox->setSelectionMode(Q3ListBox::Extended);
  QObject::connect(listBox, SIGNAL(selectionChanged()), this, SLOT(listSelectionChangedSlot()));

  setCentralWidget(listBox);


  QObject::connect(listBox,
		   SIGNAL(selected(int)),
		   this,
		   SLOT(doubleKlickSlot(int)));

  setMinimumSize(100,100);
  setMaximumSize(9999,9999);

  views = new Q3PtrList<ViewCanvas>;
  closingAll = false;

  svgWindows = new QList<GSVGWindow>;
  closingAllSVG = false;

  QObject::connect(this, SIGNAL(optionChangedSignal()), this, SLOT(optionChangedSlot()));

  dataMgr = new GDataManager(this);
  QObject::connect(dataMgr, SIGNAL(dataModifiedSignal(void)),
		   this, SLOT(updateListBox(void)));
  QObject::connect(dataMgr, SIGNAL(viewerSignal(void)),
		   this, SLOT(viewerChangedSlot(void)));
}


MainWindow::~MainWindow() {
  closeAllSlot();

  delete dataMgr;

  dataMgr = NULL;
}


void MainWindow::slotSave() {
  Q3ValueVector<int> indices(listBox->selectedItems());

  if (indices.size() > 1)
    dataMgr->saveData(indices);
  else
    dataMgr->saveData(indices[0]);
}


void MainWindow::slotLoad() {
  dataMgr->loadData(this);
}

void MainWindow::slotLoadLRS() {
  QString filename = QFileDialog::getOpenFileName(
    this, "Open LRS file", ".", "LRS files (*.lrs);;All files (*)");
  if (filename.isNull())
    return;
    
  using namespace std;
  
  string expr;
  
  {
    ifstream os(filename.toStdString().c_str());
    
    copy(istream_iterator<char>(os), istream_iterator<char>(),
      back_inserter(expr)
      );
  }
  
  std::string conf = (qApp->applicationDirPath() + "/conf/lrsviewer.conf").toStdString();
  LRS::MainWindow* lrsViewer;
  
  try {
    lrsViewer = new LRS::MainWindow(expr, conf, this);
  }
  catch (const LRS::ExpressionParser::SyntaxError& e) {
    QMessageBox::critical(this, "Open LRS file", e.what());
    return;
  }
  
  lrsViewer->setCaption("LRS");
  lrsViewer->show();
}


void MainWindow::updateListBox() {
  listBox->clear();
  
  for (int i = 0; i < dataMgr->numData(); i++) {
    QString help;
    switch (dataMgr->getData(i)->type()) {
    case DT_FEATURE_STRUCTURE:
      help = "(FS) ";
      break;
    case DT_TYPE_HIERARCHY:
      help = "(TH) ";
      break;
    }
    help += (dataMgr->getData(i)->getName()).copy();
    listBox->insertItem(help);
  }

  listSelectionChangedSlot();

  if (dataMgr->numData() > 0)
    dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, true);
  else
    dataMenu->setItemEnabled(MID_DATA_DELETE_ALL_DATA, false);
}


void MainWindow::doubleKlickSlot(int index) {
  GDataHolder* data = dataMgr->getData(index);
  if (data->type() == DT_FEATURE_STRUCTURE) {
    GStructureData* doc = data->featureStructure();
    if (doc->showDefault != 0)
      data->openViewer(this, doc->showDefault);
    else {
      if (doc->tree)
	data->openViewer(this, DRAW_TREE);
      else
	data->openViewer(this, DRAW_STRUC);
    }
  }
  else if (data->type() == DT_TYPE_HIERARCHY) {
    data->openViewer(this, 0);
  }

  viewerChangedSlot();
}


/** Opens viewers for all currently selected data packages. */
void MainWindow::showDataSlot() {
  for (unsigned int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i)) {
      GDataHolder* data = dataMgr->getData(i);
      data->openViewer(this, DRAW_AUTO);
    }
  }

  viewerChangedSlot();
}


void MainWindow::showStructureSlot() {
  for (unsigned int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i)) {
      GDataHolder* data = dataMgr->getData(i);
      data->openViewer(this, DRAW_STRUC);
    }
  }

  viewerChangedSlot();
}
    

void MainWindow::showTreeSlot() {
  for (unsigned int i = 0; i < listBox->count(); i++) {
    if (listBox->isSelected(i)) {
      GDataHolder* data = dataMgr->getData(i);
      data->openViewer(this, DRAW_TREE);
    }
  }

  viewerChangedSlot();
}


void MainWindow::deleteData(bool selOnly) {
  Q3ValueVector<int> indices(listBox->selectedItems());

  dataMgr->deleteData(indices);
}
  

void MainWindow::deleteDataSlot() {
  deleteData(true);
}


void MainWindow::deleteAllDataSlot(void) {
  dataMgr->deleteAllData();
}


void MainWindow::closeAllSlot() {
  dataMgr->closeAllViewers();
}


void MainWindow::optionsSlot() {
  GSpacingDialog* dlg = new GSpacingDialog(globalOptions, this, NULL, true);
  //dlg->setIcon(application->icon());
  if (dlg->exec() == QDialog::Accepted)
    optionChanged = true;
}


void MainWindow::toggleAutoExpandSlot(void) {
  globalOptions->expandReentrancies = !globalOptions->expandReentrancies;
  optionMenu->setItemChecked(MID_OPTIONS_EXPAND_REENT, globalOptions->expandReentrancies);
  optionChanged = true;
}

void MainWindow::toggleFitSlot(void) {
  globalOptions->fitToDrawing = !globalOptions->fitToDrawing;
  optionMenu->setItemChecked(MID_OPTIONS_FIT, globalOptions->fitToDrawing);
  optionChanged = true;
}

void MainWindow::toggleHighlightSlot(void) {
  globalOptions->highlightStructures = !globalOptions->highlightStructures;
  optionMenu->setItemChecked(MID_OPTIONS_HIGHLIGHT, globalOptions->highlightStructures);
  optionChanged = true;
}

void MainWindow::toggleAutoOpenSlot(void) {
  globalOptions->autoOpenWindows = !globalOptions->autoOpenWindows;
  optionMenu->setItemChecked(MID_OPTIONS_AUTOOPEN, globalOptions->autoOpenWindows);
  optionChanged = true;
}

void MainWindow::changeNormalColorSlot() {
  emit changeColorSignal(Options::NormalColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTypeColorSlot() {
  emit changeColorSignal(Options::TypeColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeErrorColorSlot() {
  emit changeColorSignal(Options::ErrorColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeBackgroundColorSlot() {
  emit changeColorSignal(Options::BackgroundColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTextColorSlot() {
  emit changeColorSignal(Options::TextColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTreeColorSlot() {
  emit changeColorSignal(Options::TreeColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeTreeLineColorSlot() {
  emit changeColorSignal(Options::TreeLineColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeLeafColorSlot() {
  emit changeColorSignal(Options::LeafColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeArcLabelColorSlot() {
  emit changeColorSignal(Options::ArcLabelColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeArcLabelBgColorSlot() {
  emit changeColorSignal(Options::ArcLabelBgColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeHiddenColorSlot() {
  emit changeColorSignal(Options::HiddenColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeMarkerColorSlot() {
  emit changeColorSignal(Options::MarkerColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeFoundColorSlot() {
  emit changeColorSignal(Options::FoundColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeDifferentNodesColorSlot() {
  emit changeColorSignal(Options::DifferentNodeColor, this);
  emit optionChangedSignal();
}

void MainWindow::changeNormalFontSlot() {
  emit changeFontSignal(Options::NormalFont);
  emit optionChangedSignal();
}

void MainWindow::changeTypeFontSlot() {
  emit changeFontSignal(Options::TypeFont);
  emit optionChangedSignal();
}

void MainWindow::changeErrorFontSlot() {
  emit changeFontSignal(Options::ErrorFont);
  emit optionChangedSignal();
}

void MainWindow::changeTextFontSlot() {
  emit changeFontSignal(Options::TextFont);
  emit optionChangedSignal();
}

void MainWindow::changeTreeFontSlot() {
  emit changeFontSignal(Options::TreeFont);
  emit optionChangedSignal();
}

void MainWindow::changeLeafFontSlot() {
  emit changeFontSignal(Options::LeafFont);
  emit optionChangedSignal();
}

void MainWindow::changeArcLabelFontSlot() {
  emit changeFontSignal(Options::ArcLabelFont);
  emit optionChangedSignal();
}  


/** Displays a font selection dialog that allows the user
    to change all fonts at once. */
void MainWindow::changeAllFontsSlot(void) {
  globalOptions->changeAllFonts();
}


void MainWindow::optionChangedSlot() {
  ViewCanvas* vc = views->first();
  while (vc) {
    vc->canvas->updateOptions(*globalOptions);
    vc->redrawSlot();
    vc = views->next();
  }
}

void MainWindow::listSelectionChangedSlot() {
  Q3ValueVector<int> selItems = listBox->selectedItems();
  int marked = selItems.size();
  bool hasFeatureStructure = false;
  bool hasTypeHierarchy = false;

  Q3ValueVector<int>::iterator it;
  for (it = selItems.begin(); it != selItems.end(); it++) {
    switch (dataMgr->getData(*it)->type()) {
    case DT_FEATURE_STRUCTURE:
      hasFeatureStructure = true;
      break;
    case DT_TYPE_HIERARCHY:
      hasTypeHierarchy = true;
      break;
    }

    if (hasFeatureStructure && hasTypeHierarchy)
      break;
  }
  
  if (marked > 0) {
    fileMenu->setItemEnabled(MID_FILE_SAVE, true);
    mainToolBar->setItemEnabled(TID_FILE_SAVE, true);

    mainToolBar->setItemEnabled(TID_DATA_DELETE_DATA, true);
    dataMenu->setItemEnabled(MID_DATA_DELETE_DATA, true);

    dataMenu->setItemEnabled(MID_DATA_SHOW, true);
    //mainToolBar->setItemEnabled(TID_DATA_SHOW, true);
 
    if ((marked == 1 && hasFeatureStructure) ||
	(marked > 1 && !hasTypeHierarchy)) {
      dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, true);
      mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, true);
      
      dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, true);
      mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, true);
    }
    else {
      dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, false);
      mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, false);
      
      dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, false);
      mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, false);
    }
  }
  else {
    fileMenu->setItemEnabled(MID_FILE_SAVE, false);
    mainToolBar->setItemEnabled(TID_FILE_SAVE, false);

    dataMenu->setItemEnabled(MID_DATA_DELETE_DATA, false);
    mainToolBar->setItemEnabled(TID_DATA_DELETE_DATA, false);

    dataMenu->setItemEnabled(MID_DATA_SHOW, false);
    //mainToolBar->setItemEnabled(TID_DATA_SHOW, false);

    dataMenu->setItemEnabled(MID_DATA_SHOW_TREE, false);
    mainToolBar->setItemEnabled(TID_DATA_SHOW_TREE, false);

    dataMenu->setItemEnabled(MID_DATA_SHOW_STRUC, false);
    mainToolBar->setItemEnabled(TID_DATA_SHOW_STRUC, false);
  }
}
  
void MainWindow::closeEvent(QCloseEvent* ev) {
#if 0 // moved to main()
  if (optionChanged) {
    if (QMessageBox::question(NULL, "Modified Options",
				    "You have changed several options during this GRALE session.\n\n"
				    "Would you like to make the new settings permanent?",
				    QMessageBox::Yes | QMessageBox::Default, QMessageBox::No
				    ) == QMessageBox::Yes)
      globalOptions->writeOptionToConfig();
  }
#endif
  Q3MainWindow::closeEvent(ev);
  qApp->quit();
}

void MainWindow::aboutGRALESlot(void) {
  QMessageBox::about(this, "About GRALE", GRALE_ABOUT_STRING);
}

void MainWindow::previousStructureSlot(GDataViewer* replace) {
  int index = -1;
  for (int i = 0; i < dataMgr->numData(); i++) {
    if (dataMgr->getData(i)->hasViewer(replace)) {
      index = i;
      break;
    }
  }

  if (index > -1) {
    if (index > 0)
      index--;
    else
      index = dataMgr->numData() - 1;

    int x, y, w, h;
    x = replace->x();
    y = replace->y();
    w = replace->width();
    h = replace->height();

    replace->close();

    GDataViewer* viewer = dataMgr->getData(index)->openViewer(this, DRAW_AUTO); // DRAW_AUTO only applies to feature structures
    viewer->move(x, y);
    viewer->resize(w, h);
  }
}


void MainWindow::nextStructureSlot(GDataViewer* replace) {
  int index = -1;
  for (int i = 0; i < dataMgr->numData(); i++) {
    if (dataMgr->getData(i)->hasViewer(replace)) {
      index = i;
      break;
    }
  }
  
  if (index > -1) {
    if (index < dataMgr->numData() - 1)
      index++;
    else
      index = 0;

    int x, y, w, h;
    x = replace->x();
    y = replace->y();
    w = replace->width();
    h = replace->height();

    replace->close();
    delete replace;
    //GDataViewer* viewer =
    dataMgr->getData(index)->openViewer(this, DRAW_AUTO); // DRAW_AUTO only applies to feature structures
    //viewer->move(x, y);
    //viewer->resize(w, h);
  }
}


/** Called when the status of the socket connection has changed.
    @param status the new connection status (a <code>GDE_STATE_xxxx</code> constant,
      see GDataExchange)
    @param hostname the name of the host GRALE is running on
    @param port the number of the port used in the connection. */
void MainWindow::connectionStateSlot(unsigned char status, char* hostname, int port) {
  char msg[1024];
  bool isConnected = false;

  switch (status) {
  case GDE_STATE_NO_CONNECTION:
    strcpy(msg, " No connection ");
    break;
  case GDE_STATE_READY_INET: 
    sprintf(msg, " %s:%d ", hostname, port);
    break;
  case GDE_STATE_READY_FILE:
    strcpy(msg, " Local ");
    break;
  case GDE_STATE_CONNECTED:
    isConnected = true;
    if (port > -1)
      sprintf(msg, " %s:%d ", hostname, port);
    else
      sprintf(msg, " Local ");
    break;
  default:
    strcpy(msg, "!Unknown status!");
  }
  
  sbItemConnectionInfo->setText(msg);
  reinterpret_cast<ConnectionStatusIcon*>(sbItemConnectionIcon)->connected(isConnected);
}


/** Updates the UI state depending on the number of open viewers. */
void MainWindow::viewerChangedSlot(void) {
  if (dataMgr && dataMgr->numViewers() > 0) {
    windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, true);
    mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, true);
  }
  else {
    windowMenu->setItemEnabled(MID_WINDOW_CLOSEALL, false);
    mainToolBar->setItemEnabled(TID_WINDOW_CLOSEALL, false);
  }
}


/** Calls viewerChangedSlot() when a viewer was closed.
    @param viewer a pointer to the viewer that was closed. */
void MainWindow::viewerClosed(GDataViewer* viewer) {
  viewerChangedSlot();
}


/** Saves all options on demand. */
void MainWindow::saveOptionsSlot(void) {
  globalOptions->writeOptionToConfig();
  optionChanged = false;
}
