/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    This file contains the definitions of several application-wide available functions.

    @author Holger Wunsch */


#ifndef __TOOLS_H
#define __TOOLS_H


#include <qstring.h>
#include <qwidget.h>
#include <q3toolbar.h>
#include <qtoolbutton.h>
#include <qmap.h>
#include <qmessagebox.h>
#include <qpixmap.h>

#include <stdlib.h>

namespace GRALE {
  //
  // qt3to4 port
  //
  struct _Q3ToolBar : Q3ToolBar {
    _Q3ToolBar(Q3MainWindow* parent);
    
    QToolButton* insertButton(const QPixmap& pixmap, int id, const char *signal,
      const QObject *receiver, const char *slot,
      bool enabled, const QString& text);
    
    QToolButton* insertButton(const QPixmap& pixmap, int id, const char *signal,
      const QObject *receiver, const char *slot,
      const QString& text)
    {
      return insertButton(pixmap, id, signal, receiver, slot, false, text);
    }
  
    void setItemEnabled(int itemId, bool enabled);
    void insertSeparator(int pos, int id);
    void insertSeparator();
    
  private:
    QMap<int,QToolButton*> _buttons;
  };
  
  struct KIcon {
    enum { User };
  };
  
  struct _IconLoader {
    QString _iconPath;
    _IconLoader();
    QPixmap loadIcon(QString iconName, int);
  };
  
  struct Null {};
  
  template <class A1 = Null, class A2 = Null, class A3 = Null>
  struct _ErrorMessageBox {
    _ErrorMessageBox(
      QWidget* parent, const char* caption, const char* format,
      A1 arg1 = A1(), A2 arg2 = A2(), A3 arg3 = A3()
      )
    {
      char msg[1024];
      sprintf(msg, format, arg1, arg2, arg3);
      QMessageBox::critical(parent, caption, msg);
    }
  };
  
  template <class A1, class A2, class A3>
  void error_message_box(QWidget* p, const char* cap, const char* fmt, A1 arg1, A2 arg2, A3 arg3) {
    _ErrorMessageBox<A1,A2,A3>(p, cap, fmt, arg1, arg2, arg3);
  }
  
  template <class A1, class A2>
  void error_message_box(QWidget* p, const char* cap, const char* fmt, A1 arg1, A2 arg2) {
    _ErrorMessageBox<A1,A2>(p, cap, fmt, arg1, arg2);
  }
  
  template <class A1>
  void error_message_box(QWidget* p, const char* cap, const char* fmt, A1 arg1) {
    _ErrorMessageBox<A1>(p, cap, fmt, arg1);
  }
  
  /**
    Returns true iff the input c-string is a UTF-8 encoded string.
  */
  bool isUtf8String(const char* s, int len = -1);
  
  /**
    Returns a QString from a c-string s by calling QString::fromUtf8, if s is
      UTF-8 encoded, or by calling QString::fromLocal8Bit, otherwise.
  */
  QString autoString(const char* s, int len = -1);
}


/** Presents a "Save As" dialog to the user.
    @param parent the parent widget.
    @param caption the dialog's caption.
    @param existCheck if <code>true</code>, the dialog will show a warning dialog
      if a file with the selected name already exists.
    @param filter the filter string (see <code>QFileDialog</code>).
    @return the filename the user selected, or <code>QString::null</code> if
      the user cancelled the "Save As" dialog or the warning dialog about an
      existing file. */
QString getSaveFilename(QWidget* parent, const QString& caption, bool existCheck, 
                        const QString& filter);


/**
 * This function returns a temporary file name based on
 * the user ID, the current date and time, and the current
 * pid.
 * @return the temporary name, a pointer to a static buffer.
 */
const char* getTempName(void);

QString getDirPath(const QString& filepath);

#endif
