/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file

    Implementation of the <code>ViewCanvas</code> class.

    @author Holger Wunsch */

#include <qmessagebox.h>
#include <q3accel.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qicon.h>
//Added by qt3to4:
#include <QResizeEvent>
#include <Q3PopupMenu>
#include <QDesktopWidget>

#include "global.h"
#include "vcanvas.h"
#include "gspacingdialog.h"
//#include "vcanvas.moc"

#include "find.h"
#include "NodeAddress.h"
#include "config.h"

#include "glatexrenderer.h"

ViewCanvas::ViewCanvas( GStructureData* d, int what, Options *op, GDataHolder* holder,
                        QWidget *parent, const char *name ): GDataViewer(holder, parent, name) {
  QPixmap pix;
  QImage image;
  QIcon icon;
  
  GRALE::_IconLoader _iconLoader;
  GRALE::_IconLoader *loader = &_iconLoader;
  
  using GRALE::KIcon;

  Q3Accel* accel = new Q3Accel(this);

  showing = what;

  document=d;
  
  options=new Options(*op);

  QString help;
  if (op->showAppName)
    help = d->getName() + "  -  " + APP_NAME + " " + APP_VERSION;
  else
    help = d->getName();
  //setPlainCaption(help);
  setCaption(help);

  if (what == DRAW_STRUC)
    scrollCanvas   = new ScrollCanvas(options, d, d->data->at(0), this);
  else if (what == DRAW_TREE) {
    if (d->tree)
      scrollCanvas   = new ScrollCanvas(options, d, d->tree, this);
    else
      scrollCanvas   = new ScrollCanvas(options, d, d->data->at(0), this);
  }

  setCentralWidget(scrollCanvas);
  canvas = scrollCanvas->getCanvas();
    
  fileMenu = new Q3PopupMenu;
  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  fileMenu->insertItem(icon, "&Save...", this,
                       SLOT(saveSlot()), Qt::ALT + Qt::Key_S);

  fileMenu->insertSeparator();

  pix = loader->loadIcon("fileprint", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  fileMenu->insertItem(icon, "&Print...", this,
                       SLOT( printSlot() ), Qt::ALT + Qt::Key_P);
  
  fileMenu->insertSeparator();
  fileMenu->insertItem("Render &LaTeX...", this,
                         SLOT(renderLatexSlot(void)));

  fileMenu->insertSeparator();

  fileMenu->insertItem ("&Close This Window", this,
                        SLOT( quitWindowSlot() ), Qt::ALT + Qt::Key_W );
  
  menuBar()->insertItem("&File", fileMenu);

  accel->connectItem(accel->insertItem(Qt::ALT + Qt::Key_F4),
                     this, SLOT(quitWindowSlot()));
  
  
  editMenu = new Q3PopupMenu;
  editMenu->setCheckable(true);
  
  pix = loader->loadIcon("expand", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  editMenu->insertItem (icon, "&Expand Tags", this,
                        SLOT( expandReentranciesSlot() ), Qt::CTRL + Qt::Key_E );
  editMenu->insertItem("&Reset Structure", this,
                       SLOT(resetStructureSlot()), Qt::CTRL + Qt::Key_R);
  editMenu->insertSeparator();
  pix = loader->loadIcon("magglass", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  editMenu->insertItem(icon, "Find...", this,
                       SLOT(searchSlot()), Qt::CTRL + Qt::Key_F);

  accel->connectItem(accel->insertItem(Qt::Key_F3),
                     this, SLOT(findNextSlot()));
  accel->connectItem(accel->insertItem(Qt::CTRL + Qt::Key_G),
                       this, SLOT(findNextSlot()));
  
  menuBar()->insertItem("&Edit",editMenu);
  

  dataMenu = new Q3PopupMenu;
  pix = loader->loadIcon("showtree", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  dataMenu->insertItem (icon, "Show &Tree", this,
                        SLOT( showTreeSlot()), Qt::ALT + Qt::Key_T );
  pix = loader->loadIcon("showstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  dataMenu->insertItem (icon, "Show &Structure", this,
                        SLOT( showStructureSlot() ), Qt::ALT + Qt::Key_S);

  dataMenu->insertSeparator();

  pix = loader->loadIcon("prevstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  dataMenu->insertItem(icon, "&Previous Data", this, 
                            SLOT(previousStructureSlot()), Qt::Key_P);
  pix = loader->loadIcon("nextstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  dataMenu->insertItem(icon, "&Next Data", this, 
                            SLOT(nextStructureSlot()), Qt::Key_N);

  menuBar()->insertItem("&Data", dataMenu);
  

  optionsMenu = new Q3PopupMenu;
  
  optionsMenu->insertItem("Window &Fits Structure Size", this, SLOT(toggleFitSlot()), 0, MID_VCOPTIONS_FIT);
  optionsMenu->setItemChecked(MID_VCOPTIONS_FIT, options->fitToDrawing);
  optionsMenu->insertItem("&Highlight Structures", this, SLOT(toggleHighlightSlot()), 0, MID_VCOPTIONS_HIGHLIGHT);
  optionsMenu->setItemChecked(MID_VCOPTIONS_HIGHLIGHT, options->highlightStructures);
  optionsMenu->insertItem("&Display Hidden Nodes", this, SLOT(toggleShowHiddenSlot()), 0, MID_VCOPTIONS_SHOWHIDDEN);
  optionsMenu->setItemChecked(MID_VCOPTIONS_SHOWHIDDEN, options->drawHidden);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem("&Structure Spacing...", this,
                          SLOT(optionsSlot()));
  
  menuBar()->insertItem("&Options", optionsMenu);

  windowMenu = new Q3PopupMenu;

  pix = loader->loadIcon("closeviews", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  windowMenu->insertItem(icon, "Close &All Data Windows", parent,
                       SLOT(closeAllSlot()), Qt::ALT + Qt::Key_A);

  windowMenu->insertSeparator();

  pix = loader->loadIcon("raisewindow", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIcon(pix);
  windowMenu->insertItem(icon, "Raise &Main Window", this,
                       SLOT(raiseMainWindowSlot()), Qt::ALT + Qt::Key_M);
  
  menuBar()->insertItem("Window", windowMenu);


  QObject::connect(this, SIGNAL(previousStructureSignal(GDataViewer*)), 
                   parent, SLOT(previousStructureSlot(GDataViewer*)));
  QObject::connect(this, SIGNAL(nextStructureSignal(GDataViewer*)), 
                   parent, SLOT(nextStructureSlot(GDataViewer*)));


  
  // Create toolbar...
  mainToolBar = new GRALE::_Q3ToolBar(this);
  addToolBar(mainToolBar);
  
  mainToolBar->setLabel("GRALE Data Toolbar");
  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
                            this, SLOT(saveSlot()),
                            TRUE, "Save");

  pix = loader->loadIcon("fileprint", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
                            this, SLOT(printSlot()),
                            TRUE, "Print");

  mainToolBar->insertSeparator();
  //mainToolBar->insertSeparator();
  
  pix = loader->loadIcon("expand", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
                            this, SLOT(expandReentranciesSlot()),
                            TRUE, "Expand Tags");
  
  pix = loader->loadIcon("maximize", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
                            this, SLOT(toggleFitSlot()),
                            TRUE, "Window Fits Structure Size ON/OFF");

  mainToolBar->insertSeparator();

  pix = loader->loadIcon("showtree", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(showTreeSlot()),
                            TRUE, "Show Tree");

  pix = loader->loadIcon("showstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(showStructureSlot()),
                            TRUE, "Show Structure");
  
  mainToolBar->insertSeparator();

  pix = loader->loadIcon("magglass", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(searchSlot()),
                            TRUE, "Find");

  mainToolBar->insertSeparator();

  pix = loader->loadIcon("prevstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(previousStructureSlot()),
                            TRUE, "Previous Data");

  pix = loader->loadIcon("nextstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(nextStructureSlot()),
                            TRUE, "Next Data");

  mainToolBar->insertSeparator();
  //mainToolBar->insertSeparator();

  pix = loader->loadIcon("closewindow", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(quitWindowSlot()),
                            TRUE, "Close This Window");
  
  mainToolBar->insertSeparator();

  // Crashes on KDE 3.4+ when clicked due to a of
  // KToolBarButton::mouseReleaseEvent after button has
  // already been destroyed.
  //pix = loader->loadIcon("closeviews", KIcon::User);
  //mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
  //                        parent, SLOT(closeAllSlot()),
  //                        TRUE, "Close All Data Windows");

  //mainToolBar->insertSeparator();

  pix = loader->loadIcon("raisewindow", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
                            this, SLOT(raiseMainWindowSlot()),
                            TRUE, "Raise Main Window");

  sb = statusBar();

  //sb->insertItem("", 1, 1);
  //sb->setItemAlignment(1, Qt::AlignLeft);


  //    KMainWindow::toolBar()->show();
  //resizeEvent(NULL);
  
  connect( canvas, SIGNAL(redrawSignal()),
           this, SLOT(redrawSlot()) );
  
  connect( this, SIGNAL(redrawSignal()),
           this, SLOT(redrawSlot()) );
  
  setMaximumSize(qApp->desktop()->width(), 
                 qApp->desktop()->height());

  finder = NULL;
  
  findDlg = NULL;

  if (globalOptions->expandReentrancies)
    expandReentranciesSlot();
  else
    redrawSlot();

}


ViewCanvas::~ViewCanvas() {
  if (findDlg != NULL) {
    findDlg->close();
  }
  if (finder != NULL)
    delete finder;

  delete options;
}


void ViewCanvas::resizeEvent(QResizeEvent *ev) {
  Q3MainWindow::resizeEvent(ev);
}

void ViewCanvas::redrawSlot() {
  // repaint has the side effect of computing
  // the size of the canvas widget
  canvas->repaint();

  if (options->fitToDrawing
#ifdef _WIN32
    && !(windowState() & Qt::WindowMaximized)
#endif    
    ) {
    QRect geom = qApp->desktop()->availableGeometry();
    QSize maxSize(geom.width(), geom.height());
    
    maxSize -= frameSize() - scrollCanvas->size();
    
    int lnw = scrollCanvas->lineWidth() + 1;
    QSize sz = canvas->size() + QSize(lnw * 2, lnw * 2);
    
    if (sz.width() > maxSize.width())
      sz.setWidth(maxSize.width());
    if (sz.height() > maxSize.height())
      sz.setHeight(maxSize.height());
    
    QSize szDelta = sz - scrollCanvas->size();
    
    if (!szDelta.isNull()) {
#ifndef _WIN32
    if (windowState() & Qt::WindowMaximized) {
        scrollCanvas->resize(sz + QSize(3,3));
        resize(size() + szDelta + QSize(3,3));
    }
    else
#endif
      {
        scrollCanvas->resize(sz);
        resize(size() + szDelta);
      }
    }
  }
  
  canvas->update();
}


void ViewCanvas::toggleShowHiddenSlot() {
  options->drawHidden = !(options->drawHidden);
  optionsMenu->setItemChecked(MID_VCOPTIONS_SHOWHIDDEN, options->drawHidden);
  
  emit redrawSignal();
}


void ViewCanvas::redraw() {
  emit redrawSignal();
}


void ViewCanvas::toggleFitSlot() {
  options->fitToDrawing = !(options->fitToDrawing);
  optionsMenu->setItemChecked(MID_VCOPTIONS_FIT, options->fitToDrawing);
  if (!options->fitToDrawing) {
    //resize(480, 480);
  }
  emit redrawSignal();
}


void ViewCanvas::toggleHighlightSlot() {
  options->highlightStructures = !(options->highlightStructures);
  optionsMenu->setItemChecked(MID_VCOPTIONS_HIGHLIGHT, options->highlightStructures);
  emit redrawSignal();
}


void ViewCanvas::expandReentranciesSlot() {
  GenericNode* node = 0;
  if (showing == DRAW_STRUC)
    node = document->data->at(0);
  else if (showing == DRAW_TREE)
    node = (GenericNode*) document->tree;
    
  Q3PtrList<int>* expRef = new Q3PtrList<int>();

  NodeAddress na1;
  NodeAddress na2(na1, node);
  expandReentrancies(node, na2, expRef);

  delete expRef;

  emit redrawSignal();
}


void ViewCanvas::expandReentrancies(GenericNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  DisplayableNode* nd = canvas->getDispNodes()->findDisplayableNode(addr);
  if (nd) {
    if (nd->flags & NODEFLAG_HIDDEN)
      return;
  }
  else {
    if (node->flags & NODEFLAG_HIDDEN)
      return;
  }

  switch (node->nodetype) {
  case NODETYPE_TREE_NODE:
    expandReentranciesTreeNode((TreeNode*) node, addr, expRef);
    break;
  case NODETYPE_STRUC_NODE:
    expandReentranciesStrucNode((StrucNode*) node, addr, expRef);
    break;
  case NODETYPE_FEATVAL_NODE:
    expandReentranciesFeatvalNode((FeatvalNode*) node, addr, expRef);
    break;
  case NODETYPE_LIST_NODE:
    expandReentranciesListNode((ListNode*) node, addr, expRef);
    break;
  case NODETYPE_TAIL_NODE:
    expandReentranciesTailNode((TailNode*) node, addr, expRef);
    break;
  case NODETYPE_SET_NODE:
    expandReentranciesSetNode((SetNode*) node, addr, expRef);
    break;
  case NODETYPE_REST_NODE:
    expandReentranciesRestNode((RestNode*) node, addr, expRef);
    break;
  case NODETYPE_REF_NODE:
    expandReentranciesRefNode((RefNode*) node, addr, expRef);
    break;
  case NODETYPE_FUNCTION_NODE:
    expandReentranciesFunctionNode((FunctionNode*) node, addr, expRef);
    break;
  case NODETYPE_REL_NODE:
    expandReentranciesRelationNode((RelationNode*) node, addr, expRef);
    break;
  case NODETYPE_ATOM_NODE:
    break;
  case NODETYPE_DISJUNCTION_NODE:
    expandReentranciesDisjunctionNode((DisjunctionNode*) node, addr, expRef);
    break;
  case NODETYPE_CONJUNCTION_NODE:
    expandReentranciesDisjunctionNode((DisjunctionNode*) node, addr, expRef);
    break;
  default:
    printf("Illegal node type %d in expandReentrancies\n", node->nodetype);
    break;
  }
}


void ViewCanvas::expandReentranciesTreeNode(TreeNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  GenericNode* struc = canvas->getDocument()->findNodeById(node->linkid);
  
  if (struc) {
    NodeAddress addrKid(addr, struc);
    expandReentrancies(struc, addrKid, expRef);
  }

  List* subtrees = node->subtrees;
  while (subtrees) {
    NodeAddress addrKid(addr, subtrees->item);
    expandReentrancies(subtrees->item, addrKid, expRef);
    subtrees = subtrees->tail;
  }
}

    
void ViewCanvas::expandReentranciesStrucNode(StrucNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* featvals = node->featvals;
  
  while (featvals) {
    NodeAddress addrKid(addr, (featvals->item));
    expandReentrancies(featvals->item, addrKid, expRef);
    featvals = featvals->tail;
  }
}


void ViewCanvas::expandReentranciesFeatvalNode(FeatvalNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  NodeAddress addrKid(addr, node->value);
  expandReentrancies(node->value, addrKid, expRef);
}


void ViewCanvas::expandReentranciesListNode(ListNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }

  if (node->tail)
    // Tail nodes themselves are not drawn -> their ID is not included in node address!!
    expandReentrancies(node->tail, addr, expRef);
}


void ViewCanvas::expandReentranciesTailNode(TailNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  NodeAddress addrKid(addr, node->tail);
  expandReentrancies(node->tail, addrKid, expRef);
}


void ViewCanvas::expandReentranciesSetNode(SetNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }

  if (node->rest)
    // Rest nodes themselves are not drawn -> their ID is not included in node address!!
    expandReentrancies(node->rest, addr, expRef);
}


void ViewCanvas::expandReentranciesRestNode(RestNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  NodeAddress addrKid(addr, node->rest);
  expandReentrancies(node->rest, addrKid, expRef);
}


void ViewCanvas::expandReentranciesRefNode(RefNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  int* ref = expRef->first();

  while (ref) {
    if (*ref == node->ref)
      return;
    ref = expRef->next();
  }

  DisplayableNode* nd = canvas->getDispNodes()->addDisplayableNode(addr, (GenericNode*) node);
  
  if (nd) {
    expRef->append(&node->ref);
    nd->flags |= NODEFLAG_EXPANDED;
    ReentNode* reent;
    reent = document->findReentrancy(node->ref);
    if (reent != NULL) {
      NodeAddress addrKid(addr, reent->value);
      expandReentrancies(reent->value, addrKid, expRef);
    }
    else {
      printf("ViewCanvas::expandReentranciesRefNode\n");
      printf("INTERNAL ERROR: ReentrNode with number %d not in data\n", node->ref);
    }
  }
  else {
    printf("ViewCanvas::expandReentranciesRefNode\n");
    printf("INTERNAL ERROR: Node couldn't be added to the list of displayable nodes.\n");
  }
} 


void ViewCanvas::expandReentranciesFunctionNode(FunctionNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }
}


void ViewCanvas::expandReentranciesRelationNode(RelationNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }
}


void ViewCanvas::expandReentranciesDisjunctionNode(DisjunctionNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* operands = node->operands;

  while (operands) {
    NodeAddress addrChild(addr, operands->item);
    expandReentrancies(operands->item, addrChild, expRef);
    operands = operands->tail;
  }
}


void ViewCanvas::expandReentranciesConjunctionNode(ConjunctionNode* node, NodeAddress addr, Q3PtrList<int>* expRef) {
  List* operands = node->operands;

  while (operands) {
    NodeAddress addrChild(addr, operands->item);
    expandReentrancies(operands->item, addrChild, expRef);
    operands = operands->tail;
  }
}


void ViewCanvas::saveStructure(void) {
  QString Filename = getSaveFilename(this, "Save Data", true, "GRALE data files (*.GRALE);;All files (*)");
  if (Filename.isNull())
    return;

  FILE* file = fopen(Filename.local8Bit(), "w");
  
  if (file) {
    setCursor(Qt::WaitCursor);
    
    document->write(file);
    fprintf(file, "\n");
    fclose(file);
    
    setCursor(Qt::ArrowCursor);
  }
  else {
    char buf[500];
    sprintf(buf, "Could not open file %s", (const char*) Filename.local8Bit());
    QMessageBox::critical(this, "Save", buf);
  }
}
  

void ViewCanvas::saveSlot(void) {
  saveStructure();
}


void ViewCanvas::quitWindowSlot() {
  close();
}

void ViewCanvas::expandLocalPopupSlot() {
  NodeAddress addr = canvas->getKlickedOn()->addr;
  Q3PtrList<int>* expRef = new Q3PtrList<int>();

  expandReentrancies(canvas->getDispNodes()->findDisplayableNode(addr)->node, addr, expRef);

  delete expRef;

  emit redrawSignal();
}

void ViewCanvas::printSlot() {
  QPrinter printer;
  options->readPrinterOptions(printer);
  if(printer.setup(this)) {
    options->writePrinterOptions(printer);
    canvas -> printToPrinter(printer);
  }
}


void ViewCanvas::searchSlot() {
  if (findDlg == NULL) {
    QString dlgCaption("Find in ");
    dlgCaption.append(document->getName());
    
    findDlg = new StrucFindDialog(dlgCaption, this);
    connect(findDlg, SIGNAL(findSignal()), this, SLOT(findSlot()));
    connect(findDlg, SIGNAL(findNextSignal()), this, SLOT(findNextSlot()));
    connect(findDlg, SIGNAL(cancelSignal()), this, SLOT(cancelSearchSlot()));
  }
  findDlg->show();
}


void ViewCanvas::contextMenuSearch(char* searchTerm, const NodeAddress addr) {
  if (findDlg != NULL)
    findDlg->close();

  if (finder != NULL)
    delete finder;

  QString dlgCaption("Find in ");
  dlgCaption.append(document->getName());

  findDlg = new StrucFindDialog(dlgCaption, searchTerm, this);
  connect(findDlg, SIGNAL(findSignal()), this, SLOT(findSlot()));
  connect(findDlg, SIGNAL(findNextSignal()), this, SLOT(findNextSlot()));
  connect(findDlg, SIGNAL(cancelSignal()), this, SLOT(cancelSearchSlot()));
  findDlg->show();

  Q3MemArray<int>* types = new Q3MemArray<int>(15);
  (*types)[0] = NODETYPE_TYPE_NODE;
  (*types)[1] = NODETYPE_STRUC_NODE;
  (*types)[2] = NODETYPE_FEATVAL_NODE;
  (*types)[3] = NODETYPE_LIST_NODE;
  (*types)[4] = NODETYPE_SET_NODE;
  (*types)[5] = NODETYPE_FUNCTION_NODE;
  (*types)[6] = NODETYPE_REF_NODE;
  (*types)[7] = NODETYPE_REENT_NODE;
  (*types)[8] = NODETYPE_TREE_NODE;
  (*types)[9] = NODETYPE_ATOM_NODE;
  (*types)[10] = NODETYPE_REL_NODE;
  (*types)[11] = NODETYPE_TAIL_NODE;
  (*types)[12] = NODETYPE_REST_NODE;
  (*types)[13] = NODETYPE_DISJUNCTION_NODE;
  (*types)[14] = NODETYPE_CONJUNCTION_NODE;

  finder = new Finder(document, types, findDlg->text().local8Bit(), findDlg->ignoreCaseChecked());

  if (showing == DRAW_STRUC)
    finder->findGenericNode(document->data->at(0), NodeAddress());
  else
    finder->findGenericNode((GenericNode*) document->tree, NodeAddress());
  if (document->rel->count() > 0)
    finder->findGenericNode(document->rel->at(0), NodeAddress());

  const NodeAddress* found = finder->first();

  if (found == NULL) {
    QMessageBox::information(findDlg, "Find structure", "No matches found.", QMessageBox::Ok);
    findDlg->close();
    findDlg = NULL;
    delete finder;
    finder = NULL;

    return;
  }

  const NodeAddress* start = found;

  while (true) {
    if (*found == addr)
      break;

    found = finder->next();

    if (found == NULL)
      found = finder->first();

    if (found == start) {
      QMessageBox::information(findDlg, "Find structure", "No matches found.", QMessageBox::Ok);
      findDlg->close();
      findDlg = NULL;
      delete finder;
      finder = NULL;
      
      return;
    }
  }

  found = finder->next();
  if (found == NULL)
    found = finder->first();

  canvas->ensureVisible(found);
  canvas->markFound(found);
  
  QPoint pt = canvas->nodeOrigin(found);
  scrollCanvas->ensureVisible(pt.x(), pt.y());
}
  

void ViewCanvas::findSlot() {
  Q3MemArray<int>* types = new Q3MemArray<int>(15);
  (*types)[0] = NODETYPE_TYPE_NODE;
  (*types)[1] = NODETYPE_STRUC_NODE;
  (*types)[2] = NODETYPE_FEATVAL_NODE;
  (*types)[3] = NODETYPE_LIST_NODE;
  (*types)[4] = NODETYPE_SET_NODE;
  (*types)[5] = NODETYPE_FUNCTION_NODE;
  (*types)[6] = NODETYPE_REF_NODE;
  (*types)[7] = NODETYPE_REENT_NODE;
  (*types)[8] = NODETYPE_TREE_NODE;
  (*types)[9] = NODETYPE_ATOM_NODE;
  (*types)[10] = NODETYPE_REL_NODE;
  (*types)[11] = NODETYPE_TAIL_NODE;
  (*types)[12] = NODETYPE_REST_NODE;
  (*types)[13] = NODETYPE_DISJUNCTION_NODE;
  (*types)[14] = NODETYPE_CONJUNCTION_NODE;

  if (finder != NULL)
    delete finder;

  finder = new Finder(document, types, findDlg->text().local8Bit(), findDlg->ignoreCaseChecked());

  if (showing == DRAW_STRUC)
    finder->findGenericNode(document->data->at(0), NodeAddress());
  else
    finder->findGenericNode((GenericNode*) document->tree, NodeAddress());
  if (document->rel->count() > 0)
    finder->findGenericNode(document->rel->at(0), NodeAddress());


  const NodeAddress* found = finder->first();

  if (found != NULL) {
    canvas->ensureVisible(found);
    canvas->markFound(found);
    
    QPoint pt = canvas->nodeOrigin(found);
    scrollCanvas->ensureVisible(pt.x(), pt.y());
  }
  else {
    QMessageBox::information(findDlg, "Find structure", "No matches found.", QMessageBox::Ok);
    findDlg->close();
    findDlg = NULL;
    delete finder;
    finder = NULL;
  }
}


void ViewCanvas::findNextSlot() {
  if (findDlg == NULL) {
    searchSlot();
    return;
  }

  if (finder == NULL) {
    findSlot();
    return;
  }
    
  const NodeAddress* found = finder->next();
  if (found == NULL) {
    QMessageBox::information(findDlg, "Find", 
                             "The end of the structure has been reached. The search will start again from the top.", 
                             QMessageBox::Ok);
    found = finder->first();
  }
  canvas->ensureVisible(found);
  canvas->markFound(found);
  
  QPoint pt = canvas->nodeOrigin(found);
  scrollCanvas->ensureVisible(pt.x(), pt.y());
}


void ViewCanvas::cancelSearchSlot() {
  findDlg = NULL;
  delete finder;
  finder = NULL;
  canvas->markFound(NULL);
}


void ViewCanvas::resetStructureSlot() {
  canvas->reset();
}


void ViewCanvas::showTreeSlot() {
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  if (findDlg != NULL) {
    findDlg->close();
    findDlg = NULL;
  }
  showing = DRAW_TREE;
  canvas->setRootNode((GenericNode*) document->tree);
}


void ViewCanvas::showStructureSlot() {
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  if (findDlg != NULL) { 
    findDlg->close();
    findDlg = NULL;
  }
  showing = DRAW_STRUC;
  canvas->setRootNode(document->data->at(0));
}


void ViewCanvas::showTreeNewWindowSlot() {
  //  mainw->newViewWindow(document, DRAW_TREE);
}


void ViewCanvas::showStructureNewWindowSlot() {
  //  mainw->newViewWindow(document, DRAW_STRUC);
}


bool ViewCanvas::showsDoc(GStructureData* doc) {
  return doc == document;
}


void ViewCanvas::optionsSlot() {
  GSpacingDialog* dlg = new GSpacingDialog(options, this, NULL, true);
  if (dlg->exec() == QDialog::Accepted) {
    emit redrawSignal();
  }
}


void ViewCanvas::raiseMainWindowSlot(void) {
  mainw->setActiveWindow();
  mainw->raise();
}


void ViewCanvas::previousStructureSlot(void) {
  if (findDlg != NULL) {
    findDlg->close();
    findDlg = NULL;
  }
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  emit previousStructureSignal(this);
}


void ViewCanvas::nextStructureSlot(void) {
  if (findDlg != NULL) {
    findDlg->close();
    findDlg = NULL;
  }
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  emit nextStructureSignal(this);
}


/** Changes the status bar message.
    @param message the new message. */
void ViewCanvas::setStatusBarMessage(const char* message) {
  //sb->changeItem(QString::fromLocal8Bit(message), 1);
  sb->showMessage(message);
}


/** Writes the currently selected data package to a LaTeX file. */
void ViewCanvas::renderLatexSlot(void) {
  QString Filename = getSaveFilename(this, "Render LaTeX", true, "LaTeX files (*.tex);;All files (*)");
  if (Filename.isNull())
    return;

  GLatexRenderer latex(document, showing, canvas->getDispNodes(), Filename);
  latex.render();
}

void ViewCanvas::hideUnhidePopupSlot() {
  //canvas->hideUnhide(canvas->getDispNodes()->findDisplayableNode(canvas->getKlickedOn()->addr));
  //qDebug("S/H Type %x", canvas->getDispNode());
  //qDebug("S/H Type '%s'", ((TypeNode*) canvas->getDispNode())->name);
  canvas->hideUnhide(canvas->getDispNode());
}
