%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: graphviz.pl,v $
%%  $Revision: 1.17 $
%%      $Date: 2005/03/29 21:59:28 $
%%     Author: Detmar Meurers (dm@ling.osu.edu)
%%Modified by: Gerald Penn (gpenn@cs.toronto.edu)
%%    Purpose: Provides graphical hierarchy output
%%             using graphviz package, cf.
%%             http://www.research.att.com/sw/tools/graphviz/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: graphviz.pl,v $
%% Revision 1.17  2005/03/29 21:59:28  stefan
%% implemented code that can be used to hide glbs or types that are needed for
%% feature introduction only. Such types have to start with glb_ or intro_ respectively.
%%
%% Revision 1.16  2004/03/13 10:05:56  stefan
%% added code for hiding types in a type hierarchy
%%
%% Revision 1.15  2004/03/12 12:03:56  stefan
%% added hook for hiding types
%%
%% Revision 1.14  2003/11/28 20:23:08  dm
%% - remodeled code a bit since it now also gets called from
%%   lr_compiler/arc2vcg.pl
%% - user can include graphviz_option(ps,gv) or (jpg,xv) or ...
%%   in grammar code or
%% - call set_graphviz_option(ps,gv) or (jpg,xv) or ...
%% - default is (svg,grisu)
%%
%% Revision 1.13  2003/10/02 22:24:49  dm
%% Catch and output special type a_ _ as a string to keep the two symbols
%% together in output. Also changed to use system/1 instead of shell/1
%% calls now to use default shell.
%%
%% Revision 1.12  2003/10/02 21:14:15  dm
%% Added grisu as being usable with graphviz_option(svg,grisu) - from
%% Version 1.4.0 on it can display svg files!
%%
%% Revision 1.11  2003/09/29 14:48:18  dm
%% Correction of last check in: I wanted gv not ghostview as default for
%% viewing.
%%
%% Revision 1.10  2003/09/29 01:28:57  dm
%% Changed graphviz_option_default to ps and ghostview since ps includes
%% text as text instead of as graphics and ghostview supports scaling and
%% panning in lager-than-screen images well.
%%
%% Revision 1.9  2003/05/13 04:58:49  dm
%% Allow possibility of intro/2 being part of a subs/2 declaration (cf. the
%% old ALE style of declaring a signature)
%%
%% Revision 1.8  2003/05/13 04:28:27  dm
%% Added faile_if_undefined/1 around intro/2 to catch case in which grammar
%% doesn't contain any.
%%
%% Revision 1.7  2003/05/13 04:26:30  dm
%% simplified draw_local/4
%%
%% Revision 1.6  2003/05/13 04:13:08  dm
%% Added draw_hierarchy/0 and draw_signature/0 to output entire
%% hierarchy.
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% provides the following user commands:

draw_hierarchy(Type) :-
   draw_hierarchy(Type,false).

draw_signature(Type) :-
   draw_hierarchy(Type,true).

% and two special versions to output the entire hierarchy:

draw_hierarchy :-
   draw_hierarchy(bot).

draw_signature :-
   draw_signature(bot).


:- dynamic asserted_graphviz_option/2.

% An option to determine the output format and how its shown: 
set_graphviz_option(UserFormat,UserProgram) :-
   set_graphviz_option_aux(UserFormat,UserProgram,Format,Program),
   retractall(asserted_graphviz_option(_,_)),
   assert(asserted_graphviz_option(Format,Program)).

% ------------------------------------------------------
% The predicate graphviz_option/2 can be defined by
% the user as part of the grammar to specify which format 
% is supposed to be generated and which program can view
% that format. It can also be set dynamically with
% set_graphviz_option/2.

% ----------------------------
% graphviz_option_/2: returns value of option

graphviz_option_(Format,Program) :-
   asserted_graphviz_option(Format,Program),
   !.

graphviz_option_(Format,Program) :-
   current_predicate(graphviz_option/2),
   graphviz_option(UserFormat,UserProgram),
   set_graphviz_option_aux(UserFormat,UserProgram,Format,Program),
   !.

graphviz_option_(Format,Program) :-
   graphviz_option_default(Format,Program).

% ----------------------------

% Specify the default used if the user hasn't defined graphviz_option/2
% and hasn't called set_graphviz_option/2.

%graphviz_option_default(jpg,xv).
%graphviz_option_default(ps,gv).
graphviz_option_default(svg,grisu).

% ----------------------------

set_graphviz_option_aux(UserFormat,UserProgram,Format,Program) :-
   ((atom(UserFormat),atom(UserProgram)) 
   ->  UserFormat=Format,UserProgram=Program
   ;   graphviz_option_default(Format,Program),
       format(user_error,'~N**Error: Both arguments of graphviz_option(<format>,<program>) must be atoms!~n(surround ~w and ~w by single quotes to obtain atoms)~nUsing fallback graphviz_option(~w,~w).~n',[UserFormat,UserProgram,Format,Program])
   ).

% -----------------------------------------------------------------------

% GBP: system/1 and tmpnam/1 imports moved to sics{4}.pl
% GBP: charsio now takes place through access predicates in sics{4}.pl
%:- use_module(library(charsio),[format_to_chars/3,format_to_chars/4]).


% hide_in_type_hierarchy/1 specifies prefixes of types that will not be displayed.
% The following two declarations prevent types starting with `glb' and 'intro'
% from being displayed. The grammar writer has to take care of appropriate naming of
% the respective types. `intr' types are types that are needed only for the introduction
% of features. These cannot be suppressed if the signature is shown. glb types
% may always be suppressed.

hide_in_type_hierarchy(Type) :-
         (hide_glbs(on) ->
             atom_codes(Type,[103,108,98|_])).

hide_in_type_hierarchy(Type) :-
         (hide_intro(on) ->
             atom_codes(Type,[105,110,116,114,111|_])).


% These flags will be set via the EMACS menu.

:- dynamic hide_glbs/1.

glbs_on  :- retractall(hide_glbs(_)),
            assert(hide_glbs(off)).
glbs_off :- retractall(hide_glbs(_)),
            assert(hide_glbs(on)).



:- glbs_on.


% Types which are used for the introduction of features only.
% For drawing signatures these have to displayed, in hierarchies
% they may be suppressed.
:- dynamic hide_intro/1.


intros_on  :- retractall(hide_intro(_)),
              assert(hide_intro(off)).
intros_off :- retractall(hide_intro(_)),
              assert(hide_intro(on)).


:- intros_on.



draw_hierarchy(Type,ShowApprops) :-
	type_hierarchy_compiler:ensure_signature_present,
	type_hierarchy_compiler:ensure_type_defined(Type),
	type_hierarchy_compiler:get_all_subtypes(Type,Result,[],_),
%        trace,
        ( (current_predicate(user_hide_in_type_hierarchy/1)
          ; hide_glbs(on)
          ; hide_intro(on)
          ) ->
            hide_intro(OldHideIntro),
            ( ShowApprops ->
                intros_on
            ; true ),
            eliminate_hidden_types(Result,Result1),
            retractall(hide_intro(_)),
            assert(hide_intro(OldHideIntro))
        ; Result1 = Result
        ),
	draw_hierarchy_act(Type,Result1,ShowApprops,Filename),
	call_graphviz(Filename).

draw_hierarchy_act(Type,Result,ShowApprops,Filename) :-	
	(Result == [] ->
	    format(user_output,'**Error: Type "~w" has no subtypes.~n',[Type])
	;
	    tell_tmpnam_dot(Filename,OldOutputStream),

	    format('digraph ~w {~n',[Type]),

            % horizontal space between nodes in inches:
	    format('ranksep="0.4 equally";~n',[]), 

            % horizontal space between nodes in inches:
	    format('nodesep="0.2 equally";~n',[]), 

	    % justification of labels (relevant if it has more lines)
	    format('labeljust="l";~n',[]),

            % set fontpath
%	    format('fontpath="/usr/share/fonts/default/TrueType/";~n',[]),

	    % font name and size to use
%	    format('node [fontname="helr____"];~n',[]),

	    % nodes are just text and there's no padding around them
	    format('node [height=0,width=0,shape=plaintext];~n',[]),

	    % here we can compress it to a certain size
%	    format('ratio="compress";~nsize="8.5,11";~n',[]),

	    % or one can just specify a certain size, without changing the ratio
%	    format('size="13,10";~n',[]),

            % or one can set size in inches at which to cut into pages, and margins
%	    format('page="8.5,11"~numargin="0.1,0.1"~n',[]), 

	    % no arrows at end of edges:
	    format('edge [arrowhead="none"];~n',[]), 

	    draw_hierarchy([Type-Result],_,ShowApprops),

	    format('~N}~n',[]),
	    told_tmpnam_dot(OldOutputStream)
	).

% print subtypes in style of signature input format
draw_hierarchy([],_,_).
draw_hierarchy([H|T],Mother,ShowApprops) :- 
   (   H=(Type-Subtypes)
   ->  draw_local(Type,Type1,Mother,ShowApprops),
       draw_hierarchy(Subtypes,Type1,ShowApprops)
   ;   draw_local(H,_,Mother,ShowApprops)
   ),
   draw_hierarchy(T,Mother,ShowApprops).

draw_local(Type,Type1,Mother,ShowApprops) :-
   (  Type=multi(Type1)		% eliminate multi wrappers
   -> true
   ;  Type=Type1),
   (  Type1=(a_ _)		% show a_ atoms as anon var string
   -> Type2= '"a_ _"'
   ;  Type1=Type2),
   (   var(Mother)		           % if top level
   ->  true			           % do nothing
   ;   format('~w -> ~w;~n',[Mother,Type2]) % else output local graph
   ),
   (  (ShowApprops, 
	  (fail_if_undefined(intro(Type2,FVs))
	  ;fail_if_undefined(subs(Type2,intro(_SubT,FVs)))) )
   ->  format('~w [label="~w\\l',[Type2,Type2]),
       draw_approps(FVs),
       format('"];~n',[])
   ;   true
   ).

draw_approps([]).
draw_approps([H|T]) :-
   format('   ~w\\l',[H]), %GBP: I guess this thing needs \l - either that or a form feed
   draw_approps(T).


call_graphviz(Filename) :-
   graphviz_option_(Format,Program),
   (  Program = grisu
   -> (  grale_flag
      -> t_send_dot_command(Filename),
	 get_output_stream(Stream),
	 format(Stream,'~N!show_svg_file "~w.svg"~n',[Filename]),
	 flush_output(Stream)	       
      ;  format(user_error,'~N**Error: No grisu Interface present. Specify other graphviz_option/2 or run grisu.~n',[]),
	 fail
      )
   ;  t_send_big_dot_command(Format,Program,Filename)
   ).


:- dynamic known_type/1.

eliminate_hidden_types(Types,Result) :-
        retractall(known_type(_)),
        expand_all_multi_types(Types,ExpandedTypes),
%        write('Type: '),write(Types),nl,
%        write('Type without multi/1: '),write(ExpandedTypes),nl,
        eliminate_hidden_types_(ExpandedTypes,R1),
        remove_duplicate_subtypes(R1,Result).


% Without previous expansion of the multi/1 entries the following situation
% can arise:

% [attr_adjective_word-[attr_np_adj-[],simple_attr_adj-[]],
%  complementizer_like_sign-[complementizer_word-[]],
%  multi(attr_adjective_word)-[],
%  noun_mod_preposition-[location_noun_mod_prep-[]]]],
%  preposition_word-[comp_preposition-[],
%                    mod_preposition-[multi(noun_mod_preposition)-[]]]]

% The elemenitaion of a glb type caused a type (noun_mod_prposition) that
% has subtype information to be raised to the next higher level. Since
% it is subsumed by preposition_word, it has to be removed, but removing it
% would make us loosing the subtype information, since the other instance
% of the type is multi/1.


expand_all_multi_types([],[]).
expand_all_multi_types([multi(Type)-[]|Tail],[Type-ExpandedSubtypes|ExpandedTypes]) :- !,
        type_hierarchy_compiler:get_all_subtypes(Type,Subtypes,[],_),
        expand_all_multi_types(Subtypes,ExpandedSubtypes),
        expand_all_multi_types(Tail,ExpandedTypes).
                      
expand_all_multi_types([Type-Subtypes|Tail],[Type-ExpandedSubtypes|ExpandedTypes]) :-
        expand_all_multi_types(Subtypes,ExpandedSubtypes),
        expand_all_multi_types(Tail,ExpandedTypes).
        

eliminate_hidden_types_([],[]).
eliminate_hidden_types_([H|T],Res) :-
     (   H=(Type-Subtypes)
     ->  (
%           format('looking at ~w~n',[Type]),
           

           % We instantiate Type1 depending on the value of Type.
           % To be able to call hide_in_type_hierarchy/1 with Type1 as an argument.
           
           (  Type=(a_ _)      % show a_ atoms as anon var string
              -> Type1= '"a_ _"'
             ;  Type1=Type),
             
           eliminate_hidden_types_(Subtypes,SubtypesNoGLB),
             
           (hide_in_type_hierarchy(Type1) ->
               HNoGLB = SubtypesNoGLB
%           , format('eliminating ~w, Subtypes: ~w~n',[Type2,SubtypesNoGLB])
           ;
               HNoGLB = [Type1-SubtypesNoGLB]
           )
         )
     ; % write(H),nl,
         true
     ),
     eliminate_hidden_types_(T,TNoGLB),
     union(HNoGLB,TNoGLB,Res1),
%     write(Res1),nl,
     
     keep_only_most_general_types(Res1,Res1,Res),
%     write('from the union '),
     %write_type_list(Res1),
%     write(Res1),
%     write('I only keep the following most general types: '), write_type_list(Res), nl,
     
%     write('* '),
%     write(Res),nl,nl,
     true.

% --- set union 
   
union([],X,X).
union([X|R],Y,Z) :- member(X,Y), !, union(R,Y,Z).
union([X|R],Y,[X|Z]) :- union(R,Y,Z).

keep_only_most_general_types([],_,[]).
% if there is a supertype of H in the list of all types, ignore H
% we have to restrict it to the case were H does not introduce
% subtype information. Otherwise it will be deleted in the following case:
%     pos 
%       agr_pos
%       verb
%       prep
%     agr_pos agr:agr
%       det
%       noun          

% Such a signiture should not be specified, but humans do this and Trale
% does not warn them about this.

%keep_only_most_general_types([H-[]|T],AllTypes,Res) :-
%        member(X-_,AllTypes),
%        sub_type(X,H),!,
%        keep_only_most_general_types(T,AllTypes,Res).

% I nevertheless take this version, since I work with
% LKB generated signatures and they do not contain the
% above redundancies.
keep_only_most_general_types([H-_|T],AllTypes,Res) :-
        member(X-_,AllTypes),
        X \== H,
        sub_type(X,H),!,
        keep_only_most_general_types(T,AllTypes,Res).

/*

This should not arise, since we expanded all multi/1 types.

% if there is a duplicate of H in the list
% the member in AllTypes contains information about subtypes, i.e.
% is more specific.


keep_only_most_general_types([H-[]|T],AllTypes,Res) :-
        member(H-[_|_],AllTypes),!,
        keep_only_most_general_types(T,AllTypes,Res).

*/

% there is one element in the reminder of the list
% that has empty or identical subtype list -> ignore it.
% This is like the classical remove_duplicates
keep_only_most_general_types([H-Subtypes|T],AllTypes,Res) :-
        member(H-TSubtypes,T),
        ( Subtypes = []
        ; Subtypes = TSubtypes
        ),!,
        keep_only_most_general_types(T,AllTypes,Res).

keep_only_most_general_types([H|T],AllTypes,[H|Res]) :-            
        keep_only_most_general_types(T,AllTypes,Res).


% This predicate has to be defined by the user.
% for instance the following hides all types that
% have the prefix 'glb'
% hide_in_type_hierarchy(Type) :-
%         atom_codes(Type,[103,108,98|_]).



remove_duplicate_subtypes([],[]).
remove_duplicate_subtypes([H-[]|T],[H-[]|Result]) :- !,
        remove_duplicate_subtypes(T,Result).

remove_duplicate_subtypes([Type-SubTypes|T],[Type-SubTypesOut|Result]) :-
        ( known_type(Type) ->
            SubTypesOut = []
        ; assert(known_type(Type)),
            remove_duplicate_subtypes(SubTypes,SubTypesOut)
        ),

        remove_duplicate_subtypes(T,Result).



write_type_list([]).
write_type_list([Type-_|Tail]) :-
        write(Type),
        write(' '),
        write_type_list(Tail).

