/*  $Id$

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of the <code>GStructureData</code> class.

    @author Holger Wunsch */

#include <stdlib.h>

#include "config.h"
#include "gstructuredata.h"

GStructureData::GStructureData(const QString &s) {
  name=s.copy();

  data = new QList<GenericNode>;
  Reent = new QList<ReentNode>;
  rel = new QList<RelationNode>;

  tree = NULL;
  
  map = NULL;

  showDefault = 0;
}

GStructureData::~GStructureData() {
  delete data;
  delete Reent;
  delete rel;

  if (map != NULL)
    destroyMap(map);
}

void GStructureData::setData(List* newData, NodeIdMap* m) {
  while (newData) {
    if (newData->item->nodetype == NODETYPE_REENT_NODE)
      Reent->append((ReentNode* ) (newData->item));
    else if (newData->item->nodetype == NODETYPE_TREE_NODE)
      tree = (TreeNode*) newData->item;
    else if (newData->item->nodetype == NODETYPE_REL_NODE)
      rel->append((RelationNode*) (newData->item));
    else
      data->append(newData->item);
    newData = newData->tail;
  }
  
  // hack: if there is no real structure data, use relation data
  if (data->count() == 0 && rel->count() > 0) {
    data->append(rel->first());
    rel->remove();
  }

  map = m;
}

ReentNode* GStructureData::findReentrancy(int ref) {
  ReentNode* nd;

  nd = Reent->first();

  while (nd) {
    if (nd->ref == ref)
      return nd;
    nd = Reent->next();
  }

  return NULL;
}

GenericNode* GStructureData::findNodeById(char* id) {
  return ::findNodeById(map, id);
}

void GStructureData::write(FILE* file) {
  switch (showDefault) {
  case 0:
    fprintf(file, "!newdata \"%s\" ", (const char*) (name.local8Bit()));
    break;
  case DRAW_TREE:
    fprintf(file, "!newdata(tree) \"%s\" ", (const char*) (name.local8Bit()));
    break;
  case DRAW_STRUC:
    fprintf(file, "!newdata(structure) \"%s\" ", (const char*) (name.local8Bit()));
    break;
  }

  for (int i = 0; i < data->count(); i++)
    writeGenericNode(file, data->at(i));

  ReentNode* nd = Reent->first();

  while (nd) {
    writeGenericNode(file, (GenericNode*) nd);
    nd = Reent->next();
  }

  if (tree)
    writeGenericNode(file, (GenericNode*) tree);

  RelationNode* rnd = rel->first();
  while (rnd) {
    writeGenericNode(file, (GenericNode*) rnd);
    rnd = rel->next();
  }
}

QString GStructureData::getName()
{
  return name;
}


/** Returns the highest node ID in the data.
    @return the highest node ID. */
unsigned int GStructureData::maxID(void) {
  return map->maxID;
}
