/*  $Id$

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file

   Implementation of the GSVGWidget class.

   @author Holger Wunsch */


#include <qpainter.h>
#include <qpicture.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qprinter.h>

#include "gsvgwidget.h"


/** Creates a new widget that can display SVG graphics. 
    @param dev the IO device that the widget will read SVG data from.
    @param parent the parent widget. */
GSVGWidget::GSVGWidget(QIODevice* dev, QWidget* parent):
  QScrollView(parent) {
  svgImage = new QImage();

  svgPicture = new QPicture;
  if (svgPicture->load(dev, "svg")) {
    viewport()->setEraseColor(white);
  
    scale(50);
  }
  else {
    QMessageBox::information(this, "Error", "Invalid data format.");
    emit parent->close();
  }
}


/** Destroys this instance. */
GSVGWidget::~GSVGWidget(void) {
  delete svgImage;
  delete svgPicture;
}


/** Scales the bitmap representation of the SVG picture.
    @param sizePercent the new size in percent of the original size. */
void GSVGWidget::scale(int sizePercent) {
  QRect r = svgPicture->boundingRect();
  QPixmap pm(r.right() - r.left() + 1, r.bottom() - r.top() + 1);
  pm.fill();

  QPainter painter(&pm);
  painter.drawPicture(*svgPicture);

  *svgImage = pm.convertToImage();
  QImage* scaledImg = new QImage;

  int width;
  int height;

  width = (r.right() - r.left() + 1) * sizePercent / 100;
  height = (r.bottom() - r.top() + 1) * sizePercent / 100;

  *scaledImg = svgImage->smoothScale(width, height);
  delete svgImage;
  svgImage = scaledImg;

  viewport()->repaint();

  resizeContents(width, height);
}


/** Draws the bitmap representation of the SVG picture.
    @param p the painter object to be used.
    @param clipx the left boundary of the clipping region.
    @param clipy the top boundary of the clipping region.
    @param clipw the width of the clipping region.
    @param cliph the height of the clipping region. */
void GSVGWidget::drawContents(QPainter* p, int clipx, int clipy, int clipw, int cliph) {
  QPixmap pm(svgImage->width(), svgImage->height());
  QPainter painter(&pm);
  painter.drawImage(0, 0, *svgImage);

  p->drawPixmap(0, 0, pm);
}


/** Prints the SVG picture. */
void GSVGWidget::print(void) {
}
