/*  $Id$

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file

   Definition of the GSVGWidget class.

   @author Holger Wunsch */


#ifndef __GSVGWIDGET_H
#define __GSVGWIDGET_H

#include <qscrollview.h>
#include <qpicture.h>
#include <qiodevice.h>


/** This class provides a scrollable widget that displays an SVG picture.
    The picture can be scaled within a certain range.

    @author Holger Wunsch */
class GSVGWidget: public QScrollView {
  /** The SVG picture. */
  QPicture* svgPicture;


  /** A bitmap representation of the SVG picture. This bitmap can be scaled. */
  QImage* svgImage;


 public:
  /** Creates a new widget that can display SVG graphics. 
      @param dev the IO device that the widget will read SVG data from.
      @param parent the parent widget. */
  GSVGWidget(QIODevice* dev, QWidget* parent = 0);
  
  
  /** Destroys this instance. */
  virtual ~GSVGWidget(void);


  /** Scales the bitmap representation of the SVG picture.
      @param sizePercent the new size in percent of the original size. */
  void scale(int sizePercent);


  /** Prints the SVG picture. */
  void print(void);


 protected:
  /** Draws the bitmap representation of the SVG picture.
      @param p the painter object to be used.
      @param clipx the left boundary of the clipping region.
      @param clipy the top boundary of the clipping region.
      @param clipw the width of the clipping region.
      @param cliph the height of the clipping region. */
  virtual void drawContents(QPainter* p, int clipx, int clipy, int clipw, int cliph);
};

#endif
