/*  $Id$

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    This file contains the definitions of several application-wide available functions.

    @author Holger Wunsch */


#ifndef __TOOLS_H
#define __TOOLS_H


#include <qstring.h>
#include <qwidget.h>

#include "global.h"

#if (KDE_VERSION >= 210)
#if (QT_VERSION >= 0x030000)

#include <qpixmap.h>
#include <qiconset.h>
#include <ktoolbar.h>


/** Creates a grayscale version of the normal pixmap in a QIconSet and adds this
    grayscale pixmap to the QIconSet as disabled.

    Remark: Not supported on KDE 2.0.x, where this function evaluates to an empty
    function.

    @param iconSet the icon set.
    @param backgroundColor the background color of the widget the icon set is
    displayed on. */
void setGrayscalePixmap(QIconSet& iconSet, const QColor& backgroundColor);


/** This function creates a grayscale version of an icon given its color pixmap 
    and assigns it to a disabled toolbar button.

    By default, KToolBar displays pixmaps on disabled buttons as solid gray shapes
    (all opaque pixels are set to gray). With this function, real grayscale icons
    can be used instead.

    Remark: Not supported on KDE 2.0.x, where this function evaluates to an empty
    function.

    @param toolBar the toolbar that contains the button
    @param pixmap the color pixmap of the button
    @param id the button ID */
void setGrayscaleToolbarButton(KToolBar* toolbar, QPixmap& pixmap, int id);

#endif // QT_VERSION
#endif // KDE_VERSION


/** Presents a "Save As" dialog to the user.
    @param parent the parent widget.
    @param caption the dialog's caption.
    @param existCheck if <code>true</code>, the dialog will show a warning dialog
      if a file with the selected name already exists.
    @param filter the filter string (see <code>QFileDialog</code>).
    @param kdeFilter the filter string for a KDE file dialog (see <code>KFileDialog</code>).
      This argument may be <code>QString::null</code>. 
    @return the filename the user selected, or <code>QString::null</code> if
      the user cancelled the "Save As" dialog or the warning dialog about an
      existing file. */
QString getSaveFilename(QWidget* parent, const QString& caption, bool existCheck, 
			const QString& filter, const QString& kdeFilter = QString::null);


/**
 * This function returns a temporary file name based on
 * the user ID, the current date and time, and the current
 * pid.
 * @return the temporary name, a pointer to a static buffer.
 */
const char* getTempName(void);


#endif
