:- use_module(library(assoc),[empty_assoc/1,put_assoc/4,get_assoc/5]).
:- use_module(library(ugraphs),[vertices_edges_to_ugraph/3,vertices/2,del_vertices/3]).
:- use_module(library(lists),[prefix/2]).
% Bug corrected: CHR sometimes leaves dead constraints in suspension.  Must check
%  the status of the mutable in the second argument to determine whether one is still
%  active.
% 7/12/07 - G. Penn

% Adapted to new featural pivot/root interpretation.
% 8/20/08 - G. Penn

:- op(1100,fx,srec).

build_sem_residue([],[],[]).
build_sem_residue([ResGoal|Residue],SemResidue,Rest) :-
  ResGoal = _-Goal,
  build_sem_residue_act(Goal,ResGoal,Residue,SemResidue,Rest).

build_sem_residue_act(semres(v(_BitMask,Nodes,Lits,Sts,Ists,StStars,Anchors,SemVars,SVUpdates,Contrib,
                               ContribLists,FD,_,_,_,_,_,_,_,_,_,_,_,_)),
		      _ResGoal,Residue,SemResidue,Rest) :-

  nonvar(Anchors),
  !,
  ( var(Nodes) -> SemResidue = SemResNodes ; append(Nodes,SemResNodes,SemResidue) ),
  ( var(Lits) -> SemResNodes = SemResLits ; append(Lits,SemResLits,SemResNodes) ),  
  ( var(Sts) -> SemResLits = SemResSts ; append(Sts,SemResSts,SemResLits) ),  
  ( var(Ists) -> SemResSts = SemResIsts ; append(Ists,SemResIsts,SemResSts) ),  
  ( var(StStars) -> SemResStStars = SemResIsts ; append(StStars,SemResStStars,SemResIsts) ),  
  append(Anchors,SemResAnchors,SemResStStars),
  ( var(SemVars) -> SemResSemVars = SemResAnchors ; append(SemVars,SemResSemVars,SemResAnchors) ),
  ( var(SVUpdates) -> SemResSVUpdates = SemResSemVars ; append(SVUpdates,SemResSVUpdates,SemResSemVars) ),
  ( var(Contrib) -> SemResContrib = SemResSVUpdates ; append(Contrib,SemResContrib,SemResSVUpdates) ),  
%  ( var(NegContrib) -> SemResNegContrib = SemResContrib ; append(NegContrib,SemResNegContrib,SemResContrib)),
  ( var(ContribLists) -> SemResContribLists = SemResContrib ; append(ContribLists,SemResContribLists,SemResContrib) ),  
%  ( var(NegContribLists) -> SemResNegContribLists = SemResContribLists ; append(NegContribLists,SemResNegContribLists,SemResContribLists)),
  ( var(FD) -> SemResFD = SemResContribLists ; append(FD,SemResFD,SemResContribLists) ),
	  build_sem_residue(Residue,SemResFD,Rest).
build_sem_residue_act(_,ResGoal,Residue,SemResidue,[ResGoal|Rest]) :-
  build_sem_residue(Residue,SemResidue,Rest).

% srec(Words,X,Template), e.g. Template = (lf:X)
srec(Words) :-
  nl, write('STRING: '),
  nl, number_display(Words,0),
  ttynl, 
  \+ \+ (rec(Words,FS,@sem_residuate(LF,LFTerm,Pivot,Root),Residue,Index),
	 ((current_predicate(portray_cat,portray_cat(_,_,_,_,_)),  % see also gen/1 - portray_cat/5
	   portray_cat(Words,bot,FS,Residue,Index)) -> true         %  can be called with var 1st arg.
	 ; nl, write('CATEGORY: '),nl, ttyflush,
	   pp_fs_res(FS,Residue), nl,
  	   build_iqs_restore_atts(Residue,_Iqs,Residue0),
	   build_sem_residue(Residue0,SemResidue,_FSResidue),
           pp_sem(LFTerm,SemResidue,LF,Pivot,Root,SemString),
	   name(Semantics,SemString),
	   write(Semantics),nl, nl, ttyflush
         ),
	 query_proceed).
	    
pp_sem(LFTerm,SemResidue,LF,Pivot,Root,SemString) :-
	retractall(semvarnum(_)), retractall(metavarnum(_)),
	 assert(semvarnum(0)), assert(metavarnum(0)),
	 functor(LFTerm,sem,7), arg(2,LFTerm,1), % mark root node as to be printed
         print_triage(SemResidue,LFTerm,LFNodes,LF,Pivot,Root,ContribLists),
         sort(LFNodes,_LFTops),  % this eliminates duplicates and puts nodes with var first argument first
	 empty_assoc(EAssoc),
         sem_print(LFTerm,EAssoc,_VarNames,SemString,[],ContribLists).
% KNOWN BUG: we're throwing away LFTops - don't know where to print it in GUI

%pp_sem(LFTerm,LRSResidue,LF) :-
%	 retractall(semvarnum(_)), retractall(metavarnum(_)),
%	 assert(semvarnum(0)), assert(metavarnum(0)),
%	 functor(LFTerm,sem,7), arg(2,LFTerm,1), % mark root node as to be printed
%         print_triage(LRSResidue,LFTerm,LFNodes,LF),
%         sort(LFNodes,LFTops),  % this eliminates duplicates and puts nodes with var first argument first
%	 empty_assoc(EAssoc),
%         sem_print_list([LFTerm|LFTops],EAssoc,_VarNames).
%	 nl,lrs_print_types([LFTerm|LFTops],LRSResidue,VarNames), % DEBUG
%	 dump_stgraph(STGraph,VarNames), % DEBUG
%	 nl,nl,dump_stgraph_act(ElNodes), % DEBUG
%    	 dump_residue(LRSResidue,VarNames),


print_triage(LRSResidue,LFTerm,LFNodes,LFRef,Pivot,Root, ContribLists) :-
  print_triage(LRSResidue,LFTerm,[],LFNodes,LFRef,Pivot,Root, ContribLists).

print_triage([],_,Ns,Ns,_,_,_,[]).
print_triage([Suspension|LRSResidue],Term,NsIn,NsOut,LFRef,Pivot,Root, ContribLists) :-
  arg(2,Suspension,Mref),
  ( get_mutable(active,Mref) -> arg(3,Suspension,subterm:C),
                                print_triage(C,LRSResidue,Term,NsIn,NsOut,LFRef,Pivot,Root, ContribLists)
  ; print_triage(LRSResidue,Term,NsIn,NsOut,LFRef,Pivot,Root, ContribLists)
  ).

print_triage('ststar/3_1'(M,N,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  !,functor(M,sem,7), functor(N,sem,7),
  ( N == Term -> NsMid = [M|NsIn], arg(2,M,1)
  ; NsMid = NsIn
  ),
  print_triage(Rest,Term,NsMid,NsOut,Ref,Pivot,Root,ContribLists).
print_triage('ist/4_1'(M,N,A,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  !, functor(M,sem,7), functor(N,sem,7),
  arg(1,M,IsSubTerm), arg(2,N,IsTerm), arg(6,N,NArgs),
  when(nonvar(IsTerm), IsSubTerm = 1),
  when(nonvar(NArgs), nth(A,NArgs,M)),
  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists).
print_triage('st/3_1'(M,N,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  !, functor(M,sem,7), functor(N,sem,7),
  arg(1,M,IsSubTerm), arg(2,N,IsTerm), 
  when(nonvar(IsTerm), IsSubTerm = 1),
  arg(7,N,NSubTerms), stderef(NSubTerms,M),
  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists).
print_triage('literal/4_1'(N,Literal,Arity,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  !, functor(N,sem,7),
  arg(3,N,LitToBind), arg(6,N,NArgs),
  bind_literal(Literal,LitToBind),
  length(NArgs,Arity),
  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root,ContribLists).
%print_triage('root/3_1'(X,N,_,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(N,sem,7),
%  ( X == Ref -> arg(4,N,1) ; true),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
%print_triage('pivot/3_1'(X,N,_,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(N,sem,7),	
%  ( X == Ref -> arg(5,N,1) ; true),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
print_triage('contribList/3_1'(X,N,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  !,
  ContribLists=[(X,N)|ContribListsRest], % HACK: unlikely that contribLists was worth this - switch to single contribs
  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root,ContribListsRest).
%print_triage('negcontribList/3_1'(X,N,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists, NegContribLists) :-
%  !,
%  NegContribLists=[(X,N)|NegContribListsRest],
%  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root,ContribLists,NegContribListsRest).
print_triage('node/4_1'(N,_ATypes,_RType,_,_),Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  !, functor(N,sem,7),
  (N == Root -> arg(4,N,1) ; true),
  (N == Pivot -> arg(5,N,1) ; true),
%  write(_Atypes),write(' '),write(_RType),nl,
  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists).
print_triage(_,Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists) :-
  print_triage(Rest,Term,NsIn,NsOut,Ref,Pivot,Root, ContribLists).
%print_triage([],_,Ns,Ns,_).
%print_triage([C|LRSResidue],Term,NsIn,NsOut,LFRef) :-
%  print_triage(C,LRSResidue,Term,NsIn,NsOut,LFRef).

%print_triage(ststar(M,N,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !,functor(M,sem,7), functor(N,sem,7),
%  ( N == Term -> NsMid = [M|NsIn], arg(4,M,1)
%  ; NsMid = NsIn
%  ),
%  print_triage(Rest,Term,NsMid,NsOut,Ref).
%print_triage(ist(M,N,A,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(M,sem,7), functor(N,sem,7),
%  arg(1,M,IsSubTerm), arg(2,N,IsTerm), arg(6,N,NArgs),
%  when(nonvar(IsTerm), IsSubTerm = 1),
%  when(nonvar(NArgs), nth(A,NArgs,M)),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
%print_triage(st(M,N,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(M,sem,7), functor(N,sem,7),
%  arg(1,M,IsSubTerm), arg(2,N,IsTerm), 
%  when(nonvar(IsTerm), IsSubTerm = 1),
%  arg(7,N,NSubTerms), stderef(NSubTerms,M),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
%print_triage(literal(N,Literal,Arity,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(N,sem,7),
%  arg(3,N,LitToBind), arg(6,N,NArgs),
%  bind_literal(Literal,LitToBind),
%  length(NArgs,Arity),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
%print_triage(root(X,N,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(N,sem,7),
%  ( X == Ref -> arg(4,N,1) ; true),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
%print_triage(pivot(X,N,_),Rest,Term,NsIn,NsOut,Ref) :-
%  !, functor(N,sem,7),	
%  ( X == Ref -> arg(5,N,1) ; true),
%  print_triage(Rest,Term,NsIn,NsOut,Ref).
%print_triage(_,Rest,Term,NsIn,NsOut,Ref) :-
%  print_triage(Rest,Term,NsIn,NsOut,Ref).

% Might need this when we start printing LFTops
%sem_print_list([],VarNames,VarNames,[]).
%sem_print_list([Node|Nodes],VarNamesIn,VarNamesOut,SemString) :-
%  arg(1,Node,Printed),
%  ( nonvar(Printed) -> VarNamesMid = VarNamesIn, SemString = SSRest
%  ; sem_print(Node,VarNamesIn,VarNamesMid,SemString,SSRest),
%  ),
%  sem_print_list(Nodes,VarNamesMid,VarNamesOut,SSRest).
writeContribList([],[41]). 
writeContribList([_|Tail],OutString) :-
	OutStr = [91,42,93],
	(Tail=[] -> CommaString = OutStr; append(OutStr,[44],CommaString)),
	writeContribList(Tail,RestStr),
	append(CommaString,RestStr,OutString).

sem_print(Node,VarNamesIn,VarNamesOut,SemString,SemStringRest,ContribLists) :-
  Node = sem(1,_1,Functor,IsRoot,IsPivot,Args,SubTerms),
    % 1st arg means "we printed this" - set here to true
    % 2nd arg means "subterm of LFTerm," i.e., it should be printed - already set
  ( var(IsRoot) -> SemString = SSRoot ; SemString = [94|SSRoot] ), % '^'
  ( var(IsPivot) -> SSRoot = SSPivot ; SSRoot = [123|SSPivot] ), % '{'
  ( var(Functor) -> sem_print_var(Functor,VarNamesIn,VarNamesMid,SSPivot,SSFun)
  ; name(Functor,FunString), append(FunString,SSFun,SSPivot), VarNamesMid = VarNamesIn
  ),
  ( Args = [] -> VarNamesMid2 = VarNamesMid, SSArgs = SSFun
  ; sem_print_args(Args,40,41,VarNamesMid,VarNamesMid2,SSFun,SSArgs,ContribLists) % '(',')'
  ),
  ( var(SubTerms) -> VarNamesOut = VarNamesMid2, SSSubTs = SSArgs
  ; SSArgs = [58|SSSubTs0],  % ':'
    sem_print_args(SubTerms,91,93,VarNamesMid2,VarNamesOut,SSSubTs0,SSSubTs,ContribLists)  % '[',']'
  ),
  %( var(IsPivot) -> SSSubTs = SemStringRest; SSSubTs = [125|SemStringRest] ). % '}'
    ( var(IsPivot) -> SSSubTs = SSContribs ; SSSubTs = [125|SSContribs] ), % '}'
    contrib_member(Node,ContribLists,ContribList),
%    contrib_member(Node,NegContribLists,NegContribList),
    (ContribList = [] -> SSContribs = SemStringRest
    ; SSContribs = [47|SSContrib], % '/'
      writeContribList(ContribList,ContribString),
      ContribStringHead=[40|ContribString],
      append(ContribStringHead,SemStringRest,SSContrib)
    ).
%    ( NegContribList = [] -> SSNegContrib  = 
%    ;(
%	    writeContribList(NegContribList,NegContribString), 
%	    NegContribStringHead=[126,40|NegContribString],	
%	    append(NegContribStringHead,SemStringRest,SSNegContrib))).
    % need to check things out of the residue to see if there was any contribution constraint at all
 
contrib_member(_,[],[]):-!.
contrib_member(N,[(M,List)|Tail],ContribList):-
	N = sem(_,_,FunctorN,IsRootN,IsPivotN,ArgsN,SubTermsN),
	M = sem(_,_,FunctorM,IsRootM,IsPivotM,ArgsM,SubTermsM),
	N == M,List=ContribList,!.
contrib_member(N,[H|T],ContribList):-contrib_member(N,T,ContribList).
	

sem_print_var(Var,VarNamesIn,VarNamesOut,SString,SSRest) :-
%  write(Var),VarNamesOut = VarNamesIn. % DEBUG
  get_assoc(Var,VarNamesIn,NameString) -> append(NameString,SSRest,SString), VarNamesOut = VarNamesIn
  ; retract(metavarnum(N)),
    NewN is N + 1,
    assert(metavarnum(NewN)),
    Letter is 65 + (N rem 26),  % upper-case
    Script is 48 + (N // 26),
    (Script == 48 -> NameString = [Letter]
    ; NameString = [Letter,Script]
    ),
    append(NameString,SSRest,SString),
    put_assoc(Var,VarNamesIn,NameString,VarNamesOut).

sem_print_args([],_Prefix,Suffix,VarNames,VarNames,[Suffix|SString],SString, _) :- !.  % close open tails
sem_print_args([Node|Rest],Prefix,Suffix,VarNamesIn,VarNamesOut,[Prefix|SString],SSRest,ContribLists) :-
  sem_print(Node,VarNamesIn,VarNamesMid,SString,SSMid,ContribLists),
  sem_print_args(Rest,44,Suffix,VarNamesMid,VarNamesOut,SSMid,SSRest,ContribLists).  % ','

%sem_print_list([],VarNames,VarNames).
%sem_print_list([Node|Nodes],VarNamesIn,VarNamesOut) :-
%  arg(1,Node,Printed),
%  ( nonvar(Printed) -> VarNamesMid = VarNamesIn
%   ; sem_print(Node,VarNamesIn,VarNamesMid),nl
%  ),
%  sem_print_list(Nodes,VarNamesMid,VarNamesOut).

%sem_print(Node,VarNamesIn,VarNamesOut) :-
%  Node = sem(1,_1,Functor,IsRoot,IsPivot,Args,SubTerms),
%    % 1st arg means "we printed this" - set here to true
%    % 2nd arg means "subterm of LFTerm," i.e., it should be printed - already set
%  ( var(IsRoot) -> true ; write('^') ),
%  ( var(IsPivot) -> true ; write('{') ),
%  ( var(Functor) -> sem_print_var(Functor,VarNamesIn,VarNamesMid) ; write(Functor), VarNamesMid = VarNamesIn),
%  ( Args = [] -> VarNamesMid2 = VarNamesMid ; sem_print_args(Args,'(',')',VarNamesMid,VarNamesMid2) ),
%  ( var(SubTerms) -> VarNamesOut = VarNamesMid2 ; sem_print_args(SubTerms,':[',']',VarNamesMid2,VarNamesOut) ),
%  ( var(IsPivot) -> true ; write('}') ).

%sem_print_var(Var,VarNamesIn,VarNamesOut) :-
%%  write(Var),VarNamesOut = VarNamesIn. % DEBUG
%  get_assoc(Var,VarNamesIn,Name) -> write(Name), VarNamesOut = VarNamesIn
%  ; retract(metavarnum(N)),
%    NewN is N + 1,
%    assert(metavarnum(NewN)),
%    Letter is 65 + (N rem 26),  % upper-case
%    Script is 48 + (N // 26),
%    (Script == 48 -> name(Name,[Letter])
%    ; name(Name,[Letter,Script])
%    ),
%    write(Name),
%    put_assoc(Var,VarNamesIn,Name,VarNamesOut).

%sem_print_args([],_Prefix,Suffix,VarNames,VarNames) :-
%  !,  % close open tails
%  write(Suffix).
%sem_print_args([Node|Rest],Prefix,Suffix,VarNamesIn,VarNamesOut) :-
%  write(Prefix),
%  sem_print(Node,VarNamesIn,VarNamesMid),
%  sem_print_args(Rest,',',Suffix,VarNamesMid,VarNamesOut).

stderef(SubTerms,SubTerm) :-
  var(SubTerms),
  !, SubTerms = [SubTerm|_].
stderef([_|SubTerms],SubTerm) :-
  stderef(SubTerms,SubTerm).

bind_literal(var(V),V) :-
  !,( var(V) -> retract(semvarnum(N)),
                NewN is N + 1,
                assert(semvarnum(NewN)),
                Letter is 97 + (N rem 26),  % lower case letters
                Script is 48 + (N // 26),   % weird chars if Script index > 9
                ( Script == 48 -> name(V,[Letter])  % don't print zero subscripts
		; name(V,[Letter,Script])
		)
    ; true
    ).
bind_literal(Lit,Lit).

:- multifile ale_flags/1.
ale_flags([semclose]).

:- multifile ale_flag_msg/3.
ale_flag_msg(semclose,on,'{ALE: CL-LRS will close semantic terms with srec/1}').
ale_flag_msg(semclose,det,'{ALE: CL-LRS will close semantic terms with srec/1 when determinate}').
ale_flag_msg(semclose,off,'{ALE: CL-LRS will not close semantic terms}').

:- multifile flag_modes/2.
flag_modes(semclose,[on,det,off]).

:- assert(ale_flag(semclose,off)). % initialize

% Debugging code ------------------------------------------------------------
sem_print_types([],_,_).
sem_print_types([N|Ns],LRSResidue,VarNames) :-
  member(node(M,ATypes,RType,_)#_,LRSResidue),
  M == N,
  !, sem_print(N,VarNames,_),
  write(':: '), write(ATypes),write('->'),write(RType),nl,
  sem_print_types(Ns,LRSResidue,VarNames).

dump_stgraph([],_).
dump_stgraph([V-Ns|GraphRest],VarNames) :-
  arg(3,V,VFunctor), write(VFunctor),write('-['),
  dump_stgraph_act(Ns),
  dump_stgraph(GraphRest,VarNames).

dump_stgraph_act([]) :- write(']'),nl.
dump_stgraph_act([N|Ns]) :-
  arg(3,N,NFunctor),write(NFunctor),write(','),
  dump_stgraph_act(Ns).

dump_residue([],_).
dump_residue([C#_|LRSResidue],VarNames) :-
  dump_residue_act(C,VarNames),
  dump_residue(LRSResidue,VarNames).

dump_residue_act(ststar(M,N,_),VarNames) :-  % must export ststar/3 in subterm module to call this
  !,write('ststar: '),write_node(M,VarNames), nl, write('        '),write_node(N,VarNames),nl.
dump_residue_act(st(M,N,_),VarNames) :-
  !,write('st: '),write_node(M,VarNames), nl, write('    '),write_node(N,VarNames),nl.
dump_residue_act(ist(M,N,_,_),VarNames) :-
  !,write('ist: '),write_node(M,VarNames), nl, write('     '),write_node(N,VarNames),nl.
dump_residue_act(_,_).

write_node(sem(A,B,C,D,E,_,_),VarNames) :-
  (var(C) -> sem_print_var(C,VarNames,_) ; write(C)),
  (var(A) -> true ; write(' subterm')),
  (var(B) -> true ; write(' print')),
  (var(D) -> true ; write(' root')),
  (var(E) -> true ; write(' pivot')).

