:- module(subterm,[node/4,literal/4,st/3,ist/4,anchor/1,semvars/1,semvars_update/1,
	   findall_constraints/3,  % expanded into code by CHR
	   contrib/3,contribList/3,fd/3,ststar_req/3,ist_req/4,literal_req/4,contrib_req1/3,contrib_req2/3,
           sem_close/1,sem_aggregate/2]).  % negcontrib/3 and negcontribList/3 removed
:- use_module(library(chr)).
:- use_module(library(lists),[nth/3,member/2,memberchk/2]).
:- use_module(library(assoc)).
:- use_module(library(ordsets),[ord_union/3,ord_intersection/3]).

handler subterm.
constraints node/4,literal/4,st/3,ist/4,ststar/3,anchor/1,semvars/1,semvars_update/1,contrib/3,contribList/3,fd/3,ststar_req/3, ist_req/4, literal_req/4, contrib_req1/3, contrib_req2/3, sem_close/1, not_contributed/2, not_connected/2, contributed/2, sem_aggregate/2, ists/3, collect_ists/2.

option(debug_compile,on).

:- discontiguous attribute_goal/2.
attribute_goal(AttVar,semres(Att)) :-
  '$get_attributes'(AttVar,Att,_),
  nonvar(Att).
  
% semvars/1
uniqueness @ semvars(Vs1) # Id \ semvars(Vs2) <=> Vs1 = Vs2 pragma passive(Id).
initialise @ semvars(Vs) ==> var(Vs) | empty_assoc(Vs).

% semvars_update/1
update @ semvars(VsOld) # Id, semvars_update(VsNew) <=> semvars(VsNew)
           pragma passive(Id).

% anchor/1
uniqueness @ anchor(N) # Id \ anchor(M) <=> N=M
          pragma passive(Id).

% node(N,ArgTypes,RType,A):  N maps Args of ArgTypes to RType with anchor A
type_consistency @ node(N,ATypes1,RType1,_) # Id \ node(N,ATypes2,RType2,_)
                   <=> RType1 = RType2, ATypes1 = ATypes2
                               pragma passive(Id).

% root(X,N,A):  The root of X is N, with anchor A
%dereference @ root(X,N,A) <=> nonvar(X) | deref(X,Ref,_),root(Ref,N,A).
%function @ root(X,N,_) # Id \ root(X,M,_) <=> N=M
%          pragma passive(Id).

% pivot(X,N,A): The pivot of X is N, with anchor A
%dereference @ pivot(X,N,A) <=> nonvar(X) | deref(X,Ref,_),pivot(Ref,N,A).
%function @ pivot(X,N,_) # Id \ pivot(X,M,_) <=> N=M
%           pragma passive(Id).

% literal(N,Literal,Arity,A): N has functor Literal/Arity, with Anchor A
consistency @ literal(N,F1,A1,_) # Id \ literal(N,F2,A2,_) <=> F1 = F2, A1 = A2
                    pragma passive(Id).
well_typedness @ node(N,ATypes,_,_) # Id, literal(N,_,A,_) ==> length(ATypes,A)
                       pragma passive(Id).
extensionality @ literal(N,F,A,_) # Id, literal(M,F,A,Anchor) ==> ext_arg(A,N,M,Anchor)
                     pragma passive(Id).

% fd(N,Domain,A): N is one of the literals named in Domain
member @ literal(N,F,A,_) \ fd(N,Domain,_) <=> nonvar(F) | memberchk(F,Domain).
singleton @ literal(N,F,A,_) \ fd(N,[Lit],_) <=> F = Lit.
intersection @ fd(N,Dom1,A) # Id, fd(N,Dom2,_) <=> ord_intersection(Dom1,Dom2,Dom), fd(N,Dom,A) pragma passive(Id).

% ist/4
irreflexivity @ ist(N,N,_,_) ==> fail.
%node_uniqueness @ ist(M,N,A1,_) # Id \ ist(M,N,A2,_) <=> A1 = A2
%                   pragma passive(Id).
%  - metavars can now occur in multiple arguments of the same term
arg_uniqueness @ ist(M1,N,A,_) # Id \ ist(M2,N,A,_) <=> M1 = M2
                  pragma passive(Id).
well_typedness @ node(M,_,MResType,_) # MId, node(N,NArgTypes,_,_) # NId,
                ist(M,N,A,_) ==> nth(A,NArgTypes,MResType)
                         pragma passive(MId),                passive(NId).
unit @ ist(L,M,_,A) ==> ststar(L,M,A).  % was st(L,M,A) - had to add atomicity_ist and
                                        % transitivity_ist below

% st/3
reflexivity @ st(N,N,_) <=> true.
idempotence @ st(M,N,_) # Id \ st(M,N,_) <=> true
           pragma passive(Id).
subsumption @ ist(M,N,_,_) \ st(M,N,_) <=> true.
antisymmetry @ st(M,N,_) # Id, st(N,M,_) <=> M = N  % maybe don't need this - ststar has antisym.
            pragma passive(Id).
sub_antisym @ ist(M,N,_,_), st(N,M,_) <=> M = N.    % maybe don't need this - ststar has antisym.
constant @ literal(N,_,0,_) \ st(M,N,_) <=> M = N.
unit @ st(L,M,A) ==> ststar(L,M,A).


% ststar/3
reflexivity @ ststar(N,N,_) <=> true.  % may need to get rid of this so
            % that we can use ststar/3 on LHS of universally quantified impl's
idempotence @ ststar(M,N,_) # Id \ ststar(M,N,_) <=> true
               pragma passive(Id).
antisymmetry @ ststar(M,N,_), ststar(N,M,_) # Id <=> M=N
                               pragma passive(Id).
% atomicity_ist @ ist(L,M,_,_), ststar(M,N,_), ist(L,N,_,_) ==> fail.
%  - metavars can now occur in multiple arguments of the same term
%  - even if they couldn't the right action would be to equate M=N, not fail.
% atomicity_st @ st(L,M,_), ststar(M,N,_), ist(L,N,_,_) ==> fail.
%  - even if metavars couldn't occur in multiple arguments, this one would
%    have no deterministic consequences
transitivity_st @ st(L,M,A), ststar(M,N,A) ==> ststar(L,N,A).
base_st @ st(L,M,_), ststar(M,N,_) \ st(L,N,_) <=> L \== M | true.
base_ist @ ist(L,M,_,_), ststar(M,N,_) \ st(L,N,_) <=> L \== M | true.
transitivity_ist @ ist(L,M,_,A), ststar(M,N,A) ==> ststar(L,N,A).

%taomod
%negativity @ contrib(X,N,A) # Id, negcontribList(X,List,A) <=> member(N,List) | fail.
%negativity @ negcontrib(X,N,A) # Id, contribList(X,List,A) <=> member(N,List) | fail.

%contrib(X,N,A): N contributes to X, with anchor A
% HACK: poor choice of var names here - also unclear why contributions should be aggregated like this
contrib(N,NewC,_), contrib_req1(N,C,WGoal) # Id ==> copy_with_replacement(WGoal,C,NewC,CopyGoal), call(CopyGoal) pragma passive(Id).
contrib(X,N,A) <=> contribList(X,[N],A). 
contribList(X,List1,A) # Id, contribList(X,List2,A) <=> ord_union(List1,List2,Result), contribList(X,Result,A) pragma passive(Id).

%negcontrib(X,N,A): N does not contribute to X, with anchor A
%negcontrib(X,N,A) <=> negcontribList(X,[N],A).
%negcontribList(X,List1,A) # Id, negcontribList(X,List2,A) <=> ord_union(List1,List2,Result), negcontribList(X,Result,A) pragma passive(Id).

%%unique contribution
%uniqueness @ uniqcontrib(X,N,A) # Id \ uniqcontrib(X,M,A) <=> N=M
%	pragma passive(Id).
%negativity @ uniqcontrib(X,N,A) # Id, negcontribList(X,List,A) <=> member(N,List) | fail.
%negativity @ negcontrib(X,N,A) # Id, uniqcontrib(X,M,A) ==> fail.
%subsumption @ uniqcontrib(X,N,A) # Id \ contribList(X,[N],A) <=> true.

% Semantic term closure
sem_aggregate(_,Anchor) ==> contributed([],Anchor).
sem_aggregate(Term,Anchor), ststar(N,Term,_) ==> not_contributed(N,Anchor).  %, not_connected(Term,N).

sem_aggregate(Term,Anchor), st(_,Term,Anchor) # Id ==> collect_ists(Term,Anchor) pragma passive(Id).
sem_aggregate(Term,Anchor), ststar(N,Term,Anchor) # Id1, st(_,N,Anchor) # Id2 ==> collect_ists(N,Anchor) pragma passive(Id1), passive(Id2).

% HACK: we really shouldn't use CHR for subtree connection.
collect_ists(N,Anchor), ist(M,N,_,Anchor) # Id1, literal(M,MLit,MArity,Anchor) # Id2 
  ==> ists(N,[ist(M,MLit,MArity)],Anchor) pragma passive(Id1), passive(Id2).
collect_ists(N,Anchor), ststar(Q,N,Anchor) # Id3, ist(M,Q,_,Anchor) # Id1, literal(M,MLit,MArity,Anchor) # Id2 
  ==> ists(N,[ist(M,MLit,MArity)],Anchor) pragma passive(Id1), passive(Id2), passive(Id3).

ists(N,Ists1,Anchor), ists(N,Ists2,Anchor) # Id <=> ord_union(Ists1,Ists2,Ists), ists(N,Ists,Anchor) pragma passive(Id).

contribList(N,_,Anchor) # Id1, node(N,ATypes,RType,Anchor) # Id2 \ not_contributed(N,Anchor), contributed(Accum,Anchor) # Id3
  <=> write(ATypes),write(' '),write(RType),nl, sort([contrib(N,ATypes,RType)|Accum],NewAccum), contributed(NewAccum,Anchor)
                                                        pragma passive(Id1), passive(Id2), passive(Id3).
%ist(N,Term,_,_) \ not_connected(Term,N) <=> true.
%ist(N,M,_,_), ststar(M,Term,_) # Id \ not_connected(Term,N) <=> true pragma passive(Id).

sem_close(Anchor), node(N,NATypes,NRType,Anchor) # Id1 \ not_contributed(N,Anchor) # Id2, contributed(Contribs,Anchor) # Id3
  <=> bind_contrib(N,NATypes,NRType,Contribs), sort(Contribs,NewContribs), contributed(NewContribs,Anchor) 
  pragma passive(Id1), passive(Id2), passive(Id3).

sem_close(Anchor), st(N,P,Anchor) # Id1, literal(N,NLit,NArity,Anchor) # Id2, ists(P,PIsts,Anchor) # Id3
   ==> bind_ist(PIsts,N,NLit,NArity) pragma passive(Id1), passive(Id2), passive(Id3).

%node(N,NATypes,NRType,_) # Id1, node(Term,TATypes,TRType,_) # Id2 \ not_connected(Term,N)
%  <=> true & NATypes=TATypes, NRType=TRType | Term=N pragma passive(Id1), passive(Id2).
%node(N,NATypes,NRType,_) # Id1, node(M,MATypes,MRType,_) # Id2, ist(M,Term,_,_) # Id3 \ not_connected(Term,N)
%  <=> true & NATypes=MATypes, NRType=MRType | M=N pragma passive(Id1), passive(Id2), passive(Id3).
%node(N,NATypes,NRType,_) # Id1, node(M,MATypes,MRType,_) # Id2, ist(M,P,_,_) # Id3, ststar(P,Term,_) # Id4 \ not_connected(Term,N)
%  <=> true & NATypes=MATypes, NRType=MRType | M=N pragma passive(Id1), passive(Id2), passive(Id3), passive(Id4).

% sem_close, not_contributed(_,_) ==> fail.
not_connected(_,_) ==> fail.

% Constraints for delaying
ststar_req(M,N1,WGoal), ststar(M,N2,_) ==> copy_with_replacement(WGoal,N1,N2,CopyGoal), call(CopyGoal).
  % We should really be using a wpred here
ist(M,N2,A,_) \ ist_req(M,N1,A,R) <=> N1=N2, R = 1.
literal(N,F,A,_) \ literal_req(N,F,A,R) <=> R = 1.
contrib_req1(N,C,WGoal), contribList(N,CList,_) # Id ==> copy_with_replacements(CList,WGoal,C) pragma passive(Id).
contribList(N,CList,_) \ contrib_req2(N,C,R) <=> memberchk(C,CList), R = 1.

copy_with_replacement(Var,N1,N2,CopyVar) :-
  var(Var),
  !, ( Var == N1 -> CopyVar = N2
     ; CopyVar = Var).
copy_with_replacement(Term,N1,N2,CopyTerm) :-
  functor(Term,Functor,Arity),
  functor(CopyTerm,Functor,Arity),
  copy_with_replacement_act(0,Arity,N1,N2,Term,CopyTerm).

copy_with_replacement_act(A,A,_,_,_,_) :- !.
copy_with_replacement_act(I,A,N1,N2,Term,CopyTerm) :-
  NewI is I + 1,
  arg(NewI,Term,Arg),
  arg(NewI,CopyTerm,CopyArg),
  copy_with_replacement(Arg,N1,N2,CopyArg),
  copy_with_replacement(NewI,A,N1,N2,Term,CopyTerm).

copy_with_replacements([],_WGoal,_C).
copy_with_replacements([C1|CList],WGoal,C) :-
  copy_with_replacement(WGoal,C,C1,CopyGoal),
  call(CopyGoal),
  copy_with_replacements(CList,WGoal,C).


bind_contrib(N,NATypes,NRType,Contribs) :-
  type_compatible(Contribs,NATypes,NRType,TypeCompats),  type_compat_dump(NATypes,NRType,TypeCompats),
  length(TypeCompats,Len),
  ( user:ale_flag(semclose,det) -> ( Len == 1 -> member(contrib(N,NATypes,NRType),TypeCompats) ; true)
  ; member(contrib(N,NATypes,NRType),TypeCompats)
  ).

type_compatible([],_,_,[]).
type_compatible([contrib(C,CATypes,CRType)|Contribs],NATypes,NRType,TypeCompats) :-
  unifiable(CATypes,NATypes) -> ( unifiable(CRType,NRType) -> TypeCompats = [contrib(C,CATypes,CRType)|TypeCompatsRest],
                                                              type_compatible(Contribs,NATypes,NRType,TypeCompatsRest)
                                ; type_compatible(Contribs,NATypes,NRType,TypeCompats)
                                )
  ; type_compatible(Contribs,NATypes,NRType,TypeCompats).

bind_ist(PIsts,N,NLit,NArity) :-
  literal_compatible(PIsts,NLit,NArity,LitCompats), lit_compat_dump(NLit,NArity,LitCompats),
  length(LitCompats,Len),
  ( user:ale_flag(semclose,det) -> (Len == 1 -> member(ist(N,NLit,NArity),LitCompats) ; true)
  ; member(ist(N,NLit,NArity),LitCompats)
  ).

literal_compatible([],_,_,[]).
literal_compatible([ist(I,ILit,IArity)|Ists],NLit,NArity,LitCompats) :-
  unifiable(ILit,NLit) -> ( unifiable(IArity,NArity) -> LitCompats = [ist(I,ILit,IArity)|LitCompatsRest],
                                                        literal_compatible(Ists,NLit,NArity,LitCompatsRest)
                          ; literal_compatible(Ists,NLit,NArity,LitCompats)
                          )
  ; literal_compatible(Ists,NLit,NArity,LitCompats).

unifiable(Term1,Term2) :-
  call_residue((copy_term(Term1,Copy1),copy_term(Term2,Copy2)),_),
  \+ \+ Copy1 = Copy2.

%--------------------------------------------------------------------------------
% use this to fork checks for argument identity
type_compat_dump(ATypes,RType,TypeCompats) :-
  write(ATypes-RType),nl,nl,
  type_compat_dump_act(TypeCompats).
type_compat_dump_act([]) :- write('------------'),nl.
type_compat_dump_act([contrib(_,ATypes,RType)|Contribs]) :-
  write(ATypes-RType),nl,
  type_compat_dump_act(Contribs).

lit_compat_dump(Lit,Arity,LitCompats) :-
  write(Lit/Arity),nl,nl,
  lit_compat_dump_act(LitCompats).
lit_compat_dump_act([]) :- write('------------'),nl.
lit_compat_dump_act([ist(_,Lit,Arity)|Ists]) :-
  write(Lit/Arity),nl,
  lit_compat_dump_act(Ists).

ext_arg(A,N,M,Anchor) :-
  functor(E,e,A), when(ground(E),N=M),
  ext_arg_act(0,A,E,N,M,Anchor).

ext_arg_act(A,A,_,_,_,_) :- !.
ext_arg_act(I,A,E,N,M,Anchor) :-
  NewI is I + 1,
  arg(NewI,E,EArg),
  ist(NArg,N,NewI,Anchor),
  ist(MArg,M,NewI,Anchor),
  when(?=(NArg,MArg),(NArg==MArg -> EArg = 1 ; true)),
  ext_arg_act(NewI,A,E,N,M,Anchor).

