%%% ==================================================================
%%% $Id: socket_interface.pl,v 1.6 2005/03/29 22:03:25 stefan Exp $
%%%
%%% Communication over sockets: a prolog client
%%%
%%% Author: Detmar Meurers
%%%         based on his code originally written June 1997 for the
%%%         communication between ConTroll and CUF, and other code
%%%         by Frederik Fouvry.
%%%
%%% ==================================================================

:- use_module(library(sockets),[socket_client_open/3]).

%%% --------------------------------------------------------------- %%

:- dynamic interface/2.

start_client:-
   % does not work in runtime executables
   % current_prolog_flag(argv,[Hostname,Atom]),
   (
      interface(Hostname,Atom)
      ;
      (environ('INTERFACE_HOST', Hostname), environ('INTERFACE_PORT', Atom))
   ),
   
   % two calls to name/2 to turn atom '234' into number 234
   % but leaving file socket names '/tmp/...' unchanged
   name(Atom,Codes),		
   name(PortOrFilename,Codes),
   start_client(Hostname,PortOrFilename).

% start_client(+PortOrFilename)
% e.g., start_client(5002) or start_client('/tmp/fileXYZ').
start_client(PortOrFilename) :-
   start_client(localhost,PortOrFilename).

% start_client(+Hostname,+PortOrFilename)
% e.g., start_client(horatio,5002) or start_client(localhost,'/tmp/fileXYZ')
start_client(Hostname,PortOrFilename) :-
   (  number(PortOrFilename)
   -> start_client_on_port(Hostname,PortOrFilename)
   ;  start_client_on_filesocket(PortOrFilename) % always local, ignores Hostname
   ).
  
stop_client:-
   (  (retract(socket_info(_,Stream)),
       catch(close(Stream),  % SP4: must close stream, not socket
	     system_error('close/1: ''close'' failed: Bad file number'),
	     print_message(warning,trale(socket(disconnect,failed)))))
   ->  print_message(informational,trale('Client stopped'))
   ;   print_message(warning,trale('No client information found.'))
   ).

%%% --------------------------------------------------------------- %%
% auxiliary predicates:

:- dynamic socket_info/2.
:- volatile socket_info/2.

start_client_on_port(Hostname,Port) :-
   format('Establishing connection to interface on port ~p of ~p.',[Port,Hostname]),
   catch(socket_client_open(inet(Hostname,Port),Stream,[type(text)]),
         system_error('socket_connect/3: ''connect'' failed: Connection refused'),
        (format(user_error,'~N**Error: Connection refused to port ~w of host ~w.~n',[Port,Hostname]),
         fail)
        ),
   asserta(socket_info(_,Stream)), % SP4: socket variable no longer bound
   format('~NConnection established.~n',[]).

start_client_on_filesocket(Filename) :-
   format('Establishing connection to interface on local filesocket ~p.',[Filename]),
   (  name(Filename,[47|_])	% filesocket names start with a /
   -> true
   ;  format(user_error,'~N**Error: File socket name ~p supplied is not an absolute path (i.e., starting with / ).~N',[Filename]),
      abort ),
   catch(socket_client_open(unix(Filename),Stream,[type(text)]),
         system_error('socket_connect/3: ''connect'' failed: Connection refused'),
        (format(user_error,'~N**Error: Connection failed to local filesocket ~w.~n',[Filename]),
         fail)
        ),
   asserta(socket_info(_,Stream)), % SP4: socket variable no longer bound
   format('~NConnection established.~n',[]).
