:- multifile alec_code_hook/2.
alec_code_hook(Code, Phase) :-
    save_alec_code(Code, Phase).

save_alec_code(Code, Phase) :-
    open('_alec_code.pl', append, S),
    format(S, "% alec phase: ~w~n", [Phase]),
    save_alec_code_act(Code, S),
    nl(S),
    close(S).

save_alec_code_act([], _).
save_alec_code_act([end_of_file|_], S) :- !,
    write(S, '% end_of_file'), nl(S).
save_alec_code_act([Clause|T], S) :-
    save_clause(S, Clause),
    save_alec_code_act(T, S).

save_clause(S, Clause0) :-
    copy_term(Clause0, Clause1, Goals),
    (   Goals == []
    ->  Out = Clause0
    ;   Out = (:- call_list(Goals), assertz(Clause1))
    ),
    format(S, "~k.~n", [Out]).

% called before the grammar is compiled
:- multifile abolish_user_preds/1.
abolish_user_preds(all) :-
   open('_alec_code.pl', write, FS),
   close(FS).


% called after the grammar has been compiled
:- multifile compile_grammar_hook/0.
compile_grammar_hook :-
    (   member(Goal, [  %sub(A,B),
                        fcolour(A,B,C),
                        fun_spec(A,B,C),
                        marity(A,B),
                        type_num(A,B),
                        introduce(A,B)
                      ]),
        findall(Goal, Goal, Code0),
        (   Code0 == []
        ->  Code = [(Goal :- fail)]
        ;   Code = Code0
        ),
        save_alec_code(Code, Goal),
        fail
    
    ;   findall(
            ale_instance(Ref, (Head :- true)),
            (   fspal_ref(_Source, Ref),
                clause(Head, Body, Ref),
                call(Body)
            ),
            Code
        ),
        save_alec_code(Code, fspal)
    ).
