%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: ale_redefinitions.pl,v $
%%  $Revision: 1.18 $
%%      $Date: 2005/02/13 10:33:20 $
%%     Author: Stefan Mueller (Stefan.Mueller@uni-bremen.de)
%%    Purpose: redefinitions of some ALE stuff
%%             needed to have pointers to solutions
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: ale_redefinitions.pl,v $
%% Revision 1.18  2005/02/13 10:33:20  stefan
%% *** empty log message ***
%%
%% Revision 1.17  2003/10/10 16:36:03  stefan
%% fixed a backtracking bug
%%
%% Revision 1.16  2003/10/08 20:40:33  stefan
%% reverse_count_lex_check is now back to ale_redefinitions since it is
%% needed in both [incr TSDB()] and in the chart_display interactive level.
%%
%% ale.pl causes an exception which is too strong. It breaks the interactive level.
%% This code records all missing words, produces a warning and fails.
%%
%% Revision 1.15  2003/08/13 12:06:31  stefan
%% fixed edge_limit check and exception catch in trale-itsdb and ale_redefinitions check and exception catch in trale-itsdb and ale_redefinition
%%
%% Revision 1.14  2003/08/09 15:53:07  dm
%% aufgeraeumt und modularisiert
%%
%% Revision 1.13  2003/08/09 14:42:41  dm
%% Removed temporary redefinition of reverse_count_lex_check/5. We now
%% have the redefinition from trale_patches.pl as part of the
%% redefinitions that are used for itsdb as defined in trale-itsdb.pl
%%
%% Revision 1.12  2003/08/06 10:08:05  stefan
%% reverse_... now fails if words are missing
%%
%% Revision 1.11  2003/08/06 10:04:58  stefan
%% reverse_count_lex_check as modified by Gerald did not work
%% for tsdb since I need all missing words. If Gerald insists
%% on throwing exceptions he should throw an exception that
%% informs the user of all missing words.
%%
%% Revision 1.10  2003/06/11 14:40:45  stefan
%% reverse_count -> reverse_count_lex_check
%%
%% Revision 1.9  2003/05/22 09:25:53  stefan
%% fixed multiple error messages in case we reach the edge_limit
%%
%% Revision 1.8  2003/05/19 15:39:21  stefan
%% Added exception handling in case of memory overflow. Error information
%% is asserted that is used by [incr TSDB()].
%%
%% Revision 1.7  2003/05/13 20:06:47  stefan
%% put init_chart_display before reverse_count because otherwise
%% we may have an old chart display open that displays invalid edges
%% after an unsuccessful reverse_count (missing lexical entries).
%%
%% Revision 1.6  2003/05/13 09:10:49  stefan
%% included a (wrong) fix to rule/1 in ale_redefinitions.pl
%%
%% Revision 1.5  2003/05/09 09:21:00  stefan
%% *** empty log message ***
%%
%% Revision 1.4  2003/05/04 17:20:37  stefan
%% removed all_words_known, since the check is done in reverse_count now
%%
%% Revision 1.3  2003/05/02 17:27:11  dm
%% Added init_chart_display_if_on to rec so that we get chart display
%% output with ordinary rec calls
%%
%% Revision 1.2  2003/05/02 16:36:36  dm
%% - eliminated redefinition of rect and get_and_show_solutions
%%   since test_suite_handling.pl now integrates what they did
%% - eliminated all_words_known/1 and error/2 code since
%%   trale_patches.pl now contains a redefined version of
%%   reverse_count taking care of early lookup of words
%%
%% Revision 1.1  2003/05/02 02:07:12  dm
%% added Stefan's chart_display code
%%
%%
%% Revision 1.1  2003/04/14 10:06:13  stefan
%% Initial revision
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- dynamic rule_counter/1.

:- assert(rule_counter(-1)).

% --------------------------------


:- abolish(rec/5).

rec(Ws,FS,Desc,Residue,Time) :-
  clear,
  assert(parsing(Ws)),


  ( current_predicate(lex,lex(_,_))
  -> reverse_count_lex_check(Ws,[],WsRev,0,Length),

     ( current_predicate(rec_hook/1) -> call_all(rec_hook(Length)) ; true),
     CLength is Length - 1,
     functor(Chart,chart,CLength),
     statistics(walltime,_),
     on_exception(Exception,build(WsRev,Length,Chart),build_exn(Exception)),
     statistics(walltime,[_,Time]),
     retract(to_rebuild(Index)),
     call_residue((clause(edge(Index,0,Length,FS,_,_),true),
                   (Desc == bot -> true
		   ; secret_noadderrs_toggle(Mode),
                     add_to(Desc,FS),
                     secret_adderrs_toggle(Mode)
		   )
%		   ,extensionalise(FS)
		  ),Residue),
     assert(solution(Index))
  ; raise_exception(ale(no_lex))
  ).

% -----------------------------------------------------------------------------

:- abolish(edge_assert/6).


edge_assert(Left,Right,FSOut,Dtrs,RuleName,N) :- 
  ale_flag(interp,off),                                      
  !,gen_edgenum(N),
  asserta(to_rebuild(N)),
  asserta(edge(N,Left,Right,FSOut,Dtrs,RuleName)),
  ( current_predicate(edge_assert_hook/5) -> call_all(edge_assert_hook(RuleName,N,Left,Right,Dtrs)) ; true).
%  format('Edge added: Number: ~w, Left: ~w, Right: ~w, Rule: ~w~n',
%         [N,Left,Right,RuleName]), % DEBUG
%  ttyflush. % DEBUG

edge_assert(Left,Right,FSOut,Dtrs,RuleName,N) :-
  !,nl,
  length(Dtrs,ND),
  (ale_flag(residue,show) -> frozen_term(FSOut,Frozen) ; Frozen = []),
  ( print_edge(Left,Right,FSOut,Dtrs,RuleName,ND,Frozen)
  -> gen_edgenum(N),
     asserta(to_rebuild(N)),
     asserta(edge(N,Left,Right,FSOut,Dtrs,RuleName))).





/*

It's better to do the memory check in an exception handler and call
the edgelimit test in gennum directly.

ts_error(Type):-
	( clause(error(_,_), true) -> % There is already an error
		 fail
	; Type == edgecount ->
		 ((current_predicate(edge_limit/1),
                     edge_limit(Max)) -> 
                     num(N),
                     ( N < Max -> true
                     ; assertz(error(edge_limit,'Maximum edge count')),
                       format(user_error,'~N**ERROR: Edge limit ~w reached.~N',[Max]),
                         fail
                     )
                 ; true
                 )
	; Type == memory ->
		 statistics(memory,[Mem,_]),
		 ( Mem < 253000000 -> true % SICStus Prolog maximum = 256000000 (not linux!)
		 ; trimcore,
			assertz(error(memory_limit,'Maximum memory usage')),
                        format(user_error,'~N**ERROR: Maximum memory usage!~N'),
			fail
		 )
	; assertz(error(unknown,'Unkown error')),
	  fail
	).

*/



/*:- abolish(rule/1).

rule(RuleName):-
  call_residue(clause(alec_rule(RuleName,DtrsDesc,_,Moth,_,_),true),_),
%  (RuleName rule Moth ===> DtrsDesc),
  empty_assoc(AssocIn),  
  call_residue((satisfy_dtrs(DtrsDesc,DtrCats,[],Dtrs,gdone),
                (secret_noadderrs
                ; secret_adderrs,
                  fail),
		add_to(Moth,TagMoth,bot),
                CatsOut = [TagMoth-bot|DtrCats],
		extensionalise_list(CatsOut)),Residue),
  \+ \+ (((current_predicate(portray_rule,portray_rule(_,_,_,_)),
	   portray_rule(TagMoth,bot,Dtrs,Residue)) -> true
	 ;
           nl, write('RULE: '), write(RuleName),
           build_iqs(Residue,Iqs,FSResidue),
	   (show_res -> residue_args(FSResidue,ResArgs,CatsOut) ; ResArgs = CatsOut),
           duplicates_list(ResArgs,AssocIn,DupsMid,AssocIn,VisMid,0,NumMid),
  	   duplicates_iqs(Iqs,DupsMid,DupsMid2,VisMid,_,NumMid,_),
           nl, nl, write('MOTHER: '), nl,
           nl, tab(2), pp_fs(TagMoth,bot,DupsMid2,DupsMid3,AssocIn,VisMid2,2,AssocIn,HDMid),
           nl, nl, write('DAUGHTERS/GOALS: '),   
           show_rule_dtrs(Dtrs,DupsMid3,DupsMid4,VisMid2,VisMid3,HDMid,HDMid2),
	   nl,nl, tab(2), pp_iqs(Iqs,DupsMid4,DupsOut,VisMid3,VisOut,2,HDMid2,HDOut),
  	   ((show_res,FSResidue \== [])
  	   -> nl, nl, write('Residue:'), pp_residue(FSResidue,DupsOut,_,VisOut,_,HDOut,_)
           ; true), nl
	 ),
         query_proceed).
*/


% -------------------------------------------
% add unknown word lookup -> fails if unknown words were detected
%                            reports all missing words, asserts them for [incr TSDB()]

% In addition words under consideration are asserted as le/4 (le(Phon,FS,Position))
% If the grammar writer provides a procedure clean/0, lexical entries can be removed before
% parsing. The redefined version of build/5 refers to le/4 instead of lex/2.

/*
:- dynamic error/2,le/4.

:- abolish([reverse_count_lex_check/5]).

:- use_module(library(lists),[memberchk/2]).

reverse_count_lex_check(ListIn,Acc,ListOut,CountIn,CountOut) :-
   retractall(error(word,_)),
   reverse_count_lex_check(ListIn,Acc,ListOut,CountIn,CountOut,[],Errors),
   ( Errors==[] ->
       (lookup(ListIn),  % if no word is missing, lookup all words and store them in le/3.
           current_predicate(clean/0) ->
           clean
       ; true
       )
   ; otherwise -> format(user_error,'~n* ERROR: The following words are unknown: ',[]),
     list_to_atom(Errors,32,ErrorA),
     format(user_error,'~w~n~n',[ErrorA]),
     fail
   ).           % This fails if there are errors,
                % instead of raising an exception
                % with raise_exception(ale(unk_word(Errors))) since
                % processing with itsdb should continue even if words 
                % are missing
   
reverse_count_lex_check([],Xs,Xs,Count,Count,Errors,Errors).
reverse_count_lex_check([X|Xs],Ys,Zs,CountIn,Count,ErrorsIn,ErrorsOut):-
  (  lex(X,_)
  -> ErrorsOut=ErrorsMid
  ;  (  memberchk(X,Ys)
     -> ErrorsOut=ErrorsMid
     ;  ErrorsOut=[X|ErrorsMid],
        assert(error(word,X))
     )
  ),
  CountInPlus1 is CountIn+1,
  reverse_count_lex_check(Xs,[X|Ys],Zs,CountInPlus1,Count,ErrorsIn,ErrorsMid).

lookup(Ws) :-
        retractall(le(_,_,_,_)),
        lookup(Ws,0).

lookup([W|Ws],Pos) :-
  Pos2 is Pos + 1,
  ( lex(W,FS),
      ( current_predicate(clean_lex_item/2) ->
          clean_lex_item(FS,FSClean)
      ; otherwise ->
          FS = FSClean
      ),
      assert(le(W,FSClean,Pos,Pos2)),
      fail
  ; true
  ),
  lookup(Ws,Pos2).

lookup([],_).


% identical to build in ale.pl, except that lexical entries in le/3 are used instead of lex/2.
:- abolish([build/3]).

build([W|Ws],Right,Chart):-
  RightMinus1 is Right - 1,
  (
% empty_cat(N,Right,Tag,SVs,Iqs,_,_),
%    rule(Tag,SVs,Iqs,Right,Right,empty(N,Right))
%  ;
    le(W,FS,RightMinus1,Right), deref(FS,Tag,SVs), % KNOWN BUG: should pass FS rather than Tag-SVs pair
%    lex_goal(_-(a_ W),Tag-SVs),
    add_edge(RightMinus1,Right,Tag,SVs,[],lexicon,Chart)
  ; ( RightMinus1 =:= 0
      -> true
       ; rebuild_edges(Edges),
         arg(RightMinus1,Chart,Edges),
         build(Ws,RightMinus1,Chart)
    )
  ).
%build([],_):-
%  empty_cat(N,0,Tag,SVs,Iqs,_,_),
%  rule(Tag,SVs,Iqs,0,0,empty(N,0)).
build([],_,_).



list_to_string([Last],_,LastS) :-
  name(Last,LastS).

list_to_string([H|T],D,String) :-
  list_to_string(T,D,S1),
  name(H,HS),
  append(HS,[D|S1],String).

list_to_atom(List,Delimiter,Atom) :-
  list_to_string(List,Delimiter,String),
  name(Atom,String).





%% added a window name for 'initial category'
%% list of words is also printed to the emacs buffer.
:- abolish([gen/1]).
gen(Cat) :-
  call_residue((add_to(Cat,Tag,bot),
		frozen_term(Tag,Frozen),
                ((current_predicate(portray_cat,portray_cat(_,_,_,_,_)),
		  portray_cat('Initial Category',Cat,Tag,bot,Frozen)) -> true
		; nl, write('INITIAL CATEGORY: '), nl, ttyflush,
                  pp_fs_res(Tag,bot,Frozen), nl
		),
                gen(Tag,bot,Words)),Residue),
  ((current_predicate(portray_cat,portray_cat(_,_,_,_,_)),
    portray_cat(Words,Cat,Tag,bot,Residue)) -> true,
    nl, write_list(Words),
    flush_output % added St. M.
  ; nl, write('STRING: '),
    nl, write_list(Words),
    \+ \+ (nl, write('FINAL CATEGORY: '),nl, ttyflush,
	   pp_fs_res(Tag,bot,Residue)), nl
  ),
  query_proceed.



*/

