% $Id: lr_utils.pl,v 1.3 2002/08/08 15:17:13 dm Exp $
%
% This file defines some predicates used in the lexical rule compiler
% to interface with the trale/ale system code.

% load the pretty printer for ale descriptions

:- ensure_loaded(trale_home(fs2desc)).

% ================================================
% desc2fs(+Desc:<desc>, -FS:<fs>)
%
% turns an ALE description Desc into feature structure FS

desc2fs(Desc,RefOut-SVsOut) :-
   add_to(Desc,Ref,bot),   
   deref(Ref-bot,RefOut,SVsOut).


% ================================================
% desc_fs2fs(+Desc:<desc>, +FS:<fs>)
%
% Adds an ALE description Desc to the feature structure FS

desc_fs2fs(Desc,RefIn0-SVsIn0) :-
   deref(RefIn0-SVsIn0,RefIn,SVsIn),
   add_to(Desc,RefIn,SVsIn).

% ================================================
% fs2type_feats(+FS:<fs>, -Type:<type>, -Vs:<vs>)
% 
% Returns top-level type Type and feature-value pairs Vs
% for feature structure FS

fs2type_feats(FS,Type,Vs) :-
   deref(FS,_Ref,SVs),
   SVs =.. [Type|Vs].

% ================================================
% path_fs2pathval(+Path:<path>, +FSIn:<fs>, -FSOut:<fs>)
%
% FSOut is the undereferenced value of FSIn at path Path

path_fs2pathval(P,RefIn0-SVsIn0,RefOut-SVsOut) :-
   deref(RefIn0-SVsIn0,RefIn,SVsIn),
   pathval(P,RefIn,SVsIn,RefOut,SVsOut).

% ================================================
% feat_fs2featval(+F:<feat>, +FSIn:<fs>, -FSOut:<fs>)
%
% FSIn's value for feature F is FSOut -- may involve coercion; 
% FSIn is fully dereferenced; FSOut may not be

feat_fs2featval(F,RefIn0-SVsIn0,RefOut-SVsOut) :-
   deref(RefIn0-SVsIn0,RefIn,SVsIn),
   featval(F,SVsIn,RefIn,RefOut-SVsOut).
