%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: print_mrs.pl,v $
%%  $Revision: 1.5 $
%%      $Date: 2003/07/11 06:36:15 $
%%     Author: Stefan Mueller (Stefan.Mueller@uni-bremen.de)
%%    Purpose: Prints an MRS, used by chart_display and itsdb
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: print_mrs.pl,v $
%% Revision 1.5  2003/07/11 06:36:15  stefan
%% Hm, seem to have forgotten to check some of this in.
%% Sorry
%%
%% Revision 1.4  2003/05/20 20:39:00  stefan
%% generalized code for MRS output of MERGE
%%
%% Revision 1.3  2003/05/19 15:28:51  stefan
%% simplified code since I now know that [incr TSDB()] ignores
%% newlines.
%%
%% Revision 1.2  2003/05/14 11:40:55  stefan
%% Added feature ordering for relation output
%% Added error message if H-Cons is not a closed list
%%
%%
%% Revision 1.1  2003/05/13 09:10:50  stefan
%% put MRS printing in a different file
%% included a (wrong) fix to rule/1 in ale_redefinitions.pl
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

/*

according to lkb-2003-10-01/src/mrs/basemrs.lisp the syntax of MRS output should be:

MRS -> [ LTOP INDEX LISZT HCONS ]

LTOP -> top: VAR

INDEX -> index: VAR

LISZT -> liszt: < REL* >

HCONS -> hcons: < QEQ* >

REL -> [ PREDNAME handel: VAR FEATPAIR* ]

FEATPAIR -> FEATNAME: VAR | CFEATNAME: CONSTNAME

VAR -> VARNAME | VARNAME [ VARTYPE EXTRAPAIR* ]

EXTRAPAIR -> PATHNAME: CONSTNAME

QEQ -> VARNAME RELNNAME VARNAME

*/

print_mrs(FS) :-
        (mrs(on) ->
            print_mrs(FS,MRS,[]),
            name(MRSA,MRS),
            write(MRSA)
        ; true
        ).

print_mrs(FS,MRS,MRSRest) :-
        ((cont_path_(CPath),
        
            path_fs2pathval(CPath,FS,CFS),
            retractall(mynum(_)),
            assert(mynum(0)),
            feat_fs2featval(top,CFS,TRef-TSVs),
            inst(TRef-TSVs),
            ind_feat_(IndFeat),
            feat_fs2featval(IndFeat,CFS,Ind),
%            format_to_chars('<~w,',[TRef],MRS,C1),
            format_to_chars('[ top: ~w,',[TRef],MRS,C1),
            write_o(Ind,C1,C2),
%            format_to_chars(',~n{',[],C2,C3),
            format_to_chars('liszt: < ',[],C2,C3),
          
            liszt_path_(LPath),
            path_fs2pathval(LPath,FS,Liszt),

            write_rels(Liszt,C3,C4),
%            format_to_chars('~n}~n{',[],C4,C5),
            format_to_chars('> hcons: <',[],C4,C5),

            hcons_path_(HPath),
            path_fs2pathval(HPath,FS,HCons),
          
            write_hcons(HCons,C5,C6),
%            format_to_chars('~n} >~n~n',[],C6,MRSRest)
            format_to_chars('>~n~n',[],C6,MRSRest)
         ) -> true
        ; otherwise ->
            format_to_chars('MRS output failed.~n',[],MRS,MRSRest),
            assert(error(mrs,'MRS output failed.'))
        ).

inst(O-_) :-
        nonvar(O),!.

inst(H-SVs) :-
        fs2type_feats(H-SVs,handle,_),
        mygennum(Num),
        sformat(H,'h~w',[Num]).

inst(E-SVs) :-
        event_type_(EventT),
        fs2type_feats(E-SVs,Type,_Vs),
        sub_type(EventT,Type),
        mygennum(Num),
        sformat(E,'e~w',[Num]).

inst(I-SVs) :-
        index_type_(IndexT),
        fs2type_feats(I-SVs,Type,_Vs),
        sub_type(IndexT,Type),
        mygennum(Num),
        sformat(I,'x~w',[Num]).


% instantiated handle, event, index
write_o(Ind-_,Chars,CharsRest) :-
        atom(Ind),
%        format_to_chars('~w',[Ind],Chars,CharsRest).
        format_to_chars('index: ~w',[Ind],Chars,CharsRest).

% an atomic value, for instance (a_ karl) as the value of the feature named
write_o(_-(a_ Val),Chars,CharsRest) :-
        format_to_chars('~w',[Val],Chars,CharsRest).

write_o(H-SVs,Chars,CharsRest) :-
        fs2type_feats(H-SVs,handle,_),
        inst(H-handle),
        format_to_chars('~w',[H],Chars,CharsRest).

write_o(IRef-ISVs,Chars,CharsRest) :-
        fs2type_feats(IRef-ISVs,Type,_Vs),
        event_type_(EventT),
        sub_type(EventT,Type),!,
        ( % Stefan's grammar
            feat_fs2featval(temo,IRef-ISVs,_-Temo),
            feat_fs2featval(sort,IRef-ISVs,_-Sort),
            inst(IRef-ISVs),
            format_to_chars('~w:~w:~w',[IRef,Temo,Sort],Chars,CharsRest)
        ; % Merge
            feat_fs2featval(e,IRef-ISVs,E),
            feat_fs2featval(aspect,E,_-Aspect),
            feat_fs2featval(mood,  E,_-Mood),
            feat_fs2featval(tense, E,_-Tense),
            inst(IRef-ISVs),
            format_to_chars('~w:~w:~w:~w',[IRef,Aspect,Mood,Tense],Chars,CharsRest)
        ).


write_o(IRef-ISVs,Chars,CharsRest) :-
        fs2type_feats(IRef-ISVs,Type,_Vs),
        index_type_(IndexT),
        sub_type(IndexT,Type),
        ( % Stefan's grammar
            feat_fs2featval(per,IRef-ISVs,_-Per),
            feat_fs2featval(num,IRef-ISVs,_-Num),
            feat_fs2featval(gen,IRef-ISVs,_-Gen),
            feat_fs2featval(sort,IRef-ISVs,_-Sort),
            inst(IRef-ISVs),
            format_to_chars('~w:~w:~w:~w:~w',[IRef,Per,Num,Gen,Sort],Chars,CharsRest)
        ; % Merge
            feat_fs2featval(png,IRef-ISVs,PNG),
            feat_fs2featval(pn,PNG,_-PN),
            feat_fs2featval(gen,PNG,_-Gen),
            inst(IRef-ISVs),
            format_to_chars('~w:~w:~w',[IRef,PN,Gen],Chars,CharsRest)
        ).

write_o(FS,Chars,Chars) :-
        fs2type_feats(FS,Type,_Vs),
        format(user_error,'* ERROR: Type "~w" is neither "event" nor "index".~n',[Type]),
        assert(error(mrs,'MRS output failed.')).
%,
%        fail.

write_rels(_-list,Chars,Chars) :- !,
        format(user_error,"~n* ERROR: LISZT list is not closed~n~n",[]),
        assert(error(mrs,'MRS: LISZT open.')).

write_rels(_-e_list,Chars,Chars) :- !.

write_rels(Liszt,Chars,CharsRest) :-
        feat_fs2featval(hd,Liszt,Rel),
        write_rel(Rel,Chars,C1),
        feat_fs2featval(tl,Liszt,Rels),
        write_rels(Rels,C1,CharsRest).

write_rel(RRef-RSVs,Chars,CharsRest) :-
        ( RSVs = (a_ _) -> Type = RSVs, FRs = [], Vs = []
        ; RSVs =.. [Type|Vs], approps(Type,FRs,_)
        ),
        build_keyed_feats(FRs,Vs,KeyedFeats),
        select(fval(handel,H-_,handle),KeyedFeats,KeyedFeats0),
        order_features(KeyedFeats0,KeyedFeats1),

        inst(H-handle),
        format_to_chars('~n ~w:',[H],Chars,C1),
        pred_feat_(PredFeat),
        ( select(fval(PredFeat,_-(a_ Pred),_),KeyedFeats1,KeyedFeats2) -> true
        ; KeyedFeats2 = KeyedFeats1
        ),
        ( var(Pred) -> fs2type_feats(RRef-RSVs,Pred,_Vs)
        ; true
        ),
        format_to_chars('~w(',[Pred],C1,C2),
        write_args(KeyedFeats2,C2,C3),
        format_to_chars(')',[],C3,CharsRest).

write_args([],Chars,Chars).
write_args([fval(F,V,_)|T],Chars,CharsRest) :-
        format_to_chars('~w:',[F],Chars,C1),
        write_o(V,C1,C2),
        (T = [_|_] ->
            format_to_chars(',',[],C2,C3)
        ; C3 = C2
        ),
        write_args(T,C3,CharsRest).

write_hcons(_-list,Chars,Chars) :- !,
        format(user_error,"~n* ERROR: H-CONS list is not closed~n~n",[]),
        assert(error(mrs,'MRS: H-CONS open.')).

write_hcons(_-e_list,Chars,Chars) :- !.

write_hcons(HC,Chars,CharsRest) :-
        feat_fs2featval(hd,HC,Rel),
        write_hc(Rel,Chars,C1),
        feat_fs2featval(tl,HC,HCs),
        write_hcons(HCs,C1,CharsRest).

write_hc(Rel,Chars,CharsRest) :-
        feat_fs2featval(outscpd,Rel,Out-OutT), inst(Out-OutT),
        feat_fs2featval(sc_arg,Rel,Sc-ScT),    inst(Sc-ScT),
        fs2type_feats(Rel,Type,_),
        ( (Type = qeq; Type = geq) ->
            format_to_chars('~n ~w >= ~w',[Sc,Out],Chars,CharsRest)
        ; Type = gt ->
            format_to_chars('~n ~w >  ~w',[Sc,Out],Chars,CharsRest)
        ; Type = eq ->
            format_to_chars('~n ~w =  ~w',[Sc,Out],Chars,CharsRest)
        ).




