% -*- prolog -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: sic.pl,v $
%%  $Revision: 1.5 $
%%      $Date: 2004/03/31 17:24:09 $
%%     Author: Stefan Mueller (Stefan.Mueller@uni-bremen.de)
%%    Purpose: some supporting predicates
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- dynamic last_time/1.

last_time(0).

reset_timer :-  retractall(last_time(_)),
                ale_statistics(runtime,_),
                assert(last_time(0)).   % reset Timer

timer(N)    :-  retract(last_time(L)),
                ale_statistics(runtime,[_,X]),
                N is L + X,
                assert(last_time(N)).

:- dynamic last_no_of_gcs/1, last_bytes_freed/1, last_time_spent_in_gc/1.

reset_gc_stat :-
        retractall(last_no_of_gcs(_)),
        retractall(last_bytes_freed(_)),
        retractall(last_time_spent_in_gc(_)),
        ale_statistics(garbage_collection,[GCs,Bytes,TimeSpent]),
        assert(last_no_of_gcs(GCs)),
        assert(last_bytes_freed(Bytes)),
        assert(last_time_spent_in_gc(TimeSpent)).

gc(NGCs,NBytes,NTimeSpent) :-
        retract(last_no_of_gcs(LGCs)),
        retract(last_bytes_freed(LBytes)),
        retract(last_time_spent_in_gc(LTimeSpent)),
        ale_statistics(garbage_collection,[GCs,Bytes,TimeSpent]),
        assert(last_no_of_gcs(GCs)),
        assert(last_bytes_freed(Bytes)),
        assert(last_time_spent_in_gc(TimeSpent)),
        NGCs is GCs - LGCs,
        NBytes is Bytes - LBytes,
        NTimeSpent is TimeSpent - LTimeSpent.
        


:- dynamic max_memory_used/1.
max_memory_used(0).
 
compute_max_memory :-
  memory_used(X),
  retract(max_memory_used(Y)),
  M is max(X,Y),
  assert(max_memory_used(M)),!.
 
reset_max_memory :-
  retractall(max_memory_used(_)),
  assert(max_memory_used(0)).
 
% disabled in runtime systems
memory_used(0) :-
	prolog_flag(system_type,runtime),!,write(runtime).


memory_used(X) :-  
        ale_statistics(heap,[A,_]),
        ale_statistics(local_stack,[B,_]),
        ale_statistics(global_stack,[C,_]),
        ale_statistics(trail,[D,_]),
        ale_statistics(choice,[E,_]),
        X is A + B + C + D + E.

%get_time(X) :-
%        now(X).

get_host(X) :-
        host_name(X).

get_user(X) :-
        environ('USER',X), !.
get_user(X) :-
        environ('USERNAME',X), !.
get_user(user(unknown)).

% cheating a bit with the milliseconds
convert_time(Time,Year,Month,Day,Hour,Minute,Second,0) :-
        datime(Time,datime(Year,Month,Day,Hour,Minute,Second)).


prolog_version_number(V) :-
        prolog_flag(version,V).


% Definition of a SWI command
sformat(Atom,Format,Args) :-
  format_to_chars(Format,Args,String),
  name(Atom,String).


% From The Craft of Prolog, O'Keefe
should_be_determinate(Goal) :-
  ale_statistics(local_stack,[Before|_]),
  call(Goal),
  ale_statistics(local_stack,[After|_]),
  Delta is After-Before,
  ( Delta = 0 -> true
  ; write([Goal,left,Delta,bytes]),nl,
    !
  ).

