%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: utool-interface.pl,v $
%%  $Revision: 1.2 $
%%      $Date: 2006/11/29 10:51:02 $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Communication with the scope resolver/USR display utool
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- use_module(library(xml)).

% Command =
%     solve        Solve an underspecified description.
%     solvable     Check solvability without enumerating solutions.
%     convert      Convert underspecified description from one format to another.
%     classify     Check whether a description belongs to special classes.
%     display      Start the Underspecification Workbench GUI.
%     server       Start Utool in server mode.
%     help         Display help on a command.

% Name = String to be displayed in the GUI

% InCodec = mrs-prolog, ...

% InString = 'psoa(h1,e2,
%   [ rel('prop-or-ques_m',h1,
%         [ attrval('ARG0',e2),
%           attrval('MARG',h3),
%           attrval('PSV',u4),
%           attrval('TPC',u5) ]),
%     rel('_every_q',h6,
%         [ attrval('ARG0',x7),
%           attrval('RSTR',h9),
%           attrval('BODY',h8) ]),
%     rel('_dog_n_1',h10,
%         [ attrval('ARG0',x7) ]),
%     rel('_bark_v_1',h11,
%         [ attrval('ARG0',e2),
%           attrval('ARG1',x7) ]) ],
%   hcons([ qeq(h3,h11), qeq(h9,h10) ]))'


utool_request(Command,Name,InCodec,InString,OutCodec,Result,ErrorMessage) :-
  socket('AF_INET',Socket),
  Hostname=localhost,
  Port=2802,
  catch(socket_connect(Socket,'AF_INET'(Hostname,Port),UStream),
        system_error('socket_connect/3: ''connect'' failed: Connection refused'),
        (format(user_error,'~N**Warning: Connection refused to port ~w of host ~w.~nUtool not running in server mode?~nTrying to start utool. Calling "utool server" ...~n~n',[Port,Hostname]),
            start_utool(Socket,Hostname,Port,UStream))),

  (OutCodec = '' ->
%   format(user_error,'<utool cmd="~w">~n',[Command]),
    format(UStream,   '<utool cmd="~w">~n',[Command])
  ; format(UStream,   '<utool cmd="~w" output-codec="~w">~n',[Command,OutCodec])
  ),

  name(InAtom,InString),
%  format(user_error,'<usr codec="~w" string="~w"/>~n',[InCodec,InAtom]),
  ( (Name = ''
     ;Command=solvable   % This is a workaround to cope with some unicode problem in PVM/ITSDB communication.
                         % The predicates are English and the NAME is not really needed for checks for solvability. 10.03.2007
    ) ->
%      format(user_error,'Name is ''~w''. Not sending it.~n',[Name]),
    format(UStream,   '<usr           codec="~w"',[InCodec])
  ; %write(user_error,'Name is:'),write(user_error,Name),nl(user_error),
    format(UStream,   '<usr name="~w" codec="~w"', [Name,InCodec])
  ),

  format(UStream,   ' codec-options="labelStyle=full',[]),

  ( mrs_normalization(off) ->
      format(UStream,   ',normalisation=none',[])
  ; true
  ),
  

  format(UStream,   '" string="~w"/>~n',[InAtom]),


  % optionally we can pass a set of equivalences, if the file `equivalences.xml' exists
  % in the grammar directory.
  ( absolute_file_name('equivalences.xml',
			_,
			[access(read),file_errors(fail)]) ->
  
      open('equivalences.xml',read,Equi),
      read_chars(Equi,[],EquivalencesString),
      close(Equi),
      transform_xml_characters(EquivalencesString,EquivalencesT_XML),
      format(UStream,   '<eliminate equations="~s" />~n',[EquivalencesT_XML])
  ; true
  ),
%   format(UStream,   '<eliminate equations="&lt;equivalences style=&quot;Chinese-B-Gram&quot;&gt;
%                                            &lt;permutesWithEverything label=&quot;proper_q&quot;  hole=&quot;1&quot; /&gt;;
% 	                                   &lt;permutesWithEverything label=&quot;pronoun_q&quot; hole=&quot;1&quot; /&gt;;
%                                            &lt;/equivalences&gt;" />~n',[]),
  
%  format(user_error,'</utool>~n',[]),flush_output(user_error),
  format(UStream,   '</utool>~n',[]),

  flush_output(UStream),
  %format(user_error,'flushed output stream~n',[]),  
  get_results(UStream,Command,Result,ErrorMessage),
  %format(user_error,'got results ... ~n',[]),  
  close(UStream).
  %format(user_error,'closed stream ~n',[]).



start_utool(Socket,Hostname,Port,UStream) :-
            catch(exec('utool server',[null,null,null],_Pid),
                  system_error('exec/3: ''utool server'' failed'),
                  (format(user_error,'~N**Warning: Starting utool failed.~n',[]),
                       fail)),
            connect_utool(45,Socket,Hostname,Port,UStream).

connect_utool(0,_) :-
        format(user_error,'~N**Warning: Connection to Utool timed out.~n',[]),
        fail.

connect_utool(Sek,Socket,Hostname,Port,UStream) :-
        format(user_error,".",[]),
        sleep(1),
        catch((socket_connect(Socket,'AF_INET'(Hostname,Port),UStream),
               nl),
              system_error('socket_connect/3: ''connect'' failed: Connection refused'),
              (SekN is Sek - 1,
               connect_utool(SekN,Socket,Hostname,Port,UStream))).


get_results(Stream,Command,Result,ExplanationString) :-
        read_chars(Stream,[],Chars),
        xml_parse(Chars,XML),
        %xml_pp(XML),
        
        ( xml_subterm( XML, element(error, Attributes, _Content)) ->
            member(explanation=ExplanationString,Attributes),
            %format(user_error,"~N* ERROR: ~s~n",[ExplanationString]),
            Result = 0
        ;
            ExplanationString = "",
            (Command = solve ->
                ( xml_subterm( XML, element(solution, [string=SolutionString], _Content) ),
                    name(Solution,SolutionString),
                    write(Solution),nl,
                    fail
                ; true )
            
            ; Command = solvable ->
                xml_subterm( XML, element(result, Attributes, _Content) ),
                                % This just prints the number of scopings.
                                % These numbers are provided like the number of residues.
                ( member(solvable="true",Attributes) ->
                    member(count=CountString,Attributes),
                    name(Result,CountString)
                ; format(user_error,"~N* ERROR: The MRS does not scope!~n",[])
                )
            ; % Command = display -> we do not want to return anything
                true
            )
        ).

read_chars(Stream,In,Out) :-
        read_line(Stream,Line),
        (Line \== end_of_file ->
            append(In,Line,New),
            read_chars(Stream,New,Out)
        ; In = Out ).


transform_xml_characters([],[]).
transform_xml_characters([Ci|Ti],Transformed) :-
  xml:character_entity( Ci1,Ci),!,
  transform_xml_characters(Ti,To),
  append([38|Ci1],[59],TransformedCi),
  append(TransformedCi,To,Transformed).

transform_xml_characters([Ci|Ti],[Ci|To]) :-
  transform_xml_characters(Ti,To).

        
:- dynamic mrs_normalization/1.


mrs_normalization   :- retractall(mrs_normalization(_)),assert(mrs_normalization(on)).
nomrs_normalization :- retractall(mrs_normalization(_)),assert(mrs_normalization(off)).

:- mrs_normalization.