% Patches to use ALE SLD with TRALE

%
% SLD Compiler patches
%

% Now partially evaluating debugging hooks in constraint bodies.
% 9/29/02 - G. Penn

% Changed lcons/4 to lcons/5 and trale_lcons/3 to trale_lcons/4 - added label
%  for first line of constraint for command-line spypoint commands.
% 6/5/03 - G. Penn

% Changed lmacro/2 to lmacro/3 - added label for first line of description for
%  command-line spypoint commands.
% 6/6/03 - G. Penn

% Changed subs/2 to type/1 for new signature compiler.
% 8/10/03 - G. Penn

% Bug corrected: label_complex_macrohead/7 was decomposing layouts with =.. instead of =.
% 10/10/03 - G. Penn

% Ported to new data structure.
% 7/20/06 - G. Penn

% Bug corrected: cmv_act/3 not renamed cmv_desc/3.
% 7/25/06 - G. Penn

% Bug corrected: abolish_lpreds/0 had not been updated to reflect the new
% arities of empty_cat/5 and lclosedrule/9.
% 1/26/09 - G. Penn

% RCS banners
% $Id$
%
% $Log$


:- ale_abolish([abolish_lpreds/0,dcompile_cons/0]).

abolish_lpreds :-
  ale_abolish('l--->',3),ale_abolish(lif,2),ale_abolish('l+++>',2),ale_abolish(llr,3),
  ale_abolish(lrule,4),ale_abolish(lmacro,3),ale_abolish(lemptyentry,2), ale_abolish(lcons,5),
  ale_abolish(lsemantics,2), ale_abolish(lclosedrule,9), ale_abolish(empty_cat,5),
  retractall(ale_debugging), retractall(ale_debug(_)), ale_abolish(ale_break,1),
  retractall(complex_lcons(_,_,_,_,_)).  

dcompile_cons :-
%  touch('.alec_throw'),
  absolute_file_name('.alec_throw',AbsFileName),
  retractall(ale_compiling(_)),
  assert(ale_compiling(AbsFileName)),
  dtrale_link_assert,
  dcompile_cons_act,
  retract(ale_compiling(_)).

dtrale_link_assert :-
  findall(Type,type(Type),Types), % maxcount/3 and deranged/2 are already
  compute_lcons(Types).     % there - just re-attach to user constraints

compute_lcons([]).
compute_lcons([Type|Types]) :-
  (retrieve_user_lcons(Type,ConsLabel,LabelledCons,FS,Goal)
  -> ale_assert(lcons(Type,ConsLabel,LabelledCons,FS,Goal))
  ; true
  ),
  compute_lcons(Types).

retrieve_user_lcons(Type,ConsLabel,LabelledCons,FS,Goal) :-
  current_predicate(trale_lcons,trale_lcons(_,_,_,_)),
  trale_lcons(Type,ConsLabel,LabelledCons,TGoal),
  !,findall(lcc(CondVar,Cond,LabelledConseq,CGoal),
            (complex_lcons(Type,Antec,LabelledConseq,CGoal,NarrowVars),
             apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
	    ComplexConss),
  ( ComplexConss == [] -> Goal = TGoal
  ; apply_lccs(ComplexConss,FS,Type,TGoal,Goal)
  ).
retrieve_user_lcons(Type,[],LabelledCons,FS,Goal) :-  % no line label for complex antecedent constraints
	                                              %  because they are anonymous (for now)
  findall(lcc(CondVar,Cond,LabelledConseq,CGoal),
          (complex_lcons(Type,Antec,LabelledConseq,CGoal,NarrowVars),
           apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
          ComplexConss),
  ComplexConss \== [], % otherwise fail in order to add no code
  LabelledCons = ltype(bot,none),
  apply_lccs(ComplexConss,FS,Type,true,Goal).

apply_lccs([],_,_,TGoal,TGoal).
apply_lccs([lcc(FS,Cond,LabelledConseq,CGoal)|CCs],FS,Type,TGoal,Goal) :-
  empty_assoc(AssocIn),
  compile_cond(Cond,prolog((deref(FS,TFS,FType,AttPos),
			    d_add_to(LabelledConseq,FS,FType,TFS,AttPos,bot,cons(Type)),CGoal)),
	       PGoals,[],AssocIn,_,AssocIn,AssocIn,AssocIn),  % there shouldn't be any FSs, NVs or Vars here.
  goal_list_to_seq(PGoals,PGoal),
  apply_lccs(CCs,FS,Type,(TGoal,PGoal),Goal).

% alternatively, the most beautiful line of Prolog ever written:
% apply_lccs(CCs,FS,Type,(TGoal,query_goal(when(Cond,prolog((deref(FS,Tag,SVs),d_add_to(LabelledConseq,Tag,SVs,cons(Type)),CGoal))))),Goal).
%
% but alas, it is too slow...

%
% SLD Parser patches
%

:- multifile label_term/4.
label_term(trale_cons(T,RHS),[_,_,RHSLayout],Va,AbsFile) :-
  !,
  ( nonvar(RHS), RHS = (Cons goal Goal) -> RHSLayout = [_,ConsLayout,GoalLayout],
                                         label_desc(Cons,ConsLayout,LabelledCons,FirstLine,Va,AbsFile),
                                         label_body(Goal,GoalLayout,LabelledGoal,Va,AbsFile),
                                         CGoal = d_query(LabelledGoal)
  ; label_desc(RHS,RHSLayout,LabelledCons,FirstLine,Va,AbsFile),
    CGoal = true
  ),
  ConsLabel = [AbsFile|FirstLine],
  ale_assert(trale_lcons(T,ConsLabel,LabelledCons,CGoal)).
label_term('*>'(Antecedent,RHS),[_,_,RHSLayout],Va,AbsFile) :-
  !,
  find_narrow_vars(Antecedent,AntecNoQs,NarrowVars),
  find_trigger_type(AntecNoQs,bot,Trigger),
  ( nonvar(RHS), RHS = (Consequent goal Goal) -> RHSLayout = [_,ConseqLayout,GoalLayout],
                                                 label_desc(Consequent,ConseqLayout,LabelledConseq,_,Va,AbsFile),
                                                 label_body(Goal,GoalLayout,LabelledGoal,Va,AbsFile),
                                                 ale_assert(complex_lcons(Trigger,AntecNoQs,LabelledConseq,d_query(LabelledGoal),NarrowVars))
  ; label_desc(RHS,RHSLayout,LabelledConseq,_,Va,AbsFile),
    ale_assert(complex_lcons(Trigger,AntecNoQs,LabelledConseq,true,NarrowVars))
  ).
label_term(':='(LVLHS,RHS),[_,LVLHSLayout,RHSLayout],Va,AbsFile) :-
  !,
  label_desc(RHS,RHSLayout,LabelledRHS,FirstLine,Va,AbsFile),
  label_complex_macrohead(LVLHS,LVLHSLayout,MacroHead,LabelledRHS,LabelledMacroDesc,Va,AbsFile),
  ale_assert(lmacro(MacroHead,FirstLine,LabelledMacroDesc)).

label_complex_macrohead(LVLHS,LVLHSLayout,LHS,LRHS,LMD,Va,AbsFile) :-
  functor(LVLHS,MacroName,Arity),
  functor(LHS,MacroName,Arity),
  ( Arity == 0 -> MacroParms = []  % in this case, LVLHSLayout isn't a list, just a number
  ; LVLHSLayout = [_|LVArgLayouts],
    add_lv_lmacro_descs(0,Arity,LVArgLayouts,LVLHS,LHS,MacroParms,Va,AbsFile)
  ),
  cmv_desc(LRHS,MacroParms,LMD).

add_lv_lmacro_descs(A,A,[],_,_,[],_,_) :- !.
add_lv_lmacro_descs(OldP,A,[LVArgLayout|LVArgLayouts],LVLHS,LHS,[LVParm|LVParms],Va,AbsFile) :-
  P is OldP + 1,
  arg(P,LVLHS,LVArg),
  arg(P,LHS,TArg),
  ( var(LVArg) -> LVArg = LVParm, LVParm = (TArg,lvar(_,'_',[]))
  ; LVArg = LVParm-Desc, LVParm = (TArg,LabelledDesc,lvar(_,'_',[])),
    LVArgLayout = [_,_,DescLayout],
    label_desc(Desc,DescLayout,LabelledDesc,_,Va,AbsFile)
  ),
  add_lv_lmacro_descs(P,A,LVArgLayouts,LVLHS,LHS,LVParms,Va,AbsFile).

