;; -*- Mode: Emacs-Lisp; dummy:""; buffer-read-only:t; dummy2:""; -*-

;; [MLA] This file is based on sicstus-support.el. My modifications consist
;; only of renaming ('sicstus -> 'swi) and commenting out the irrelevant
;; functions. The goal was to make emacs react to pltraces when used with 
;; SWI as inferior process. Nothing of the other functionality provided by
;; sicstus-support.pl is in fact needed (for the time being).

;; [PM] 1.43 The strange EMACS_MODE_LINE_BUFFER_READONLY is
;; substituted by configure when generating sicstus-support.el to make
;; it read-only. See configure.in


;; sicstus-support.el -- SICStus specific support for prolog.el
;;; Copyright (C) 1998, 1999 SICS and IQSoft
;;
;; Author: Rozman Tamas, IQSoft
;;         Emil Astrom, SICS <emil@sics.se>
;; Maintainer: Per.Mildner@sics.se
;; Version: 1.43 (for SICStus Prolog 3.12.1)
;; (update sicstus-support-version below)
;;
;; Maintainer: Per.Mildner@sics.se
;;
;; Should work with FSF/GNU Emacs as well as XEmacs.
;;
;; This package depends on being automatically loaded from prolog.el
;; when prolog-system is set to sicstus.
;; 

;; To use this put the following in your ~/.emacs file
;; (load "SICSTUS_EMACS_PATH/sicstus_emacs_init")
;; where SICSTUS_EMACS_PATH is the folder containing this file and
;; sicstus_emacs_init.el
;;
;;
;; Obsolete:
;; To use this put the following in your ~/.emacs file
;; (setq load-path (cons (expand-file-name "/usr/local/lib/sicstus3/emacs") load-path))
;; (autoload 'run-prolog "prolog" "Start a Prolog sub-process." t)
;; (autoload 'prolog-mode "prolog" "Major mode for editing Prolog programs." t)
;; (setq prolog-system 'sicstus) ; This will ensure that sicstus-sup is loaded!
;; (setq auto-mode-alist (cons '("\\.pl$" . prolog-mode) auto-mode-alist))

;; where the path in the first line is the file system path to the
;; directory containing this file as well as prolog.el.
;; MS Windows paths can be written like "c:/sicstus3/emacs".
;;
;; The last line above makes sure that files ending with .pl are
;; assumed to be Prolog files and not Perl, which is the default Emacs
;; setting. If this is not wanted, remove that line. It is then necessary
;; for the user to manually switch to prolog mode by typing
;; M-x prolog-mode after opening a prolog file.
;;
;; If the shell command to start the prolog process ('sicstus') is not
;; available in the default path, then it is necessary to set the
;; value of the environment variable EPROLOG to a shell command to
;; invoke the prolog process.

;;; Version history
;; 1.4beta (with SICStus 3.9b4)

;; . Update for new message syntax (when not
;;   prolog-use-standard-consult-compile-method-flag)
;; . added det, nondet as keywords
;; . Moved some code here that did not belong in prolog.el
;;
;; 1.3 Released (as sicstus-support.el) with SICStus 3.8
;; . Now preserves existing process filter of the inferior prolog
;;   buffer. This is good since it allows composing of filter based
;;   features.
;; . Generalize, clean-up
;; . Better support for XEmacs
;; . Sending commands to SICStus now works when debugger is waiting
;;   for input (SICStus >= 3.8)
;; . C-X SPC now sets and clears line breakpoints using the
;;   programmable debugger interface of SICStus 3.8.
;; . `M-x sicstus-download-new-packages' checks for and downloads new
;;   versions of emacs lisp code and Info files.
;; 
;; 1.2pre (under the name pltrace.el): ?
;; 1.1 (under the name pltrace.el): Released with SICStus Prolog 3#7


(require 'prolog)

;; The overlay stuff is in a separate package under XEmacs. In FSF Emacs
;; this is builtin.
(if (string-match "XEmacs\\|Lucid" emacs-version)
    (require 'overlay))

(eval-when-compile
  (if (string-match "XEmacs\\|Lucid" emacs-version)
      (require 'overlay)))

(defvar swi-program-name "swipl"
  "*Program name for invoking an inferior SWI Prolog with `run-prolog'.")

(defvar swi-program-switches '("-G128m")
  "*Switches given to inferior SWI Prolog run with `run-prolog'.")

(defvar pltrace-mode-map
  (if (boundp 'prolog-mode-map-swi)
      prolog-mode-map-swi
    prolog-mode-map))

;; (define-key pltrace-mode-map [(control c) (space)] 'pltrace-break)

(defvar pltrace-process "prolog"
  "Name of prolog process.")

(defvar pltrace-debug nil
  "*Non-nil to debug the pltrace package.")

(defvar pltrace-cmd-prefix
  "\t"
  "Magic string prefixed to commands sent to prolog.")

(defun pltrace-debug-msg (msg)
  (and pltrace-debug
       (message "PLTRACE: %s" msg)))

(defun pltrace-send-cmd (goal)
  "Send GOAL to prolog. GOAL must not have any terminating period."
  (prolog-ensure-process)
  (let ((cmd (format "%s%s.\n" pltrace-cmd-prefix goal)))
    (pltrace-debug-msg cmd)
    (process-send-string pltrace-process cmd)))


(defun pltrace-break (arg &optional action)
  "Set a break-point at the current source line.
Numeric argument (C-U) removes breakpoint for this line.
Negative argument (C-U -) removes all breakpoints for this file.
When called noninteractively optional ACTION should be one of
set, clear, clear_all in which case ARG is ignored"
;; temp not documented since it is same as set in traceui.
;; set, temp, clear, clear_all
  (interactive "P")
  (let ((file (prolog-bsts buffer-file-name))
        (line (save-restriction
                (widen)
                (beginning-of-line)
                (1+ (count-lines 1 (point))))))
    (or action
        (setq action (cond ((null arg)
                            'set)
                           ((> (prefix-numeric-value arg) 0)
                            'clear)
                           (t 'clear_all))))
    (pltrace-send-cmd (format "prolog:lbp_breakpoint_emacs(%s,'%s',%d)" action file line))))




;; executes a Prolog-trace command
;;
;; the format of a command:
;;
;;   <begin>c,Buffer
;;   Text
;;   <end>
;;        Creates a buffer named Buffer, or empties it if it already
;;        exists; and deposits Text into it.
;;
;;  <begin>f,Port,Line,File<end>
;;        Shows line Line in file File with an overlay corresponding to Port.
;;
;;  <begin>b,Port,Line,Buffer<end>
;;        Shows line Line in buffer Buffer with an overlay corresponding to
;;        Port.
;;
;;  <begin>u<end>
;;        Removes the last highlight made by ..._show
;;
;;  <begin>k,Buffer<end>
;;        Kills buffer named Buffer created by a c (create_buffer) command.
;;

(defvar pltrace-command-alist
  '(
    (?e . pltrace-execute-breakpoint-command)

    ;; 1.2pre
    (?f . pltrace-execute-normal-command-wrapper)
    (?b . pltrace-execute-normal-command-wrapper)
    (?u . pltrace-execute-normal-command-wrapper)
    (?k . pltrace-execute-normal-command-wrapper)
    (?c . pltrace-execute-extended-command)))
  

(defun pltrace-execute-normal-command-wrapper (command-char param-list ignore)
  (pltrace-execute-normal-command command-char param-list))

;; this is called by the process filter pltrace-filter, so
;; proc and global variable pltrace-process is always the same
;; (defvar foo nil)
(defun pltrace-execute-command (proc command)
  (if (string-match "\\(.*\\)\\(\n\\)?" command)
      (let* ((first-line (substring command 0 (match-end 1)))
             (text (and (match-end 2) (substring command (match-end 2))))
             ;; (pos (pltrace-find command "\n"))
             ;; (first (substring command 0 pos))
             ;; (second (substring command pos))
             (param-list (pltrace-split-command first-line))
             (command-char (car param-list))
             (ent (assoc (elt command-char 0) pltrace-command-alist))
             (handler-fun (cdr ent)))
        (if ent
            (progn
              (pltrace-debug-msg (format "%s" (list handler-fun command-char (cdr param-list) text)))
              (pltrace-apply-command handler-fun command-char (cdr param-list) text))))))


(defun pltrace-execute-breakpoint-command (command-char param-list ignore)
  (let* ((what (nth 0 param-list))
         (line (nth 1 param-list))
         (file (nth 2 param-list)))
    (cond ((equal what "lbp_set")
           (pltrace-mark-breakpoint file 'set)
           (message "Breakpoint: line %s@%s" line (file-name-nondirectory file)))
          ((equal what "lbp_temp")
           (pltrace-mark-breakpoint file 'temp)
           (message "Breakpoint: line %s@%s" line (file-name-nondirectory file)))
          ((equal what "lbp_clear")
           (pltrace-mark-breakpoint file 'clear)
           (message "Breakpoint removed: line %s@%s" line (file-name-nondirectory file)))
          ((equal what "lbp_clear_all")
           (pltrace-mark-breakpoint file 'clear_all)
           (message "Breakpoints removed: %s" (file-name-nondirectory file)))
          ((equal what "lbp_error")
           (message "Could not set breakpoint on line %s@%s" line (file-name-nondirectory file))))))
           


(defun pltrace-mark-breakpoint (file kind)
  ;; dummy
  )
  
(defun pltrace-apply-command (fun &rest args)
  ;; Would like this to be interruptible (which is not the case when
  ;; run from a process filter) Unfortunately I did not get Idle
  ;; timers to work in XEmacs 21.1.7
  (apply fun args))


;;; Menus [PM]


;;; ORIGINAL pltrace.el starts here
;;; 
;; pltrace.el --- support for the source debugger of SICStus Prolog >= 3.8
;;; Copyright (C) 1998, 1999 SICS and IQSoft
;;
;; Author: Rozman Tamas, IQSoft
;;         Emil Astrom, SICS <emil@sics.se>
;; Maintainer: Per.Mildner@sics.se
;; Version: 1.2.1
;; Keywords: source-level debugger sicstus

;;; Commentary:
;;
;; It is recommended that this file (pltrace.el) is kept in the same
;; directory as prolog.el, and it probably already is since this is
;; the default. No other installation is required for the debugger
;; mode. However prolog.el must of course be properly installed, see
;; that file for instructions of how to do that.

;;; Usage:
;;
;; When running SICStus in a shell buffer of Emacs and not in the
;; standard inferior buffer then the debugging support is turned on by
;; typing "M-x pltrace-current". To turn it on for the inferior buffer
;; use the Prolog menu or type "M-x pltrace-on". To turn it off for
;; either the inferior buffer or for a shell buffer, use the Prolog
;; menu or type "M-x pltrace-off".

;;; Version history
;; 1.2pre ?
;; 1.1 Released with SICStus Prolog 3#7


;;; Code:



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; global variables and initializations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar pltrace-port-arrow-assoc '( 
	("call" . ">>>") ("exit" . "+++") ("ndexit" . "?++")
	("redo" . "<<<") ("fail" . "---") ("exception" . "==>"))
"*Association list for mapping Prolog ports to overlay arrows")

(defvar pltrace-source-debug-on "prolog:source_info_on.\n" 
"*When the Prolog-trace is switched on, the contents of this variable
is sent to the standard input of the Prolog process")

(defvar pltrace-source-debug-off "prolog:source_info_off.\n"
"*When the Prolog-trace is switched off, the contents of this variable
is sent to the standard input of the Prolog process")

(defvar pltrace-overlay (make-overlay (point) (point))
"Overlay used by the Prolog-trace to highlight a line")

;; set the default highlight properties
(overlay-put pltrace-overlay 'face 'highlight)

;; initially make the overlay invisible
(delete-overlay pltrace-overlay)

(defvar pltrace-command-begin "$([{"
"defines a string that identifies the beginning of a Prolog-trace 
command")

(defvar pltrace-command-end "\n$)]}"
"Defines a string that identifies the end of a Prolog-trace command
Note that a command have to end with a newline, but that newline need 
not be put at the end of this string")

(defvar pltrace-queue ""
"A string that stores the characters came from the Prolog process")

(defvar pltrace-command-coming nil
"A bool value indicating whether currently a Prolog-trace command is
coming")

(defvar pltrace-process nil
"Identifies the Prolog process if Prolog-trace is running")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; general string-manipulating functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; finds a pattern in string
;; the beginning position of the first occurrence of the 
;; pattern is returned; if pattern is not found and the string ends 
;; with a prefix of the pattern, returns that position
(defun pltrace-find (string pattern)
  (let ((i 0) 
        (to (length string))
        (char (aref pattern 0)))
    (catch 'loop
      (while (< i to)
        (if (and 
	     (= char (aref string i))
	     (let ((m ((lambda (a b) (if (< a b) a b))
		       (- to i) (length pattern))))
	       (pltrace-substring-compare 1 m pattern (1+ i) string)))
	    (throw 'loop i))
        (setq i (1+ i)))
      to)))

;; compares substrings; substring1 is (substring pattern i-pat to)
;; substring2 is (substring string i-str (+ (- i-str i-pat) to))

(defun pltrace-substring-compare (i-pat to pattern i-str string)
  (or (= i-pat to)
    (and
     (= (aref pattern i-pat) (aref string i-str))
     (pltrace-substring-compare (1+ i-pat) to pattern (1+ i-str) string))))

;; returns comma-separated substrings of string in a list
(defun pltrace-split-command (string)
  (let* ((pos (pltrace-find string ","))
	 (sub (substring string 0 pos)))
    (if (= pos (length string))
	 (list sub)
         (cons sub (pltrace-split-command (substring string (1+ pos)))))))





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; the implementation of the interace
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
;; finds a new command in pltrace-queue and displays all parts of 
;; pltrace-queue that are not part of a command;
;; the beginning of the command is identified by the pattern defined in 
;; pltrace-command-begin
;;
;; this is called by the process filter pltrace-filter, so
;; proc and global variable pltrace-process is always the same
(defun pltrace-find-command (proc)
  (let* ((pos (pltrace-find pltrace-queue pltrace-command-begin))
         (first (substring pltrace-queue 0 pos))
         (second (substring pltrace-queue pos)))
    (pltrace-insert-process proc first)
    (setq pltrace-queue second)
    (if (<= (length pltrace-command-begin) (length second))
      (progn
	(setq pltrace-queue (substring pltrace-queue (length pltrace-command-begin)))
        (setq pltrace-command-coming t)
        (pltrace-parse-command proc)))))

;; finds the end of an already-found command in pltrace-queue and 
;; executes it;
;; the command end is identified by the pattern defined in 
;; pltrace-command-end
;;
;; this is called by the process filter pltrace-filter, so
;; proc and global variable pltrace-process is always the same
(defun pltrace-parse-command (proc)
  (let* ((real-end (concat pltrace-command-end "\n"))
	 (pos (pltrace-find pltrace-queue real-end))
         (first (substring pltrace-queue 0 pos))
         (second (substring pltrace-queue pos)))
    (if (<= (length real-end) (length second))
      (progn 
        (pltrace-execute-command proc first)
        (setq pltrace-command-coming nil)
        (setq pltrace-queue (substring second (length real-end)))
        (pltrace-find-command proc)))))

;; [PM] reimplemented above
;; ;; executes a Prolog-trace command
;; ;;
;; ;; the format of a command:
;; ;;
;; ;;   <begin>c,Buffer
;; ;;   Text
;; ;;   <end>
;; ;;        Creates a buffer named Buffer, or empties it if it already
;; ;;        exists; and deposits Text into it.
;; ;;
;; ;;  <begin>f,Port,Line,File<end>
;; ;;        Shows line Line in file File with an overlay corresponding to Port.
;; ;;
;; ;;  <begin>b,Port,Line,Buffer<end>
;; ;;        Shows line Line in buffer Buffer with an overlay corresponding to
;; ;;        Port.
;; ;;
;; ;;  <begin>u<end>
;; ;;        Removes the last highlight made by ..._show
;; ;;
;; ;;  <begin>k,Buffer<end>
;; ;;        Kills buffer named Buffer created by a c (create_buffer) command.
;; ;;
;; ;; this is called by the process filter pltrace-filter, so
;; ;; proc and global variable pltrace-process is always the same
;; (defun pltrace-execute-command (proc command)
;;   (let* ((pos (pltrace-find command "\n"))
;;          (first (substring command 0 pos))
;;          (second (substring command pos))
;;          (param-list (pltrace-split-command first)))
;;     (if (= 0 (length second))
;;         (pltrace-execute-normal-command (car param-list) (cdr param-list))
;;       (pltrace-execute-extended-command (car param-list) (cdr param-list)
;;                                       (substring second 1)))))
;;


;; executes commands f, b, u, k
(defun pltrace-execute-normal-command (command param-list)
  (cond ((string= "u" command)
	 (pltrace-unshow))
	((string= "k" command)
	 (pltrace-kill-buffer (car param-list)))
	(t
	 (let ((port (car param-list))
	       (line-num (string-to-number (nth 1 param-list)))
	       (rest (nth 2 param-list)))
	   (cond ((string= "f" command)
		  (pltrace-file-show port line-num rest))
		 ((string= "b" command)
		  (pltrace-buffer-show port line-num rest)))))))

;; executes command c
(defun pltrace-execute-extended-command (command param-list text)
  (if (string= "c" command)
      (progn
	(pltrace-unshow)
	(pltrace-create-buffer (car param-list) text))))

	     	
;; displays file file-name and highlights line line-num
;; with the overlay-arrow corresponding to port
(defun pltrace-file-show (port line-num file-name)
;  (pltrace-show port line-num (find-file-noselect file-name)))

;  (message "pltrace, line %d" line-num)

  ;; Now we should make absolutely sure that this file is
  ;; not already visited under another name 
  (let ((bufs (buffer-list))
	(truefilename (file-truename file-name))
	buf
	buffer)
    ;; Go through all open buffers to see if their true file names
    ;; match file-name.
    (let ((case-fold-search (memq system-type '(windows-nt cygwin32 ms-dos vax-vms)))
          (truefilename-regexp (regexp-quote truefilename)))
      (while bufs
        (setq buf (car bufs))
        (if (and
             (buffer-file-name buf)

             ;; [PM] However, we should make sure that the case tables
             ;;      used are not arbitrarily taken from the current
             ;;      buffer. Investigate this.
             ;; [PM] on windows sicstus will lowercase all paths
             ;;      This is not true of Emacs so there might be
             ;;      spurious mismatches. Arguably Xemacs
             ;;      file-truename is buggy and should return the
             ;;      casified name if given a matching all lowercase
             ;;      name on these systems. However, using case
             ;;      insensitive compare might be a good idea even
             ;;      when file-truename does the right thing (such as
             ;;      GNU Emacs 20.4.1)
             ;; (string= truefilename (file-truename (buffer-file-name buf)))
             (string-match truefilename-regexp 
                           (file-truename (buffer-file-name buf)))
             )
            (progn
              (setq buffer buf)
              (setq bufs nil)))		; End while loop
        (setq bufs (cdr bufs))))        ; Get next buffer

    ;; If the file was not already opened, then open it
    (if (not buffer)
	(setq buffer (find-file-noselect file-name)))

    (pltrace-show port line-num buffer)))

;; displays buffer named buffer-name and highlights line line-num
;; with the overlay-arrow corresponding to port
(defun pltrace-buffer-show (port line-num buffer-name)
  (pltrace-show port line-num (get-buffer buffer-name)))

;; clears buffer named buffer-name and puts text in it;
;; or if it does not exist it creates a new buffer and puts text into it
(defun pltrace-create-buffer (buffer-name text)
  (save-excursion
    (let ((new-buffer (get-buffer-create buffer-name)))
      (set-buffer new-buffer)
      (let ((buffer-read-only nil))
        (widen)
        (let ((valid                    ; existing content is OK
               (and (= (buffer-size) (length text)) ; redundant
                 (string= (buffer-substring-no-properties (point-min) (point-max))
                          text))))
          ;; [PM] avoid reinserting and re-fontifying.
          (if (not valid)
              (progn
                (erase-buffer)
                (insert text)
                ;; Start the Prolog mode if it is available (for
                ;; font-locking)
                ;; [PM] used to require non-nil window-system.
                ;;      This is a bad idea since
                ;;      . font locking also works for color capable
                ;;        terminals. Instead added a test that font-lock is
                ;;        on
                ;;      . prolog-mode has other benefits for prolog code
                (if (fboundp 'prolog-mode)
                    (progn
                      (prolog-mode)
                      ;; [PM] should be done by major mode.
                      ;;      prolog-mode now does this.
                      ;; (and font-lock-mode       ; prolog-mode-hook may change this
                      ;;      (font-lock-fontify-buffer))
                      ))))
          (set-buffer-modified-p nil))))))

;; hides pltrace-overlay an overlay-arrow
(defun pltrace-unshow ()
  (setq overlay-arrow-string nil)
  (delete-overlay pltrace-overlay))

;; kills a buffer
(defun pltrace-kill-buffer (buffer)
  (kill-buffer buffer))

;; displays a buffer and highlights line 
;; line-num with the overlay-arrow corresponding to port
(defun pltrace-show (port line-num buffer)
  (let ((pltrace-window (display-buffer buffer)))
  (save-excursion
    (set-buffer buffer)
    (goto-line (+ 1 line-num))
    (let ((end-pos (point)))
      (goto-line line-num)
      (move-overlay pltrace-overlay (point) end-pos buffer)
      ;; [PM] Why was this done again?
      ;; (goto-line line-num)
      (setq overlay-arrow-position (point-marker))
      (setq overlay-arrow-string (cdr (assoc port pltrace-port-arrow-assoc)))

      (if (pos-visible-in-window-p (point) pltrace-window)
          ;; [PM] This has the poorly documented side effect of
          ;; forcing the next redisplay to ensure that window-start is
          ;; not changed by point motion.  Without set-window-start
          ;; the newly created "*Prolog source*" buffer will always
          ;; show the end of the buffer.  I suspect that the problem
          ;; has to do with the newly inserted buffer contents (the
          ;; source to display) and the fact that this is all
          ;; happening in a process filter so there has been no
          ;; redisplay of this window yet.
          (set-window-start pltrace-window (window-start pltrace-window))
        (forward-line (/ (- (window-height pltrace-window) 1) -2))
        (set-window-start pltrace-window (point)))))))


;; [PM] Old code did not handle pre-existing process filter (e.g.,
;;      comint-filter)
;; [PM] Make sure pre-existing filter gets a shot at whatever is not
;;      a pltrace command. This way it should be possible for several 
;;      filters to coexist peacefully.
;;      Also, make it harmless to call pltrace-prolog (and thus
;;      pltrace-current) repeatedly.
(defvar pltrace-old-process-filter nil
  ;; note that FILTER may be nil or t as well
  "A cons (PROCESS . FILTER) of the process and its old filter.
PROCESS is needed since the user may have restarted the prolog
process, possibly in the same buffer.")

;; [PM]
;; searches for a command in the output of the Prolog process 
;; and displays all parts of the output that are not 
;; part of a command
;;
;; this is a process filter for Prolog-trace, so
;; proc and global variable pltrace-process is always the same
(defun pltrace-filter (proc string)
  (setq pltrace-queue (concat pltrace-queue string)) 
  (if pltrace-command-coming 
    (pltrace-parse-command proc)
    (pltrace-find-command proc)))

;; [PM] We always want the process filter to look for emacs commands
;; However, we may not want prolog to use source debugging.
;; treats the process of the buffer buf as a Prolog process and (by
;; default) starts Prolog-trace
(defun pltrace-prolog (buf &optional no-debug-on)
  (let ((process (get-buffer-process buf)))
    (if process
        (progn
          (setq pltrace-process process) ; [PM] only change if non-nil
          ;; zap if obsolete
          (and pltrace-old-process-filter
               (not (equal (car pltrace-old-process-filter)
                           pltrace-process))
               (setq pltrace-old-process-filter nil))

          (if pltrace-old-process-filter ; we're already installed
              nil                       ; do nothing
            (progn
              (setq pltrace-old-process-filter
                    (cons pltrace-process
                          (process-filter pltrace-process)))
              (setq overlay-arrow-string nil
                    overlay-arrow-position nil
                    pltrace-queue ""
                    pltrace-command-coming nil)
              (set-process-filter pltrace-process 'pltrace-filter)
              ))
          (if (not no-debug-on)
              (process-send-string pltrace-process pltrace-source-debug-on))))))



;; [PM] insert into buffer using old process-filter, if any
(defun pltrace-insert-process (proc string)
  (let ((old-filter
         (and (equal proc (car pltrace-old-process-filter))
              (cdr pltrace-old-process-filter))))
    (if (or (eq old-filter t)
            (eq old-filter nil))
        (pltrace-insert-process-default proc string)
      (funcall old-filter proc string))))
           
           

;; inserts string to the buffer of process proc
;; prolog.el has a corresponding function called
;; `prolog-process-insert-string' but it might be better not to
;; rely on the existance of it.
;; [PM] This used to be pltrace-insert-process
(defun pltrace-insert-process-default (proc string)
  (let ((old-buffer (current-buffer)))
    (unwind-protect                     ; aka with-current-buffer
        (let (moving)
          (set-buffer (process-buffer proc))
          (setq moving (= (point) (process-mark proc)))
          (save-excursion
            ;; Insert the text, moving the process-marker.
            (goto-char (process-mark proc))
            (insert string)
            (set-marker (process-mark proc) (point)))
          (if moving (goto-char (process-mark proc))))
      (set-buffer old-buffer))))

  


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; hooks and interactive functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; [PM]
(defvar swi-support-hook '()
  "List of functions to call when entering SWI prolog-mode.
This hook is run by run-swi-support-hook if prolog-system is `swi'.
Also see swi-support-inferior-hook.")

;; [PM]
(defvar swi-support-inferior-hook '()
  "List of functions to call when entering SWI prolog-inferior-mode.
This hook is run by run-swi-support-inferior-hook if prolog-system is `swi'.
Also see swi-support-hook.")

;; [PM]

(defun swi-support-initialization ()
  "Initialize SWI support.
Does nothing if `prolog-system' is not `swi'.
Intended to always be on prolog-mode-hook"
  (if (eq prolog-system 'swi)
      (progn
        ;; Key bindings

        (define-key pltrace-mode-map 
          "\C-x "                       ; also works in GNU Emacs 19.34.1
          'pltrace-break)
        ;; [PM] SP 3.9b4
        (make-variable-buffer-local 'prolog-parse-compilation-errors)
;        (setq prolog-parse-compilation-errors 'sicstus-prolog-parse-compilation-errors)

        (run-hooks 'swi-support-hook))))

(add-hook 'prolog-mode-hook 'swi-support-initialization)

(defun swi-support-inferior-initialization ()
  "Initialize SWI support for inferior prolog.
Does nothing if `prolog-system' is not `swi'.
Intended to always be on prolog-inferior-mode-hook"
  (and (eq prolog-system 'swi)
       (run-hooks 'swi-support-inferior-hook)))

(add-hook 'prolog-inferior-mode-hook 'swi-support-inferior-initialization)
(add-hook 'swi-support-inferior-hook 'pltrace-on)


;; [PM] Used to be in prolog.el
;(defun prolog-enable-sicstus-sd ()
;  "Enable the source level debugging facilities of SICStus 3.7 and later.
;To turn source debugging on (off) only for the currently running
;inferior prolog, use \\[pltrace-on] (\\[pltrace-off])."
;  (interactive)
;  ;; [PM] Hook always on, it now looks at prolog-use-sicstus-sd
;  ;; ;; Turn on the source level debugging by default
;  ;; (add-hook 'swi-support-inferior-hook 'pltrace-on t)
;
;  ;; [PM] Logic is lacking, if set after inferior prolog starts then
;  ;; enabling will turn it on in the running inferior mode.
;
;  (if (not prolog-use-sicstus-sd)
;      (progn
;        ;; If there is a *prolog* buffer, then call pltrace-on
;        (if (get-buffer "*prolog*")
;            (pltrace-on))
;        (setq prolog-use-sicstus-sd t))))

;; [PM] Used to be in prolog.el
;(defun prolog-disable-sicstus-sd ()
;  "Disable the source level debugging facilities of SICStus 3.7 and later.
;To turn source debugging on (off) dynamically, use pltrace-on (pltrace-off)."
;  (interactive)
;  (setq prolog-use-sicstus-sd nil)      ; disable for future run-prologs
;  ;; [PM] the hook now looks at prolog-use-sicstus-sd
;  ;; ;; Remove the hook
;  ;; (remove-hook 'swi-support-inferior-hook 'pltrace-on)
;
;  ;; If there is a *prolog* buffer, then call pltrace-off
;  (if (get-buffer "*prolog*")
;      (pltrace-off)))
  
;; turn on the trace for the current buffer
(defun pltrace-current ()  
  "Turn on source level debugging for the current buffer."
   (interactive)
   (pltrace-prolog (current-buffer)))

;; treats the process of the current buffer as a Prolog process and starts
;; Prolog-trace
(defun pltrace-on ()  
  "Turn on source level debugging for the *prolog* buffer."
   (interactive)
   (pltrace-prolog (get-buffer "*prolog*")))

(defun pltrace-off (&optional remove-process-filter)
  "Turn off source level debugging."
  (interactive)
  (if pltrace-process 
      (progn
        ;; [PM] Now we want the process filter to look for emacs
        ;; commands even if not source debugging.
        (if remove-process-filter
            (progn
              ;; [PM] used to set to nil unconditionally, but
              ;;      e.g., the comint filter should be reinstalled
              (set-process-filter pltrace-process
                                  (and pltrace-old-process-filter
                                       (equal (car pltrace-old-process-filter)
                                              pltrace-process)
                                       (cdr pltrace-old-process-filter)))
              (setq pltrace-old-process-filter nil)))

	(delete-overlay pltrace-overlay)
	(setq overlay-arrow-string nil
              overlay-arrow-position nil)
	(process-send-string pltrace-process pltrace-source-debug-off)

	(if remove-process-filter (setq pltrace-process nil)))))

;; [PM] SP 3.9b4 moved from prolog.el, adapted for 3.9
;(defun sicstus-prolog-parse-compilation-errors (limit &optional find-at-least)
;  "Parse the prolog compilation buffer for errors.
;Argument LIMIT is a buffer position limiting searching.
;For use with the `compilation-parse-errors-function' variable."
;  (setq compilation-error-list nil)
;  (message "Parsing SICStus error messages...")
;
;  ;; In case limit is within a line this will extend the limit to
;  ;; encompass the entire line.
;
;  ;; Actually, We should probably do what compile.el does, look at
;  ;; each line manually.
;  (if limit
;      (save-excursion
;        (goto-char limit)
;        (forward-line 1)
;        (setq limit (point))))
;  (let ((found 0)
;        (last-error-end compilation-parsing-end)
;        filepath dir file errorline
;        error-sign                      ; "!" or "*"
;        )
;    (goto-char compilation-parsing-end)
;    (while
;        (and
;         ;; [PM] ignore find-at-least
;         ;;      1. XEmacs will call with find-at-least zero!
;         ;;      2. Want to always process a complete file since we
;         ;;         want markers in case the user edits the file upon
;         ;;         visiting the first error.
;         ;; (or (null find-at-least) (< found find-at-least))
;         (re-search-forward
;          ;; 3.8 "{\\([a-zA-Z ]* ERROR\\|Warning\\):.* in line[s ]*\\([0-9]+\\)"
;          "^\\\([*!]\\) Approximate lines: \\([0-9]+\\)"
;          limit t))
;
;      (setq errorline (string-to-number (match-string 2)) ; may be adjusted below
;            error-sign (regexp-quote (match-string 1)))
;      
;      (save-excursion
;	(re-search-backward
;         
;	 ;; 3.8 "{\\(consulting\\|compiling\\|processing\\) \\(.*\\)\\.\\.\\.}"
;	 "% \\(consulting\\|compiling\\|processing\\) \\(.*\\)\\.\\.\\."
;	 limit t)
;	(setq filepath (match-string 2)))
;
;      ;; Does this work with SICStus under Windows (i.e. backslashes and stuff?)
;      ;; [PM] SP 3.9b4 SICStus does not print backslashes even on Windows
;      ;;               'C:' and UNC paths is what differs from UNIX
;      (if (string-match "\\(.*/\\)\\([^/]*\\)$" filepath)
;	  (progn
;	    (setq dir (match-string 1 filepath))
;	    (setq file (match-string 2 filepath))))
;      
;      ;; In 3.9 error messages looks like
;      ;; ! Syntax error
;      ;; ! , cannot start an expression
;      ;; ! in line 21
;      ;; ! main1 :- 
;      ;; ! <<here>>
;      ;; ! , , foo ( [ 1 , 2 , 3 ] ) . 
;      ;; ! Approximate lines: 17-23, file: '/home/perm/sicstus/foo.pl'
;      ;; ! Syntax error
;      ;; ! operator expected after expression
;      ;; ! in line 24
;      ;; ! main1 :- a 
;      ;; ! <<here>>
;      ;; ! a a foo ( [ 1 , 2 , 3 ] ) . 
;      ;; ! Approximate lines: 17-26, file: '/home/perm/sicstus/foo.pl'
;      ;;
;      ;; The error-sign ! can be * for warnings
;      ;;
;      ;; We have now found the line containing "Approximate lines: "
;      ;;
;      ;; We should scan backwards over ! (*) until we reach a non-! 
;      ;; (non-*) or we reach part of the previous error.
;      ;; If we find a "in line NN we should adjust errorline to the more
;      ;; precise info
;      ;;
;
;      (let ((error-point (point))       ; beginning of this error message
;            error-end)                  ; last-error-end to use for next error
;        (save-excursion
;          (end-of-line)
;          (setq error-end (point))
;          (beginning-of-line 0)         ; beginning of previous line
;          (while (and (< last-error-end (point))
;                      (looking-at error-sign))
;            (if (looking-at (concat error-sign " in line \\([0-9][0-9]*\\)"))
;                (setq errorline (string-to-number (match-string 1))))
;            (setq error-point (point))
;            (beginning-of-line 0))
;          (goto-char error-point)       ; beginning of this error
;          
;          (setq last-error-end error-end
;                error-point (point-marker)))
;        (setq compilation-error-list
;              (cons
;               (cons error-point
;                     (list (list file dir) errorline))
;               compilation-error-list)
;              found (+ found 1)
;              )))
;    )
;  (setq compilation-error-list (nreverse compilation-error-list)))
;
;;; [PM] This is so wrong. The faces should be customizable
;(defun sicstus-prolog-font-lock-keywords ()
;  "Set up font lock keywords for the SICStus Prolog system."
;  (if (or window-system
;          ;; [PM] XEmacs does font-lock on terminals as well
;          ;;      Also, window-system is bogus in a multi-display
;          ;;      environment
;          (eq prolog-emacs 'xemacs))
;      (progn
;        (require 'font-lock)
;        ;; This is only for backward compatibility. Not needed with
;        ;; Emacs 19.34 or later.
;        (if (boundp 'font-lock-background-mode)
;            ()
;          (make-local-variable 'font-lock-background-mode)
;          (setq font-lock-background-mode 'light)) ; Assume light bg
;        (if (boundp 'font-lock-display-type)
;            ()
;          (make-local-variable 'font-lock-display-type)
;          (setq font-lock-display-type 'color)) ; Assume color
;
;        ;; Create faces
;        ;; Format: (FACE FOREGROUND BACKGROUND BOLD-P ITALIC-P UNDERLINE-P)
;        (let* ((dark-bg (eq font-lock-background-mode 'dark))
;               (faces
;                (cond
;                 ((memq font-lock-display-type '(mono monochrome))
;                  '((prolog-warning-face nil nil t t nil)
;                    (prolog-builtin-face nil nil nil nil t)
;                    (prolog-redo-face nil nil nil t nil)
;                    (prolog-exit-face nil nil nil nil t)
;                    (prolog-exception-face nil nil t t t)))
;                 ((memq font-lock-display-type '(grayscale greyscale
;                                                           grayshade greyshade))
;                  '((prolog-warning-face nil nil t t nil)
;                    (prolog-builtin-face nil nil nil nil t)
;                    (prolog-redo-face nil nil nil t nil)
;                    (prolog-exit-face nil nil nil nil t)
;                    (prolog-exception-face nil nil t t t)))
;                 (dark-bg 		; dark colour background
;                  '((prolog-warning-face "red" nil t nil nil)
;                    (prolog-builtin-face "LightSkyBlue" nil nil nil nil)
;                    (prolog-redo-face "darkorchid" nil nil nil nil)
;                    (prolog-exit-face "green" nil nil nil nil)
;                    (prolog-exception-face "black" "Khaki" t nil nil)))
;                 (t			; light colour background
;                  '((prolog-warning-face "red" nil t nil nil)
;                    (prolog-builtin-face "Orchid" nil nil nil nil)
;                    (prolog-redo-face "darkorchid" nil nil nil nil)
;                    (prolog-exit-face "ForestGreen" nil nil nil nil)
;                    (prolog-exception-face "black" "Khaki" t nil nil))))))
;
;          (while faces
;            (if (fboundp 'font-lock-make-face)
;                ;; The preferred way
;                (font-lock-make-face (car faces))
;              ;; The clumsy way
;              (let ((facename (nth 0 (car faces)))
;                    (fg (nth 1 (car faces)))
;                    (bg (nth 2 (car faces)))
;                    (bold (nth 3 (car faces)))
;                    (ital (nth 4 (car faces)))
;                    (under (nth 5 (car faces))))
;                (make-face facename)
;                (if fg (set-face-foreground facename fg))
;                (if bg (set-face-background facename bg))
;                (if bold (make-face-bold facename))
;                (if ital (make-face-italic facename))
;                (if bold (make-face-bold facename))
;                (set-face-underline-p facename under)
;                ;; This is needed under Emacs 20 for some reason.
;                (set facename facename)
;                ))
;            (setq faces (cdr faces))))
;      
;        ;; Font Lock Patterns
;        (let (
;              ;; "Native" Prolog patterns
;              (head-predicates
;               (list (format "^%s" prolog-atom-regexp)
;                     0 font-lock-function-name-face))
;              (variables
;                                        ;xx
;                                        ;	       '("\\<\\([_A-Z][a-zA-Z0-9_]*\\)"
;                                        ;		 1 font-lock-variable-name-face))
;               (list (format "\\<\\([_%s][%s%s0-9_]*\\)"
;                             prolog-upper-case-string
;                             prolog-lower-case-string
;                             prolog-upper-case-string)
;                     1 'font-lock-variable-name-face))
;
;              (important-elements
;               (list "[][}{!;|]\\|\\*->"
;                     0 'font-lock-keyword-face))
;              (important-elements-1
;               '("[^-*]\\(->\\)" 1 font-lock-keyword-face))
;              (predspecs		; module:predicate/cardinality
;               (list (format "\\<\\(%s:\\|\\)%s/[0-9]+"
;                             prolog-atom-regexp prolog-atom-regexp)
;                     0 font-lock-function-name-face 'prepend))
;              (keywords                 ; directives (queries)
;               (list
;                (concat
;                 "^[?:]- *\\("
;                 (prolog-make-keywords-regexp prolog-keywords-i)
;                 "\\)\\>")
;                1 'prolog-builtin-face))
;
;              (quoted_atom (list prolog-quoted-atom-regexp
;                                 2 'font-lock-string-face 'append))
;              (string (list prolog-string-regexp
;                            1 'font-lock-string-face 'append))
;              ;; SICStus specific patterns
;              (sicstus-object-methods
;               (list
;                'prolog-font-lock-object-matcher
;                1 font-lock-function-name-face))
;              ;; Inferior mode specific patterns
;              (prompt
;               (list prolog-prompt-regexp-i 0 'font-lock-keyword-face))
;              (trace-exit
;               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Exit\\):"
;                 1 prolog-exit-face))
;              (trace-fail
;               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Fail\\):"
;                 1 prolog-warning-face))
;              (trace-redo
;               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Redo\\):"
;                 1 prolog-redo-face))
;              (trace-call
;               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Call\\):"
;                 1 font-lock-function-name-face))
;              (trace-exception
;               '("[ \t]*[0-9]+[ \t]+[0-9]+[ \t]*\\(Exception\\):"
;                 1 prolog-exception-face))
;              (error-message-identifier
;               '(
;                 ;; 3.8 "{\\([A-Z]* ?ERROR:\\)"
;                 ;; any line that starts with an exclamation mark
;                 ;; extend any lines containing 'error' to the end of line
;                 "\\(^! \\(.*error.*$\\)?\\)"
;                 1 prolog-exception-face prepend))
;              (error-whole-messages
;               '(
;                 ;; 3.8 "{\\([A-Z]* ?ERROR:.*\\)}[ \t]*$"
;                 "\\(^! .*\\)$"           ; any line that starts with an exclamation mark
;                 1 font-lock-comment-face append))
;              (error-warning-messages
;               ;; Mostly errors that SICStus asks the user about how to solve,
;               ;; such as "NAME CLASH:" for example.
;               '("^[A-Z ]*[A-Z]+:" 0 prolog-warning-face))
;              (warning-messages
;               '(
;                 ;; 3.8 ;; We allow one newline in the warning message
;                 ;; 3.8 "\\({ ?\\(Warning\\|WARNING\\) ?:[^\n]*\n?[^\n]*}\\)[ \t]*$" 
;                 "\\(^\\* .*\\)$"         ; any line that starts with an asterisk
;                 ;; 3.8 2
;                 1
;                 prolog-warning-face prepend))
;              )
;
;          ;; Make font lock list
;          (delq
;           nil
;           (cond
;            ((eq major-mode 'prolog-mode)
;             (list
;              head-predicates
;              quoted_atom
;              string
;              variables
;              important-elements
;              important-elements-1
;              predspecs
;              keywords
;              sicstus-object-methods
;                                        ; nil                     ; types
;                                        ; nil                     ; modes
;                                        ; nil                     ; directives
;              ))
;            ((eq major-mode 'prolog-inferior-mode)
;             (list
;              prompt
;              error-message-identifier
;              error-whole-messages
;              error-warning-messages
;              warning-messages
;              predspecs
;              trace-exit
;              trace-fail
;              trace-redo
;              trace-call
;              trace-exception))
;            ((eq major-mode 'compilation-mode)
;             (list
;              error-message-identifier
;              error-whole-messages
;              error-warning-messages
;              warning-messages
;              predspecs))))
;          ))))
;
;(put 'prolog-font-lock-keywords 'sicstus 'sicstus-prolog-font-lock-keywords)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; (provide 'pltrace)

;;; ORIGINAL pltrace.el ends here

(provide 'swi-support)

;;; swi-support.el ends here
