%

fs2tree(FS, Tree, Vars) :-
    fs_to_tree(FS, Tree, Vars).
    
fs_to_tree(FS, Tree, Vars) :-
    empty_assoc(EmptyAssoc),
    duplicates(FS, EmptyAssoc, Dups, EmptyAssoc, _Visited, 0, _NumDups),
    fs_to_tree_act(FS, Dups, Tree),
    findall(N1-T, (gen_assoc(Dup, Dups, N), fs_to_tree_act(Dup, Dups, N, T), N1 is N+1), Vars0),
    keysort(Vars0, Vars1),
    %pairs_keys_values(Vars1, _, Vars2),
    Vars =.. [vars | Vars1].

fs_to_tree_act(FS, Dups, Tree) :-
    fs_to_tree_act(FS, Dups, _Tag, Tree).
fs_to_tree_act(FS, Dups, Tag, Tree) :-
    (   get_assoc(FS, Dups, N), Tag \== N
    ->  N1 is N+1, Tree = '$VAR'(N1)
    ;   get_type(FS, Type),
        approps(Type, FRs, _),
        get_vals(FRs, FS, Vals),
        frs_to_ftrees(FRs, Vals, Dups, FTrees),
        (   Type == 0
        ->  Functor = (mgsat)
        ;   Type = (a_ X)
        ->  Functor = '$ATOM'(X)
        ;   Functor = Type
        ),
        Tree =.. [Functor|FTrees]
    ).

frs_to_ftrees([], [], _Dups, []).
frs_to_ftrees([F:R|FRs], [V|Vs], Dups, [F:T|Rest]) :-
    fs_to_tree_act(V, Dups, T0),
    ( T0 == (mgsat)
    ->  T = '$MGST'(R)
        %bind_mgsat(R, FS1, _),
        %fs_to_tree_act(FS1, Dups, T)
    ;   T = T0
    ),
    frs_to_ftrees(FRs, Vs, Dups, Rest).
