% $Id: ghooks.pl,v 1.51 2003/10/16 04:47:37 dm Exp $
%
% This file defines a pretty printing hook for the grisu interface.
%
% Authors: Detmar Meurers and Dale Gerdemann
% Extensively modified but still considered atrocious by: Gerald Penn
% 
% TODO:
%   - relational attachments
%
% ------------------------------------------------------

% Extended allowable a_/1 atoms to general Prolog terms.  Still need to render
%  variables more intelligently (currently, all are "_").
% 6/19/07 - G. Penn

% Bug corrected: atom_to_double_quoted_string/2 needs a case for a_/1 atoms (strip off
%  the wrapper and convert).
% 4/20/07 - G. Penn

% Bug corrected: insert_duplicates_list/5 was unhooking the Dups AVL.
% 6/4/07 - G. Penn

% Bug corrected: relink_parse_tree/6,8 were not relinking empty categories because
%  they were looking up edge/6 on every index.
% 9/11/07 - G. Penn

% Adapted portray_fs_by_type/12 to new featural pivot/root interpretation.
% 8/20/08 - G. Penn

:- multifile portray_fs/11, portray_approps/2.
% GBP - ALE provides no suppport for portray_approps/2; what is this?

:- use_module(library(lists),[select/3,memberchk/2]).
:- use_module(library(ugraphs),[vertices_edges_to_ugraph/3]).
:- use_module(library(assoc),[assoc_to_list/2,put_assoc/4,
                              empty_assoc/1,get_assoc/3]).

% also: t_samsort/3 from generally loaded sicstus site additions

% ------------------------------------------------------
% Load some other parts of the system this builds on:

% GBP: not really patches anymore; these are hook predicate definitions for the GUI
:- ensure_loaded(trale_home(trale_patches)).

% Feature ordering
:- ensure_loaded(trale_home(feature_ordering)).

% Socket support:
:- ensure_loaded(trale_home(socket_interface)).

% Unfilling code:
:- ensure_loaded(trale_home(unfill)).

% should always be loaded before other hooks are:
:- [trale_home(save_results)].

% webtrale
:- ensure_loaded(trale_home('webtrale/wtx')).

% ------------------------------------------------------
% switches:

:- dynamic grale_flag/0, grisu_debug_flag/0.

grale_on :-
   retractall(grale_flag),
   assert(grale_flag).

grale_off :- 
   retractall(grale_flag).

:- grale_on.

% --------------------------

grisu_debug :- % shows the output normally sent to grisu on standard out
   retractall(grisu_debug_flag),
   assert(grisu_debug_flag).

grisu_nodebug :-
   retractall(grisu_debug_flag).

% --------------------------

:- dynamic pref_struc_flag/1.

pref_struc_on :-
   asserta(pref_struc_flag(true)).

pref_struc_off :-
   asserta(pref_struc_flag(fail)).

pref_struc_restore :-  % restore to previous value
   retract(pref_struc_flag(_)).

pref_struc_flag :-     % 0-arity pred for testing the value of this flag
   pref_struc_flag(X), !,
   call(X).

% ------------------------------------------------------

% GBP - ported.  This code seems to exist for the benefit of diff/4 -
%  why can't ALE's hooks be used?
grisu_pp_fs(FS,FlagAss,MGType) :-
   \+ \+ (% deref(FS,Ref,SVs),
          grisu_pp_fs_act(FS,FlagAss,MGType)).
%grisu_pp_fs(Ref0,SVs0,FlagAss) :-
%   \+ \+ (deref(Ref0,SVs0,Ref1,SVs1),
%          grisu_pp_fs_act(Ref1,SVs1,FlagAss)).

grisu_pp_fs_act(FS,FlagAss,MGType) :-
   empty_assoc(Empty),
   duplicates(FS,Empty,DupsIn,Empty,_,0,_),
   grisu_pp_fs(FS,DupsIn,_,Empty,_,Empty,_,FlagAss,MGType).
	 
grisu_pp_fs_res(FS,Residue,FlagAss,MGType) :-
   \+ \+ (% deref(FS,Ref,SVs),
          grisu_pp_fs_res_act(FS,Residue,FlagAss,MGType)).
%grisu_pp_fs_res(Ref0,SVs0,Residue,FlagAss) :-
%   \+ \+ (deref(Ref0,SVs0,Ref1,SVs1),
%          grisu_pp_fs_res_act(Ref1,SVs1,Residue,FlagAss)).

% GBP - fixed ResArgs binding (reference var was being unhooked)
grisu_pp_fs_res_act(FS,Residue,FlagAss,MGType) :-
  empty_assoc(AssocIn),
  build_iqs_restore_atts(Residue,Iqs,FSResidue),
  (ale_flag(residue,show) -> residue_args(FSResidue,ResArgs,[FS]) ; ResArgs = [FS]),
  duplicates_list(ResArgs,AssocIn,DupsMid,AssocIn,VisMid,0,NumMid),
  duplicates_iqs(Iqs,DupsMid,DupsMid2,VisMid,_,NumMid,_),
  % print feature structure in grisu
  grisu_pp_fs(FS,DupsMid2,DupsMid3,AssocIn,VisMid2,AssocIn,HDMid,FlagAss,MGType),
  grale_pp_complete(Iqs,FSResidue,DupsMid3,VisMid2,HDMid).

grisu_pp_fs(FS,Dups0,Dups,Vis0,Vis,HD0,HD,FlagAss,MGType) :-
   begin_grisu_output(Stream),
   fs2keyed_feats(FS,KeyedFeats,Type),
   % We check for a low level hook, i.e. portray_fs here;
   % this hook is e.g. used by the code in save_results.pl
   ( (current_predicate(portray_fs,portray_fs(_,_,_,_,_,_,_,_,_,_,_)),
     portray_fs(Type,FS,MGType,KeyedFeats,Vis0,Vis,Dups0,Dups,0,HD0,HD)) % GBP - not sure MGType should be 0 here
   -> true  % GBP, and it's really weird that you don't know whether portray_fs/11 is defined - you're the
            % one who is supposed to define it.
   % GBP - in fact, it appears that portray_fs/11 is defined, but is failing
    ; (Type == 0 -> PortrayType = MGType ; PortrayType = Type),
         % GBP - this is a total hack, but hopefully this predicate won't care that KeyedFeats sometimes
         %   doesn't match PortrayType
      grisu_portray_fs(Stream,PortrayType,FS,KeyedFeats,
                       Vis0,Vis,Dups0,Dups,HD0,HD,FlagAss)
   ).

% ------------------------------------------------------


grisu_portray_fs(Stream,Type,FS,KeyedFeats,
                 Vis0,Vis,Tags0,Tags,HD0,HD,ToplevelFlagAss) :-

   toplevel_init(FS,Tags0,ToplevelFlagAss,HD0,HD1),

   % retrieve the information we stored in the HD arg. between callbacks:
   get_assoc(current,HD1,c(Index0,Index,Mother,Hist0,Hist,FlagAss)),
   put_assoc(current,HD1,no_assoc,HD2),

   % deref(FS,Ref,_SVs),
   get_flags(FlagAss,FS,Flags),

   % GBP - Tag is not a tag in the sense of the old ALE data structure!
   (get_assoc(FS,Tags0,Tag),   % This is a structure sharing and
    Mother \= tag(_)            % we were'nt called by writeReentrancyList
   ->  % store the Type/FS/KeyedFeats in Visited under key Ref
       put_assoc(FS,Vis0,f(Type,FS,KeyedFeats),Vis2),
       addTagToHist(Mother,Tag,Hist0,Hist2),
       writeTag(Stream,Mother,Tag,Index0,Index3,Flags),
       Tags1=Tags0, HD3=HD2
   ;   beginNonTag(Type,Mother,Index0,Index2,Stream,MissingEnds,Flags),
       addToHist(Mother,Type,Index0,Hist0,Hist1),
       put_assoc(FS,Vis0,not_a_tag,Vis1),
       print_feats(KeyedFeats,Type,Vis1,Vis2,Tags0,Tags1,
                   col(FS,Index2,Index3,Index0,Hist1,Hist2,
                       FlagAss,Stream),
                   HD2,HD3),
       endNonTag(MissingEnds,Stream) % closes brackets from beginNonTag
   ),
   end_portray(HD0,HD3,HD,Vis2,Vis,Tags1,Tags,Index3,Index,Hist2,Hist,FlagAss,Stream),
   !.

% Catch if something fails in the code above, since else we would
% backtrack into the default pretty printer.
% GBP - why is aborting execution with an error message preferable to using the
%  default pretty printer?
grisu_portray_fs(_,_,_,_,Vis,Vis,Tags,Tags,HD,HD,_) :-
   print_message(warning,'Unexpected failure in pretty printing.'),
   abort.

toplevel_init(FS,Tags0,ToplevelFlagAss,HD0,HD1) :-
   topLevel(HD0), !, 
   % check if a ToplevelFlagAss was provided (i.e. is not a variable):
   (  var(ToplevelFlagAss)
   -> empty_assoc(Empty),
      ToplevelFlagAss = flagAss(_IdentAss,Empty)
   ;  true
   ),
   ToplevelFlagAss = flagAss(IdentAss,UnfAss0),
   get_unfill(FS,Tags0,UnfAss0,UnfAss),
   % The assoc entry "current" we use to memorize between callbacks is:
   % c(Index0,Index,Mother,Hist0,Hist,FlagAss)
   put_assoc(current,HD0,
             c(0,_Index,hd(root),[],_Hist,flagAss(IdentAss,UnfAss)),
             HD1).
   % this put_assoc into HD also ensures we fail topLevel/1 after callback

toplevel_init(_,_,_,HD,HD).

% beginNonTag/7: Ref is not a reentrant node, and mother isn't a tag
% Deal with cases where it's
% a) a list:
beginNonTag(Type,Mother,Index0,Index,Stream,MissingEnds,Flags) :-
   treatList(Type,Mother,Index0,Index,Stream,MissingEnds,Flags), 
   !.

% b) anything else 
% This can have features, so it uses MissingEnds to close paren after
% output of features and their value in the calling predicate.
beginNonTag(Type,_,Index0,Index,Stream,[_],Flags) :-
   beginType(Type,Index0,Index,Stream,Flags).

endNonTag([],_).
endNonTag([_|T],Stream) :-
   end(Stream),
   endNonTag(T,Stream).

writeTag(Stream,Mother,Tag,Index0,Index,Flags) :-
   (   Mother = tl(_)
   ->  begin(Stream,'Z',Index0,Index1,Flags),
       beginTag(Stream,Tag,Index1,Index,Flags),
       end(Stream),             % of beginTag
       end(Stream),             % of begin...'Z'
       end(Stream)              % of mother list
   ;   beginTag(Stream,Tag,Index0,Index,Flags),
       end(Stream)
   ).


end_portray(HD0,HD2,HD,Vis1,Vis,Tags1,Tags,Index3,Index,Hist2,Hist,FlagAss,Stream) :-
   topLevel(HD0),
   !,
   xnl(Stream),xnl(Stream),
   assoc_to_list(Tags1,TagsList),
   writeReentrancyList(TagsList,Vis1,Vis,Tags1,Tags,
                       col(Index3,Index,Hist2,Hist,FlagAss,Stream),
                       HD2,HD3,Stream),
   writeTree(Stream,Hist,Index,HD3),
%   nl(Stream),
%   flush_output(Stream),
%   HD=HD0.% we exit the top level call with an empty assoc list in HD
   put_assoc(id_index,HD0,Index,HD).

% second clause deals with when we're not on topLevel:
end_portray(_,HD,HD,Vis,Vis,Tags,Tags,Index,Index,Hist,Hist,_,_).

writeReentrancyList([],Vis,Vis,Tags,Tags,
                    col(Index,Index,Hist,Hist,_FlagAss,Stream),
                    HD,HD,Stream).
writeReentrancyList(Reentrancies0,Vis0,Vis,Tags0,Tags,
                    col(Index0,Index,Hist0,Hist,FlagAss,Stream),
                    HD0,HD,Stream) :-
% This is done inefficiently here and should probably be changed.
% As far as I understand Dale's code, it proceeds as follows:
% 1. select picks out a ref
% 2. get_assoc sees if we've come across this node already and stored 
%    the feature structure under that node
% 3. if get_assoc fails, it backtracks into 1. and selects another
%    ref, until we find one that we know the feature structure of
%    If get_assoc succeeds, we print the value of the reentrancy,
%    by calling portray_fs, wrapping the tag with tag() so that
%    we actually print the value this time (and not just the tag).  
% 4. If step 3 never suceeds, we have a structure sharing that didn't
%    actually occur in the structure. That can occur in case a when 
%    statement, which refers to something our structure was derived 
%    from (and therefore does not have to be reentrant with something 
%    in our structure itself).
   ( select(Ref-Tag,Reentrancies0,Reentrancies),  % GBP - Ref is now a FS; why is select/3 being used here?
     get_assoc(Ref,Vis0,f(Type,FS,KeyedFeats)) 
   ->  beginReentr(Stream,Tag,Index0,Index1),
       put_assoc(current,HD0,c(Index1,Index2,tag(Tag),Hist0,Hist1,FlagAss),HD1),
       grisu_portray_fs(Stream,Type,FS,KeyedFeats,Vis0,Vis1,Tags0,Tags1,HD1,HD2,_),
       end(Stream),             % of beginReentr
       xnl(Stream)
   ;   Vis0=Vis1,Tags0=Tags1,Index0=Index2,Hist0=Hist1,HD0=HD2,
       print_message(warning,'Found reentrancy not rooted in this structure.')
   ),
   writeReentrancyList(Reentrancies,Vis1,Vis,Tags1,Tags,
                       col(Index2,Index,Hist1,Hist,FlagAss,Stream),
                       HD2,HD,Stream).


print_feats(Feats,Type,VisIn,VisOut,TagsIn,TagsOut,Col,HDIn,HDOut) :-
   order_features(Feats,OrdFeats),
   print_ord_feats(OrdFeats,Type,VisIn,VisOut,TagsIn,TagsOut,Col,HDIn,HDOut).

print_ord_feats([],_Type,Vis,Vis,Tags,Tags,
            col(_Ref,Index,Index,_Mother,Hist,Hist,_FlagAss,_Stream),
            HD,HD).
print_ord_feats([Feat|Feats],Type,Vis0,Vis,Tags0,Tags,
            col(MotherRef,Index0,Index,Mother,Hist0,Hist,FlagAss,Stream),
            HD0,HD) :-
   print_feat(Feat,Type,Vis0,Vis1,Tags0,Tags1,
              col(MotherRef,Index0,Index2,Mother,Hist0,Hist1,FlagAss,Stream),
              HD0,HD1),
   print_ord_feats(Feats,Type,Vis1,Vis,Tags1,Tags,
               col(MotherRef,Index2,Index,Mother,Hist1,Hist,FlagAss,Stream),
               HD1,HD).


print_feat(fval(F,Val,Restr),Type,Vis0,Vis,Tags0,Tags1,
           col(_,Index0,Index2,Mother,Hist0,Hist1,FlagAss,Stream),
           HD0,HD) :-
   % deref(Val,Ref,SVs),            % get the ref of the feature value
   get_flags(F,FlagAss,Val,Flags),
   beginFeature(Type,F,Index0,Index1,NeedsEnd,Flags,Stream),
% store the type of things with a phon attribute (for tree, in partic. leafs)
  deal_with_phon(F,Restr,Val,Mother,Type,HD0,HD1),
  getNextMother(F,Restr,Type,Mother,Mother1,HD1,HD2),
  put_assoc(current,HD2,c(Index1,Index2,Mother1,Hist0,Hist1,FlagAss),HD3),
% The main recursion into the feature values:
   fs2keyed_feats(Val,KeyedFeats,X0Type),
   (X0Type == 0 -> XType = mgsat(Restr) ; XType = X0Type), % GBP - get rid of 0 in output, while keeping KeyedFeats empty
   grisu_portray_fs(Stream,XType,Val,KeyedFeats,
                       Vis0,Vis,Tags0,Tags1,HD3,HD,_),
  (   NeedsEnd
  ->  end(Stream)
  ;   true 
  ).

get_flags(Feat,flagAss(IdentAss,UnfAss),FS,flags(Differ,Unfill)) :-
   get_flags(flagAss(IdentAss,UnfAss),FS,flags(Differ,TmpUnfill)),
   (   no_write_feat_flag(Feat)	% when user tells us so
   ->  Unfill=true		% unfill a particular feature
   ;   Unfill=TmpUnfill).	% else deal according to its informativeness

get_flags(flagAss(IdentAss,UnfAss),FS,flags(Differ,Unfill)) :-
   (   (var(IdentAss);		% if variable -> wasn't specified
        get_assoc(FS,IdentAss,_) )
   ->  Differ=false             % uninterresting = common in diff
   ;   Differ=true 
   ),
   (  get_assoc(FS,UnfAss,_)       
   -> Unfill=true
   ;  Unfill=false
   ).

deal_with_phon(Feat,Restr,FS,Mother,Type,HD0,HD1) :-
   phon_feat_(Feat),		% user defined as storing the phonology
   sub_type(list,Restr),	% and has appropriate value list
   !,
   % deref(FS,TagMid,SVsMid),
   (  fs_string2prolog_list(FS,Phon,Restr) 
   -> true
   ;  Phon = [none]
   ),
   put_assoc(phon_info(Mother),HD0,Type:Phon,HD1).
deal_with_phon(_,_,_,_,_,HD,HD).

% GBP - added Restr so that we can interpret lazy structures
fs_string2prolog_list(FS,List,Restr) :-
   get_type(FS,Type),
   ( Type == 0 -> FSType = Restr
   ; FSType = Type
   ),
   ( sub_type(e_list,FSType) -> List = []  % GBP - corrected from == e_list
   ; FSType == list   -> List = list
   ; sub_type(ne_list,FSType),  % GBP - added this guard
     List = [H|T],
     get_feat(hd,FS,Hd),
     % deref(Hd,_TagHd,SVsHd),
     get_type(Hd,HdType),
     % remove an atom wrapper if one exists:
     (  HdType = (a_ H) -> true
     ;  HdType = H
     ),
     get_feat(tl,FS,Tl),
     % deref(Tl,TagTl,SVsTl),
     approp(tl,FSType,TlRestr),
     fs_string2prolog_list(Tl,T,TlRestr)
   ).


treatList(ne_list,Mother,Index0,Index,Stream,[],Flags) :- 
   !,
   (   Mother = tl(_)
   ->  Index is Index0 + 1      % We want a different id
                                % than for tl:hd for use in
                                % the assoc lists. 
   ;   begin(Stream,'L',Index0,Index,Flags)  
       % the end for this begin is in next two clauses
   ).
treatList(e_list,Mother,Index0,Index,Stream,[],Flags) :- 
   !,
   (   Mother = tl(_)
   ->  end(Stream),             % of mother list 
       Index=Index0
   ;   begin(Stream,'L',Index0,Index,Flags),
       end(Stream)
   ).

% The hd and the tl of a list are displayed by the special
% [...] notation, so we never unfill those attribute values.
% The argument of flags/2 in the following two clause thus
% is always set to false.

% Type = list or other value that can occur as value of tl attribute
treatList(Type,tl(_),Index0,Index,Stream,[_,_,_],flags(Differ,_)) :-
   begin(Stream,'Z',Index0,Index1,flags(Differ,false)),
   beginType(Type,Index1,Index,Stream,flags(Differ,false)).
   % end of beginType -> one true in needsEnd continuation
   % end of begin ... Z -> one true in needsEnd continuation
   % end of mother list -> one true in needsEnd continuation

treatList(Type,hd(_),Index0,Index,Stream,[_],flags(Differ,_)) :-
   % Note that we never unfill the tl of a list since lists 
   % are displayed through the special notation, so the second 
   % argument of flags/2 is always specified to be false.
   beginType(Type,Index0,Index,Stream,flags(Differ,false)).
   % end of beginType -> one true in needsEnd continuation
   

% When the pretty printing hook is called for the first time in
% printing a structure, the HDIn Argument is an empty_assoc (i.e. t).
topLevel(A) :-
   empty_assoc(A).

addTagToHist(none,_,Hist,Hist) :- !.
addTagToHist(Mother,Tag,Hist,[Mother=tag(Tag)|Hist]) :- !.   

addToHist(tag(N),_Type,Index,Hist,[tag(N)-Index|Hist]).
addToHist(hd(I),_Type,Index,Hist,[I-hd(Index)|Hist]).
addToHist(tl(I),Type,Index,Hist,[Tail|Hist]) :-
   makeTail(Type,I,Index,Tail),
   !.
addToHist(tl(_I),_Type,_Index,Hist,Hist).
addToHist(dtrs(I),Type,Index,Hist,[Dtrs|Hist]) :-
   makeDtrs(Type,I,Index,Dtrs).
addToHist(none,_Type,_Index,Hist,Hist).

makeTail(ne_list,I,Index,I-tl(Index)).
makeTail(list,I,_Index,I-tl(list)).
makeTail(e_list,I,_Index,I-tl(e_list)).
makeTail(mgsat(T),I,Index,Tail) :- % GBP - added for lazy types;
  makeTail(T,I,Index,Tail).        %  but I have no idea what this predicate is doing.

makeDtrs(ne_list,I,Index,I-dtrs(Index)).
makeDtrs(list,I,_Index,I-dtrs(list)).
makeDtrs(e_list,I,_Index,I-dtrs(e_list)).
makeDtrs(mgsat(T),I,Index,Dtrs) :-  % GBP - added for lazy types;
  makeDtrs(T,I,Index,Dtrs).         %  but I have no idea what this predicate is doing.

getNextMother(hd,_,ne_list,Index,hd(Index),HD,HD) :- !.
getNextMother(tl,_,ne_list,Index,tl(Index),HD,HD) :- !.
getNextMother(Feat,Restr,Type,Index,dtrs(Index),HD0,HD1) :- 
   dtrs_feat_(Feat),		% user defined as storing the dtrs
   sub_type(list,Restr),	% and has appropriate value list
   !,
   put_assoc(tree_info(Index),HD0,Type,HD1). % store tree mother info by Index
getNextMother(_,_,_,_,none,HD,HD).

% Note on getNextMother: We explicitly test for the type ne_list since 
% attributes hd and/or tl could be appropriate for other types as well,
% but we only have a special list syntax for type ne_list.

get_output_stream(Stream) :-
  (   \+ grisu_debug_flag,
      current_predicate(socket_info,socket_info(_,_)),
      socket_info(_,Stream)
  ->  true
  ;   current_output(Stream)
  ).


% beginType(+Type:<typename>,
%           +CurrentIndx:<integer>,-NextIndex:<integer>,
%           +Stream:<stream>,+Flags)

% a) a_ atoms
beginType((a_ Atom),Index0,Index,Stream,Flags) :-
   !,
   begin(Stream,'A',Index0,Index,Flags), 
   writeqt(Stream,Atom).

% b) ordinary types
beginType(Type,Index0,Index,Stream,Flags) :-
   begin(Stream,'S',Index0,Index1,Flags),
   write(Stream,'('),
   writeFlags(Stream,Flags),
   writeIndex(Stream,Index1,Index),
   writeqt(Stream,Type),
   end(Stream).

% Note on beginFeature: We explicitly test for the type ne_list since 
% attributes hd and/or tl could be appropriate for other types as well,
% but we only have a special list syntax for type ne_list.

beginFeature(ne_list,_Feat,Index, Index,false,_Flags,_Stream) :- !.
beginFeature(_Type,   Feat,Index0,Index,true,  Flags, Stream) :-
   begin(Stream,'V',Index0,Index,Flags),
   writeFeat(Stream,Feat).

% begin a tag
beginTag(Stream,Num,Index0,Index,Flags) :-      % new 1.0.6 syntax
   begin(Stream,'#',Index0,Index,Flags),
   write(Stream,' '),           % new syntax: space required
   write(Stream,Num).

% begin a reentr
beginReentr(Stream,Num,Index0,Index) :- % new 1.0.6 syntax
   begin(Stream,'R',Index0,Index),
   write(Stream,' '),           % new syntax: space required
   write(Stream,Num).


% begin general cases: X is one of the different node kinds
begin(Stream,X,Index0,Index) :-
   begin(Stream,X,Index0,Index,false).
begin(Stream,X,Index0,Index,Flags) :-
   write(Stream,'('),
   write(Stream,X),
   writeFlags(Stream,Flags),
   writeIndex(Stream,Index0,Index).

end(Stream) :- 
   write(Stream,')').

writeFlags(Stream,Flags) :-
   (   (nonvar(Flags),Flags=flags(Differ,Unfill))
   ->  (Differ -> write(Stream,'*');true),
       (Unfill -> write(Stream,'+');true) 
   ; true).

writeIndex(Stream,Index0,Index) :-
   write(Stream,Index0),        % new syntax: index no longer quoted
   Index is Index0+1.

writeqt(Stream,X) :-
   write(Stream,'"'),
   write(Stream,X),
   write(Stream,'"').


writeFeat(Stream,Feat):-
   writeqt(Stream,Feat). % we send the feat as is, i.e. no capitalize_atom 

writeTree(Stream,Hist,Index0,HD) :-
   removeTagsFromHistory(Hist,HistNoTags),
   vertices_edges_to_ugraph([],HistNoTags,Graph),
   bindListsInHist(Graph,T0),
   ( select(dtrs(0,_),T0,_) 
   -> writeTree(Stream,0,T0,_T,Index0,_Index,HD)
   ;  true % no top-level tree found, so none is displayed
   ).

% ----------------------------------------------------------
% removeTagsFromHistory/2: connects all paths to their
% actual values, i.e. in case the value is a tag inserts the
% value that tag is pointing to.
% Example:
% | ?- removeTagsFromHistory([tag(0)-11,5-tl(e_list),hd(5)=tag(0),3-tl(5),hd(3)=tag(0)],L).
%      L = [5-tl(e_list),5-hd(11),3-tl(5),3-hd(11)] ? 

removeTagsFromHistory(Hist0,Hist) :-
   empty_assoc(Bind0),
   store_bindings(Hist0,Hist1,Bind0,Bind1),
   apply_bindings(Hist1,Hist,Bind1).

store_bindings([],[],Bind,Bind).
store_bindings([tag(N)-S|R0],R,Bind0,Bind) :-
   !,
   put_assoc(N,Bind0,S,Bind1),
   store_bindings(R0,R,Bind1,Bind).
store_bindings([F|R0],[F|R],Bind0,Bind) :-
   store_bindings(R0,R,Bind0,Bind).

apply_bindings([],[],_Bind).
apply_bindings([Term=tag(N)|Hist0],[Index-NewTerm|Hist],Bind) :-
   !,
   get_assoc(N,Bind,S), 
   Term =.. [What,Index],
   NewTerm =.. [What,S],
   apply_bindings(Hist0,Hist,Bind).
apply_bindings([F|Hist0],[F|Hist],Bind) :-
   apply_bindings(Hist0,Hist,Bind).

% ----------------------------------------------------------

bindListsInHist(Hist0,Hist) :-
   empty_assoc(Binding),
   put_assoc(e_list,Binding,[],Binding1),
   put_assoc(list,Binding1,_,Binding2),
   bindListsInHist(Hist0,Hist,Binding2).

bindListsInHist([],[],_Binding).
bindListsInHist([N-[hd(H),tl(TL0)]|Hist0],Hist,Binding0) :- !,
   makeNewTail(TL0,TL,Binding0,Binding1),
   get_binding(N,Var,Binding1,Binding2),
   Var=[H|TL],
   bindListsInHist(Hist0,Hist,Binding2).
bindListsInHist([N-[dtrs(D)]|Hist0],[dtrs(N,Dtrs)|Hist],Binding0) :- !,
   get_binding(D,Dtrs,Binding0,Binding1),
   bindListsInHist(Hist0,Hist,Binding1).   
bindListsInHist([_F|Hist0],Hist,Binding0) :- !,
   bindListsInHist(Hist0,Hist,Binding0).

get_binding(S,BoundTo,Binding0,Binding) :-
   get_assoc(S,Binding0,BoundTo), !,
   Binding = Binding0.
get_binding(S,BoundTo,Binding0,Binding) :-
   put_assoc(S,Binding0,BoundTo,Binding).

makeNewTail(e_list,[],Binding,Binding) :- !.
makeNewTail(list,_,Binding,Binding) :- !.
makeNewTail(N,TL,Binding0,Binding) :-
   get_binding(N,TL,Binding0,Binding).

% ----------------------------------------------------------

writeTree(Stream,N,T0,T,Index0,Index,HD) :-
   begin(Stream,'T',Index0,Index1),
   ( select(dtrs(N,Dtrs),T0,T1)
   ->  true
   ;   Dtrs=[], T1=T0
   ),
   % node label:
   writeNodeLabel(Stream,Dtrs,N,HD),
   % arc label:
%   writeqt(Stream,some-feat),
   % link id (no longer quoted):
   write(Stream,N),
   writeDtrs(Dtrs,T1,T,Index1,Index,Stream,HD),
   end(Stream).

writeNodeLabel(Stream,_Dtrs,N,HD) :-
   (get_assoc(phon_info(N),HD,Info) 
   -> writeqt(Stream,Info)
   ;  (get_assoc(tree_info(N),HD,Info)
      -> writeqt(Stream,Info)
      ;  writeqt(Stream,'no label'))).

writeDtrs([],T,T,Index,Index,_Stream,_HD).
writeDtrs([N|Rest],T0,T,Index0,Index,Stream,HD) :-
   writeTree(Stream,N,T0,T1,Index0,Index1,HD),
   writeDtrs(Rest,T1,T,Index1,Index,Stream,HD).

xwrite(Stream,Term0) :-
   rm_nl(Term0,Term),
   write(Stream,Term),
   write(Term).


% For xnl/1 and xtab/1 below, I commented out to get screen output
% which can directly be loaded into grisu. Comment back in to human
% readable output on screen. (wdm, 1.August.02)

%xnl(user_output) :- !,
%   nl(user_output).
xnl(_).

%xtab(user_output,N) :- !,
%   tab(user_output,N).
xtab(_,_).


% ========================================================


begin_grisu_output(Stream,Title) :-
   set_title(Title),
   begin_grisu_output(Stream).
begin_grisu_output(Stream) :-
   get_output_stream(Stream),
   format(Stream,'!newdata',[]),
   (  pref_struc_flag 
   -> format(Stream,'(structure)',[])
   ;  true
   ),
   get_title(Title),
   format(Stream,'"~w"',[Title]).

% ========================================================
% specific portray hooks
% ========================================================

portray_lex(Word,FS,Residue) :-
   grale_flag,			% fails immediately if grisu not on
   set_title((lex):Word),
   grisu_pp_fs_res(FS,Residue,_,0),
   clear_title.

% ------------------------------------------------------
% portray_cat/5: basic feature structure output
% called by rec/1, rec/2, rec_best/2, rec_list/2, gen/1, gen/2 

%portray_cat(Words,Desc,FS,Residue) :-
%   grale_flag,			% fails immediately if grisu not on
%   (  Desc = bot 
%   -> set_title(Words)
%   ;  set_title(Words-Desc)
%   ),
%   grisu_pp_fs_res(FS,Residue,_,0),
%   clear_title.

% HACK: no calling description display yet
portray_cat(Words,_Desc,FS,Residue,Index) :-
  grale_flag,
  list_to_double_quoted_string(Words,DQWords),  % KNOWN BUG: must handle var Words arg for gen/1.
  ( ale_lists_defined -> clause(fcolour(hd,HdPos,_),true),
                         clause(fcolour(tl,TlPos,_),true),
                         relink_parse_tree(FS,Words,Index,Tree,[FS1|TreeFSs],[],HdPos,TlPos)
  ; relink_parse_tree(FS,Words,Index,Tree,[FS1|TreeFSs],[])  % remove root so that default structural view
  ),                                                         % will be expanded
  append("!newdata",DQWords,GraleCommandPrefix),  % (structure)
  grale_write_chars(GraleCommandPrefix),
  \+ \+ (empty_assoc(AssocIn),
	 build_iqs_restore_atts(Residue,Iqs,Residue0),  % HACK: no graphical residue or ineq display yet
	 ( ale_flag(cllrs,on) -> build_sem_residue(Residue0,SemResidue,FSResidue) %, debug_semres(SemResidue)
	 ; SemResidue = [], FSResidue = Residue0
	 ),
         (ale_flag(residue,show) -> residue_args(FSResidue,ResArgs,[FS]) ; ResArgs = [FS]),
%   	 duplicates(ResArgs,AssocIn,DupsMid,AssocIn,_,0,NumMid),
	 duplicates_list(ResArgs,AssocIn,Dups0,AssocIn,Vis0,0,Num0),
	 duplicates_iqs(Iqs,Dups0,Dups1,Vis0,_Vids1,Num0,Num1),
	 insert_duplicates_list(TreeFSs,Dups1,Dups2,Num1,Num2),  % HACK: the re-entrancies list is the only place
	 put_assoc(top_index,AssocIn,Num2,HD0),                  % in the present interface where I can provide these
	 put_assoc(tree_struc,HD0,Tree,HD1),
	 put_assoc(sem_residue,HD1,SemResidue,HD2),
	 pp_fs(FS,0,Dups2,Dups3,AssocIn,Vis2,0,HD2,HD3),
	 grale_pp_iqs(Iqs,Dups3,Dups4,Vis2,Vis3,HD3,HD4), 
         (  FSResidue \== [], ale_flag(residue,show) 
         -> grale_write_chars("/"),
            pp_residue(FSResidue,Dups4,_,Vis3,_,HD4,_) 
         ;  true
         )),
  grale_nl,grale_flush_output.

debug_semres([]).
debug_semres([Suspension|SemRes]) :-
  arg(3,Suspension,subterm:C),
  arg(2,Suspension,Mref),
  (get_mutable(active,Mref) -> MFlag = 1 ; MFlag = 0),
  functor(C,F,A), arg(1,C,One), arg(2,C,Two),
  write(F/A), write(' '),write(One),
  ( A \== 2 -> write(','),write(Two),
               (F == 'ist/4_1' -> arg(3,C,Three), write(','),write(Three) ; true)
  ; true),
  write(' '),write(MFlag),
  nl, flush_output,
  debug_semres(SemRes).

% ------------------------------------------------------

portray_mgsat(Desc,FS,Residue,MGType) :-
   grale_flag,			% fails immediately if grisu not on
   set_title((mgsat):Desc),
   grisu_pp_fs_res(FS,Residue,_,MGType),
   clear_title.

% ------------------------------------------------------
portray_ale_macro(MacroName,_Desc,FS,Residue) :-
   grale_flag,
   set_title((macro):MacroName),
   grisu_pp_fs_res(FS,Residue,_,0),
   clear_title.

% ------------------------------------------------------

portray_empty(I,FS,_Dtrs,RuleName,ND,Residue) :-
   grale_flag,
   set_title('Empty category'),
   grisu_pp_fs_res(FS,Residue,_,0),
   (no_interpreter 
    -> true
     ; nl, write('     index: '),(functor(I,empty,2)
                                 -> arg(1,I,E),
                                    write(E)
                                  ; write(I)),
       nl, write('      rule: '),write(RuleName),
       nl, write(' # of dtrs: '),write(ND),
       nl
    ),
    clear_title.

% ------------------------------------------------------

% portray_unif_failure(Ref1,SVs1,Frozen1,Ref2,SVs2,Frozen2) :-
% portray_path_failure(Path1,Path2,Tag,SVs,Frozen) :-
% portray_feat_failure(Feat,Ref,SVs,Frozen) :-
% portray_macro_failure(MacroName,Ref,SVs,Frozen) :-
% portray_addtype_failure(Type,Ref,SVs,Frozen) :-
% portray_undef_type(Atom,Ref,SVs,Frozen) :-
% portray_desc_failure(Desc,Ref,SVs,Frozen) :-
% portray_featpath_failure(Feat,Path,Tag,SVs,Frozen) :-
% portray_edge_discard(LReln,I,Left,Right,Tag,SVs,RuleName,ND,Res) :-
% portray_edge_retract(I,Left,Right,ETag,ESVs,ERuleName,NED,ERes) :-
% portray_incoming_edge(Left,Right,Tag,SVs,RuleName,ND,Res) :-
% portray_edge(pending,Left,Right,TagOut,SVsOut,RuleName,ND,Res) :-
% portray_dtr_edge(D,DLeft,DRight,DTag,DSVs,DRule,DND,DRes) :-
% portray_ale_goal(Goal,Residue) :-
% portray_edge(I,M,N,Tag,SVs,RuleName,ND,Residue) :-
% portray_lex_rule(RuleName,Desc1,Desc2,Tag1,bot,Tag2,bot,Residue,Goal,Morphs) :-
% portray_ale_clause(Head,Body) :-
% portray_rule(TagMoth,bot,Dtrs,Residue) :-

% ------------------------------------------------------

% GBP - is ALE suppposed to call this?  It isn't...
portray_approps(Type,FVs) :-
   grale_flag,                  % fails immediately if flag not on
   unfill_off,
   get_output_stream(Stream),
   format(Stream,'!newdata"show_approp(~w)"',[Type]),
   Index0=1,
   beginType(Type,Index0,Index1,Stream,false),
   pa_feature_values(FVs,Index1,_FinalIndex,Stream),
   end(Stream),                 %'S'
   nl(Stream),
   flush_output(Stream),
   unfill_restore.

pa_feature_values([],Index,Index,_Stream).
pa_feature_values([Feat:Type|Rest],Index0,Index,Stream) :-
   pa_feature_value(Feat,Type,Index0,Index1,Stream),
   pa_feature_values(Rest,Index1,Index,Stream).

pa_feature_value(Feat,Type,Index0,Index,Stream):-
   begin(Stream,'V',Index0,Index1),
   writeFeat(Stream,Feat),
   beginType(Type,Index1,Index,Stream,false),
   end(Stream),                 % Type   
   end(Stream).                 %'V'

% ========================================================
% redefinition of pp_residue/7 until output of residues
% is integrated in ale.pl in a way allowin output in grisu

% Compared to the original in ale.pl, the only change in this predicate 
% is to switch grisu off for residue output so that the 
% default pretty printer handles them

% GBP - in other words, Grisu can't support residues?  Anyway,
%  ALE now has a portray_resgoal/7 hook.  The fact that you need
%  to define it this way indicates that something is wrong
%  - either portray_fs/11 should be passing you more information, or
%  you're not using the hooks properly.

% :- abolish(pp_residue/7).

portray_resgoal(Goal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut) :-
    grale_flag,
    get_output_stream(Stream),
    grale_portray_resgoal(Goal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).

% the original definition from ale.pl:
%original_pp_residue([],Dups,Dups,Vis,Vis,HD,HD).
%original_pp_residue([_-Goal|Residue],DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut) :-
%  pp_resgoal
%  original_pp_residue(Residue,DupsMid,DupsOut,VisMid,VisOut,HDMid,HDOut).

% ==============================================================================

% this catches the pp_fs/1 call in portray_cat/4
portray_fs(Type,FS,MGType,KeyedFeats,VisIn0,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut) :-
  % MLA: it seems that no one touches Vis when portray_fs is called
  % in the context of portray_cat. The trouble is that save_results
  % uses it to determine is it's a top-level structure and because
  % VisIn is always empty, every single FS is saved as result.
  (  empty_assoc(VisIn0)
  -> put_assoc(hack_save_results,VisIn0,for_diff,VisIn)
  ;  VisIn = VisIn0
  ),
  ( get_assoc(id_index,HDIn,ID0,HDMid,IDMid) -> PortrayTreeReEntrancies = false
  ; get_assoc(top_index,HDIn,ID0), get_assoc(tree_struc,HDIn,Tree),
    put_assoc(id_index,HDIn,IDMid,HDMid),
    PortrayTreeReEntrancies = true
    % this is a top-level call - prepare to print PS tree and re-entrancies below
  ),

  %( del_assoc(dup_descend,HDMid,ID1,HDMid1) -> Dup = nodup
  ( get_assoc(dup_descend,HDMid,ID1), ID1 \== no_assoc ->
    put_assoc(dup_descend,HDMid,no_assoc,HDMid1),
    Dup = nodup
    % dup_descend is a flag passed by portray_reentrancies/7 to indicate that we must descend one level
    % into the structure of re-entrant nodes while printing re-entrancies.
  ; get_assoc(FS,DupsIn,TagNum) -> Dup = dup, HDMid1 = HDMid, IDMid is ID0 + 1
  ; Dup = nodup, HDMid1 = HDMid
  ),
  
  ( Dup == dup -> name(TagNum,TagNumChars), name(ID0,ID0Chars),
                  append_list(["(#",ID0Chars," ",TagNumChars,")"],GraleString),
                  HDMid3 = HDMid1, VisMid = VisIn, DupsMid = DupsIn,
                  grale_write_chars(GraleString)
  ; Type == 0 -> ( MGType == 0 -> MGTypeString = "\"mgsat\""
		 ; approps(MGType,_,0) -> type_to_double_quoted_string(MGType,MGTypeString)
		 ; mgsat_to_double_quoted_string(MGType,MGTypeString)
		 ),
                 ( var(ID1) -> ID1 is ID0 + 1, IDMid is ID1 + 1 ; IDMid is ID0 + 1 ),
                 name(ID1,ID1Chars), name(ID0,ID0Chars),
                 append_list(["(S",ID1Chars,"(",ID0Chars,MGTypeString,"))"],GraleString),
                 HDMid3 = HDMid1, VisMid = VisIn, DupsMid = DupsIn,
                 grale_write_chars(GraleString)
  ; portray_fs_by_type(Type,IDMid,ID1,ID0,KeyedFeats,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid1,HDMid3)
  ),

  ( PortrayTreeReEntrancies == true -> portray_tree(Tree,DupsMid,HDMid3,HDMid4),
                                       portray_reentrancies(DupsMid,DupsOut,VisMid,VisOut,Col,HDMid4,HDOut)
  ; DupsOut = DupsMid, VisOut = VisMid, HDOut = HDMid3
  ).

portray_fs_by_type(e_list,IDMid,ID1,ID0,_EmptyList,Vis,Vis,Dups,Dups,_Col,HD,HD) :-
  !, (var(ID1) -> IDMid is ID0 + 1, UseID = ID0 ; UseID = ID1, IDMid = ID0),
  name(UseID,UseIDChars),
  append_list(["(L",UseIDChars,")"],GralePrefix),
  grale_write_chars(GralePrefix).
portray_fs_by_type(ne_list,IDMid,ID1,ID0,KeyedFeats,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid1,HDMid3) :-
  !, (var(ID1) -> IDMid is ID0 + 1, UseID = ID0 ; UseID = ID1, IDMid = ID0),
  name(UseID,UseIDChars),
  append_list(["(L",UseIDChars],GralePrefix),
  grale_write_chars(GralePrefix),
  portray_nelist_vs(KeyedFeats,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid1,HDMid3),
  grale_write_chars(")").
% HACK: for now, display polymorphically typed lists as normal feature structures.

portray_fs_by_type(sem,IDMid,ID1,ID0,KeyedFeats,Vis,Vis,Dups,Dups,_Col,HD,HD) :-
  ale_flag(cllrs,on),
  !, get_assoc(sem_residue,HD,SemResidue),
  memberchk(fval(sem,SemFS,_),KeyedFeats), SemFS = (a_ sem(Term,_)),
  memberchk(fval(pivot,PivotFS,_),KeyedFeats), PivotFS = (a_ sem(Pivot,_)),
  memberchk(fval(root,RootFS,_),KeyedFeats), RootFS = (a_ sem(Root,_)),
  ( var(ID1) ->  IDMid is ID0 + 1, UseID = ID0 ; UseID = ID1, IDMid = ID0),
  name(UseID,UseIDChars),
  \+ \+ (pp_sem(Term,SemResidue,SemFS,Pivot,Root,SemString),       % e.g., ^{every(x,A:[x],B:[x])}
%	    name(SemName,SemString), write(SemName), nl, % DEBUG
         append_list(["(A",UseIDChars,"\"::LRS::",SemString,"\")"],GralePrefix),
         grale_write_chars(GralePrefix)
	).  % HACK: for now, unwind this, so that we can print a given subterm again from another
            %  LF value.  We don't have a good way of representing structure sharing across underspecified
            %  terms right now, so we just start over with each one.
portray_fs_by_type(Type,IDMid,ID1,ID0,KeyedFeats,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid1,HDMid3) :-
  type_to_double_quoted_string(Type,TypeString),
  ( var(ID1) ->  ID1 is ID0 + 1, IDMid is ID1 + 1 ; IDMid is ID0 + 1),
  name(ID1,ID1Chars), name(ID0,ID0Chars),
  append_list(["(S",ID1Chars,"(",ID0Chars,TypeString,")"],GralePrefix),
  grale_write_chars(GralePrefix),
  sort_keyed_feats(Type,KeyedFeats,SortedKeyedFeats),
  portray_vs(SortedKeyedFeats,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid1,HDMid3).

portray_vs([],Vis,Vis,Dups,Dups,_,HD,HD) :-
  grale_write_chars(")").
portray_vs([fval(F,V,R)|KeyedFeats],VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut) :-
  get_assoc(id_index,HDIn,ID0,HDMid,IDMid),
  IDMid is ID0 + 1, name(ID0,ID0Chars), atom_to_double_quoted_string(F,FString),
  append_list(["(V",ID0Chars,FString],GraleFPrefix),
  grale_write_chars(GraleFPrefix),
  print_fs(_,V,R,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid,HDMid1),
  grale_write_chars(")"),
  portray_vs(KeyedFeats,VisMid,VisOut,DupsMid,DupsOut,Col,HDMid1,HDOut).

portray_nelist_vs(KeyedFeats,VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut) :-
  memberchk(fval(hd,V,R),KeyedFeats),  % HACK: ignore features other than hd and tl
  memberchk(fval(tl,VT,RT),KeyedFeats),
  print_fs(_,V,R,VisIn,VisMid,DupsIn,DupsMid,Col,HDIn,HDMid),
  get_type(VT,TailType),
  approps(TailType,FRs,_),
  build_keyed_feats(FRs,VT,TailKeyedFeats),
  portray_tail(TailType,VT,RT,TailKeyedFeats,VisMid,VisOut,DupsMid,DupsOut,Col,HDMid,HDOut).

portray_tail(0,FS,ne_list,_,Vis,Vis,Dups,Dups,_,HD,HD) :-
  !,
  ( get_assoc(FS,Dups,TagNum) -> get_assoc(id_index,HDIn,ID0,HDOut,IDMid), ID1 is ID0 + 1, IDMid is ID1 + 1,
                                 name(TagNum,TagNumChars), name(ID0,ID0Chars), name(ID1,ID1Chars),
                                 append_list(["(Z",ID0Chars,"(#",ID1Chars," ",TagNumChars,"))"],GraleString),
                                 grale_write_chars(GraleString)

  ; get_assoc(id_index,HDIn,ID0,HDOut,IDMid), ID1 is ID0 + 1, IDMid is ID1 + 1,
    name(ID0,ID0Chars), name(ID1,ID1Chars),
    append_list(["(Z",ID0Chars,"(A",ID1Chars,"\"...\"))"],GraleString),
    grale_write_chars(GraleString)
  ).
portray_tail(0,FS,e_list,_,Vis,Vis,Dups,Dups,_,HDIn,HDOut) :-
  !,
  ( get_assoc(FS,Dups,TagNum) -> get_assoc(id_index,HDIn,ID0,HDOut,IDMid), ID1 is ID0 + 1, IDMid is ID1 + 1,
                                 name(TagNum,TagNumChars), name(ID0,ID0Chars), name(ID1,ID1Chars),
                                 append_list(["(Z",ID0Chars,"(#",ID1Chars," ",TagNumChars,"))"],GraleString),
                                 grale_write_chars(GraleString)

  ; HDOut = HDIn
  ).
portray_tail(e_list,FS,_,_,Vis,Vis,Dups,Dups,_,HDIn,HDOut) :-
  !,
  ( get_assoc(FS,Dups,TagNum) -> get_assoc(id_index,HDIn,ID0,HDOut,IDMid), ID1 is ID0 + 1, IDMid is ID1 + 1,
                                 name(TagNum,TagNumChars), name(ID0,ID0Chars), name(ID1,ID1Chars),
                                 append_list(["(Z",ID0Chars,"(#",ID1Chars," ",TagNumChars,"))"],GraleString),
                                 grale_write_chars(GraleString)
  ; HDOut = HDIn
  ).
portray_tail(ne_list,FS,_,KeyedFeats,VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut) :-
  !,
  ( get_assoc(FS,DupsIn,TagNum) -> get_assoc(id_index,HDIn,ID0,HDOut,IDMid), ID1 is ID0 + 1, IDMid is ID1 + 1,
                                   name(TagNum,TagNumChars), name(ID0,ID0Chars), name(ID1,ID1Chars),
                                   append_list(["(Z",ID0Chars,"(#",ID1Chars," ",TagNumChars,"))"],GraleString),
                                   VisOut = VisIn, DupsOut = DupsIn,
                                   grale_write_chars(GraleString)
  ; portray_nelist_vs(KeyedFeats,VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut)
  ).
portray_tail(Type,FS,MGType,KeyedFeats,VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut) :-
  % this is the case that the type, list, uses, among possibly others
  get_assoc(id_index,HDIn,ID0,HDMid,IDMid), IDMid is ID0 + 1,
  name(ID0,ID0Chars),
  append_list(["(Z",ID0Chars],GralePrefix),
  grale_write_chars(GralePrefix),
  portray_fs(Type,FS,MGType,KeyedFeats,VisIn,VisOut,DupsIn,DupsOut,Col,HDMid,HDOut),
  grale_write_chars(")").

% HACK: we're really supposed to use GRALE ids rather than re-entrant tags here, but
% since trees must be marhsalled before re-entrancies, we don't have the ids yet.  So
% conflate ids with tags in all S-containers with re-entrancies.
portray_tree(tree(RuleName,SubWs,RootFS,SubTrees),Dups,HDIn,HDOut) :-
  get_assoc(id_index,HDIn,ID0,HDMid,IDMid),
  IDMid is ID0 + 1, name(ID0,ID0Chars),
  ( get_assoc(RootFS,Dups,LinkID) -> true    % if not in Dups, then we used TopIndex + 1
  ; get_assoc(top_index,HDMid,TopIndex), LinkID is TopIndex + 1
  ),			
  name(LinkID,LinkIDChars),
  name(RuleName,RuleNameChars),
  list_to_double_quoted_string(SubWs,[34|SubWsString]),  % 34 => '"', 58 => ':'
  append([34|RuleNameChars],[58|SubWsString],LabelString),
  append_list(["(T",ID0Chars," ",LabelString," ",LinkIDChars],TreePrefix),
  grale_write_chars(TreePrefix),
  portray_trees(SubTrees,Dups,HDMid,HDOut).
portray_trees([],_,HD,HD) :- grale_write_chars(")").
portray_trees([Tree|Trees],Dups,HDIn,HDOut) :-
  portray_tree(Tree,Dups,HDIn,HDMid),
  portray_trees(Trees,Dups,HDMid,HDOut).

portray_reentrancies(DupsIn,DupsOut,VisIn,VisOut,Col,HDIn,HDOut) :-
  assoc_to_list(DupsIn,DupsList),
  portray_reentr_act(DupsList,VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut).
portray_reentr_act([],Vis,Vis,Dups,Dups,_,HD,HD).
portray_reentr_act([FS-TagNum|DupsList],VisIn,VisOut,DupsIn,DupsOut,Col,HDIn,HDOut) :-
  get_assoc(id_index,HDIn,ID0,HDMid,IDMid),
  name(ID0,ID0Chars), name(TagNum,TagNumChars),
  IDMid is ID0 + 1,
  append_list(["(R",ID0Chars," ",TagNumChars],GralePrefix),
  grale_write_chars(GralePrefix),
  put_assoc(dup_descend,HDMid,TagNum,HDMid1),
  print_fs(_,FS,bot,VisIn,VisMid,DupsIn,DupsMid,Col,HDMid1,HDMid2),  % HACK: we need to store type context in HD
  grale_write_chars(")"),
  portray_reentr_act(DupsList,VisMid,VisOut,DupsMid,DupsOut,Col,HDMid2,HDOut).

:- dynamic grale_psrule_varlink/3.

relink_parse_tree(FS,Words,Index,tree(RuleName,SubWs,FS,SubTrees),[FS|TreeFSsMid],TreeFSsRest) :-
  get_edge_ref(Index,Left,Right,_,DtrNums,RuleName),
  substring(Left,Right,Words,SubWs),
  ( grale_psrule_varlink(Index,LinkedFS,(a_ LinkedDtrs))
  -> empty_assoc(EAssoc),
     edge_to_dtr_map(LinkedFS,FS,EAssoc,Map),
     map_dtrs(LinkedDtrs,Dtrs,Map),
     relink_parse_tree_dtrs(Dtrs,Words,DtrNums,SubTrees,TreeFSsMid,TreeFSsRest)
  ; SubTrees = [], TreeFSsRest = TreeFSsMid
  ).
relink_parse_tree_dtrs([],_,_,[],TreeFSs,TreeFSs).
relink_parse_tree_dtrs([Dtr|Dtrs],Words,[Num|Nums],[Tree|Trees],TreeFSs,TreeFSsRest) :-
  relink_parse_tree(Dtr,Words,Num,Tree,TreeFSs,TreeFSsMid),
  relink_parse_tree_dtrs(Dtrs,Words,Nums,Trees,TreeFSsMid,TreeFSsRest).

relink_parse_tree(FS,Words,Index,tree(RuleName,SubWs,FS,SubTrees),[FS|TreeFSsMid],TreeFSsRest,HdPos,TlPos) :-
  get_edge_ref(Index,Left,Right,_,DtrNums,RuleName),
  substring(Left,Right,Words,SubWs),
  ( grale_psrule_varlink(Index,LinkedFS,LinkedDtrs)
  -> empty_assoc(EAssoc),
     edge_to_dtr_map(LinkedFS,FS,EAssoc,Map),
     map_dtrs(LinkedDtrs,Dtrs,Map,HdPos,TlPos),
     relink_parse_tree_dtrs(Dtrs,Words,DtrNums,SubTrees,TreeFSsMid,TreeFSsRest,HdPos,TlPos)
  ; SubTrees = [], TreeFSsRest = TreeFSsMid
  ).
relink_parse_tree_dtrs([],_,_,[],TreeFSs,TreeFSs,_,_).
relink_parse_tree_dtrs([Dtr|Dtrs],Words,[Num|Nums],[Tree|Trees],TreeFSs,TreeFSsRest,HdPos,TlPos) :-
  relink_parse_tree(Dtr,Words,Num,Tree,TreeFSs,TreeFSsMid,HdPos,TlPos),
  relink_parse_tree_dtrs(Dtrs,Words,Nums,Trees,TreeFSsMid,TreeFSsRest,HdPos,TlPos).

edge_to_dtr_map(Edge,Dtr,AssocIn,AssocOut) :-
  ( get_assoc(Edge,AssocIn,_) -> AssocOut = AssocIn   % occurs check
  ; put_assoc(Edge,AssocIn,Dtr,AssocMid),
    ( compound(Edge) -> functor(Edge,Functor,Arity), functor(Dtr,Functor,Arity),
                        edge_to_dtr_map_args(0,Arity,Edge,Dtr,AssocMid,AssocOut)
    ; AssocOut = AssocMid
    )
  ).
edge_to_dtr_map_args(A,A,_,_,Assoc,Assoc) :- !.
edge_to_dtr_map_args(NPrev,A,Edge,Dtr,AssocIn,AssocOut) :-
  N is NPrev + 1,
  arg(N,Edge,EdgeArg), arg(N,Dtr,DtrArg),
  edge_to_dtr_map(EdgeArg,DtrArg,AssocIn,AssocMid),
  edge_to_dtr_map_args(N,A,Edge,Dtr,AssocMid,AssocOut).

map_dtr(In,Out,Map) :-
  empty_assoc(EAssoc),
  map_dtr(In,Out,Map,EAssoc).
map_dtr(In,Out,Map,Occurs) :-
  ( get_assoc(In,Occurs,Out) -> true
  ; ( get_assoc(In,Map,Out) -> true
    ; compound(In) -> functor(In,Functor,Arity), functor(Out,Functor,Arity),
   	              put_assoc(In,Occurs,Out,NewOccurs),
	              map_dtr_args(0,Arity,In,Out,Map,NewOccurs)
    ; Out = In, put_assoc(In,Occurs,Out,NewOccurs)  % must place put_assoc/4 call last in this case
    )                                               % or else variable In may move in standard order.
  ).
map_dtr_args(A,A,_,_,_,_) :- !.
map_dtr_args(NPrev,A,In,Out,Map,Occurs) :-
  N is NPrev + 1,
  arg(N,In,InArg), arg(N,Out,OutArg),
  map_dtr(InArg,OutArg,Map,Occurs),
  map_dtr_args(N,A,In,Out,Map,Occurs).

map_dtrs([],[],_).
map_dtrs([LD|LDs],[D|Ds],Assoc) :-
  map_dtr(LD,D,Assoc),
  map_dtrs(LDs,Ds,Assoc).
map_dtrs(LDs,Ds,Assoc,HdPos,TlPos) :-
  get_type(LDs,Type),
  ( sub_type(e_list,Type) -> Ds = []
  ; Ds = [D|DsRest], arg(HdPos,LDs,LD), arg(TlPos,LDs,LDsRest),
    map_dtr(LD,D,Assoc),
    map_dtrs(LDsRest,DsRest,Assoc,HdPos,TlPos)
  ).

% HACK: we really shouldn't be doing this, but in the current interface, there's no
%  way to describe parse tree nodes other than by specifying them in the re-entrancies list.
%  And because subtrees refer to GRALE ids, and the tree must be marshalled out before the re-entrancies,
%  we need to equate ids of S containers with re-entrant tags where they are re-entrant.
insert_duplicates_list([],Assoc,Assoc,Num,Num).
insert_duplicates_list([FS|FSs],AssocIn,AssocOut,NumIn,NumOut) :-
  ( get_assoc(FS,AssocIn,_) -> NumMid = NumIn, AssocMid = AssocIn
  ; put_assoc(FS,AssocIn,NumIn,AssocMid),
    NumMid is NumIn + 1
  ),
  insert_duplicates_list(FSs,AssocMid,AssocOut,NumMid,NumOut).

substring(N,M,Ws,SubWs) :-
  Diff is M - N,
  length(SubWs,Diff),
  length(Prefix,N),
  append(Prefix,Rest,Ws),
  append(SubWs,_,Rest).

atom_to_double_quoted_string(Atom,[34|Suffix]) :-
  name(Atom,AtomChars),
  append(AtomChars,[34],Suffix).

type_to_double_quoted_string((a_ Atom),Suffix) :-
  !,term_to_double_quoted_string(Atom,Suffix).
type_to_double_quoted_string(Atom,[34|Suffix]) :-
  name(Atom,AtomChars),
  append(AtomChars,[34],Suffix).

mgsat_to_double_quoted_string(Type,[34|Suffix]) :-
  name(Type,TypeChars),
  append_list(["mgsat(",TypeChars,")",[34]],Suffix).

list_to_double_quoted_string([],[34,34]).
list_to_double_quoted_string([W|Ws],[34|Suffix]) :-
  name(W,WChars),
  append(WChars,Rest,Suffix),
  ldqs_act(Ws,Rest).

ldqs_act([],[34]).
ldqs_act([W|Ws],[32|Suffix]) :-
  name(W,WChars),
  append(WChars,Rest,Suffix),
  ldqs_act(Ws,Rest).

term_to_double_quoted_string(Term,[34|Suffix]) :-
  term_to_double_quoted_string_act(Term,Suffix,[34]).

term_to_double_quoted_string_act(Var,Suffix,Rest) :-
  var(Var),
  !, Suffix = [95|Rest].  % KNOWN BUG: really should carry around names to label these with
term_to_double_quoted_string_act(Atom,Suffix,Rest) :-
  atomic(Atom),
  !, name(Atom,AtomString),
  append(AtomString,Rest,Suffix).
term_to_double_quoted_string_act(Term,Suffix,Rest) :-
  functor(Term,Functor,Arity),
  name(Functor,FunString),
  append(FunString,ArgString,Suffix),   % HACK: really should look up operator precedence
  args_to_double_quoted_string_act(0,Arity,Term,40,ArgString,[41|Rest]).

args_to_double_quoted_string_act(A,A,_,_,AString,AString) :- !.
args_to_double_quoted_string_act(N,A,Term,Start,[Start|ArgString],Rest) :-
  NewN is N + 1,
  arg(NewN,Term,Arg),
  term_to_double_quoted_string_act(Arg,ArgString,ArgStringMid),
  args_to_double_quoted_string_act(NewN,A,Term,44,ArgStringMid,Rest).

:- dynamic grdebug/1.

grale_write_chars(Chars) :-
  dget_output_stream(GraleOStream),
  name(String,Chars),
  write(GraleOStream,String).

grale_nl :-
  dget_output_stream(GraleOStream),
  nl(GraleOStream).

grale_flush_output :-
  dget_output_stream(GraleOStream),
  flush_output(GraleOStream).

dget_output_stream(Stream) :-
  ( clause(grdebug(Stream),true) -> true
  ; get_output_stream(Stream) % complements of ghooks.pl
  ).
      
append_list([],[]).
append_list([L|Ls],Result) :-
  append(L,Rest,Result),
  append_list(Ls,Rest).

% --- residue

begin_x(X,HD0,HD,S) :-
    get_assoc(id_index,HD0,ID0),
    begin(S,X,ID0,ID1),
    put_assoc(id_index,HD0,ID1,HD).
    
begin_relation(RelationName,HD0,HD,Stream) :-
    begin_x('D',HD0,HD,Stream),
    term_to_double_quoted_string(RelationName,RelationName1),
    name(RelationName2,RelationName1),
    write(Stream,RelationName2).

grale_portray_as_atom(Term,HD0,HD,Stream) :-
    begin_x('A',HD0,HD,Stream),
    term_to_double_quoted_string(Term,TermCodes),
    name(Name,TermCodes),
    write(Stream,Name),
    end(Stream).

grale_portray_resgoal(Goal,Dups,Dups,Vis,Vis,HD0,HD,Stream) :-
    (var(Goal) ; atomic(Goal)), !,
    grale_portray_as_atom(Goal,HD0,HD,Stream).

grale_portray_resgoal(when(_,user:when_type_delayed0(Type,FS,_,WGoal)),
        DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :- !,
    grale_pp_resgoal_act(Type,FS,WGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).
grale_portray_resgoal(prolog:when(_,user:when_type_delayed0(Type,FS,_,WGoal)),
        DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :- !,
    grale_pp_resgoal_act(Type,FS,WGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).    
grale_portray_resgoal((map_assoc(nv_fresh,_,_),(_=_),query_goal0(_,_,_,NBody,_,_)),DupsIn,DupsOut,
             VisIn,VisOut,HDIn,HDOut,Stream) :-
  !,grale_pp_goal(NBody,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).
grale_portray_resgoal((map_assoc(nv_fresh,_,_),query_goal0(_,_,_,NBody,_,_)),DupsIn,DupsOut,
             VisIn,VisOut,HDIn,HDOut,Stream) :-
  !,grale_pp_goal(NBody,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).
  
grale_portray_resgoal(Goal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
    Goal =.. [Functor | Args],
    begin_relation(Functor,HDIn,HDMid,Stream),
    grale_portray_resgoal_args(Args,DupsIn,DupsOut,VisIn,VisOut,HDMid,HDOut,Stream),
    end(Stream).

grale_portray_resgoal_args([],Dups,Dups,Vis,Vis,HD,HD,_Stream).   
grale_portray_resgoal_args([Arg|Args],DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
    grale_portray_resgoal(Arg,DupsIn,DupsMid,VisIn,VisMid,HDIn,HDMid,Stream),
    %xxx grale_pp_res_wgoal(Arg,DupsIn,DupsMid,VisIn,VisMid,HDIn,HDMid,Stream),
    grale_portray_resgoal_args(Args,DupsMid,DupsOut,VisMid,VisOut,HDMid,HDOut,Stream).
    
grale_pp_resgoal_act(Type,FS,WGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
    begin_relation(when_type,HDIn,HDMid,Stream),
    grale_portray_as_atom(Type,HDMid,HDMid1,Stream),
    grale_pp_fs(FS,DupsIn,DupsMid,VisIn,VisMid,HDMid1,HDMid2,Stream),
    grale_pp_res_wgoal(WGoal,DupsMid,DupsOut,VisMid,VisOut,HDMid2,HDOut,Stream),
    end(Stream).

%xxx
%grale_pp_res_wgoal((map_assoc(nv_fresh,_,_),(_=_),query_goal0(_,_,_,NBody,_,_)),DupsIn,DupsOut,
%             VisIn,VisOut,HDIn,HDOut,Stream) :-
%  !,grale_portray_resgoal(NBody,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).
%grale_pp_res_wgoal((map_assoc(nv_fresh,_,_),query_goal0(_,_,_,NBody,_,_)),DupsIn,DupsOut,
%             VisIn,VisOut,HDIn,HDOut,Stream) :-
%  !,grale_portray_resgoal(NBody,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).

grale_pp_res_wgoal((Arg1=Arg2),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  %xxx!, ( integer(Arg2) -> grale_portray_resgoal((Arg1=Arg2),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream)
  !, ( integer(Arg2) -> grale_portray_as_atom((Arg1=Arg2),HDIn,HDOut,Stream), DupsIn=DupsOut,VisIn=VisOut
     ; begin_relation((=),HDIn,HDMid,Stream),
       grale_pp_fs(Arg1,DupsIn,DupsMid,VisIn,VisMid,HDMid,HDMid1,Stream),
       grale_pp_fs(Arg2,DupsMid,DupsOut,VisMid,VisOut,HDMid1,HDOut,Stream),
       end(Stream)
     ).
grale_pp_res_wgoal(arg(N,FS1,FS2),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  !, begin_relation(arg,HDIn,HDMid,Stream),
  grale_portray_as_atom(N,HDMid,HDMid1,Stream),
  grale_pp_fs(FS1,DupsIn,DupsMid,VisIn,VisMid,HDMid1,HDMid2,Stream),
  grale_pp_fs(FS2,DupsMid,DupsOut,VisMid,VisOut,HDMid2,HDOut,Stream),
  end(Stream).
grale_pp_res_wgoal(deref(FS,TFS,Type,AttPos),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  !, begin_relation(deref,HDIn,HDMid,Stream),
  grale_pp_fs(FS,DupsIn,DupsOut,VisIn,VisOut,HDMid,HDMid1,Stream),
  grale_portray_as_atom(TFS,HDMid1,HDMid2,Stream),
  grale_portray_as_atom(Type,HDMid2,HDMid3,Stream),
  grale_portray_as_atom(AttPos,HDMid3,HDOut,Stream),
  end(Stream).
grale_pp_res_wgoal((G1 -> G2 ; G3),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  !,begin_relation(';',HDIn,HD1,Stream),
  begin_relation('->',HD1,HD2,Stream),
  grale_pp_res_wgoal(G1,DupsIn,DupsMid,VisIn,VisMid,HD2,HD3,Stream),
  grale_pp_res_wgoal(G2,DupsMid,DupsMid2,VisMid,VisMid2,HD3,HD4,Stream),
  end(Stream), % relation ->
  grale_pp_res_wgoal(G3,DupsMid2,DupsOut,VisMid2,VisOut,HD4,HDOut,Stream),
  end(Stream). % relation ;
grale_pp_res_wgoal((G1,G2),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  !,begin_relation(',',HDIn,HD1,Stream),
  grale_pp_res_wgoal(G1,DupsIn,DupsMid,VisIn,VisMid,HD1,HD2,Stream),
  grale_pp_res_wgoal(G2,DupsMid,DupsOut,VisMid,VisOut,HD2,HDOut,Stream),
  end(Stream).
grale_pp_res_wgoal(\+ G,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  !,begin_relation('\\+',HDIn,HD1,Stream),
  grale_pp_res_wgoal(G,DupsIn,DupsOut,VisIn,VisOut,HD1,HDOut,Stream),
  end(Stream).

grale_pp_res_wgoal(when_type(Type,FS,WGoal),DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  !,grale_pp_resgoal_act(Type,FS,WGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).

grale_pp_res_wgoal(CompGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
  nonvar(CompGoal), CompGoal =.. [CompRel|Args],
  name(CompRel,CompRelName),
  append("fs_",RelName,CompRelName),
  !,name(Rel,RelName), Goal =.. [Rel|Args],
  grale_pp_goal(Goal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).

%xxx
%grale_pp_res_wgoal(WGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream) :-
%  grale_portray_resgoal(WGoal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).

grale_pp_res_wgoal(WGoal,Dups,Dups,Vis,Vis,HDIn,HDOut,Stream) :-
  grale_portray_as_atom(WGoal,HDIn,HDOut,Stream).


% MLA: the code for grale_pp_goal code is adapted from from ale.pl

%grale_pp_goal(Goal,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream).

grale_pp_goal(X,Dups,Dups,Vis,Vis,HD0,HD,S):-
   (var(X) ; atomic(X)), !,
   grale_portray_as_atom(X,HD0,HD,S).
grale_pp_goal(\+ Goal,Dups0,Dups,Vis0,Vis,HD0,HD,S):-
  !, begin_relation((\+),HD0,HD1,S),
  grale_pp_goal(Goal,Dups0,Dups,Vis0,Vis,HD1,HD,S),
  end(S).
grale_pp_goal((G1 -> G2 ; G3),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !,begin_relation(';',HD0,HD1,S),
  begin_relation('->',HD1,HD2,S),
  grale_pp_goal(G1,Dups0,Dups1,Vis0,Vis1,HD2,HD3,S),
  grale_pp_goal(G2,Dups1,Dups2,Vis1,Vis2,HD3,HD4,S),
  end(S), % relation ->
  grale_pp_goal(G3,Dups2,Dups,Vis2,Vis,HD4,HD,S),
  end(S). % relation ;
grale_pp_goal((Goal1;Goal2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !,begin_relation(';',HD0,HD1,S),
  grale_pp_goal(Goal1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_goal(Goal2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_goal((Goal1,Goal2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !,begin_relation(',',HD0,HD1,S),
  grale_pp_goal(Goal1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_goal(Goal2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_goal(prolog(Hook),Dups,Dups,Vis,Vis,HD0,HD,S) :-
  !, grale_portray_as_atom(prolog(Hook),HD0,HD,S).
grale_pp_goal(prolog(NVs,Hook),Dups,Dups,Vis,Vis,HD0,HD,S) :-
  !, grale_portray_as_atom(prolog(NVs,Hook),HD0,HD,S).
grale_pp_goal(when(Cond,Goal),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !,begin_relation('when',HD0,HD1,S),
  grale_pp_cond(Cond,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_goal(Goal,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_goal(Desc1 =@ Desc2,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !, begin_relation('=@',HD0,HD1,S),
  grale_pp_desc(Desc1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_desc(Desc2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
%grale_pp_goal(true,Dups,Dups,Vis,Vis,_,HD,HD) :-
%  !, write(true).
%grale_pp_goal(!,Dups,Dups,Vis,Vis,_,HD,HD) :-
%  !, write(!).
grale_pp_goal((G1 -> G2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation('->',HD0,HD1,S),
  grale_pp_goal(G1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_goal(G2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
%grale_pp_goal(fail,Dups,Dups,Vis,Vis,_,HD,HD) :-
%  !, write(fail).
grale_pp_goal((Desc1 = Desc2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !, begin_relation('=',HD0,HD1,S),
  grale_pp_desc(Desc1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_desc(Desc2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_goal(Goal,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  Goal =.. [Rel|Args],
  begin_relation(Rel,HD0,HD1,S),
  grale_pp_goal_args(Args,Dups0,Dups,Vis0,Vis,HD1,HD,S),
  end(S).

grale_pp_goal_args([],Dups,Dups,Vis,Vis,HD,HD,_S).
grale_pp_goal_args([Arg|Args],Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  grale_pp_desc(Arg,Dups0,Dups1,Vis0,Vis1,HD0,HD1,S),
  grale_pp_goal_args(Args,Dups1,Dups,Vis1,Vis,HD1,HD,S).

grale_pp_cond(X^(Cond),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation('^',HD0,HD1,S),
  grale_pp_desc(X,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),        
  grale_pp_cond(Cond,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_cond((C1,C2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation(',',HD0,HD1,S),
  grale_pp_cond(C1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_cond(C2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_cond((C1;C2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation(';',HD0,HD1,S),
  grale_pp_cond(C1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_cond(C2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_cond(FS=Desc,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation('=',HD0,HD1,S),
  grale_pp_desc(FS,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_desc(Desc,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).

grale_pp_desc(X,Dups,Dups,Vis,Vis,HD0,HD,S) :-
  atomic(X),
  !, grale_portray_as_atom(X,HD0,HD,S).
grale_pp_desc(X,Dups0,Dups,Vis0,Vis,HD0,HD,_S) :-
  var(X),
  !, pp_fs(X,Dups0,Dups,Vis0,Vis,0,HD0,HD).
grale_pp_desc(FS,Dups0,Dups,Vis0,Vis,HD0,HD,_S) :-
  functor(FS,Module,Arity),
  clause(marity(Module,Arity),true),
  !,
  pp_fs(FS,Dups0,Dups,Vis0,Vis,0,HD0,HD).
%pp_desc([],Dups,Dups,Vis,Vis,_,HD,HD) :-
%  !,write([]).
grale_pp_desc([H|T],Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !, begin_x('L',HD0,HD1,S),
  grale_pp_desc(H,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_tail(T,Dups1,Dups,Vis1,Vis,HD2,HD,S).
grale_pp_desc(F:Desc,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !, begin_x('S',HD0,HD01,S), begin_x('V',HD01,HD1,S),
  writeqt(S,F),
  grale_pp_desc(Desc,Dups0,Dups,Vis0,Vis,HD1,HD,S),
  end(S), end(S).
grale_pp_desc(@ Macro,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation('@',HD0,HD1,S),
  pp_desc(Macro,Dups0,Dups,Vis0,Vis,HD1,HD),
  end(S).
                                % will fall through to function clause
grale_pp_desc((D1,D2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !,begin_relation(',',HD0,HD1,S),
  grale_pp_desc(D1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_desc(D2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_desc((D1;D2),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  !,begin_relation(';',HD0,HD1,S),
  grale_pp_desc(D1,Dups0,Dups1,Vis0,Vis1,HD1,HD2,S),
  grale_pp_desc(D2,Dups1,Dups,Vis1,Vis,HD2,HD,S),
  end(S).
grale_pp_desc((=\= Desc),Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  % xxx should probably do something else here?
  !,begin_relation('=\=',HD0,HD1,S),
  grale_pp_desc(Desc,Dups0,Dups,Vis0,Vis,HD1,HD,S),
  end(S).
grale_pp_desc(Path1 == Path2,Dups,Dups,Vis,Vis,HD0,HD,S) :-
  !,begin_relation('==',HD0,HD1,S),
  grale_portray_as_atom(Path1,HD1,HD2,S),
  grale_portray_as_atom(Path2,HD2,HD,S),
  end(S).
grale_pp_desc(a_ X,Dups,Dups,Vis,Vis,HD0,HD,S) :-
  !,begin_relation('a_',HD0,HD1,S),
  grale_portray_as_atom(X,HD1,HD,S),
  end(S).
grale_pp_desc(Other,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
                                % handles types, functions and macros
  Other =.. [Head|Args],
  begin_relation(Head,HD0,HD1,S),
  grale_pp_descs(Args,Dups0,Dups,Vis0,Vis,HD1,HD,S),
  end(S).

grale_pp_descs([D|Ds],DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,S) :-
  grale_pp_desc(D,DupsIn,DupsMid,VisIn,VisMid,HDIn,HDMid,S),
  grale_pp_descs(Ds,DupsMid,DupsOut,VisMid,VisOut,HDMid,HDOut,S).

grale_pp_tail([],Dups,Dups,Vis,Vis,HD,HD,S) :-
  !,end(S).
grale_pp_tail([H|T],DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,S) :-
  !,grale_pp_desc(H,DupsIn,DupsMid,VisIn,VisMid,HDIn,HDMid,S),
  grale_pp_tail(T,DupsMid,DupsOut,VisMid,VisOut,HDMid,HDOut,S).
grale_pp_tail(NonList,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_x('Z',HD0,HD1,S),
  grale_pp_desc(NonList,Dups0,Dups,Vis0,Vis,HD1,HD,S),
  end(S), % tail
  end(S). % list

% -- end of grale_pp_goal related code

grale_pp_fs(FS,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,_Stream) :-
  pp_fs(FS,DupsIn,DupsOut,VisIn,VisOut,0,HDIn,HDOut)
   %-> true
  %; grale_portray_resgoal(FS,DupsIn,DupsOut,VisIn,VisOut,HDIn,HDOut,Stream)
  .
  %; grale_portray_as_atom(pp_fs_failed:FS,HDIn,HDOut,Stream),
  %  DupsOut = DupsIn, VisOut = VisIn.

% --- inequations
grale_pp_iqs([],Dups,Dups,Vis,Vis,HD,HD) :- !.
grale_pp_iqs(Iqs,Dups0,Dups,Vis0,Vis,HD0,HD) :-
  get_output_stream(S),
  write(S,'-'),
  grale_pp_iqs_(Iqs,Dups0,Dups,Vis0,Vis,HD0,HD,S).

grale_pp_iqs_([],Dups,Dups,Vis,Vis,HD,HD,_S).
grale_pp_iqs_([ineq(FS1,FS2)|Iqs],Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
  begin_relation('=\\=',HD0,HD1,S),
  grale_pp_fs(FS1,Dups0,Dups1,Vis0,Vis1,HD0,HD1,S),
  grale_pp_fs(FS2,Dups1,Dups2,Vis1,Vis2,HD1,HD2,S),
  end(S),
  grale_pp_iqs_(Iqs,Dups2,Dups,Vis2,Vis,HD2,HD,S).

portray_rule(RuleName,FSMoth,Dtrs,DtrCats,Residue) :-
    grale_flag,

    CatsOut = [FSMoth|DtrCats],
    build_iqs_restore_atts(Residue,Iqs,FSResidue),
    (   ale_flag(residue,show)
    ->  residue_args(FSResidue,ResArgs,CatsOut)
    ;   ResArgs = CatsOut
    ),

    empty_assoc(EmptyAssoc),
    duplicates_list(ResArgs,EmptyAssoc,DupsMid,EmptyAssoc,VisMid,0,NumMid),
    duplicates_iqs(Iqs,DupsMid,DupsMid2,VisMid,_,NumMid,Num),

    put_assoc(id_index,EmptyAssoc,Num,HD0),

    begin_grisu_output(S,(rule):RuleName),

    grale_write_list(['["Rule: ', RuleName, '"-""'],S), % table RuleName as header
    grale_pp_fs(FSMoth,DupsMid2,DupsMid3,EmptyAssoc,VisMid2,HD0,HDMid,S),

    write(S, '"--->"[-'), % table without a header
    grale_show_rule_dtrs(Dtrs,DupsMid3,DupsMid4,VisMid2,VisMid3,HDMid,HDMid2,S),

    write(S,']]'),
    
    portray_reentrancies(DupsMid4,DupsOut,VisMid3,VisOut,0,HDMid2,HDOut),
    grale_pp_complete(Iqs,FSResidue,DupsOut,VisOut,HDOut).
    
grale_pp_complete(Iqs,FSResidue,Dups0,Vis0,HD0) :-
    grale_pp_iqs(Iqs,Dups0,Dups1,Vis0,Vis1,HD0,HD1),
    grale_pp_residue(FSResidue,Dups1,_,Vis1,_,HD1,_), 
    grale_nl, grale_flush_output,
    clear_title.

grale_pp_residue(FSResidue,Dups0,Dups,Vis0,Vis,HD0,HD) :-
    (   (ale_flag(residue,show),FSResidue \== [])
    ->  grale_write_chars("/"), pp_residue(FSResidue,Dups0,Dups,Vis0,Vis,HD0,HD)
    ;   Dups = Dups0, Vis = Vis0, HD = HD0
    ).


grale_show_rule_dtrs([],Dups,Dups,Vis,Vis,HD,HD,_S).
grale_show_rule_dtrs([goal>Goal|Dtrs],Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
    !,grale_write_list(['"', goal, '"'],S),
    grale_pp_goal(Goal, Dups0, Dups1, Vis0, Vis1, HD0, HD1, S),
    grale_show_rule_dtrs(Dtrs,Dups1,Dups,Vis1,Vis,HD1,HD,S).
grale_show_rule_dtrs([Label>FS|Dtrs],Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
    grale_write_list(['"', Label, '"'],S),
    pp_fs(FS, Dups0, Dups1, Vis0, Vis1, 0, HD0, HD1),
    grale_show_rule_dtrs(Dtrs,Dups1,Dups,Vis1,Vis,HD1,HD,S).
    

grale_portray_goal(Goal,Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
    Goal =.. [Functor | FSs],
    begin_relation(Functor,HD0,HD1,S),
    grale_portray_goal_args(FSs,Dups0,Dups,Vis0,Vis,HD1,HD,S),
    end(S).

grale_portray_goal_args([],Dups,Dups,Vis,Vis,HD,HD,_S).
grale_portray_goal_args([FS|FSs],Dups0,Dups,Vis0,Vis,HD0,HD,S) :-
    pp_fs(FS, Dups0, Dups1, Vis0, Vis1, 0, HD0, HD1),
    grale_portray_goal_args(FSs,Dups1,Dups,Vis1,Vis,HD1,HD,S).


grale_write_list([],_S).
grale_write_list([H|T],S) :-
    write(S,H),
    grale_write_list(T,S).


portray_lex_rule(RuleName,FS1,FS2,Residue,Goal,Morphs,ArgsOut,Cond) :-
    grale_flag,

    build_iqs_restore_atts(Residue,Iqs,FSResidue),
    (   ale_flag(residue,show)
    ->  residue_args(FSResidue,ResArgs,ArgsOut)
    ;   ResArgs = ArgsOut
    ),

    empty_assoc(Empty),
    duplicates_list(ResArgs,Empty,Dups0,Empty,Vis0,0,Num0),
    duplicates_iqs(Iqs,Dups0,Dups1,Vis0,_Vis1,Num0,Num),

    put_assoc(id_index,Empty,Num,HD0),

    begin_grisu_output(S,(lex_rule):RuleName),
    
    % table, RuleName as header
    grale_write_list(['["Lexical rule: ', RuleName, '"-"input category"'],S),
    grale_pp_fs(FS1,Dups1,Dups2,Empty,Vis2,HD0,HD1,S),

    write(S,'"output category"'),
    grale_pp_fs(FS2,Dups2,Dups3,Vis2,Vis3,HD1,HD2,S),

    (   Cond == true
    ->  Dups3 = Dups4, Vis3 = Vis4, HD2 = HD3
    ;   write(S,'"condition"'),
        grale_pp_goal(Goal,Dups3,Dups4,Vis3,Vis4,HD2,HD3,S)
    ),
    
    write(S,'"morphs"[-'),
%    grale_portray_as_atom(Morphs,HD3,HD4,S),
    numbervars(Morphs,0,_),
    grale_pp_morphs(Morphs,HD3,HD4,S),
   /*write('\n\n\n'),
   write(Morphs),
   write('\n\n\n'),
   HD3=HD4,*/

    write(S,']]'),
    Dups = Dups4, Vis = Vis4, HD = HD4,
    portray_reentrancies(Dups,DupsOut,Vis,VisOut,0,HD,HDOut),
    grale_pp_complete(Iqs,FSResidue,DupsOut,VisOut,HDOut).

grale_pp_morphs((Morph,Morphs),HD0,HD,S):-
  grale_pp_morph(Morph,HD0,HD1,S),
  grale_pp_morphs(Morphs,HD1,HD,S).
grale_pp_morphs(Morph,HD0,HD,S):-
  grale_pp_morph(Morph,HD0,HD,S).

grale_pp_morph((P1 becomes P2),HD0,HD,S):-
  write(S,'""'),        % empty first column
  begin_relation(becomes,HD0,HD1,S),
  grale_pp_patt(P1,HD1,HD2,S),
  grale_pp_patt(P2,HD2,HD, S),
  end(S).
grale_pp_morph((P1 becomes P2 when Cond),HD0,HD,S):-
  write(S,'""'),
  begin_relation(when,HD0,HD1,S),
  grale_portray_as_atom(Cond,HD1,HD2,S),
  begin_relation(becomes,HD2,HD3,S),
  grale_pp_patt(P1,HD3,HD4,S),
  grale_pp_patt(P2,HD4,HD,S),
  end(S),       % end 'becomes'
  end(S).       % end 'when'
        
grale_pp_patt((X,Xs),HD0,HD,S):-
  !,begin_relation(',',HD0,HD1,S),
  grale_pp_at_patt(X,HD1,HD2,S),
  grale_pp_patt(Xs,HD2,HD,S),
  end(S).
grale_pp_patt(X,HD0,HD,S):-
  grale_pp_at_patt(X,HD0,HD,S).

grale_pp_at_patt(Atom,HD0,HD,S):-
 atom(Atom),
  !, name(Atom,Codes),
  make_char_list(Codes,Chars),
  grale_portray_as_list_of_atoms(Chars,HD0,HD,S).
grale_pp_at_patt([H|T],HD0,HD,S):-
%  grale_portray_as_atom(List,HD0,HD,S).
  !, grale_portray_as_list_of_atoms([H|T],HD0,HD,S).

grale_pp_at_patt(X,HD0,HD,S):-
  grale_portray_as_atom(X,HD0,HD,S).

grale_portray_as_list_of_atoms(List,HD0,HD,S) :-
   begin_x('L',HD0,HD1,S),
   grale_portray_as_list_of_atoms_(List,HD1,HD,S).
grale_portray_as_list_of_atoms_([],HD,HD,S) :- end(S).
grale_portray_as_list_of_atoms_([C|Cs],HD0,HD,S) :-
  grale_portray_as_atom(C,HD0,HD1,S),
  grale_portray_as_list_of_atoms_(Cs,HD1,HD,S).

% ========================================================
%% -*-Mode: prolog;-*-

