:- module(subterm,[node/4,literal/4,st/3,ist/4,anchor/1,semvars/1,semvars_update/1
		   ,findall_constraints/3  % expanded into code by CHR
		   ]).

:- use_module(library(chr)).

%:- use_module(library(lists),[nth/3]).
:- use_module(library(assoc)).

%handler subterm.
%constraints node/4,literal/4,st/3,ist/4,ststar/3,anchor/1,semvars/1,semvars_update/1.
:- chr_constraint node/4,literal/4,st/3,ist/4,ststar/3,anchor/1,semvars/1,semvars_update/1.

%option(debug_compile,on).
:- chr_option(debug,on).

:- discontiguous attribute_goal/2.
attribute_goal(AttVar,semres(Att)) :-
  '$get_attributes'(AttVar,Att,_),
  nonvar(Att).

% the swi version of the above
attribute_goals(V, [semres(Att) | Tail], Tail) :-
    get_attr(V, subterm, Att), nonvar(Att), !.
  
% semvars/1
uniqueness @ semvars(Vs1) # Id \ semvars(Vs2) <=> Vs1 = Vs2 pragma passive(Id).
initialise @ semvars(Vs) ==> var(Vs) | empty_assoc(Vs).

% semvars_update/1
update @ semvars(_VsOld) # Id, semvars_update(VsNew) <=> semvars(VsNew)
           pragma passive(Id).

% anchor/1
uniqueness @ anchor(N) # Id \ anchor(M) <=> N=M
          pragma passive(Id).

% node(N,ArgTypes,RType,A):  N maps Args of ArgTypes to RType with anchor A
type_consistency @ node(N,ATypes1,RType1,_) # Id \ node(N,ATypes2,RType2,_)
                   <=> RType1 = RType2, ATypes1 = ATypes2
                               pragma passive(Id).

% root(X,N,A):  The root of X is N, with anchor A
%dereference @ root(X,N,A) <=> nonvar(X) | deref(X,Ref,_),root(Ref,N,A).
%function @ root(X,N,_) # Id \ root(X,M,_) <=> N=M
%          pragma passive(Id).

% pivot(X,N,A): The pivot of X is N, with anchor A
%dereference @ pivot(X,N,A) <=> nonvar(X) | deref(X,Ref,_),pivot(Ref,N,A).
%function @ pivot(X,N,_) # Id \ pivot(X,M,_) <=> N=M
%           pragma passive(Id).

% literal(N,Literal,Arity,A): N has functor Literal/Arity, with Anchor A
consistency @ literal(N,F1,A1,_) # Id \ literal(N,F2,A2,_) <=> F1 = F2, A1 = A2
                    pragma passive(Id).
well_typedness @ node(N,ATypes,_,_) # Id, literal(N,_,A,_) ==> length(ATypes,A)
                       pragma passive(Id).
extensionality @ literal(N,F,A,_) # Id, literal(M,F,A,Anchor) ==> ext_arg(A,N,M,Anchor)
                     pragma passive(Id).

% ist/4
irreflexivity @ ist(N,N,_,_) ==> fail.
%node_uniqueness @ ist(M,N,A1,_) # Id \ ist(M,N,A2,_) <=> A1 = A2
%                   pragma passive(Id).
%  - metavars can now occur in multiple arguments of the same term
arg_uniqueness @ ist(M1,N,A,_) # Id \ ist(M2,N,A,_) <=> M1 = M2
                  pragma passive(Id).
well_typedness @ node(M,_,MResType,_) # MId, node(N,NArgTypes,_,_) # NId,
                ist(M,N,A,_) ==> nth(A,NArgTypes,MResType)
                         pragma passive(MId),                passive(NId).
unit @ ist(L,M,_,A) ==> ststar(L,M,A).  % was st(L,M,A) - had to add atomicity_ist and
                                        % transitivity_ist below

% st/3
reflexivity @ st(N,N,_) <=> true.
idempotence @ st(M,N,_) # Id \ st(M,N,_) <=> true
           pragma passive(Id).
subsumption @ ist(M,N,_,_) \ st(M,N,_) <=> true.
antisymmetry @ st(M,N,_) # Id, st(N,M,_) <=> M = N  % maybe don't need this - ststar has antisym.
            pragma passive(Id).
sub_antisym @ ist(M,N,_,_), st(N,M,_) <=> M = N.    % maybe don't need this - ststar has antisym.
constant @ literal(N,_,0,_) \ st(M,N,_) <=> M = N.
unit @ st(L,M,A) ==> ststar(L,M,A).


% ststar/3
reflexivity @ ststar(N,N,_) <=> true.  % may need to get rid of this so
            % that we can use ststar/3 on LHS of universally quantified impl's
idempotence @ ststar(M,N,_) # Id \ ststar(M,N,_) <=> true
               pragma passive(Id).
antisymmetry @ ststar(M,N,_), ststar(N,M,_) # Id <=> M=N
                               pragma passive(Id).
% atomicity_ist @ ist(L,M,_,_), ststar(M,N,_), ist(L,N,_,_) ==> fail.
%  - metavars can now occur in multiple arguments of the same term
%  - even if they couldn't the right action would be to equate M=N, not fail.
% atomicity_st @ st(L,M,_), ststar(M,N,_), ist(L,N,_,_) ==> fail.
%  - even if metavars couldn't occur in multiple arguments, this one would
%    have no deterministic consequences
transitivity_st @ st(L,M,A), ststar(M,N,A) ==> ststar(L,N,A).
base_st @ st(L,M,_), ststar(M,N,_) \ st(L,N,_) <=> L \== M | true.
base_ist @ ist(L,M,_,_), ststar(M,N,_) \ st(L,N,_) <=> L \== M | true.
transitivity_ist @ ist(L,M,_,A), ststar(M,N,A) ==> ststar(L,N,A).

%--------------------------------------------------------------------------------
% use this to fork checks for argument identity
ext_arg(A,N,M,Anchor) :-
  functor(E,e,A), when(ground(E),N=M),
  ext_arg_act(0,A,E,N,M,Anchor).

ext_arg_act(A,A,_,_,_,_) :- !.
ext_arg_act(I,A,E,N,M,Anchor) :-
  NewI is I + 1,
  arg(NewI,E,EArg),
  ist(NArg,N,NewI,Anchor),
  ist(MArg,M,NewI,Anchor),
  when(?=(NArg,MArg),(NArg==MArg -> EArg = 1 ; true)),
  ext_arg_act(NewI,A,E,N,M,Anchor).

