%%% ==================================================================
%%% $Id: socket_interface.pl,v 1.6 2005/03/29 22:03:25 stefan Exp $
%%%
%%% Communication over sockets: a prolog client
%%%
%%% Author: Detmar Meurers
%%%         based on his code originally written June 1997 for the
%%%         communication between ConTroll and CUF, and other code
%%%         by Frederik Fouvry.
%%%
%%% ==================================================================

:- use_module(library(socket), [
    tcp_socket/1
    , tcp_connect/2
    , tcp_open_socket/3
    , tcp_close_socket/1
    ]).

%%% --------------------------------------------------------------- %%

:- dynamic interface/2.

start_client:-
   % does not work in runtime executables
   % current_prolog_flag(argv,[Hostname,Atom]),
   (
      interface(Hostname,Atom)
      ;
      (environ('INTERFACE_HOST', Hostname), environ('INTERFACE_PORT', Atom))
   ),
   % two calls to name/2 to turn atom '234' into number 234
   % but leaving file socket names '/tmp/...' unchanged
   name(Atom,Codes),		
   name(PortOrFilename,Codes),
   start_client(Hostname,PortOrFilename), !,
   set_client_encoding.

% if the above fails, try spawning gralej as process,
% redirecting its streams
start_client :-
    start_gralej,
    set_client_encoding.

start_gralej :-
    absolute_file_name(path(java), JavaExeFilename),
    absolute_file_name(trale_home('gralej/gralej.jar'), GralejJarFilename),
    process_create(JavaExeFilename, ['-jar', GralejJarFilename, '--stdin'], [
        stdin(pipe(GralejStdIn)),
        stdout(std),
        stderr(std),
        process(Pid)
        ]
    ),
    asserta(socket_info(gralej_pid(Pid), GralejStdIn)),
    at_halt(shutdown_gralej).

shutdown_gralej :-
    (   retract(socket_info(gralej_pid(_Pid), Stream)),
        catch(close(Stream), _, fail),
        fail
    ;   true
    ).

restart_client :-
    catch(stop_client, _, true),
    start_client.

% start_client(+PortOrFilename)
% e.g., start_client(5002) or start_client('/tmp/fileXYZ').
start_client(PortOrFilename) :-
   start_client(localhost,PortOrFilename).

% start_client(+Hostname,+PortOrFilename)
% e.g., start_client(horatio,5002) or start_client(localhost,'/tmp/fileXYZ')
start_client(Hostname,PortOrFilename) :-
   (  number(PortOrFilename)
   -> start_client_on_port(Hostname,PortOrFilename)
   ;  not_implemented(start_client_on_filesocket/1)
   ).
  
stop_client:-
   shutdown_gralej,
   (  (retract(socket_info(Socket,_)),
       catch(tcp_socket_close(Socket),
	     _Ex,
	     print_message(warning,trale(socket(disconnect,failed)))))
   ->  print_message(informational,trale('Client stopped'))
   ;   print_message(warning,trale('No client information found.'))
   ).

% to be called after assering socket_info
set_client_encoding :-
    (   environ('GRISU_STREAM_ENCODING', Encoding),
        socket_info(_, Stream)
    ->  set_stream(Stream, encoding(Encoding))
    ;   true
    ).

%%% --------------------------------------------------------------- %%
% auxiliary predicates:

:- dynamic socket_info/2.
:- volatile socket_info/2.

start_client_on_port(Hostname,Port) :-
   format('Establishing connection to interface on port ~p of ~p.',[Port,Hostname]),
   flush_output,
   tcp_socket(Socket),	% This creates the Socket
   catch(tcp_connect(Socket,Hostname:Port),
         _Ex,
        (format(user_error,'~N**Error: Connection refused to port ~w of host ~w.~n',[Port,Hostname]),
         fail)
        ),
   tcp_open_socket(Socket,_InputStream,OutputStream),
   asserta(socket_info(Socket,OutputStream)),
   format('~NConnection established.~n',[]).

