:- dynamic trace_term_id/1.
:- dynamic trace_message_id/1.

trace_term_id(_).
trace_message_id(_).

traceall :-
    notraceall,
    assert(trace_term_id(_)),
    assert(trace_message_id(_)).

notraceall :-
    retractall(trace_term_id(_)),
    retractall(trace_message_id(_)).

dbg_trace_term(Term) :-
    current_prolog_flag(debug, on) -> trace_term(Term)
    ; true.
dbg_trace_term(Term, Depth) :-
    current_prolog_flag(debug, on) -> trace_term(Term, Depth)
    ; true.

trace_term(Term) :-
    trace_term(Term, 100).
trace_term(Term, Depth) :-
    ale_next_id(term, Id),
    (   trace_term_id(Id)
    ->  format("-- term [~w]: ", [Id]),
        write_term(Term, [max_depth(Depth)]),
        nl
    ;   true
    ).

trace_message(_Msg) :-
    trace_message_id(_) -> fail
    ; !.
trace_message(Msg) :-
    ale_next_id(trace, Id),
    (   trace_message_id(Id)
    ->  format("-- trace message [~w]: ~w~n", [Id, Msg])
    ;   true
    ).

trace_message(_,_) :-
    trace_message_id(_) -> fail
    ; !.
trace_message(Fmt, Args) :-
    ale_next_id(trace, Id),
    (   trace_message_id(Id)
    ->  append("-- trace message [~w]: ", Fmt, Fmt1),
        format(user_error, Fmt1, [Id | Args])
    ;   true
    ).

dbg_trace_message(Fmt, Args) :-
    current_prolog_flag(debug, on) -> trace_term(Fmt, Args)
    ; true.

m(Msg) :-
    trace_message(Msg).

trace_goal(GoalId, Goal) :-
    trace_message("goal: ~w~n", [GoalId]),
    (   call(Goal)
    ->  trace_message("goal: ~w succeeded~n", [GoalId])
    ;   trace_message("goal: ~w failed~n", [GoalId]),
        fail
    ).
