%%%%%%%%%%%%%%%%%%%%%
% TRALE 1.0
% Gerald Penn
% University of Toronto

:- ensure_loaded(ale).

%:- ale_abolish([memberchk/2,member/2,reverse/2,append/3,ord_union/3,get_assoc/3,
%	    put_assoc/4,ord_intersect/2,ord_intersection/3,ord_add_element/3,
%	    is_list/1,ord_subtract/3]).
:- compile(sics).

:- multifile file_search_path/2.
:- dynamic file_search_path/2.
trale_search_path_add(Name,Path) :-
          (clause(file_search_path(Name,Path),true) -> true
          ;   assertz(file_search_path(Name,Path))).

:- prolog_load_context(directory,TraleHome),
   assert(file_search_path(trale_home,TraleHome)).
   
:-  (   environ('TRALE_HOME',Path)
    ->  trale_search_path_add(trale_home,Path),
        trale_search_path_add(common,Path)
    ;   true
    ).

:- compile(utilities).
:- compile(pth).

:- use_module(library(ugraphs),[vertices_edges_to_ugraph/3,neighbours/3,
                                top_sort/2]).
:- use_module(library(lists),[remove_duplicates/2]).
:- use_module(library(terms),[variant/2,term_subsumer/3]).
:- use_module(library(ordsets),[ord_union/2]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compiler patches
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile abolish_user_preds/1.
abolish_user_preds(sig) :- ale_abolish((signature)/1).
abolish_user_preds(cons) :- ale_abolish(('*>')/2), ale_abolish((trale_cons)/2).
abolish_user_preds(fun) :- ale_abolish((':=')/2), ale_abolish((macro)/2).

%
% Flags
%

:- dynamic core_ale_flags/1.
:- ale_flags(ALE_Flags),
   assert(core_ale_flags(ALE_Flags)),
   ale_abolish(ale_flags/1).
ale_flags([subintro,cllrs|Core_Flags]) :-
  clause(core_ale_flags(Core_Flags),true).
  
:- multifile ale_flag_msg/3.
ale_flag_msg(subintro,file,'{ALE: compiler will define sub/2 and intro/2 from signature file}').
ale_flag_msg(subintro,grammar,'{ALE: compiler will expect grammars to define sub/2 and intro/2}').
ale_flag_msg(cllrs,on,'{ALE: CL-LRS is active}') :- initialise_cllrs.
ale_flag_msg(cllrs,off,'{ALE: CL-LRS is inactive}').

:- multifile flag_modes/2.
flag_modes(subintro,[file,grammar]).
flag_modes(cllrs,[on,off]).

:- ale_abolish([initialise_ale_sld/0]).
initialise_ale_sld :- ensure_loaded(trale_home(trdebug)).

initialise_cllrs :- ensure_loaded(trale_home(sem)).
	
initialise_trale_flags :-
  assert(ale_flag(subintro,file)),
  assert(ale_flag(cllrs,off)),
  ale_flag(subtypecover,_,on).

%
% Signature patches
%

% no incremental signature compilation because of TRALE signature file
:- ale_abolish([compile_sub_type/1,compile_sub_type/0,
            compile_approp/1,compile_approp/0]).

:- ale_abolish(compile_sig_act/0).
compile_sig_act :-
  ( ale_flag(subintro,file) -> alec_announce('Reading signature file...'),
                               assert(alec(trale_signature)),
                               compile('.alec_throw')
  ; true
  ),
  compile_sub_type_act(_,_,_,_), % (SortedSubIntros,SortedIntros,SortedExts,STMatrix),
  compile_approp_act, % (SortedSubIntros,SortedIntros,STMatrix),
  compile_extensional_act, % (SortedExts).
  compile_modules_act,
  ( ale_flag(subtypecover,on) -> compile_deranged_act ; true),
  ( current_predicate(compile_sig_hook,compile_sig_hook) -> call_all(compile_sig_hook) ; true).

%
% Constraint compiler patches
%

:- op(1125,xfx,'*>'). % was 900
:- op(1125,xfx,trale_cons).

:- multifile alec_catch_hook/2.
:- discontiguous alec_catch_hook/2.

% cons/2 capture - must map to trale_cons/2 so that compile_deranged can add
%  constraints for exhaustive subtype covering
:- multifile term_expansion/2.
term_expansion((Type cons Cons),(Type trale_cons Cons)).

:- ale_abolish([compile_cons_act/0]).
compile_cons_act :-
  alec_announce('Compiling type constraints...'),
  ale_abolish((ct)/3), retractall(constrained(_)),
%  retract_fs_palettes(ct),
  retractall(complex_cons(_,_,_,_,_)),
  compile_complex_assert,
  link_cons_act,
  (current_predicate(cons,(_ cons _)) ->
            [bot,has,constraints] if_error
                 ( bot cons _ ),
            [multiple,constraint,declarations,for,CType] if_error
                 (bagof(CT,Cons^(CT cons Cons),CTypes),
                  duplicated(CType,CTypes)),
            [constraint,declaration,given,for,atom] if_error
                 ( (a_ _) cons _ ),
      assert(alec(ct)),
      \+ \+ compile('.alec_throw')
  ; ([no,constraints,found] if_warning true)
  ),
  ( current_predicate(compile_cons_hook,compile_cons_hook) -> call_all(compile_cons_hook) ; true).

link_cons :-
  touch('.alec_throw'),
  absolute_file_name('.alec_throw',AbsFileName),
  retractall(ale_compiling(_)),  
  assert(ale_compiling(AbsFileName)),
  link_cons_act,
  retract(ale_compiling(_)).
link_cons_act :-
  assert(alec(trale_link)),
  compile('.alec_throw').

alec_catch_hook(trale_link,Code) :-
  findall(Type,type(Type),Types), % maxcount/3 and deranged/2 are already
  compute_cons(Types,Code).     % there - just re-attach to user constraints

:- dynamic complex_cons/5.
compile_complex_assert :-
  current_predicate('*>',(_ *> _))
  -> \+ ((Antecedent *> ConsequentandGoal),
	 ( ConsequentandGoal = (Consequent goal Goal) -> true
	 ; Consequent = ConsequentandGoal, Goal = true
	 ),
         find_narrow_vars(Antecedent,AntecNoQs,NarrowVars),
         find_trigger_type(AntecNoQs,bot,Trigger),
         ale_assert(complex_cons(Trigger,AntecNoQs,Consequent,Goal,NarrowVars)),
         fail)
   ; true.

find_narrow_vars(Antecedent,AntecNoQs,NarrowVars) :-
  find_nv_act(Antecedent,AntecNoQs,[],NarrowVars).

find_nv_act(X^(Antecedent),AntecNoQs,NVsIn,NVsOut) :-
  !,find_nv_act(Antecedent,AntecNoQs,[X|NVsIn],NVsOut).
find_nv_act(AntecNoQs,AntecNoQs,NVs,NVs).

% this is a restricted form of infer_mgsat_type/3 that prohibits inequations and functions.
find_trigger_type(Var,Trigger,Trigger) :-
  var(Var),
  !.
find_trigger_type(Feat:_,TrigIn,TrigOut) :-
  introduce(Feat,Intro),
  !,unify_type(Intro,TrigIn,TrigOut).
find_trigger_type((D1,D2),TrigIn,TrigOut) :-
  !,find_trigger_type(D1,TrigIn,TrigMid),
  find_trigger_type(D2,TrigMid,TrigOut).
find_trigger_type((D1;D2),TrigIn,TrigOut) :-
  !,find_trigger_type(D1,TrigIn,TrigOut1),
  find_trigger_type(D2,TrigIn,TrigOut2),
  generalise(TrigOut1,TrigOut2,TrigOut).
find_trigger_type(Type,TrigIn,TrigOut) :-
  type(Type),
  !,unify_type(Type,TrigIn,TrigOut).
find_trigger_type(Ant,_,_) :-
  error_msg((nl,write('compile_cons: illegal description, '),write(Ant),
             write(', used in complex antecedent constraint'),nl)).

% compute_cons:  collect complex antecedent constraints into cons/2 clauses

compute_cons([],[end_of_file]).
compute_cons([Type|Types],Code) :-
  (retrieve_user_cons(Type,Cons,Goal) % user has cons, either directly or through '*>'/2
     -> Code = [(Type cons Cons goal Goal)|CodeRest]
      ; Code = CodeRest               % nothing to add
  ),
  compute_cons(Types,CodeRest).

retrieve_user_cons(Type,Cons,Goal) :-
  current_predicate(trale_cons,(_ trale_cons _)),
  (Type trale_cons TCons goal TGoal),
  !,findall(cc(CondVar,Cond,Conseq,CGoal),
            (complex_cons(Type,Antec,Conseq,CGoal,NarrowVars),
             apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
            ComplexConss),
    ( ComplexConss == []
    -> Cons = TCons,
       Goal = TGoal
     ; Cons = (FS,TCons),
       apply_ccs(ComplexConss,FS,TGoal,Goal)
    ).

retrieve_user_cons(Type,Cons,Goal) :-
  current_predicate(trale_cons,(_ trale_cons _)),
  (Type trale_cons TCons),
  !,findall(cc(CondVar,Cond,Conseq,CGoal),
            (complex_cons(Type,Antec,Conseq,CGoal,NarrowVars),
             apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
            ComplexConss),
    ( ComplexConss == []
    -> Cons = TCons,
       Goal = true
     ; Cons = (FS,TCons),
       apply_ccs(ComplexConss,FS,true,Goal)
    ).
retrieve_user_cons(Type,Cons,Goal) :-
  findall(cc(CondVar,Cond,Conseq,CGoal),
          (complex_cons(Type,Antec,Conseq,CGoal,NarrowVars),
           apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
          ComplexConss),
  ComplexConss \== [], % otherwise fail to add no code
  Cons = FS,
  apply_ccs(ComplexConss,FS,true,Goal).

apply_narrow_vars([],Cond,Cond).
apply_narrow_vars([NV|NVs],CondIn,CondOut) :-
  apply_narrow_vars(NVs,NV^(CondIn),CondOut).

apply_ccs([],_,TGoal,TGoal).
apply_ccs([cc(FS,Cond,Conseq,CGoal)|CCs],FS,TGoal,Goal) :-
  apply_ccs(CCs,FS,(TGoal,when(Cond,(FS=Conseq,CGoal))),Goal).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Signature file input - calls read_ths/4 from pth.pl and converts to ALE 
%  sub/intro representation in O(n log n) time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

alec_catch_hook(trale_signature,Code) :-
  !,(current_predicate(signature,signature(_))
    -> findall(File,signature(File),[SigFile|Rest]),
       [multiple,signature,specifications,found,'-',using,'first:',SigFile]
         if_warning (Rest \== [])
    ; SigFile = 'signature'
    ),
  read_ths(SigFile,SubL,AppL,_),
  sort(SubL,SortedSubL),
  trale_sub(SortedSubL,Code,CodeMid),
  sort(AppL,SortedAppL),
  trale_intro(SortedAppL,CodeMid,[end_of_file]).

trale_sub([X-Y|SubLMid],[(X sub [Y|Ys])|CodeMid],CodeRest) :-
  trale_sub_act(SubLMid,X,Ys,CodeMid,CodeRest).

trale_sub_act([],_,[],Code,Code).
trale_sub_act([X-Y|SubLMid],Last,Ys,Code,CodeRest) :-
  (X = Last
  -> Ys = [Y|YsRest],
     trale_sub_act(SubLMid,Last,YsRest,Code,CodeRest)
   ; Ys = [],
     Code = [(X sub [Y|YsRest])|CodeMid],
     trale_sub_act(SubLMid,X,YsRest,CodeMid,CodeRest)).

trale_intro([],CodeRest,CodeRest).
trale_intro([app(T,F,R,_)|AppLMid],[(T intro [F:R|FRs])|CodeMid],CodeRest) :-
  trale_intro_act(AppLMid,T,FRs,CodeMid,CodeRest).

trale_intro_act([],_,[],Code,Code).
trale_intro_act([app(T,F,R,_)|AppLMid],Last,FRs,Code,CodeRest) :-
  (T = Last
  -> FRs = [F:R|FRsRest],
     trale_intro_act(AppLMid,Last,FRsRest,Code,CodeRest)
   ; FRs = [],
     Code = [(T intro [F:R|FRsRest])|CodeMid],
     trale_intro_act(AppLMid,T,FRsRest,CodeMid,CodeRest)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Logical-Variable Macros with Modes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% notation:
% someMacro(X-a,Y-b) := t, abc:X, def:Y.
% OR
% someMacro(X,Y) := t, abc:X, def:Y.

:- op(1125,xfx,':=').

:- multifile compile_fun_hook/0.
compile_fun_hook :- compile_macro_act.

compile_macro_act :-
  current_predicate(':=',(_ := _)) -> assert(alec(macro)),
                                      compile('.alec_throw')
 ; true.				      

alec_catch_hook(macro,[(:- multifile (macro)/2)|Code]) :-
% this is going to create warnings whenever user defines macro/2 without
% multifile - KNOWN BUG
  findall(logmacro(LVLHS,RHS),(LVLHS := RHS),LogMacros),
  compute_macros(LogMacros,Code).

compute_macros([],[end_of_file]).
compute_macros([logmacro(LVLHS,RHS)|LogMacros],[(LHS macro RHS)|Code]) :-
  functor(LVLHS,MacroName,Arity), % match the macro by name and arity
  functor(LHS,MacroName,Arity),
  ( add_lv_macro_descs(0,Arity,LVLHS,LHS) -> compute_macros(LogMacros,Code)
  ; error_msg((nl,write('error: logical macro arguments must be unshared variables in '),
	       write(LVLHS),nl))
  ).

add_lv_macro_descs(A,A,_,_) :- !.
add_lv_macro_descs(OldP,A,LVLHS,LHS) :-
  P is OldP + 1,
  arg(P,LVLHS,LVArg),  % LVArg is either macro-var. or macrovar-Desc pair
  arg(P,LHS,TArg),
  ( var(LVArg) -> LVArg = (TArg,_)     % if Desc is missing, interpret as bot
  ; LVArg = Var-Desc -> var(Var),   % check before instantiating to catch shared vars
                        Var = (TArg,Desc,_)	% adding anonymous variable pushes structure
                                                % sharing on Prolog level in RHS into
                                                % structure sharing on (ALE) logical level.
  ),
  add_lv_macro_descs(P,A,LVLHS,LHS). 
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Banner
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- nl,write('
TRALE Version 1.0; June, 2005
Copyright (C) 1999,2003--2005, Gerald Penn
All rights reserved'),nl,
 initialise_trale_flags.

%:- set_prolog_flag(encoding, utf8).
%:- ale_flag(residue,_,show).


