%%% ==================================================================
%%%
%%% Some extensions of the core TRALE system
%%%
%%% $Id: trale_extras.pl,v 1.87 2005/06/19 17:20:47 stefan Exp $
%%%
%%% Time-stamp: <dm on 14.Nov.2003 19:22:11 EST>
%%%
%%% ==================================================================

%%% shortcuts

c :- compile_gram(theory).
dc :- dcompile_gram(theory).    % for use with source-level debugger

% Allow the user to write :- tree_extensions. to get the additional
% syntax for integrating the extralogical parts (phonology/dtrs)
% into the feature structure. Note that this has to be part of
% the grammar file of the user, since it contains parts that need
% to be compiled by the trale grammar compiler.

tree_extensions :- [tree_extensions].

%%% --------------------------------------------------------------- %%
%%% Starting up code----------------------------------------------- %%
%%% --------------------------------------------------------------- %%

% GBP: moved delete_file/2 import to sics{4}.pl
%:- use_module(library(charsio),[format_to_chars/3]). -- unused
:- use_module(toplevel).

%% command which is executed at every startup 
%% (be it from a state or from the prolog source code)

trale_startup :-
   % load user startup file unless it has already been done by loading
   % the prolog code of startup.pl (i.e. when LOAD_SOURCES is false)        
   (  environ('READ_USER_STARTUP',true)
   -> (  load_user_startup
      -> true
      ;  format(user_error,'loading startup failed~n',[])
      )
   ;  true  % GBP - moved failure message; this case is fine
   ),
   %
   % if a $INTERFACE_FILE exists, load it
   (  (environ('INTERFACE_FILE',IName),file_exists(IName))
   -> format('~NLoading Interface connection specifications (option -g):~n~n',[]),  % GBP - corrected typo 'options'
      consult(IName), delete_file(IName, [ignore])
   ;  true
   ),
   %
   % connect to graphical interface if needed
   (  (environ('INTERFACE',Interface), 
       (Interface=='graphics(remote)';Interface=='graphics(true)'))
   -> (  catch(start_client, Exception, fail)
      -> true
      ;  format(user_error,'~N**Error: startup of interface failed (~w)~n',[Exception])
      )
   ;  nl
   ),
   %
   % initialize the I/O unicode support
   %
   (  environ('TRALE_UNICODE', true) -> set_prolog_flag(encoding, utf8)
   ;  true
   ),
   %
   % output program and version header
   trale_milca_version(Version),
   format('~n~nTRALE Milca environment (version ~p)~n',[Version]),
   format('Copyright (C) 2002-6 Project MILCA A4~n',[]),
   format('All rights reserved~n~n~n',[]),
   %
   % if a $USER_CMD_FILE exists, load it
   (  (environ('USER_CMD_FILE',CName),file_exists(CName))
   -> format('~NCompiling grammar/executing user commands (options -c and -e):~n~n',[]),
      consult(CName)
   ;  true
   ).

load_user_startup :-
   environ('READ_USER_STARTUP', true),
   getenv('TRALE_USER_STARTUP', F0),
   absolute_file_name(F0, F),
   catch(ensure_loaded(F), Ex, (
        print_message(error, Ex), fail
        )).

% This is called at startup of a standalone state.
%
runtime_entry(start) :-
        % We have to get the path again, since the standalone
        % state has the path already compiled in, but it may be
        % a wrong path.
        environ('TRALE_HOME',Path),
        %trale_search_path_add(trale_home,Path),
        % there is just one trale_home and if we add the right one with
        % assertz, the chart display fails.
        retractall(file_search_path(trale_home,_)),
        asserta(file_search_path(trale_home,Path)),
                
        trale_startup,
        toplevel.

%%% --------------------------------------------------------------- %%
%%% Saving of system states --------------------------------------- %%
%%% --------------------------------------------------------------- %%

% top level predicate to save state of currently loaded trale version:
save_trale_state :-
   environ('CORE_NAME',CoreName),
   save_state(trale_home(CoreName)).

% same, but for saving a version of trale, with compiled grammar, for tsdb 
save_tsdb_state :-
   load_tsdb,
   save_state(tsdb,(trale_startup,itsdb:connect_tsdb)).

%% save_state/1: saving user states under name given as argument
save_state(Name) :-
   save_state(Name,trale_startup).


% Auxiliary predicates:
save_state(Name,Startup) :-
   current_prolog_flag(compiling,CompileFlag),
   (   set_prolog_flag(compiling,fastcode) % Sparc only
   ->  true                     
   ;   set_prolog_flag(compiling,compactcode)
   ),
   absolute_file_name(Name,AbsName),
   add_state_suffix(AbsName,FullName),
   save_program(FullName,user:Startup),
   set_prolog_flag(compiling,CompileFlag).

save_standalone_state(Name) :-
   current_prolog_flag(compiling,CompileFlag),
   (   set_prolog_flag(compiling,fastcode) % Sparc only
   ->  true                     
   ;   set_prolog_flag(compiling,compactcode)
   ),
   absolute_file_name(Name,AbsName),
%   add_state_suffix(AbsName,FullName),
   save_program(AbsName),%user:standalone_trale_startup),
   set_prolog_flag(compiling,CompileFlag).

%% add_state_suffix/2: append state suffix
%% Adds hosttype to name of state so that we can have different states
%% for each operating system. 
add_state_suffix(In,Out) :-
   (   environ('STATE_SUFFIX',Suffix) 
   ->  atom_concat(In,Suffix,Out)
   ;   In=Out
   ).

%% load_state/1: loading state
load_state(Name):-
   absolute_file_name(Name,AbsName),
   add_state_suffix(AbsName,FullName),
   restore(FullName).

% -------------------------------------------------------------------
% Enhanced query input as frontend to rec/2 
%
% rec/0 prompts user for a string, ending with a newline
%  and passes the appropriate word list to rec/1
%  with a variable Desc argument.

(rec) :-
   read_line_of_words(List,Desc),
   rec(List,Desc).

% rec_desc/1 work like rec/0, but with a Desc argument - Only parses
% matching the description are returned.

rec_desc(Desc) :-
   read_line_of_words(List,Desc1),
   rec(List,(Desc,Desc1)).

% ------------------------------

read_line_of_words(List,Desc) :-
  read_line(CharList),
  general_tokenize_sentence_string(CharList,List,Desc).

% general_tokenize_sentence_atom(+Atom,-Tokenlist,-StartSymbolDesc)
general_tokenize_sentence_atom(Atom,Tokenlist,StartSymbolDesc) :-
   atom_codes(Atom,String),
   general_tokenize_sentence_string(String,Tokenlist,StartSymbolDesc).

% general_tokenize_sentence_string(String,-Tokenlist,-StartSymbolDesc)
% This predicate allows the grammar to define the tokenization to 
% be used by defining the predicate
%
% tokenize_sentence_string(+String,-Tokenlist,-StartSymbolDesc)
%
% for a particular grammar. In that one can e.g. specify to lowercase 
% everything automatically, what to do with special punctuation,
% how to treat special symbols like apostrophes, etc.

general_tokenize_sentence_string(ItemInput0,ItemInput,Desc) :-
        (  current_predicate(tokenize_sentence_string/3)
	-> tokenize_sentence_string(ItemInput0,ItemInput,Desc)
	;  tokenize_sentence_string_default(ItemInput0,ItemInput,Desc)
	).

tokenize_sentence_string_default(String,Wordlist,bot) :-
   string2wordlist(String,Wordlist).

% string2wordlist(+,-): a very simple tokenizer, i.e. it translates
% a string into words (a list of atoms), taking any number of spaces 
% as delimiting words
%
% e.g.
% "ab"     -> [ab]
% "ab cd"  -> [ab,cd]
% "ab  cd" -> [ab,cd]
%
% i.e.
% [97,98]              -> [ab]
% [97,98,32,99,100]    -> [ab,cd]
% [97,98,32,32,99,100] -> [ab,cd]

string2wordlist([],[]) :- !.
string2wordlist(InString,AtomList) :-
   one_word(InString,WordList,RestString),
   (  WordList == []
   -> AtomList = Atoms
   ;  name(Atom,WordList),
      AtomList = [Atom|Atoms]
   ),
   string2wordlist(RestString,Atoms).

one_word([],[],[]).             % end of string reached
one_word([H|T],Word,Rest) :-
   (  H == 32                        % if space ends a word
   -> Word=[],remove_spaces(T,Rest)  %    remove, incl. additional spaces
   ;  one_char(H,Word0,Word),	     % else map one char
      one_word(T,Word0,Rest)         %      and continue on word
   ).

one_char(X,L,[X|L]).

remove_spaces([],[]).
remove_spaces([H|T],R) :-
   (  H == 32 
   -> remove_spaces(T,R)
   ;  [H|T]=R
   ).

% ------------------------------------------------------------------------

% wordlist2string(+,-)
%
% translate a wordlist into a string. This is almost the inverse
% of string2wordlist/2, but with the difference that always insert
% exactly one space between each word (and never any space at the
% beginning or the end of the string).

wordlist2string([],[]).

wordlist2string([Atom|Atoms],EndPointer0) :-
        name(Atom,WordList),
        makedifflist(WordList,EndPointer0,EndPointer1),
        (Atoms == [] ->
                 EndPointer1 = EndPointer2
        ;   EndPointer1 = [32|EndPointer2]),
        wordlist2string(Atoms,EndPointer2).
        
makedifflist([],X,X).
makedifflist([H|T],[H|L],Diff) :-
        makedifflist(T,L,Diff).


%%% --------------------------------------------------------------- %%
%%% Additional user predicates ==---------------------------------- %%
%%% --------------------------------------------------------------- %%

:- dynamic ask_questions_flag/0.

% This makes query_proceed supressable.
% If we open windows with information, asking for feedback does
% not make sense. St. M�. 27.02.2005
ask_questions_flag.

% GBP - naughty, naughty...
% Define more user friendly (i.e. less typing) query_proceed/0.
%:- abolish(query_proceed/0).

%query_proceed :-
%   (ask_questions_flag -> ( query_user('Do you want to try for more solutions? (return for <~w>) ',[yes],yes,Answer),
%                              ( ( Answer == y
%                                ; Answer == 'y.'
%                                ; Answer == yes
%                                ; Answer == 'yes.'
%                                ) -> fail 
%                              ; true 
%                              )
%                          ; fail
%                          )
%   ).

%% query_user(+Format,+Args,+Default,-Value).
%%
%% Useful for getting user input, for instance:
%%
%% query_user('File name? (return for `~w'') : ',[foo],foo,File).
%%
query_user(Format,Args,Default,Value) :-
   nl,
   format(Format,Args),
   get_line(Answer0),
   nl,
   ( Answer0 == [] -> 
                 Value = Default
        ; atom_codes(Value,Answer0)
   ).

%------------------------------------------------------------------------
% predicates for storing and accessing titles that can be displayed 
% by some of the pretty printers (e.g. the one using the grisu interface)

:- dynamic title/1.

clear_title :-
   retractall(title(_)).

set_title(Title) :-
   \+ \+ (
   numbervars(Title,0,_),
   retractall(title(_)),
   assert(title(Title))
         ).
get_title(Title) :-
   (  title(Title)
   -> true
   ;  Title=untitled
   ).

%------------------------------------------------------------------------
% lex_desc(Word:<word>,Desc:<description>)
%------------------------------------------------------------------------
% prints out all words matching the description, querying user in between
%------------------------------------------------------------------------

lex_desc(Word,Desc):-
   \+ \+ ( call_residue(lex(Word,FS),Residue),
           secret_noadderrs_toggle(Mode),
           add_to(Desc,FS),
           secret_adderrs_toggle(Mode),
           lex_output(Word,FS,Residue),
           query_proceed
  ).

lex_output(Word,FS,Residue) :-
   ((current_predicate(portray_lex,portray_lex(_,_,_)),
             portray_lex(Word,FS,Residue)) -> 
             true
           ; nl, write('WORD: '), write(Word),
             nl, write('ENTRY: '),
             pp_fs_res(FS,Residue), nl),
   !.

%------------------------------------------------------------------------
% rec_all/1 and rec_all/2 are identical to rec/1 and rec/2 from ale.pl
% except for the fact that they output all solutions without user interaction.
%-----------------------------------------------------------------------------
%
% rec_all(Words:<word>s)
%-----------------------------------------------------------------------------
% basic predicate to parse Words;  prints out all recognized categories
%-----------------------------------------------------------------------------
rec_all(Words):-  
  rec_all(Words,bot).

% ----------------------------------------------------------------------------
% rec_all(Words:<word>s,Desc:<desc>)
% ----------------------------------------------------------------------------
% Like rec_all/1, but solution FSs must satisfy Desc
% ----------------------------------------------------------------------------

% GBP - cleaned up and ported
rec_all(Words,Desc):-  % must add code to print residues
  nl, write('STRING: '),
  nl, number_display(Words,0),
  ttynl, 
     \+ on_exception(ale(Exception),
		     (rec(Words,FS,Desc,Residue,Index),
                      ((current_predicate(portray_cat,portray_cat(_,_,_,_,_)),
                        portray_cat(Words,Desc,FS,Residue,Index)) -> true
               % see also gen/1 - portray_cat/4 can be called with var 1st arg.
                      ; nl, write('CATEGORY: '),nl, ttyflush,
                        pp_fs_res(FS,Residue),nl
                      ),
                      fail),
		     alex(Exception)).

%------------------------------------------------------------------------
% Older stuff:


% Save lexicon in a file
save_lexicon(File):-
        open(File,write,Stream),
        findall(_,(lex(String,FS),
                                  write_term(Stream,lex(String,FS),[quoted(true),ignore_ops(true),portrayed(true),cycles(true),max_depth(0)]),
                                  write(Stream,'.'),
                                  nl(Stream)),_),
        flush_output(Stream),
        close(Stream).

read_lexicon(File):-
        open(File,read,Stream),
        repeat,
        read_term(Stream,Entry,[cycles(true)]),
        ( Entry == end_of_file ->
                 close(Stream),!
        ; assert(Entry),
          fail
        ).

% GBP - this would strip off attributed vars - must flatten attribute
%  structure first.  Someday...
%/*
% Save lexicon in a data base
%:- use_module(library(bdb)).
%save_lexicon:-
%        db_open_env('ALE lexicon',EnvRef),
%        db_open(lexdb,update,lex(+,-),EnvRef,DBRef),
%        db_store(DBRef,_,_),
%        db_close_env(EnvRef).
%*/

compile_gram_no_lex(File) :-
  abolish_preds,
  reconsult(File),
  compile_gram_no_lex.

% GBP - corrected and ported
compile_gram_no_lex :-
  clear_num,
  touch('.alec_throw'),
  absolute_file_name('.alec_throw',AbsFileName),
  retractall(ale_compiling(_)),
  assert(ale_compiling(AbsFileName)),
  compile_sig_act,
  compile_fun_act,
  compile_cons_act,
  compile_logic_act,
  compile_subsume_act,
  compile_dcs_act,
  compile_mgsat_act,
  compile_grammar_no_lex_act,
  retract(ale_compiling(_)),
  ( current_predicate(compile_gram_hook,compile_gram_hook) -> call_all(compile_gram_hook) ; true).

compile_grammar_no_lex_act:-
  compile_lex_rules_act,
  % here used to be the lexicon compilation
  % compile_lex_act,
  compile_rules_act,
  compile_generate_act.

% if examples exists (specified in theory.pl), the examples are printed,
% to give the user a clue what he can do with a grammar.
print_examples :-
        (current_predicate(examples/1) ->
            (language(german) -> format(user_error,"Beispiele:",[])
            ;                    format(user_error,"Examples:",[])
            ),
            examples(X),
            print_examples(X)
        ; true ).

print_examples([]) :-
        format(user_error,"~n~n",[]).
print_examples([E|Es]) :-
        format(user_error,"~11+~w~n",[E]),
        print_examples(Es).

