% GBP - nothing else in system seems to rely on this.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: test_suite_handling.pl,v $
%%  $Revision: 1.44 $
%%      $Date: 2003/10/15 18:37:31 $
%%     Author: Detmar Meurers (dm@ling.osu.edu), integrating
%%             code from Gerald Penn, Jesse Tseng, Stefan Mueller
%%    Ported to SP4 by: Gerald Penn
%%    Purpose: Provides predicates to run test suites with TRALE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: test_suite_handling.pl,v $
%% Revision 1.44  2003/10/15 18:37:31  stefan
%% The tokanizer has to be called for transforming test strings.
%%
%% Revision 1.43  2003/06/23 20:48:33  dm
%% - now callsreverse_count_lex_check/5 (used to be called reverse_count/5)
%%
%% Revision 1.42  2003/05/08 16:58:57  dm
%% changed warning to user_error
%%
%% Revision 1.41  2003/05/04 00:28:43  dm
%% bugfix in msec2time
%%
%% Revision 1.40  2003/05/02 16:41:45  dm
%% - added output of number of edges in write_result/2
%% - added msec2time/2 predicate to have better output of time
%% - modified rect/3 to allow for reverse_count to fail when unknown
%%   words are in the string
%% - modified get_and_show_solutions/3 by adding assert(solution/1),
%%   which is needed by Stefan's chart display code
%%

:- ensure_loaded([trale_home('chart_display/sic')]). % timer
                 
:- dynamic show_result_flag/0.

% user predicates: 
% - test/[1,2] shows results
% - testt/[1,2] only reports number of results and time
%
% First argument Arg can be: 
% - all
% - a test item number,
% - or a range of item numbers (two-member list)
%
% A second argument can be used to specify a description that needs
% to be satisfied of the result, i.e., a kind of "start category"

test(Arg,Desc)  :- show_result,test_aux(Arg,Desc).
testt(Arg,Desc) :- hide_result,
        chart_display(Old),
        nochart_display,
        nl, % switch to a new line, to make glosses look good
        test_aux(Arg,Desc),
        retractall(chart_display(_)),
        assert(chart_display(Old)).

test(Arg)  :- test(Arg,nothing).
testt(Arg) :- testt(Arg,nothing).

% ---------------------
% double negation used to keep suspensions from being output

test_aux(all,Desc) :-  
   get_and_do_test(_,Desc), fail; true.
test_aux(Nr,Desc) :-
   \+ \+ ( integer(Nr),
	   (  get_and_do_test(Nr,Desc) 
	   -> true 
	   ;  format(' (~w) * Test item not found *~n',[Nr]) 
	   )).
test_aux([From,To],Desc) :-
   \+ \+ ( (  From @=< To 
	   -> Next is From + 1,
	      (  get_and_do_test(From,Desc) 
	      -> true
	      ;  format(' (~w) * Test item not found *~n',[From])
	      ),
	      test_aux([Next,To],Desc)
	   ;  true
	   ) ).

% ---------------------
% auxiliary predicates

:- dynamic num_sol/1,num_res/2,num_scopings/2.

get_and_do_test(Nr,TestDesc) :-
	get_test(Nr,Words,Desc,ExpSol),
	(  TestDesc == 'nothing' 
	-> rect(Words,Desc,ExpSol) % if: use Desc provided in t/5 fact
	;  rect(Words,TestDesc,_)  % else: override Desc in t/5 and 
	).                         %       ignore expected number of solutions
                                

get_test(Nr,WordList,Desc,ExpSol) :-
	(fail_if_undefined(t(Nr,Example,ExpSol))
        ;fail_if_undefined(t(Nr,Example,ExpSol,Comment))
        ;fail_if_undefined(t(Nr,Example,Desc,ExpSol,Comment))
        ;fail_if_undefined(t(Nr,Example,Gloss,Desc,ExpSol,Comment))),
	(integer(ExpSol) ->
	    true
	;   format(user_error,'~N     Warning: Test predicate listed ~w instead of number of solutions.~n~n',[ExpSol])
	),
	([H|_] = Example, integer(H) ->  
            % Example is a string (=list of character codes): "a sentence"
	    Example=String,
	    % Obtain corresponding list of words: [a,sentence]
	    general_tokenize_sentence_string(String,WordList,_Desc)
	;   % Example is a list of words: [this,is,a,sentence]
	    Example=WordList,
            % obtain corresponding string "this is a sentence"
	    wordlist2string(WordList,String)
	), 
        % the ~+ is a tab, needed for glosses
	format(' (~w)~+~s  ',[Nr,String]),
	(nonvar(Comment), Comment \== '' ->
	    format('% ~p~n',[Comment])
	;   nl),
        (nonvar(Gloss), Gloss \== '' ->
	    format('~+~s~n~n',[Gloss])
        ; true).

% The variable Dur was not bound after the on_exception clause.
% so I explicitly call statistics again. St. M. 05.06.2006

% GBP - cleaned up and ported.
rect(Ws,Desc,ExpSol) :-
  reset_timer,
  retractall(num_res(_,_)),
  retractall(num_sol(_)),
  assert(num_sol(0)),
  \+ on_exception(ale(Exception),
		  (rec(Ws,FS,Desc,Residue,Dur,Index),
		   (show_result_flag ->
		       ( (current_predicate(portray_cat,portray_cat(_,_,_,_,_)),
			  portray_cat(Ws,bot,FS,Residue,Index)) -> true
		       ; pp_fs_res(FS,Residue), nl
		       )
		   ;   true
		   ),
		   tick_sol_count_res(Residue),
		   fail),
		  soft_alex(Exception)),
  clear_title,
  timer(Dur),
  write_time_solcount(Dur,ExpSol),!.

% Another problem is that rec/5 fails if there is no solution,
% for these cases no statistics was printed and no time was given.

rect(_Ws,_Desc,ExpSol) :-
  clear_title,
  timer(Dur),
  write_time_solcount(Dur,ExpSol).

% the following is alex from ale.pl without an abort at the end:
% we need this since exceptions may be due to memory consumtion.
% An exception should not result in the abortion of the test run,
% but in an error for the respective sentence.
soft_alex(Exception) :-
  format(user_error,'{ALE: ERROR: ',[]),
  ale_exception(Exception),
  format(user_error,'}~n~n',[]),
  flush_output(user_error).

tick_sol_count_res(Residue) :-
  retract(num_sol(OldNum)),
  NewNum is OldNum + 1,
  build_iqs_restore_atts(Residue,_Iqs,FSResidue),   % Filters out some unwanted residues. St. M. 12.09.2007
  length(FSResidue,ResLen),
  asserta(num_res(NewNum,ResLen)),
  asserta(num_sol(NewNum)).

% GBP - formerly write_result/2
write_time_solcount(Dur,ExpSol) :-
   msec2time(Dur,Time),
   format(' ==> ~w CPU time.        ', [Time]),
   num_sol(NumSol),
   (  NumSol == 0 
   -> (  write('No solution')
      ;  NumSol == 1 
      -> write('1 solution'))
   ; format('~d solutions',[NumSol])
   ),
   (  integer(ExpSol) 
   -> (  ExpSol == NumSol 
      -> write(' (ok)')
      ; format(' (!!! ~d expected !!!)',[ExpSol])
      )
   ; true
   ),
   edgenum(Edges),
   emptynum(E),             % The (negative) number of the next empty element, if there is one.
                            % Therefore we have to subtract one after having it multiplied by -1 below.
   AllEdges is Edges + (E * -1) - 1,
   format('; ~p passive edges',[AllEdges]),

   (  NumSol == 0 
   -> true
   ;  write('; residues: '),
      write_num_of_residues(1,NumSol)
   ),
   (  NumSol == 0 
   -> true
   ;  ( % If scopings were computed, print the numbers.
        num_scopings(_,_) ->
          write('; scopings: '),
          write_num_of_scopings(1,NumSol)
      ; true
      )
   ),
   nl,nl.

write_num_of_residues(X,NumSol):-
   num_res(X,ResNr),
   format('~w',[ResNr]),
   (  X == NumSol 
   -> true
   ;  write(','), X1 is X+1,
      write_num_of_residues(X1,NumSol)
   ).

write_num_of_scopings(X,NumSol):-
   num_scopings(X,ResNr),
   format('~w',[ResNr]),
   (  X == NumSol 
   -> true
   ;  write(','), X1 is X+1,
      write_num_of_scopings(X1,NumSol)
   ).


show_result :-
   retractall(show_result_flag),
   assert(show_result_flag).

hide_result :-
   retractall(show_result_flag).

% -----

:- use_module(library(codesio),[format_to_codes/3,format_to_codes/4]).

% msec2time(+milliseconds,-atom-expressing-time)
msec2time(Msec,Time) :-
   TimeMin is Msec // 60000,
   TimeSec is Msec mod 60000,
   %
   (  TimeMin == 0
   -> TimeChars = Time1
   ;  format_to_codes('~d min ',[TimeMin],TimeChars,Time1) 
   ),
   format_to_codes('~3d sec',[TimeSec],Time1),
   atom_codes(Time,TimeChars).


%%% ==================================================================
% Additional predicate, originally housed in  Gerald's testt.pl

% GBP - no one seems to be calling this.
%time(Words):-  
%  nl, write('STRING: '),
%  nl, number_display(Words,0),
%  empty_assoc(AssocIn),
%  ttynl, call_residue((statistics(runtime,_),
%		       rec(Words,Tag,SVs),
%		       statistics(runtime,W)),Residue),
%  \+ \+ ((show_res -> residue_args(Residue,ResArgs,[Tag-SVs]) ; ResArgs = [Tag-SVs]),
%	 duplicates_list(ResArgs,AssocIn,DupsMid,AssocIn,_,0,_),
%	 nl, write('CATEGORY: '),nl, ttyflush,
%	 pp_fs(Tag,SVs,DupsMid,DupsOut,AssocIn,VisMid,0,AssocIn,HDMid),
%	 nl, write('TIME: '), write(W), nl,
%	 ((show_res,Residue \== [])
%	 -> nl,nl,write('Residue:'), pp_residue(Residue,DupsOut,_,VisMid,_,HDMid,_)
%	  ; true)),
%  nl, query_proceed.

%%% ==================================================================
% Local Variables:
% mode: prolog
% End:
