%%%%%%%%%%%%%%%%%%%%%
% TRALE 1.0
% Gerald Penn
% University of Toronto
% 
% Developed with SICStus Prolog 3.10.1, December, 2003
%
% set environment variable TRALE_HOME to the directory that contains this
% file

% Added 'signature' default signature file specification.
% 6/24/99 - G. Penn

% Added logical variable macros and *> constraints
% 7/10/99 - G. Penn

% Bug corrected: parentheses missing on ale_macro and := calls.
% 7/13/99 - G. Penn

% Bug corrected: trale_abolish_preds had to be renamed to abolish_preds to
%  work with the debugger.
% 7/14/99 - G. Penn

% Bug corrected: retrieve_user_cons/3 expected only one complex_cons/5 clause 
%  for any given type.
% 8/27/99 - G. Penn

% Now loads call for CHR library, since new ale.new.pl does not.
% 9/15/99 - G. Penn

% Adapted to reference-based FS data structure.  No longer uses CHR.
% 4/28/02 - G. Penn

% Bug corrected: compile_sig_act/0 was calling link_cons/0 instead of link_cons_act/0 -
%  left choice-point in compilation chain.
% 4/29/02 - G. Penn

% Cleaned up and rewrote on-line FV map code to write dif/2 statements into compiled
%  trale_maps_init/2 to keep compiled logic code streams smaller.
% 6/9/02 - G. Penn

% Rewrote compute_maxcount/2 to compute and tabulate the actual maximal covers - cheaper
%  than decomposing joins.
% 6/11/02 - G. Penn

% Bug corrected: trale_maps_init/2 should not be inherited like other constraints - now
%  attach at ucons/7, add_to_typecons/6, and mgsat_cons/6.
% 6/14/02 - G. Penn

% Added ord_add_element/3 to the abolish list.
% 6/18/02 - G. Penn

% Bug corrected: resuspension on full-deref was not guarded by nonvar(FSVs) delay in trale_maps/4.
% 8/14/02 - G. Penn

% Bug corrected: recursive call in fully_deref_map/4 was guarded by nonvar(V) delay.
% 8/14/02 - G. Penn

% Bug corrected: logical macro argument mode declarations weren't optional.
% 8/31/02 - G. Penn

% Reorganised compilation chain slightly to do constraint linking entirely in constraint phase.
% 9/5/02 - G. Penn

% Bug corrected: new compilation chain was not integrated correctly with compile_gram/0,1 from ale.pl
% 9/12/02 - G. Penn

% Bug corrected: macro/2 was not instantiating LVLHS before looking up :=/2.
% 9/16/02 - G. Penn

% Bug corrected: add_lv_macro_descs/4 was binding variable LVArgs to Var-Desc pairs.
% 9/19/02 - G. Penn

% Bug corrected: missing macro/2 in abolish_preds/0.
% 9/29/02 - G. Penn

% Now trap shared or bound variables in logical variable macro heads.
% 11/22/02 - G. Penn

% Added fun/1 to abolish_preds/0.
% 11/22/02 - G. Penn

% Changed trale_signature hook to accommodate multiple signature files and new pth.pl.
% 11/27/02 - G. Penn

% Updated to SP 3.10.1.
% 7/14/03 - G. Penn

% Updated to matrix-based signature compiler.
% 7/16/03 - G. Penn

% Updated to new definition of imm_sub_type/2.
% 8/11/03 - G. Penn

% Updated uact/8 for symmetrically closed ud/4,5,6.
% 9/28/03 - G. Penn

% Added subintro ALE flag to control whether signature files are used.
% 11/29/03 - G. Penn

% Bug corrected: "Reading sig-file" announcement should only be printed
%  when subintro flag is file.
% 1/4/04 - G. Penn

% Bug corrected: MGSat clauses of add_to_typeact/6 and uact/8 weren't calling
%  trale_maps_init/2.
% 4/4/04 - G. Penn

% Updated uact/8, add_to_typeact/6, featval_act/8, map_mgsat/1, project_map/4,
%  and fully_deref_map/4 for addition of intensional var to ALE data structure.
% 5/26/04 - G. Penn

% Updated uact/8, add_to_typeact/6, featval_act/8, map_mgsat/1, project_map/4,
%  trale_maps_init_act/3, trale_maps/4, and update_map/9 for switch to attributed
%  variables in ALE data structure.
% 6/9/04 - G. Penn

% Updated for modular ALE data structure with maxima, added mgsat ALE flag
% 10/30/04 - G. Penn

% Moved generalise/3 to ale.pl
% 5/31/05 - G. Penn

% Updated uact/9, add_to_typeact/5 and map_mgsat/1 to support attributed data
%  structure with Prolog-level extensionality checking
% 6/17/05 - G. Penn

% Bug corrected: Missing abolish declaration for trale_maps_init/2 in compile_deranged_act/0.
%  project_map/6 was reading every feature value in module, rather than just the appropriate ones.
%  Compiled out project_map/5 in trale_deranged phase so that we can use FRs list without overhead.
% 6/18/05 - G. Penn

% Bug corrected: changed operator precedence of '*>'/2 and other TRALE
%  operators to 1125 - makes it easier to detect bugs when attachments
%  like morphs/2 are misplaced, and places it below discontiguous/1
%  and other compile-time declarations.
% 9/3/05 - G. Penn

% Cleaned up compile_complex_assert/0 to make single pass through '*>'/2 principles.
% 9/26/05 - G. Penn

% Bug corrected: update_map/9 expected interrupt var to be bound to max/1 instead of
%  max/0 with maximal types.
% 6/26/06 - G. Penn

% Bug corrected: subsume_tmap/2 was recursively calling subsume_map/2.
% 7/9/06 - G. Penn

% Bug corrected: project_val/6 did not handle sparse encodings properly.
% 7/9/06 - G. Penn

% Bug corrected: update_map/9 was not updating value types.
% 7/9/06 - G. Penn

% Added redefine for new ale_flag debugger hook.
% 7/20/06 - G. Penn

% Bug corrected: compile_sig_act/0, compile_cons_act/0 and compile_fun_act/0 had
%  not been updated to call their respective compiler hooks.
% 7/21/06 - G. Penn

% Bug corrected: update_map/9 was not updating maximal types correctly.
% 7/26/06 - G. Penn

% Added support for forall/2 universals.
% 11/7/06 - G. Penn

% Bug corrected: APMaps must be of definite length before calling atomic_prune/4, or
%  else atomic_prune_close/1 will iterate through all possible lists.
% 11/30/06 - G. Penn

% Added cllrs flag for CL-LRS constraint solver.
% 6/7/07 - G. Penn

% Dispatched subtype cover compilation code to stcover.pl, and moved compile_logic and
%  compile_sig hooks to ale.pl.
% 8/12/08 - G. Penn

% Updated to cleaner mechanism for adding ale_flags/1.
% 8/12/10 - G. Penn

% RCS banners
% $Id: trale.pl,v 1.2 1999/02/06 17:01:07 gpenn Exp gpenn $
% $Log: trale.pl,v $
% Revision 1.2  1999/02/06  17:01:07  gpenn
% Version 0.2 - bug corrections and support for a_/1 atoms.
%
% Revision 1.1  1998/10/24  17:20:15  gpenn
% Initial revision
%

:- ensure_loaded(ale).

:- clause(prolog_version(sp4),true) -> ensure_loaded(trale4diff)
   ; clause(prolog_version(sp3),true) -> ensure_loaded(trale3diff).

:- multifile file_search_path/2.
:- dynamic file_search_path/2.
trale_search_path_add(Name,Path) :-
          (clause(file_search_path(Name,Path),true) -> true
          ;   assertz(file_search_path(Name,Path))).

:- environ('TRALE_HOME',Path),
   trale_search_path_add(trale_home,Path),
   trale_search_path_add(common,Path).

:- compile(utilities).

:- use_module(library(ugraphs),[vertices_edges_to_ugraph/3,neighbours/3,
                                top_sort/2]).
:- use_module(library(terms),[variant/2,term_subsumer/3]).
:- use_module(library(ordsets),[ord_union/2]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compiler patches
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile abolish_user_preds/1.
abolish_user_preds(sig) :- static_abolish((signature)/1).
abolish_user_preds(cons) :- static_abolish(('*>')/2), static_abolish((trale_cons)/2).
abolish_user_preds(macro) :- static_abolish((':=')/2), static_abolish((macro)/2).
abolish_user_preds(all) :- static_abolish((signature)/1), static_abolish(('*>')/2), static_abolish((trale_cons)/2),
                           static_abolish((':=')/2), static_abolish((macro)/2).

%
% Flags
%

:- multifile ale_flags/1.
ale_flags([subintro,cllrs]).
  
:- multifile ale_flag_msg/3.
ale_flag_msg(subintro,file,'{ALE: compiler will define sub/2 and intro/2 from signature file}').
ale_flag_msg(subintro,grammar,'{ALE: compiler will expect grammars to define sub/2 and intro/2}').
ale_flag_msg(cllrs,on,'{ALE: CL-LRS is active}') :- initialise_cllrs.
ale_flag_msg(cllrs,off,'{ALE: CL-LRS is inactive}').

:- multifile flag_modes/2.
flag_modes(subintro,[file,grammar]).
flag_modes(cllrs,[on,off]).

:- static_abolish(initialise_ale_sld/0).
initialise_ale_sld :- ensure_loaded(trale_home('trdebug.pl')).

initialise_cllrs :- ensure_loaded(trale_home('sem.pl')).
	
initialise_trale_flags :-
  assert(ale_flag(subintro,file)),
  assert(ale_flag(cllrs,off)),
  ale_flag(subtypecover,_,on).

%
% Signature patches
%

% no incremental signature compilation because of TRALE signature file
% SP4: abolish needs options
:- static_abolish_tree([compile_sub_type/1,compile_sub_type/0,
            compile_approp/1,compile_approp/0]).

:- static_abolish(compile_sig_act/0).
compile_sig_act :-
  ( ale_flag(subintro,file) -> alec_announce('Reading signature file...'),
                               assert(alec(trale_signature)),
                               compile('.alec_throw')
  ; true
  ),
  compile_sub_type_act(_,_,_,_), % (SortedSubIntros,SortedIntros,SortedExts,STMatrix),
  compile_approp_act, % (SortedSubIntros,SortedIntros,STMatrix),
  compile_extensional_act, % (SortedExts).
  compile_modules_act,
  ( ale_flag(subtypecover,on) -> compile_deranged_act ; true),
  ( current_predicate(compile_sig_hook,compile_sig_hook) -> call_all(compile_sig_hook) ; true).

%
% Constraint compiler patches
%

:- op(1125,xfx,'*>'). % was 900
:- op(1125,xfx,trale_cons).

:- multifile alec_catch_hook/2.
:- discontiguous alec_catch_hook/2.

:- static_abolish(compile_cons_act/0).
compile_cons_act :-
  alec_announce('Compiling type constraints...'),
  static_abolish((ct)/3), retractall(constrained(_)), abolish_wpreds,
%  retract_fs_palettes(ct),
  retractall(complex_cons(_,_,_,_,_)),
  compile_complex_assert,
  link_cons_act,
  (current_predicate(cons,(_ cons _)) ->
            [bot,has,constraints] if_error
                 ( bot cons _ ),
            [multiple,constraint,declarations,for,CType] if_error
                 (bagof(CT,Cons^(CT cons Cons),CTypes),
                  duplicated(CType,CTypes)),
            [constraint,declaration,given,for,atom] if_error
                 ( (a_ _) cons _ ),
      assert(alec(ct)),
      \+ \+ compile('.alec_throw')
  ; ([no,constraints,found] if_warning true)
  ),
  ( current_predicate(compile_cons_hook,compile_cons_hook) -> call_all(compile_cons_hook) ; true).

link_cons :-
  touch('.alec_throw'),
  absolute_file_name('.alec_throw',AbsFileName),
  retractall(ale_compiling(_)),  
  assert(ale_compiling(AbsFileName)),
  link_cons_act,
  retract(ale_compiling(_)).
link_cons_act :-
  assert(alec(trale_link)),
  compile('.alec_throw').

alec_catch_hook(trale_link,Code) :-
  findall(Type,type(Type),Types), % maxcount/3 and deranged/2 are already
  compute_cons(Types,Code).     % there - just re-attach to user constraints

:- dynamic complex_cons/5.
compile_complex_assert :-
  current_predicate('*>',(_ *> _))
  -> \+ ((Antecedent *> ConsequentandGoal),
	 ( ConsequentandGoal = (Consequent goal Goal) -> true
	 ; Consequent = ConsequentandGoal, Goal = true
	 ),
         find_narrow_vars(Antecedent,AntecNoQs,NarrowVars),
         find_trigger_type(AntecNoQs,bot,Trigger),
         assert(complex_cons(Trigger,AntecNoQs,Consequent,Goal,NarrowVars)),
         fail)
   ; true.

find_narrow_vars(Antecedent,AntecNoQs,NarrowVars) :-
  find_nv_act(Antecedent,AntecNoQs,[],NarrowVars).

find_nv_act(X^(Antecedent),AntecNoQs,NVsIn,NVsOut) :-
  !,find_nv_act(Antecedent,AntecNoQs,[X|NVsIn],NVsOut).
find_nv_act(AntecNoQs,AntecNoQs,NVs,NVs).

% this is a restricted form of infer_mgsat_type/3 that prohibits inequations and functions.
find_trigger_type(Var,Trigger,Trigger) :-
  var(Var),
  !.
find_trigger_type(Feat:_,TrigIn,TrigOut) :-
  introduce(Feat,Intro),
  !,unify_type(Intro,TrigIn,TrigOut).
find_trigger_type((D1,D2),TrigIn,TrigOut) :-
  !,find_trigger_type(D1,TrigIn,TrigMid),
  find_trigger_type(D2,TrigMid,TrigOut).
find_trigger_type((D1;D2),TrigIn,TrigOut) :-
  !,find_trigger_type(D1,TrigIn,TrigOut1),
  find_trigger_type(D2,TrigIn,TrigOut2),
  generalise(TrigOut1,TrigOut2,TrigOut).
find_trigger_type(Type,TrigIn,TrigOut) :-
  type(Type),
  !,unify_type(Type,TrigIn,TrigOut).
find_trigger_type(Ant,_,_) :-
  error_msg((nl,write('compile_cons: illegal description, '),write(Ant),
             write(', used in complex antecedent constraint'),nl)).

% compute_cons:  collect complex antecedent constraints into cons/2 clauses

compute_cons([],[end_of_file]).
compute_cons([Type|Types],Code) :-
  (retrieve_user_cons(Type,Cons,Goal) % user has cons, either directly or through '*>'/2
     -> Code = [(Type cons Cons goal Goal)|CodeRest]
      ; Code = CodeRest               % nothing to add
  ),
  compute_cons(Types,CodeRest).

retrieve_user_cons(Type,Cons,Goal) :-
  current_predicate(trale_cons,(_ trale_cons _)),
  (Type trale_cons TCons goal TGoal),
  !,findall(cc(CondVar,Cond,Conseq,CGoal),
            (clause(complex_cons(Type,Antec,Conseq,CGoal,NarrowVars),true),
             apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
            ComplexConss),
    ( ComplexConss == []
    -> Cons = TCons,
       Goal = TGoal
     ; Cons = (FS,TCons),
       apply_ccs(ComplexConss,FS,TGoal,Goal)
    ).

retrieve_user_cons(Type,Cons,Goal) :-
  current_predicate(trale_cons,(_ trale_cons _)),
  (Type trale_cons TCons),
  !,findall(cc(CondVar,Cond,Conseq,CGoal),
            (clause(complex_cons(Type,Antec,Conseq,CGoal,NarrowVars),true),
             apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
            ComplexConss),
    ( ComplexConss == []
    -> Cons = TCons,
       Goal = true
     ; Cons = (FS,TCons),
       apply_ccs(ComplexConss,FS,true,Goal)
    ).
retrieve_user_cons(Type,Cons,Goal) :-
  findall(cc(CondVar,Cond,Conseq,CGoal),
          (clause(complex_cons(Type,Antec,Conseq,CGoal,NarrowVars),true),
           apply_narrow_vars(NarrowVars,(CondVar=Antec),Cond)),
          ComplexConss),
  ComplexConss \== [], % otherwise fail to add no code
  Cons = FS,
  apply_ccs(ComplexConss,FS,true,Goal).

apply_narrow_vars([],Cond,Cond).
apply_narrow_vars([NV|NVs],CondIn,CondOut) :-
  apply_narrow_vars(NVs,NV^(CondIn),CondOut).

apply_ccs([],_,TGoal,TGoal).
apply_ccs([cc(FS,Cond,Conseq,CGoal)|CCs],FS,TGoal,Goal) :-
  apply_ccs(CCs,FS,(TGoal,when(Cond,(FS=Conseq,CGoal))),Goal).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Signature file input - calls read_ths/4 from pth.pl and converts to ALE 
%  sub/intro representation in O(n log n) time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

alec_catch_hook(trale_signature,Code) :-
  !,(current_predicate(signature,signature(_))
    -> findall(File,signature(File),[SigFile|Rest]),
       [multiple,signature,specifications,found,'-',using,'first:',SigFile]
         if_warning (Rest \== [])
    ; SigFile = 'signature'
    ),
  read_ths(SigFile,SubL,AppL,_),
  sort(SubL,SortedSubL),
  trale_sub(SortedSubL,Code,CodeMid),
  sort(AppL,SortedAppL),
  trale_intro(SortedAppL,CodeMid,[end_of_file]).

trale_sub([X-Y|SubLMid],[(X sub [Y|Ys])|CodeMid],CodeRest) :-
  trale_sub_act(SubLMid,X,Ys,CodeMid,CodeRest).

trale_sub_act([],_,[],Code,Code).
trale_sub_act([X-Y|SubLMid],Last,Ys,Code,CodeRest) :-
  (X = Last
  -> Ys = [Y|YsRest],
     trale_sub_act(SubLMid,Last,YsRest,Code,CodeRest)
   ; Ys = [],
     Code = [(X sub [Y|YsRest])|CodeMid],
     trale_sub_act(SubLMid,X,YsRest,CodeMid,CodeRest)).

trale_intro([],CodeRest,CodeRest).
trale_intro([app(T,F,R,_)|AppLMid],[(T intro [F:R|FRs])|CodeMid],CodeRest) :-
  trale_intro_act(AppLMid,T,FRs,CodeMid,CodeRest).

trale_intro_act([],_,[],Code,Code).
trale_intro_act([app(T,F,R,_)|AppLMid],Last,FRs,Code,CodeRest) :-
  (T = Last
  -> FRs = [F:R|FRsRest],
     trale_intro_act(AppLMid,Last,FRsRest,Code,CodeRest)
   ; FRs = [],
     Code = [(T intro [F:R|FRsRest])|CodeMid],
     trale_intro_act(AppLMid,T,FRsRest,CodeMid,CodeRest)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Logical-Variable Macros with Modes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% notation:
% someMacro(X-a,Y-b) := t, abc:X, def:Y.
% OR
% someMacro(X,Y) := t, abc:X, def:Y.

:- op(1125,xfx,':=').

:- multifile compile_macro_hook/0.
compile_macro_hook :- compile_lvmacro_act.

compile_lvmacro_act :-
  current_predicate(':=',(_ := _)) -> assert(alec(macro)),
                                      compile('.alec_throw')
 ; true.				      

alec_catch_hook(macro,[(:- multifile macro/2)|Code]) :-
% this is going to create warnings whenever user defines macro/2 without
% multifile - KNOWN BUG
  findall(logmacro(LVLHS,RHS),(LVLHS := RHS),LogMacros),
  compute_macros(LogMacros,Code).

compute_macros([],[end_of_file]).
compute_macros([logmacro(LVLHS,RHS)|LogMacros],[(LHS macro RHS)|Code]) :-
  functor(LVLHS,MacroName,Arity), % match the macro by name and arity
  functor(LHS,MacroName,Arity),
  ( add_lv_macro_descs(0,Arity,LVLHS,LHS) -> compute_macros(LogMacros,Code)
  ; error_msg((nl,write('error: logical macro arguments must be unshared variables in '),
	       write(LVLHS),nl))
  ).

add_lv_macro_descs(A,A,_,_) :- !.
add_lv_macro_descs(OldP,A,LVLHS,LHS) :-
  P is OldP + 1,
  arg(P,LVLHS,LVArg),  % LVArg is either macro-var. or macrovar-Desc pair
  arg(P,LHS,TArg),
  ( var(LVArg) -> LVArg = (TArg,_)     % if Desc is missing, interpret as bot
  ; LVArg = Var-Desc -> var(Var),   % check before instantiating to catch shared vars
                        Var = (TArg,Desc,_)	% adding anonymous variable pushes structure
                                                % sharing on Prolog level in RHS into
                                                % structure sharing on (ALE) logical level.
  ),
  add_lv_macro_descs(P,A,LVLHS,LHS). 
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Banner
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% SP4: quoted newlines must be \n
:- nl,write('\nTRALE Version 1.0; June, 2005\nCopyright (C) 1999,2003--2005, Gerald Penn\nAll rights reserved'),nl,
 initialise_trale_flags.
