% -*- prolog -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: trale-itsdb.pl,v $
%%  $Revision: 1.18 $
%%      $Date: 2005/06/19 17:22:03 $
%%     Author: Stefan Mueller (Stefan.Mueller@uni-bremen.de)
%%    Purpose: Output of TSDB++ statistics, communication
%%             with capi interface, see itsdb.pl
%%             This code is adapted from Frederik Fouvry's
%%             code for ALE
%%             Arity of edge and empty_cat changed
%%             solutions are explicitly stored in solutions/1
%%             by TRALE.
%%   Language: Prolog
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Log: trale-itsdb.pl,v $
%% Revision 1.18  2005/06/19 17:22:03  stefan
%% moved number_of to trale_emacs.pl which is always loaded, since
%% this is used by the trale_emacs interface as well.
%%
%% Revision 1.17  2004/03/31 16:02:13  stefan
%% fixed handling if no MRS is printed, now TSDB++ works with Felix' grammar.
%% fixed printing of rule information
%%
%% Revision 1.16  2004/03/10 14:33:36  stefan
%% *** empty log message ***
%%
%% Revision 1.14  2004/03/07 22:45:53  stefan
%% better pretty printing of offline tsdb info
%%
%% Revision 1.13  2004/03/07 20:24:07  stefan
%% added (a lot of) code that makes it possible to work with itsdb without a X connection.
%%
%% Revision 1.12  2003/10/08 20:41:33  stefan
%% moved reverse_count_lex_check to ale_redefinitions since
%% it is relevant for both the chart_display and [incr TSDB()].
%%
%% Revision 1.11  2003/10/05 14:16:37  dm
%% Redid the tokenization of the string and calling of the parser.
%% - Tokenization now allows a grammar to specify the tokenization that
%%   should be used.
%% - The parser is now called in a failure driven loop instead of a
%%   findall, which ensures that no unnecessary memory is consumed and
%%   it avoids the problem with the call to solution/1 in the findall
%%   after the call to rec/5, which due to additional backtracking
%%   led to n! number of solutions being reported instead of n.
%%
%% Revision 1.10  2003/08/13 12:06:31  stefan
%% fixed edge_limit check and exception catch in trale-itsdb and ale_redefinitions check and exception catch in trale-itsdb and ale_redefinition
%%
%% Revision 1.9  2003/08/09 15:11:47  dm
%% Changed rec/6 to rec/5 plus lookup of solution/1 to avoid having to
%% have yet another redefinition in ale_redefinitions.pl
%%
%% Revision 1.8  2003/08/09 14:40:58  dm
%% Moved reverse_count_lex_check here from trale_patches.pl (and removed
%% other temporary definition from ale_redefinitions.pl) and modified it
%% to fail when there are errors - instead of raising an exception.
%%
%% Revision 1.7  2003/08/06 10:09:15  stefan
%% I now catch exceptions (for instance missing words, if this
%% is introduced into the set of exception throwing errors again.
%%
%% Revision 1.6  2003/07/09 20:33:07  stefan
%% adapted to new protocol
%%
%% Revision 1.5  2003/05/19 15:30:46  stefan
%% different call to print_mrs. circumvented length limitation for
%% capi_printf in itsdb.pl. Reordered error output and mrs printing
%% so that MRS errors are recorded by [incr TSDB()].
%%
%% Revision 1.4  2003/05/13 09:10:50  stefan
%% put MRS printing in a different file
%%
%% Revision 1.4  2003/05/13 09:10:48  stefan
%% put MRS printing in a different file
%%
%% Revision 1.3  2003/05/10 13:48:53  stefan
%% *** empty log message ***
%%
%% Revision 1.2  2003/05/09 09:21:02  stefan
%% *** empty log message ***
%%
%% Revision 1.1  2003/05/08 21:50:58  stefan
%% *** empty log message ***
%%
%% Revision 1.3  2003/05/08 19:12:58  stefan
%% MRS-Output works
%%
%% Revision 1.2  2003/05/08 09:23:12  stefan
%% before converting print_mrs from working with descriptions to
%% using fs and the code in lr_utils.pl
%%
%%
%% Revision 1.1  2003/05/03 15:30:26  stefan
%% Initial revision
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- use_module(library(system),[environ/2]).
:- use_module(library(charsio),[format_to_chars/3,format_to_chars/4]).
:- use_module(library(lists),[memberchk/2,member/2,nth/3,nth0/3]).
:- use_module(library(ordsets),[ord_union/2,list_to_ord_set/2]).

:- use_module(itsdb,[write_field/2,write_field/4,write_table/2]).

:- ensure_loaded([trale_home('tsdb/retract_lex_desc'),    % retraction of stems
                  trale_home('chart_display/print_mrs') %,
%                  trale_home('chart_display/ale_redefinitions')
                 ]).

:- version('[incr tsdb()] interface for TRALE').

:- dynamic edge_limit/1, error/2.

%----------------------------------------------------------------------
% Predicates that actually collect the information from an application
% These are for ALE/TRALE (so not 3.2!)
% They are currently hard-wired in the module user.

% Special action must be taken for compiled predicates to know their
% numbers.  What do we want: grammar numbers or compiled numbers?
user:create_run(_TSInstanceName,_RunID,_Comment,_Interactivep,Protocol,_Command):-
	( callable(Command) -> call(Command)
   ; true
   ),
        retractall(derivation_protocol(_)),
        assert(derivation_protocol(Protocol)),

        prolog_version_number(Platform),
        trale_milca_version(TraleMilcaVersion),
        sformat(Application,'Trale ~w',[TraleMilcaVersion]),
        ( current_predicate(grammar_version/1) ->
		 grammar_version(Grammar)
	; Grammar = ''
	),
	number_of(type(_),NTypes),
 	number_of((_ macro _),NMacros),
 	number_of(lex(_,_),NLex),
        number_of(lex_rule(_,_),NLex_rule),
 	number_of(rule(_,_),NRule),

        (tsdb_offline_flag ->
            % we cannot write the run relation here since we have to do the test run before
            % we know the completion time.
            date_time(Start),
            retractall(start_time(_)),
            assert(start_time(Start))
        ;    
% Apparently supplied by itsdb
%	date_time(Start),
% Apparently supplied by itsdb
%	write_field('run-id',RunID),		% unique test run identifier
% Apparently supplied by itsdb
%	write_field('r-comment',Comment),		% descriptive narrative
        write_field('platform',Platform), % implementation platform (version)
% Apparently supplied by itsdb
% 	write_field('tsdb',''),		% tsdb(1) (version) used
 	write_field('application',Application),	% application (version) used
% 	write_field('environment',''),		% application-specific information

        
	( current_predicate(grammar_version/1) ->
		 grammar_version(Grammar),
		 write_field('grammar',Grammar) % grammar (version) used
	; true
	),
% Not in ALE/TRALE
%	write_field('avms',''),		        % number of avm types in image
 	write_field('sorts',NTypes),		% number of sort types in image
 	write_field('templates',NMacros),	% number of templates in image
 	write_field('lexicon',NLex),		% number of lexical entries
 	write_field('lrules',NLex_rule),	% number of lexical rules
 	write_field('rules',NRule)		% number of (non-lexical) rules
% Apparently supplied by itsdb
%  environ('USER',User),
%  write_field('user',User),		% user who did the test run
% Apparently supplied by itsdb
%  host_name(Host),
%  write_field('host',Host),		% machine used for this run
% Apparently supplied by itsdb
% 	shell('uname -rsv',_,OS),
%  write_field('os',OS),		% operating system (version)
% Apparently supplied by itsdb
%  write_field('start',Start),		% start time of this test run
% only in complete_run! - or supplied by itsdb?
% 	write_field('end',''),		% end time for this test run
% Apparently supplied by itsdb
% 	write_field('items',''),		% number of test items in this run
% Apparently supplied by itsdb
% 	write_field('status',''),		% exit status (PVM only)

            ),
        
         % remove unnecessary lexical items
         % the german grammar uses this to get rid of
         % stems which slow down things when zero inflected
         % material occurs in the string.
         ( current_predicate(retract_before_parsing,retract_before_parsing(_)),
             retract_before_parsing(X),
             format(user_error,"Retracting items matching retract_before_parsing/1  ...~n",[]),
             open_null_stream(DevNull),
             prolog_flag(user_error,UserError,DevNull),
             retract_lex_desc(X),
             prolog_flag(user_error,_,UserError)
         ; true
         ),
	true.

% It might be useful to redefine some of the rec predicates for test
% suite purposes (more accurate statistics collection, and removal of all
% pretty printing).

%process_item(1,'U mard rā dust dāšt.',_ParseID,1000,_Exhaustivep,Derivationp,X).
%process_item(1,'er schläft.',_ParseID,1000,_Exhaustivep,Derivationp,X).
%process_item(2,'这   只   狗  叫。',_ParseID,1000,_Exhaustivep,Derivationp,X).
%process_item(1,'买    汽车   的 人    很    多。',_ParseID,1000,_Exhaustivep,Derivationp,X).
user:process_item(ItemID,ItemInput0,_ParseID,MaxEdges,_Exhaustivep,Derivationp,
						_Interactivep):-
	retractall(edge_limit(_)),
	assert(edge_limit(MaxEdges)),

        reset_all,
        general_tokenize_sentence_atom(ItemInput0,ItemInput,Desc),
        nochart_display,
	trimcore,

        memory_used(X),
        assert(initially_memory_used(X)),

	statistics(garbage_collection,[GCs0,_,TGC0]),
	statistics(walltime,[Wall0,_]),
	statistics(runtime,[Run0,_]),

	compute_all_parses(ItemInput,Desc),

	statistics(runtime,[_,T]),

	findall(SolutionId-T,
		solution(SolutionId), 
		Results,
		[fail-RunLastFail]), % this fourth argument adds fail-RunLastFail to end of Results list

	statistics(runtime,[Run1,RunLastFail]),
	statistics(walltime,[Wall1,_]),
	statistics(garbage_collection,[GCs1,_,TGC1]),

        % write results has to be done before writing the errors since
        % the MRS output called in write results may produce error messages
        
	length(ItemInput,Length),
	used_edges(Length,ListOfTrees), % includes empty edges

	write_results(ItemID,Results,ListOfTrees,ItemInput), % only _used_ passive edges
        ( error(_,_) ->
            findall(Err,error(_,Err),ErrorList),
            list_to_atom(ErrorList,32,Error)

        ; Error = '' % for real text output without [inct tsdb()]
        ),
        % in case of errors due to memory or edge limits there could be readings
        ( error(word,_) ->
            Readings = -1
        ; otherwise ->
            length(Results,Readings0),
            Readings is Readings0 - 1
        ),
        
	Results = [FirstId-FirstT|_],
 	( FirstId \== fail ->
            First = FirstT
 	; First = -1
 	),
	TTotal is Run1 - Run0,
	TCPU is Run1 - Run0,
	TGC is TGC1 - TGC0,
	TReal is Wall1 - Wall0,
	number_of(edge(_,_,_,_,_,lexicon),Words), % ! does _not_ include empty edges
%	number_of(edge(_,_,_,_,_,_,_),Pedges), %! includes lexicon! The new trale also includes the empty cats here.

        edgenum(Edges),
        emptynum(E),             % The (negative) number of the next empty element, if there is one.
                                 % Therefore we have to subtract one after having it multiplied by -1 below.
        Pedges is Edges + (E * -1) - 1,
   
	ord_union(ListOfTrees,UsedEdges),
	length(UsedEdges,RPEdges),
%	statistics(program,[Heap,_]),

%       statistics(memory,[Memory,_]),   % total size of the process does not change from item to item
                                         % TSDB shows others in performance statistics so it makes
                                         % more sense to store MUsed there
        initially_memory_used(MIni),
	compute_max_memory,
        max_memory_used(MMax),
        
        MUsed is MMax - MIni,
%	statistics(atoms,[Symbols,_,_]),
	GCs is GCs1 - GCs0,

        (tsdb_offline_flag ->

            ParseId = ItemID,
            RunId = 1,
            
            Lstasks = '-1',
            Pctasks = '-1',
            Pftasks = '-1',
            Petasks = Pedges,  % this field should be '-1' since we do not know the parser tasks
                               % but in order to use the comparison provided by Compare|Performance in itsdb
                               % we fill in the pedges here
            Pstasks = '-1',
            Aedges  = '-1',
            Raedges = '-1',
            
            Unif    = '-1',
            Copies  = '-1',
            Conses  = '-1',
            Symbols = '-1',
            ILoad   = '-1',
            ALoad   = '-1',
            
            date_time(Date),
            Comment = '',

            parse_stream(ParseStream),
            ResultList = [
          ParseId, %parse-id :integer :key                # unique parse identifier
          RunId,   %run-id :integer :key                  # test run for this parse
          ItemID,  %i-id :integer :key                    # item parsed
          Readings,%readings :integer                     # number of readings obtained
          First,   %first :integer                        # time to find first reading
          TTotal,  %total :integer                        # total time for parsing
          TCPU,    %tcpu :integer                         # total (cpu) time forprocessing
          TGC,     %tgc :integer                          # gc time used
          TReal,   %treal :integer                        # overall real time
          Words,   %words :integer                        # lexical entries retrieved
          Lstasks, %l-stasks :integer                     # successful lexical ruleapplications
          Pctasks, %p-ctasks :integer                     # parser contemplated tasks (LKB)
          Pftasks, %p-ftasks :integer                     # parser filtered tasks
          Petasks, %p-etasks :integer                     # parser executed tasks
          Pstasks, %p-stasks :integer                     # parser succeeding tasks
          Aedges,  %aedges :integer                       # active items in chart (PAGE)
          Pedges,  %pedges :integer                       # passive items in chart
          Raedges, %raedges :integer                      # active items contributing to result
          RPEdges, %rpedges :integer                      # passive items contributing to result
          Unif,    %unifications :integer                 # number of (node) unifications
          Copies,  %copies :integer                       # number of (node) copy operations
          Conses,  %conses :integer                       # cons() cells allocated
          Symbols, %symbols :integer                      # symbols allocated
          MUsed,   %others :integer                       # bytes of memory allocated
          GCs,     %gcs :integer                          # number of garbage collections
          ILoad,   %i-load :integer                       # initial load (start of parse)
          ALoad,   %a-load :integer                       # average load
          Date,    %date :date                            # date and time of parse
          Error,   %error :string                         # error string (if applicable|:-)
          Comment  %comment: string                       # application-specific comment
     ],
  format(ParseStream,'~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w~n',
         ResultList),
  flush_output(ParseStream),
  msec2time(TTotal,PrintTime),
  PrintMem is (MUsed // 1024),
  host_name(Host),
  tsdb_item(ItemID,IInput),
            ( Error = '' ->
                format(user_error,'~w: ~t~d~14| Readings: ~t~w,~28| TTotal: ~t~w,~55| Pedges: ~t~w,~69| Mem: ~t~D kb,~86| ~w~n',[Host,ItemID,Readings,PrintTime,Pedges,PrintMem,IInput])
            ; format(user_error,'~w: ~t~d~14| Readings: ~t~w,~28| TTotal: ~t~w,~55| Pedges: ~t~w,~69| Mem: ~t~D kb,~86| ~w~t~130|Error: ~w~n',[Host,ItemID,Readings,PrintTime,Pedges,PrintMem,IInput,Error])
            )

        ;
% Apparently supplied by itsdb
%	write_field('parse-id',ParseID),	% unique parse identifier
%	write_field('run-id',''),		% test run for this parse

	write_field('i-id',ItemID),             % item parsed
        write_field('readings',Readings),       % number of readings obtained
        write_field('first',First),             % time to find first reading (msec)
	write_field('total',TTotal),            % total time for parsing (msec)
	write_field('tcpu',TCPU),		% total (cpu) processing time (msec)
	write_field('tgc',TGC),	        	% gc time used (msec)
	write_field('treal',TReal),	        % overall real time (msec)
	write_field('words',Words),             % lexical entries retrieved
%	write_field('l-stasks',''),		% successful lexical rule applications
%%	write_field('p-ctasks',''),		% parser contemplated tasks (LKB)
%	write_field('p-ftasks',''),		% parser filtered tasks

        % Should not be provided, since we do not know it.
        % See comment above.
	write_field('p-etasks',Pedges),		% parser executed tasks
%	write_field('p-stasks',''),		% parser succeeding tasks
%%	write_field('aedges',''),		% active items in chart (PAGE)
	write_field('pedges',Pedges),		% passive items in chart
%%	write_field('raedges',''),		% active items contributing to result
	write_field('rpedges',RPEdges),		% passive items contributing to result
%%	write_field('unifications',''),		% number of (node) unifications
%	write_field('copies',''),		% number of (node) copy operations        
%	write_field('conses',MUsed), %Heap\),		% cons() cells allocated
        write_field('others',MUsed),
%	write_field('symbols',Symbols),		% symbols allocated
%	write_field('others',Memory),		% bytes of memory allocated
        write_field('gcs',GCs), % number of garbage collections
% Apparently supplied by itsdb
%	write_field('i-load',''),		% initial load (start of parse)
% Apparently supplied by itsdb
%	write_field('a-load',''),		% average load
% Apparently supplied by itsdb
%   date_time(Date),
%	write_field('date',Date),		% date and time of parse

        write_field('error',Error),             % error string (if applicable |:-)
%	write_field('comment',''),		% application-specific comment
	( Derivationp == 1 ->
	    % here _all_ passive edges should be stored
	    true
	; true
	),
	write_rules,
	write_output),
        !. % I do not know where the memory leaks are, but there is at least one here.


% ALE uses a chart parser, so we here don't differentiate between first and
% all solutions but just call it in a failure driven loop. rec/4 asserts
% the result edges under predicate solution/1
compute_all_parses(ItemInput,Desc) :-
   catch(rec(ItemInput,_FS,Desc,_Residue),
	 Exception, 
	 ( (Exception = ale(unk_words(Words)) -> (sformat(String,'unknown words: ~w',[Words]),
                                                     assert(error(word,String)))
           ; (sformat(String,'~w',[Exception]),
               assert(error(exception,String)))),
                   
	   fail
	 )),
   fail
   ;
   true.


:- abolish([tokenize_sentence_string_default/3]). % defined in trale_extras.pl
 
tokenize_sentence_string_default(StringList,ItemInput,Desc) :- % Stefan M"uller's version
        convert_s(StringList,WordList),
        list2list_desc(WordList,ItemInput,Desc).
%        write(user_error,WordList),nl(user_error)


% Any switches control writing tables?
write_results(ItemID,Res,UsedEdges,ItemInput):-
	findall(Rec,(
	nth(ResultId,Res,RId-RTime),
	RId \== fail,

                     	findall(L,(member(Tree,UsedEdges),
				  memberchk(RId,Tree),
				  length(Tree,L)),[RPEdges]),

        find_derivation([RId],ItemInput,DerivC,[]), % Or can this be anything?
        atom_codes(Derivation,DerivC),
                     
        edge(RId,_From,_To,FSs,_FSRes,_Daughters,_Rule),
        print_mrs(FSs,tsdb,MRSC,[]),
        atom_codes(MRS,MRSC),

      (tsdb_offline_flag ->

          ParseId = ItemID,
          Size = -1,
          RAedges = -1,
          RCtasks = -1,
          RFtasks = -1,
          REtasks = -1,
          RStasks = -1,
          RTree = '',
          
       ResultList = [
         ParseId,    % parse-id :integer :key                # parse for this result
         ResultId,   % result-id :integer                    # unique result identifier
         RTime,      % time :integer                         # time to find this result (msec)
         RCtasks,    % r-ctasks :integer                     # parser contemplated tasks
         RFtasks,    % r-ftasks :integer                     # parser filtered tasks
         REtasks,    % r-etasks :integer                     # parser executed tasks
         RStasks,    % r-stasks :integer                     # parser succeeding tasks
         Size,       % size :integer                         # size of feature structure
         RAedges,    % r-aedges :integer                     # active items for this result
         RPEdges,    %r-pedges :integer                     # passive items in this result
         Derivation, % derivation :string                    # derivation tree for this reading
         RTree,      % tree :string                          # phrase structure tree (CSLI labels)
         MRS         % mrs :string                           # mrs for this reading
       ],

        format_to_chars('~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w~n', ResultList,RecC),
          eliminate(10,RecC,RecC_no_nl),
          eliminate(92,RecC_no_nl,RecC_no_nl_no_bs),
          append(RecC_no_nl_no_bs,[10],RecC_no_nl_no_bs1),
          atom_codes(Rec,RecC_no_nl_no_bs1)


      ;

           
           
% Apparently supplied by itsdb
%	write_field('parse-id',_ParseID),  % parse for this result
	write_field('result-id',ResultId,Rec,C1), % unique result identifier
	% get from list of results
	write_field('time',RTime,C1,C2),		  % time to find this result (msec)
%	write_field('r-ctasks',''),  % parser contemplated tasks
%	write_field('r-ftasks',''),  % parser filtered tasks
%	write_field('r-etasks',''),  % parser executed tasks
%	write_field('r-stasks',''),  % parser succeeding tasks
	% get somehow the size of the term
	write_field('size',-1,C2,C3),		  % size of feature structure
%	write_field('r-aedges',''),  % active items for this result
   % get the number of passive edges for this result only

	write_field('r-pedges',RPEdges,C3,C4),  % passive items in this result


%       1 fuer alles wie immer und 2 fuer UDF 2.0.  UDF (Unified Derivation Format) 2.0
                     
	(derivation_protocol(1) ->
            write_field('derivation',Derivation,C4,C5)
        ;   write_field('derivation','',C4,C5)  % C4=C5 % We don't know this format
        ),                       % derivation tree for this reading
%	write_field('tree',''),		  % phrase structure tree (CSLI labels)

	write_field('mrs',MRS,C5,Cn),		  % mrs for this reading
	Cn = [],
	true)),Results),
	(Results \== [] ->
            ( tsdb_offline_flag ->
                result_stream(ResultStream),
                write_list2(ResultStream,Results),
                flush_output(ResultStream)
            ;
                write_table('results',Results)
            )
        ; true
        ),
	true.

:- dynamic rule_results/2.

write_rules :-

        ( tsdb_offline_flag ->

            (rule(Rule,_),
                number_of(edge(_,_,_,_,_,_,Rule),Passives),
%             %ParseId :integer :key                # parse for this rule summary
%             %Rule :string                          # rule name
%             Filtered = -1, %:integer                     # rule postulations filtered
%             Executed = -1, %:integer                     # rule postulations executed
%             Success  = -1, %:integer                    # successful rule postulations
%             Actives  = -1, %:integer                      # active edges built from this rule
%             %passives :integer                     # passive edges built from this rule

%             rule_stream(RuleStream),
%             format(RuleStream,'~w@~w@~w@~w@~w@~w@~w~n',[ParseId,Rule,Filtered,Executed,Success,Actives,Passives]),
%             flush_output(ResultStream)

            ( retract(rule_results(Rule,OldPassives)) ->
                NewPassives is OldPassives + Passives
            ; otherwise ->
                NewPassives = Passives
            ),
            assert(rule_results(Rule,NewPassives)),
            
                fail
           ;
           true )
        
	; otherwise -> findall(Rec,(
% Not supplied by itsdb?
%	write_field('parse-id',ParseID,Rec,C1), % parse for this rule summary
                                    Rec = C1,
                                    rule(Rule,_),
                                    write_field('rule',Rule,C1,C2), % rule name
                                % 	write_field('filtered','',C2,C3),  % rule postulations filtered
                                % 	write_field('executed','',C3,C4),  % rule postulations executed
                                % 	write_field('successes','',C4,C5), % successful rule postulations
                                % 	write_field('actives','',C5,C6),	  % active edges built from this rule
                                    C2 = C6,
                                    number_of(edge(_,_,_,_,_,_,Rule),Passives),
                                    write_field('passives',Passives,C6,Cn), % passive edges built from this rule
                                    Cn = [],
                                    true),Rules),
                                %	write_table('rules',Rules),
            write_table('statistics',Rules)
        ).

% When and how?
write_output:-
% Not supplied by itsdb?
%   write_field('i-id',-1),	% item for this output specification
   write_field('o-application',''),	% applicable appliaction(s)
	( current_predicate(grammar_version/1) ->
		 grammar_version(Grammar)
	; Grammar = ''
	),
   write_field('o-grammar',Grammar),	% applicable grammar(s)
   write_field('o-ignore',''),	% ignore this item flag
   write_field('o-wf',-1),	% application-specific grammaticality
   write_field('o-gc',-1),	% maximal number of garbage collections
   write_field('o-derivation',''),	% expected derivation
   write_field('o-tree',''),	% expected phrase structure tree
   write_field('o-mrs',''),	% expected mrs
   write_field('o-edges',-1),	% maximal number of edges to build
   environ('USER',User),
   write_field('o-user',User),	% author of this output specification
	itsdb:date_time(Date),
   write_field('o-date',Date),	% creation date
	true.

% Not sure if these are output or input tables ...
write_tree:-
% Probably supplied by itsdb
%  write_field('parse-id',_),
%	write_field('t-version',_),
%	write_field('t-active',_),
%	write_field('t-confidence',_),
%	write_field('t-author',_),
%	write_field('t-start',_),
%	write_field('t-end',_),
%	write_field('t-comment',''),
	true.

write_decision:-
% Probably supplied by itsdb
%	write_field('parse-id',_),
%	write_field('version',_),
%	write_field('d-state',_),
%	write_field('d-type',_),
%	write_field('d-key',_),
%	write_field('d-value',_),
%	write_field('d-start',_),
%	write_field('d-end',_),
%	write_field('d-date',_),
	true.

user:complete_run(_RunID,_AppSpecific):-
% Apparently supplied by itsdb
% 	date_time(End),
%  	write_field('end',End),		% end time for this test run
	true.

user:reconstruct_item(_):-
	true.

%---------------------------------------------------------------------
% Auxiliary predicates



used_edges(Length,UsedEdges):-
	findall(Set,(solution(Top),
                     edge(Top,0,Length,_,_,D,_),
		     recreate_tree(D,Edges,[Top]),
		     list_to_ord_set(Edges,Set)),UsedEdges).
	
recreate_tree([],List,List).
recreate_tree([Edge|Sisters],Edges,Rest0):-
	( integer(Edge) ->
		 edge(Edge,_,_,_,_,Daughters,_)
	; Edge = empty(Nr,Pos),
	  empty_cat(Nr,Pos,_,Daughters,_)
	),
	recreate_tree(Daughters,Rest,[Edge|Rest0]),
	recreate_tree(Sisters,Edges,Rest).

% Similar to recreate_tree, only rule names are collected instead of edge numbers
% and the tree shape is retained.

% For reasons I do not understand, Gerald's version of Trale needs "\\" to escape
% the quotation marks. This was not the case with the old Trale. St. Mü. 06.06.2006

find_derivation([],_,Deriv,Deriv).
find_derivation([Edge|Sisters],String,Deriv,Derivn):-
	( integer(Edge) ->
		 edge(Edge,From,To,_,_,Daughters,Rule),
		 ( Rule == lexicon -> nth0(From,String,Name) ; Rule = Name ),
	    format_to_chars(' (\\"~a\\" ~d ~d',[Name,From,To],Deriv,Deriv1)
	; Edge = empty(Nr,Pos),
	  empty_cat(Nr,Pos,_,Daughters,Rule),
            format_to_chars(' (\\"~a\\" ~d ~d',[Rule,Pos,Pos],Deriv,Deriv1)
	),
	find_derivation(Daughters,String,Deriv1,Deriv2),
	format_to_chars(')',[],Deriv2,Deriv3),
	find_derivation(Sisters,String,Deriv3,Derivn).

reset_all :-
  clear,
  retractall(error(_,_)),
  reset_timer,
  trimcore,
  reset_gc_stat,
  reset_max_memory.


:- use_module(library(system),[popen/3]).

shell(Command,_Status,Atom):-
	system:popen(Command,read,Stream),
	read_stream(Stream,Charlist),
	close(Stream),
	atom_codes(Atom,Charlist).

read_stream(Stream,Charlist):-
	get_code(Stream,C),
	(C == -1 -> Charlist = []
	; Charlist = [C|Charlist0],
	  read_stream(Stream,Charlist0)
	).

normalise_input_string([],[]).
normalise_input_string([W|Ws],[NW|NWs]):-
	atom_codes(W,[C|R]),
	( lower_upper_tab(NC,C) ->
		 atom_codes(NW,[NC|R])
	; NW = W
	),
	normalise_input_string(Ws,NWs).



%%    Purpose: Code for working with [incr TSDB()] without
%%             X connection. Should not be necessary, but
%%             currently [incr TSDB()] crashes or hangs when
%%             X connections are terminated.


% reads items from the tsdb item file and asserts them to the Prolog database
get_items(TSDir) :-
        sformat(ItemFile,'~w/item',[TSDir]),
        seeing(OldInput),
        see(ItemFile),
        repeat,
          (get_line(T) ->             % defined in sics.pl
              process_item_line(T),
              fail
          ; true
          ),
        !,
        seen, 
        see(OldInput).

process_item_line(end_of_file) :- !.

process_item_line(List) :-
        get_items(List,[Iid,_Iorigin,_Iregister,_Iformat,_Idifficulty,_Icategory,Iinput|_]),
        assert(tsdb_item(Iid,Iinput)).
        


get_items(List,[Field|Fields]) :- 
        append(FieldCs,[64|Rest],List),!,
        name(Field,FieldCs),
        get_items(Rest,Fields).

get_items(LastField,LastField).


:- dynamic tsdb_offline_flag/0, run_stream/1, parse_stream/1, result_stream/1.


tsdb_online :-
   retractall(tsdb_offline_flag).

tsdb_offline :- 
   retractall(tsdb_offline_flag),
   assert(tsdb_offline_flag).

:- tsdb_online.

% test a subset of these items, which is defined in the script that calls TRALE

test_from_to([From,To],RunID,MaxEdges,TSDir) :-
  format(user_error,'~nTesting items ~w to ~w with an edge limit of ~w and run-id ~w ...~n~n',[From,To,MaxEdges,RunID]),
  get_items(TSDir),
  tsdb_offline,
  create_run(_TSInstanceName,RunID,_Comment,_Interactivep,_Protocol,_Command),

  datime(datime(Year,Month,Day,Hour,Minute,Second)),
  sformat(FileString,'~w-~w:~w-~w-~w_~w:~w:~w',[From,To,Year,Month,Day,Hour,Minute,Second]),

  sformat(RunName,   '~w/run-~w',   [TSDir,FileString]),
  open(RunName,write,RunStream),
  retractall(run_stream(_)),
  assert(run_stream(RunStream)),
  
  sformat(ParseName, '~w/parse-~w', [TSDir,FileString]),
  open(ParseName,write,ParseStream),
  retractall(parse_stream(_)),
  assert(parse_stream(ParseStream)),
  
  sformat(ResultName,'~w/result-~w',[TSDir,FileString]),
  open(ResultName,write,ResultStream),
  retractall(result_stream(_)),
  assert(result_stream(ResultStream)),
  
  
  test_from_to_aux([From,To],MaxEdges),

  % as above in create_run
  
        prolog_version_number(Platform),
        trale_milca_version(TraleMilcaVersion),
        sformat(Application,'Trale ~w',[TraleMilcaVersion]),
        ( current_predicate(grammar_version/1) ->
		 grammar_version(Grammar)
	; Grammar = ''
	),
	number_of(type(_),NTypes),
 	number_of((_ macro _),NMacros),
 	number_of(lex(_,_),NLex),
        number_of(lex_rule(_,_),NLex_rule),
 	number_of(rule(_,_),NRule),

        Comment = '',
        Environment = '',
        Avms = -1,
        Tsdb = '',
        environ('USER',User),
        
        host_name(Host),
        shell('uname -rsv',_,OSnl),
        atom_codes(OSnl,OSString),
        append(OSString_no_nl,[_],OSString),
        atom_codes(OS,OSString_no_nl),
        Status = 'Completed',
        Items is To - From + 1,                       % lets assume that all numbers are used
        start_time(Start),
        date_time(End),

        run_stream(RunStream),
        format(RunStream,'~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w@~w~n',
         [
          RunID,       %run-id :integer :key                  # unique test run identifier
          Comment,     %comment :string                       # descriptive narrative
          Platform,    %platform :string                      # common-lisp platform (version)
          Tsdb,        %tsdb :string                          # tsdb(1) (version) used
          Application, %application :string                   # application (version) used
          Environment, %:string                               # application-specific information
          Grammar,     %grammar :string                       # grammar (version) used
          Avms,        %avms :integer                         # number of avm types in image
          NTypes,      %sorts :integer                        # number of sort types in image
          NMacros,     %templates :integer                    # number of templates in image
          NLex,        %lexicon :integer                      # number of lexical entries
          NLex_rule,   %lrules :integer                       # number of lexical rules
          NRule,       %rules :integer                        # number of (non-lexical) rules
          User,        %user :string                          # user who did the test run
          Host,        %host :string                          # machine used for this run
          OS,          %os :string                            # operating system (version)
          Start,       %start :date                           # start time of this test run
          End,         %end: date                             # end time for this test run
          Items,       %items: integer                        # number of test items in this run
          Status       %:string
         ]),
        close(RunStream),
        close(ParseStream),
        close(ResultStream).


test_from_to_aux([From,To],MaxEdges) :-
   \+ \+ ( (  From @=< To 
	   -> Next is From + 1,
	      ( tsdb_item(From,ItemInput) ->
                  process_item(From,ItemInput,_ParseID,MaxEdges,_Exhaustivep,_Derivationp,_Interactivep)
              ; otherwise ->
                  format(' (~w) * Test item not found *~n',[From])
              ),
              test_from_to_aux([Next,To],MaxEdges)
	   ;  true
	   ) ).

eliminate(_Eliminate,[],[]).
eliminate(Eliminate,[H|T],R) :-
   (  H == Eliminate 
   -> eliminate(Eliminate,T,R)
   ;  eliminate(Eliminate,T,R2),
       [H|R2]=R
   ).




write_list2(_Stream,[]).
write_list2(Stream,[H|T]) :-
        write(Stream,H),
        write_list2(Stream,T).


list_to_string([Last],_,LastS) :-
  name(Last,LastS).

list_to_string([H|T],D,String) :-
  list_to_string(T,D,S1),
  name(H,HS),
  append(HS,[D|S1],String).

list_to_atom(List,Delimiter,Atom) :-
  list_to_string(List,Delimiter,String),
  name(Atom,String).


